#pragma once

#include "Matrix4.h"

namespace IDVR{
namespace Tools{

	using namespace System::Drawing;

/**
<b>Class for handling object rotation by calculating rotaton angle out of mouse movement</b>.

This class is part of the Tools namespace and the main goal is the claculation of a rotation matrix based on
arbitrary mouse movement of the user.
<p>
The idea is that if the user moves the mouse pointer on the canvas (see class OpenGLCanvas) the corresponding angle to
that movement will be calculated around the three main axes. The different angles are defined on the surface of a specified 
sphere (<i>Arcball sphere</i>) which will be "placed" in the middle of the canvas. 
<br>Thus, the length of a mouse movement will be projected on the sufrace of this sphere and the angles to the middle 
point of the sphere can be calculated.
@author <i>Bernhard Pflugfelder</i>
@version 1.0
@date 02-18-2005
*/
public __gc class ArcBall
{
   private:
	     /**
		 Projects the specified point on the <i>Arcball sphere</i> surface.
		 This function calculates the surface coordinates of the corresponding point by projection.
		 @param[ in ] n_NewPt the actual mouse position.
		 @param[ out ] n_NewVec the projected position on the sphere surface.
	     */
		 void mapToSphere(PointF n_NewPt, Vector3 *n_NewVec);
		 /**
		 Calculates a simple SVD value of the specified 4x4 matrix.
		 This function calculates a very simple SVD value which will be used as a scale factor for actual rotation matrix.
		 @param n_DestinationObj the instance of the Matrix4 class which saves the original matrix.
		 @return the calculated SVD value.
		 */
		 float getMatrixSVD(Matrix4 *n_DestinationObj);
		 /**
		 Copies the rotational component (upper 3x3) of the input matrix to a new destination matrix.
		 @param[ in ] n_SourceObj the input matrix.
		 @param[ out ] n_DestinationObj the ouput matrix.
	     */
		 void setRotationScaleFromMatrix(Matrix4 *n_DestinationObj, Matrix4* n_SourceObj);
		 /**
		 Multiplies the the rotational component (upper 3x3) of the input matrix with the specified scale value.
		 @param[ in ] scale the scale value.
		 @param[ out ] n_DestinationObj the ouput matrix.
	     */
		 void setMatrixMulRotationScale(Matrix4 *n_DestinationObj, float scale);

   public:   
			/**
			Constructor initialization with the valid mouse bounds for canvas height and width.
			This valid bound area specifies all valied values for possible mouse point, all point outside this
			area will be clipped during the calcualtion.
			@param NewWidth the valid bound width.
			@param NewHeight the valid bound height.
			*/
			ArcBall(float NewWidth, float NewHeight);
                
			/** 
			Set new mouse bounds for heigth and width.
			@param NewWidth the valid bound width.
			@param NewHeight the valid bound height.
			*/
			void setBounds(float NewWidth, float NewHeight);

            /**
			Saves the current specified mouse click point as the new start point.
			@param NewPt the current start point of a mouse movement.
			*/
            void click(PointF NewPt);

			/**
			Saves the current specified mouse drag point as the new end point and calculates the corresponding rotation.
			@param[ in ] NewPt the current end point of a mouse movement.
			@param[ out ] NewRot the calculated quaternion of the start and end vector of the mouse movement.
			*/
            void drag(PointF NewPt, Vector4 *NewRot);

			/**
			Resets the current start and end vectors of the rotation calculation.
			*/
			void resetVectors(void);

			/**
			Sets the value of this matrix to the matrix conversion of the quaternion argument. 
			@param[ in ] n_DestinationObj the input matrix.
			@param[ out ] n_Vector4 the quaternion to be converted.
			*/
			void CreateRotationMatrixFromVector4(Matrix4 *n_DestinationObj, Vector4 *n_Vector4);

			/**
			Sets the rotational component (upper 3x3) of this matrix to the matrix
			values in the T precision Matrix3d argument. 
			The other elements of this matrix are unchanged; a singular value decomposition is performed
			on this object's upper 3x3 matrix to factor out the scale, then this
			object's upper 3x3 matrix components are replaced by the passed rotation
			components, and then the scale is reapplied to the rotational
			components.
			@param[ in ] n_SourceObj the input matrix.
			@param[ out ] NewObj the ouput matrix.
			*/
			void SetRotationFromMatrix(Matrix4* NewObj, Matrix4* n_SourceObj);

        private:
            Vector3 *StVec;		  /**< Saves the start click vector of a mouse movement. */
            Vector3 *EnVec;       /**< Saves the drag click vector of a mouse movement. */
            float AdjustWidth;    /**< Saves the mouse bounds width. */
            float AdjustHeight;   /**< Saves the mouse bounds height. */

			static const double Epsilon = 1.0e-5; /**< This constant is the minimum value which will be used for all calculations in this class.
												       That means if calculated angle values are less than Epsilon this angles will be disregarded. */ 
			//Arcball sphere constants:
			//Diameter is       40.0f
			//Radius is         20.0f
			//Radius squared is 400.0f
			static const float RADIUS = 9.0f;        /**< This constant is the radius of the <i>Arcball sphere</i>. */
			static const float RADIUSSQUARED = 3.0f; /**< This constant is the squared radius of the <i>Arcball sphere</i>. */

};
}
}

