#pragma once

#include "IShadingmodel.h"
#include "Vector3.h"
#include "Transferfunction.h"

namespace IDVR {
namespace VolumeRenderer {
namespace SWVolumeRenderer {
namespace ShadingModels {

using namespace System::Drawing;
using namespace IDVR::Tools;
using namespace IDVR::Interfaces;
using namespace IDVR::VolumeRenderer::RenderPrimitives;
using namespace IDVR::Volume::DataStructures;

/**<B>Class for the contour shading model.</B>

Namespace: IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels

Baseclass: IDVR::Interfaces::IShadingModel

This class calculates the color value of a given raysample. raysamples the lie on the contour of an object get a high,
others a low value.

@see MipShadingModelSW - the mip shading model
@see PhongShadingModelSW - the phong shading model
@see ToneShadingModelSW - the tone shading model
@author <i>Christopher Thurnher</i>
@version 1.0
@date 02-18-2005
*/

public __gc class ContourShadingModelSW : public IShadingModel
{
private:

	__value enum colorIndices{RED, GREEN, BLUE};

	short m_DensityScope; /**< the scope of the density values of the dataset. */
	double m_Exponent; /**< bigger exponent => thinner contour lines. */
	double m_GradientMagnitudeThreshold; /**< a big gradient magnitude threshold indicates a surface. */
	Vector3* m_View; /**< the view vector of the image plane. */
	float m_BaseColor __gc[]; /**< the base color that the contour lines will have. */
	float m_KTF; /**< ratio of the calculated color according to the transfer function. */
	Transferfunction* m_TransferFunc; /**< the specified transfer function. */

public:

	/**
	Standard constructor.
	*/
	ContourShadingModelSW(void);
	/**
	Constructor.
	@param n_DensityScope the scope of the density values of the dataset.
	@param n_GradientMagnitudeThreshold the gradient magnitude threshold.
	@param n_Exponent the exponent.
	@param n_Col the color of the contour lines.
	*/
	ContourShadingModelSW(unsigned short n_DensityScope, double n_GradientMagnitudeThreshold, double n_Exponent, Color n_Col);
	/**
	Constructor.
	@param n_DensityScope the scope of the density values of the dataset.
	@param n_GradientMagnitudeThreshold the gradient magnitude threshold.
	@param n_Exponent the exponent.
	@param n_Col the color of the contour lines.
	@param n_KTF the ration of the transfer function.
	@param n_TransferFunc the transfer function.
	*/
	ContourShadingModelSW(unsigned short n_DensityScope, double n_GradientMagnitudeThreshold, double n_Exponent, Color n_Col, float n_KTF, Transferfunction* n_TransferFunc);

	/**
	Calculates the Color value for a given raysample.
	@param n_RaySample the specified raysample.
	@return The modified raysample.
	*/
	virtual RaySample* doShading(RaySample* n_RaySample);

	/**
	Sets the view direction.
	@param n_View the view direction.
	*/
	__property void set_ViewDirection(Vector3* n_View);
	/**
	Returns the gradient magnitude threshold.
	@return The gradient magnitude threshold.
	*/
	__property double get_ContourGradientMagnitudeThreshold(void);
	/**
	Returns the exponent.
	@return The exponent.
	*/
	__property double get_Exponent(void);
	/**
	Returns the current color as Color object.
	@return The current color as Color object.
	*/
	__property Color get_Col(void);
	/**
	Returns the ktf value.
	@return The ktf value.
	*/
	__property float get_ContourKTF(void);
};
}
}
}
}