#pragma once

#include "VolumeData.h"
#include "DataObject.h"
#include "VoxelData.h"
#include "Histogram.h"
#include "Transferfunction.h"
#include "IRenderer.h"
#include "RendererSW.h"
#include "TFClassificationModelSW.h"
#include "DvrCompositingModelSW.h"
#include "PhongShadingModelSW.h"
#include "ProgressWin.h"
#include "GradientEstimationSW.h"

namespace IDVR {
namespace Volume {

	using namespace System;
	using namespace System::IO;
	using namespace System::Threading;
	using namespace IDVR::Volume::DataStructures;
	using namespace IDVR::VolumeRenderer::SWVolumeRenderer::RenderCalculations;

	/**
	<b>Class to load the entire data structure from a specified data file.</b>

	This class is part of the main namespace Volume in which it is the only memeber class besides the subnamespace 
	DataStructure.
	<br><br>
	With this class the loading of the entire volume data set from an arbitrary choosen *.dat file (ordinary 
	ASCII-Textfile with the file extension .dat) will be easily achived. Only the valid file name and file path must be
	specified outside of this class structure in an instance of the main GUI class Neu.
	<br><br>
	Furthermore, this class contains the essantial member object of the class VolumeData which is the controll class of
	the stored volume data structure after the loading process. Every class which accesses this volume data structure has to
	do this by using this member object.
	<br>
	Note that with the deleting of the instance of this class also the current loaded volume data structure will be lost.
	@see Neu - the main GUI class.
	@see VolumeData - the main datastructure class.
	@author <i>Bernhard Pflugfelder</i>
	@version 1.0
	@date 02-19-2005
	*/
	public __gc class DataLoader
	{
	public:
		/**
		This is the standard constructor.
		All member variables of this new instance will be set to zero/null.
		*/
		DataLoader(void);
		/**
		This is the standard deconstructor.
		All member variables will be deleted.
		*/
		~DataLoader(void);

		/**
		Reads the entire volume data structure from file and initialze the actual data structure.
		This function encapsulates the main task of this class which is the loading of the volume data structure from
		the already specified file name (see DataLoader::set_FileName). The loaded data will be automatically saved in
		an instance of the class VolumeData which is the member object DataLoader::m_Dataset.
		*/
		void readDataFile(void);
		/**
		Reads the mask data from file and sets the corresponding voxel object ID numbers.
		This function loads the mask data from the already specified file name (see DataLoader::set_FileName) and updates 
		the current member object DataLoader::m_Dataset. This means a new volume data object has to be created and the 
		corresponding voxel object ID keys have to be set.
		@param n_ObjectDescription the description of the new volume data object.
		@return The unique object ID of the newly created volume data object.
		*/
		unsigned short readMaskFile(String* n_ObjectDescription); 
		/**
		Calculates all gradient vector values of the volume data structure.
		This function will be only used if the current data set is not too big because the saving of the gradients consumes lot
		of memory. Thus, if the current data set is actually too big (see DataLoader::MaxUsableMemory for the maximum usable 
		memory size) the gradients will be calculated during the rendering process.
		<br><br>
		The member variable DataLoader::m_GradientEstimationMode gives information about which gradient calculation mode should
		be used (defined in the %GradientEstimationSW class).
		@see GradientEstimationSW
		*/
		void calculateGradients(void);
		/**
		Delets the entire data structure for saving memory.
		*/
		void clearDataSet(void);
		/**
		Resets the object IDs of all voxels of the volume data structure to the general object.
		@see DataObject
		*/
		void resetObjectIDs(void);

		/**
		Sets a new file name and file path for upcoming loading processes.
		@param n_FileName the new file path and name.
		*/
		__property void set_FileName(String *n_FileName);
		/**
		Returns the current file name and file path.
		@return The current saved file path and name.
		*/
		__property String* get_FileName(void);
		/**
		Sets a new gradient calculation mode for upcoming loading processes.
		@param n_GradientEstimationMode the new gradient calculation mode.
		@see GradientEstimationSW
		*/
		__property void set_GradientEstimationMode(unsigned char n_GradientEstimationMode);

		//Control classes for the progress window during the loading process.
		//This progress window will be updated by an own thread which is started in the main
		//GUI class Neu.
		void setProgressbarHandler(ProgressWin *n_ProgressbarHandler);
		void setGUIHandler1(Thread *n_GUIHandler1);
		void setGUIHandler2(Thread *n_GUIHandler2);

		/**
		Returns the current volume data structure.
		@return An instance of the class VolumeData.
		*/
		VolumeData* getDataset(void);
	
	private:

		/** Constant index values for the three components of the three dimensional volume data set. */
		__value enum voxelIndices {X = 0,Y,Z};

		ProgressWin *m_ProgressbarHandler;
		Thread *m_GUIHandler1, *m_GUIHandler2;

		unsigned char m_GradientEstimationMode; /**< Saves the current gradient calculation mode. */
		String *m_FileName; /**< Saves the current file name and file path. */

	public:

		VolumeData *m_Dataset; /**< Saves the current volume data structure. */
		
		static String* GeneralDescription = S"Allgemein";
		static unsigned short GeneralImportance = 0, CanvasSize = 512; 
		static const unsigned char HeaderLength = 3, NumBytes = 2, NumOfBytesPerVoxel = 14;

		static float Default_Viewdirection_X = 0.0f,
					 Default_Viewdirection_Y = 0.0f,
					 Default_Viewdirection_Z = 1.0f;

		static float Default_Lightdirection_X = 0.0f,
					 Default_Lightdirection_Y = 0.0f,
					 Default_Lightdirection_Z = 1.0f;

		//Application settings
		static unsigned short DensityScope = 4096, MaxUsableMemory = 300 /*MBytes of main memory*/;
		static String* configFileName = S"config.xml";
		static String* Default_DataFileDirectory;
		static String* Default_MaskFileDirectory;
		static String* Default_ConfigFileDirectory;
		static String* Default_TransFuncFileDirectory;

		//PhongShading Values
		static float Default_Ambient = 0.5f;
		static float Default_Diffuse = 0.8f;
		static float Default_Specular = 0.5f;
		static short Default_SpExponent = 10;

		//DepthCueing Values
		static float Default_K1DepthCueing = 1.0f;
		static float Default_K2DepthCueing = 0.0f;

		//Optimation Values
		static float Default_SamplingDistance = 0.7f;
		static float Default_GradientThreshold = 0.01f;
		static float Default_OpacityThreshold = 1.0f;

		//MIP Settings
		static int Default_MipThreshold = 255;
		static Color Default_MipColor = Color::White;

		//Contour Settings
		static float Default_ContourThreshold = 1.0f;
		static float Default_ContourGradientThreshold = 0.4f;
		static int Default_ContourExponent = 4;
		static Color Default_ContourColor = Color::White;
		static float Default_ContourKTF = 0.5f;

		//Tone Settings
		static float Default_ToneThreshold = 1.0;
		static float Default_ToneMagnitudeThreshold = 0.2f;
		static float Default_ToneKWarm = 0.5f;
		static float Default_ToneKCold = 0.5f;
		static float Default_ToneKTF = 0.5f;
		static Color Default_ToneWarmColor = Color::Yellow;
		static Color Default_ToneColdColor = Color::Blue;

		//DVR Settings
		static float Default_DvrOpacity = 0.5f;
		static Color Default_DvrColor = Color::Blue;
	};
}
}

		 