#pragma once

using namespace CsGL::OpenGL;
using namespace CsGL::Pointers;
using namespace CsGL::Util;
using namespace System;
using namespace System::Drawing;

namespace IDVR {
namespace Tools {
namespace OpenGLTools {

	/**
	<b>Class for managing the draw process onto the canvas.</b>

	This class is part of the OpenGLTools namespace which is actually a subnamespace of the main namespace Tools. 
	<br><br>
	After the final color values are calculated by the application's raycaster of the class RenderMachineSW 
	this resultshave to be drawn onto the canvas. Furthermore, this drawing process should be implemented with the
	help of OpenGL texture management for most efficient processing.<br><br>
	As every other class of the subpackage OpenGLTools this class has the base class OpenGLCode of the CSOpenGL library
	to correctly implement the OpenGL code Which will be handled by the OpenGLPanel which will show the specific drawing 
	results on the screen.<br><br>
	Thus, te main task of this class is the mapping of the final color values which are stored as a two dimensional array
	into OgenGL texture objects which will be actually drawn onto the canvas.

	@see OpenGLCanvas - the GUI class for the canvas with its draw control
	@see RenderMachineSW - the actual raycaster of the application.
	@author <i>Bernhard Pflugfelder</i>
	@version 1.0
	@date 02-20-2005
	*/
public __gc class DrawModelGLSW : public OpenGLPanel::OpenGLCode
{
private:

	short m_CanvasWidth;   /**< Saves the current width of the canvas. */
	short m_CanvasHeight;  /**< Saves the current height of the canvas. */
	short m_TextureWidth;  /**< Saves the width of the essential texture object. */
	short m_TextureHeight; /**< Saves the height of the essential texture object. */
	unsigned char m_FrameBuffer __gc[];  /**< Saves the final color values. */
	float m_CanvasBackColor __gc[];      /**< Saves the current canvas background color. */
	GLuint m_ResultTex;					 /**< Saves the index of the used OpenGL texture object. */

	 /** Constant index values for the three color components red, green and blue. */
	__value enum ColorIndices{R = 0,G,B};

public:

	/**
	This is the standard constructor.
	All member variables of this new instance will be set to zero/null.
	*/
	DrawModelGLSW(void);
	/**
	Constructor initialization with the valid canvas and texture sizes.
	All member variables will be set to zero besides the canvas dimension size member variables and the texture
	size member variables.
	@param n_CanvasWidth the width of the current canvas.
	@param n_CanvasHeight the height of the current canvas.
	@param n_TextureWidth the width of the texture which actually is the row count of the final color values.
	@param n_TextureHeight the height of the texture which actually is the row count of the final color values.
	*/
	DrawModelGLSW(short n_CanvasWidth, short n_CanvasHeight, short n_TextureWidth, short n_TextureHeight);
	/**
	Constructor initialization with the valid canvas size and canvas background color.
	All member variables will be set to zero besides the canvas dimension size member variables and the canvas background 
	member variable.
	@param n_CanvasWidth the width of the current canvas.
	@param n_CanvasHeight the height of the current canvas.
	@param n_CanvasBackColor the background color of the current canvas.
	*/
	DrawModelGLSW(short n_CanvasWidth, short n_CanvasHeight, Color n_CanvasBackColor);
	/**
	Constructor initialization with all included member variables.
	All member variables will be set to the cooresponding parameter. Thus, parameter for all included member variables have
	to be assigned.
	@param n_CanvasWidth the width of the current canvas.
	@param n_CanvasHeight the height of the current canvas.
	@param n_TextureWidth the width of the texture which actually is the row count of the final color values.
	@param n_TextureHeight the height of the texture which actually is the row count of the final color values.
	@param n_FrameBuffer the actual final color values of the rendering process.
	@param n_CanvasBackColor the background color of the current canvas.
	*/
	DrawModelGLSW(short n_CanvasWidth, short n_CanvasHeight, short n_TextureWidth, short n_TextureHeight, unsigned char n_FrameBuffer __gc[], Color n_CanvasBackColor);
	/**
	This is the standard deconstructor.
	All member variables will be deleted.
	*/
	~DrawModelGLSW(void);

	/**
	This function overwrites the base init funtion of the class OpenGLCode.
	It contains the actual initialization code in OpenGL and also performs the code.
	*/
	virtual void Init(void);
	/**
	This function overwrites the base draw funtion of the class OpenGLCode.
	It contains the actual drawing code in OpenGL and also performs the code.
	*/
	virtual void Draw(void);

	/**
	Sets a new witdh for the canvas.
	@param n_CanvasWidth the new canvas width.
	*/
	__property void set_CanvasWidth(short n_CanvasWidth);
	/**
	Sets a new height for the canvas.
	@param n_CanvasHeight the new canvas height.
	*/
	__property void set_CanvasHeight(short n_CanvasHeight);
	/**
	Sets a new width for the used texture object.
	@param n_TextureWidth the new texture object width.
	*/
	__property void set_TextureWidth(short n_TextureWidth);
	/**
	Sets a new height for the used texture object.
	@param n_TextureHeight the new texture object height.
	*/
	__property void set_TextureHeight(short n_TextureHeight);
	/**
	Sets a new canvas background color.
	@param n_CanvasBackColor the new canvas background color as an instance of the standard color class.
	*/
	__property void set_CanvasBackColor(Color n_CanvasBackColor);

	/**
	Returns the witdh of the current canvas.
	@return The current canvas width.
	*/
	__property short get_CanvasWidth(void);
	/**
	Returns the height of the current canvas.
	@return The current canvas height.
	*/
	__property short get_CanvasHeight(void);
	/**
	Returns the witdh of the current used texture object.
	@return The current texture object width.
	*/
	__property short get_TextureWidth(void);
	/**
	Returns the height of the current used texture object.
	@return The current texture object height.
	*/
	__property short get_TextureHeight(void);

	/**
	Sets new final color values as an two dimensional 8-bit array.
	Note that the dimensions of the color values array need not to be equal to the dimension  of
	the current texture object (which actually is equal  to the canvas dimension) because sampling is 
	implemented.
	@param n_FrameBuffer the final color values.
	*/
	void setFrameBuffer(unsigned char n_FrameBuffer __gc[]);
	/**
	Returns the current stored color values.
	@return The current final color values which are stored in a two dimensional array.
	*/
	unsigned char getFrameBuffer(void) __gc[];

	/**
	Returns an instance of the standard Bitmap class.
	This function will be needed to store the calculated final color values into a graphic file like JEPG, BMP, etc.
	@param n_FrameBuffer the final color values.
	@param n_CanvasWidth the width of the current canvas.
	@param n_CanvasHeight the height of the current canvas.
	@param n_Resolution the resolution og the current canvas.
	*/
	static Bitmap* GetBitmap(unsigned char n_FrameBuffer __gc[], int n_CanvasWidth, int n_CanvasHeight, int n_Resolution);
};
}
}
}