#pragma once

namespace IDVR {
namespace Volume {
namespace DataStructures {

		using namespace System;
		using namespace System::Collections;	

	/**
	<b>Class for representing a density histogram for a specific volume data object.</b>

	This class is part of the DataStructure namespace which is actually a subnamespace of the main namespace Volume. 
	Furthermore the standard class Object is inherted to the %Histogram class for better controlling of a particular 
	quantity of %Histogram instances.
	<br><br>
	An instance of this class contains a single histogram which will be included in every volume object. It gives
	information about the count of grayvalues per density value. 
	Thus, it is possible to represent the density distribution in the TransferFuncWin class and a visual interpretation could
	be drwan for better user understanding of the specific volume data object.
	@see VolumeData - the main datastructure class.
	@see DataObject - the volume data object class.
	@author <i>Bernhard Pflugfelder</i>
	@version 1.0
	@date 02-18-2005
	*/
	public __gc class Histogram : public System::Object 
		{
		public: 
				/**
				This is the standard constructor.
				All member variables of this new instance will be set to zero/null.
				*/
				Histogram(void);
				/**
				Constructor initialization with the maximum number of valid gray (<i>density</i>) values.
				All member variables will be set to zero, the Histogram::m_DensityFrequency member will be initialized with
				the given maximum number of gray values.
				@param numOfGrayvalues the maximum valid number of gray values.
				*/
				Histogram(int numOfGrayvalues);
				/**
				Constructor initialization by an user-defined histogram.
				The new instance will be set to the corresponding user-defined histogram values.
				@param n_DensityFrequency the user-defined histogram as an one dimensional array.
				@param n_Length the length of the user-defined histogram.
				*/
				Histogram(int n_DensityFrequency __gc[], int n_Length);
				/**
				This is the standard deconstructor.
				Espacially the Histogram::m_DensityFrequency member which actual is an array will be deleted.
				*/
				~Histogram(void);

				/**
				Increments the count of the specific density value.
				The current histogram will not be changed besides the particularly choosen density value.
				@param n_Density the specific density value for incrementation.
				*/
				void addNewValue(unsigned short n_Density);
				/**
				Sets the choosen value to the specific density value.
				The current histogram will not be changed besides the particularly choosen density value.
				@param n_Density the specific density value.
				@param n_Value the new density count.
				*/
				void addNewValue(unsigned short n_Density, int n_Value);
				/**
				Returns a new downsampled histogam by the chossen sample factor.
				@param n_SampleFactor the choosen sample factor for down sampling.
				@return A new instance of the %Histogram class.
				*/
				Histogram* getDownSampledHistogramValues(int n_SampleFactor);
				/**
				Returns the current density count of the specified density value.
				@param n_Density the specific density value.
				@return The current density count to the choosen density value.
				*/
				int getHistogramValue(unsigned short n_Density);

				/**
				Returns the current density histogram.
				@return The current density histogram as an one dimensional int array.
				*/
				__property int get_HistogramValues(void) __gc[];
				/**
				Returns the current density histogram length.
				@return The current density histogram length.
				@note The density histogram length is actually the maximum density scope.
				*/
				__property int get_HistogramLength(void);
				/**
				Returns the maximum voxel count of the current histogram.
				@return The maximum voxel count.
				*/
				__property int get_MaxVoxelCount(void);

		private:
				int m_Length; /**< Saves the current histogram length. */
				int m_MaxValue; /**< Saves the maximum voxel count of the histogram. */
				int m_DensityFrequency __gc[]; /**< Represents the actual histogram values. */
	};
}
}
}