#pragma once

#include ".\IIDVRCompositingModel.h"
#include ".\IIDVRModulationModel.h"
#include ".\FootprintElement.h"

namespace IDVR {
namespace VolumeRenderer {
namespace SWVolumeRenderer {
namespace IDVRCompositingModels {

using namespace IDVR::Interfaces;
using namespace IDVR::VolumeRenderer::RenderPrimitives;
using namespace System::Collections;

/**
<B>Class for the maximum importance driven volume rendering.</B>

Namespace: IDVR::VolumeRenderer::SWVolumeRenderer::IDVRCompositingModels

Baseclass: IDVR::Interfaces::IIDVRCompositingModel

This class does maximum importance driven volume rendering. To those raysamples, that have the objectID with
the maximum importance along a given ray it assigns the sparseness value 1, to all other raysamples it assigns
the value 0. It also contains a IDVRModulationModel, which modifies the raysamples of the ray with a certain
method.

@see IDVR::VolumeRenderer::SWVolumeRenderer::IDVRModulationModels::IDVRColAndOpModulatorSW - the color and opacity modulator.
@author <i>Christopher Thurnher</i>
@version 1.0
@date 02-18-2005
*/

public __gc class IDVRMImPCompositingModelSW : public IIDVRCompositingModel
{

private:

	IIDVRModulationModel *m_IIDVRModulatior; /**< Does the modulation of the raysample after the compositing. */
	ArrayList *m_FootprintArrayList; /**< List of objectIDs. */
	Hashtable *m_LevelsOfSparseness; /**< Level of Sparseness. */
	double m_CountersinkSlope; /**< Slope of the countersink geometry. */
	double m_GradientScaleFactor; /**< Factor of which the gradients on the countersink should be scaled */

public:

	/**
	This is the standard constructor.
	This standard constructor initialzed all elements with zero, exept the gradientScaleFactor with 1.
	*/
	IDVRMImPCompositingModelSW(void);
	/**
	Constuctor.
	@param n_IIDVRModulator an IDVRModulation interface.
	*/
	IDVRMImPCompositingModelSW(IIDVRModulationModel *n_IIDVRModulator);
	/**
	Constuctor.
	@param n_IIDVRModulator an IDVRModulation interface.
	@param n_CountersinkSlope the countersink slope.
	@param n_GradientScaleFactor the gradient scalefactor.
	*/
	IDVRMImPCompositingModelSW(IIDVRModulationModel *n_IIDVRModulator, double n_CountersinkSlope, double n_GradientScaleFactor);
	/**
	Standard destuctor.
	*/
	~IDVRMImPCompositingModelSW(void);

	/**
	Assigns the level of sparseness hashtable.
	*/
	virtual void doCompositing(void);
	/**
	Calls the modifyRaySample method of the IDVRModulation interface.
	@param n_RaySample the raysample that should be modified.
	*/
	virtual RaySample *modifyRaySample(RaySample *n_RaySample);
	/**
	Sets the objectID list.
	@param n_FootprintArrayList the objectID list.
	*/
	virtual void setFootprintArrayList(ArrayList *n_FootprintArrayList);

	/**
	Returns the IDVRModulation interface.
	@return The IDVRModulation interface.
	*/
	IIDVRModulationModel* getIIDVRModulationModel(void);
	/**
	Sets the IDVRModulation interface.
	@param n_IIDVRModulationModel the IDVRModulation interface.
	*/
	void setIIDVRModulatonModel(IIDVRModulationModel *n_IIDVRModulationModel);
	/**
	Sets the countersink slope.
	@param n_CountersinkSlope the countersink slope.
	*/
	void setCountersinkSlope(double n_CountersinkSlope);
	/**
	Sets the gradient scalefactor.
	@param n_GradientScaleFactor the gradient scalefactor.
	*/
	void setGradientScaleFactor(double n_GradientScaleFactor);
	/**
	Returns the countersink slope.
	@return The Icountersink slope.
	*/
	double getCountersinkSlope(void);
	/**
	Returns the gradient scalefactor.
	@return The gradient scalefactor.
	*/
	double getGradientScaleFactor(void);
};
}
}
}
}