#pragma once

#include ".\VolumeData.h"
#include ".\IShading.h"
#include ".\Light.h"
#include ".\IRenderer.h"
#include ".\ImagePlane.h"
#include ".\Footprint.h"

#using <mscorlib.dll>

namespace IDVR {
namespace Interfaces {
	
using namespace System::Threading;
using namespace IDVR::Interfaces;
using namespace IDVR::Volume::DataStructures;
using namespace IDVR::VolumeRenderer::RenderPrimitives;

/*
Interface of the basic raycaster.

namespace: IDVR::Interfaces

This interface defines the basic raycaster of this application which is actually is the "heart" of the rendering process.
This raycaster will be used in both rendering processes which are the ordinara rendering and the specific %IDVR 
rendering process.
*/
public __gc __interface IRenderMachine
{
public:

	/*
	This interface function is the main caller funtion of the raycaster which will be
	directly called form outside.<br>
	Thus, it controlls the raycaster and other important features like the handling of
	the progress window (see ProgressWin).
	*/
	void start_Raycasting(void) = 0;
	/*
	Interface function of the ordinary raycaster which uses only the general volume object and the basic
	rendering models.
	*/
	void do_NormalRayCasting(void) = 0;
	/*
	Interface function of the enhanced raycaster which uses all valid volume data objects with its own rendering
	attriutes and models.
	*/
	void do_TwoLevelRayCasting(void) = 0;
	/*
	Interface function of the additionally enhanced raycaster which uses not only two-level rendering but also the %IDVR 
	rending methods to render the different volume data object depending on its importanve values.
	attriutes and models.
	@param n_Footprint the footprint structure of the volume data set.
	*/
	void do_ImportanceDrivenRayCasting(Footprint *n_Footprint) = 0;

	/*
	Interface function to link the current loaded volume data to the raycaster.
	@param n_DataSet the current loaded data structure.
	*/
	__property void set_Data(VolumeData* n_DataSet) = 0;
	/*
	Interface function to switch between ordinary and two-level rendering.
	@param n_IsTwoLevel if true then two-level rendering will be activated, otherwise ordinary rendering will be used.
	*/
	__property void set_TwoLevelRendering(bool n_IsTwoLevel) = 0; 
	/*
	Interface function to activate %IDVR rendering.
	@param n_IsImportance if true then %IDVR rendering will be used.
	*/
	__property void set_ImportanceDrivenRendering(bool n_IsImportance) = 0;
	/*
	Interface function to set the specific sampling distance.
	@param n_SamplingDistance the new sampling distance.
	*/
	__property void set_SampingDistance(double n_SamplingDistance) = 0;
	/*
	Interface function to set the specific opacity threshold.
	@param n_ThresholdOpacity the new opacity threshold.
	*/
	__property void set_OpacityThreshold(double n_ThresholdOpacity) = 0;
	/*
	Interface function to set the specific gradient threshold.
	@param n_ThresholdGradient the new gradient threshold.
	*/
	__property void set_GradientThreshold(double n_ThresholdGradient) = 0;
	/*
	Interface function to set the specific image plane with its current size and orientation 
	(depends on the view direction).
	@param n_ImagePlane the new image plane.
	*/
	__property void set_ImagePlane(ImagePlane* n_ImagePlane) = 0;
	/*
	Interface function to set the specific gradient calculation mode.
	@param n_GradientEstimationMode the new gradient calculation mode.
	*/
	__property void set_GradientEstimationMode(char n_GradientEstimationMode) = 0;
	/*
	Interface function to set the specific density interpolation mode.
	@param n_DensityInterpolationMode the new density interpolation mode.
	*/
	__property void set_DensityInterpolationMode(char n_DensityInterpolationMode) = 0;
	/*
	Interface function to set the specific gradient interpolation mode.
	@param n_GradientInterpolationMode the new gradient interpolation mode.
	*/
	__property void set_GradientInterpolationMode(char n_GradientInterpolationMode) = 0;
	/*
	Interface function to set the specific object ID interpolation mode.
	@param n_VoxelIDInterpolationMode the new object ID interpolation mode.
	*/
	__property void set_VoxelIDInterpolationMode(char n_VoxelIDInterpolationMode) = 0;

	/*
	Interface function to get the current image plane.
	@return The current image plane which is linked to the raycaster.
	*/
	__property RenderPrimitives::ImagePlane* get_ImagePlane(void) = 0;
	/*
	Interface function to get the current volume data structure.
	@return The current volume data structure which is linked to the raycaster.
	*/
	__property VolumeData* get_Data(void) = 0;

};
}
}


/*! \interface IDVR::Interfaces::IRenderMachine
	\brief <B>Interface of the basic raycaster.</B>

	namespace: IDVR::Interfaces

	This interface defines the basic raycaster of this application which is actually is the "heart" of the rendering process.
	This raycaster will be used in both rendering processes which are the ordinara rendering and the specific %IDVR rendering process.

	@author <i>Bernhard Pflugfelder</i>
	@version 1.0
	@date 02-20-2005

	<hr>
	<b>Public member function:</b><br><br><ul>
	<li><b>void start_Raycasting(void)</b><br><br>
	 This interface function is the main caller funtion of the raycaster which will be
	 directly called form outside.<br>
	 Thus, it controlls the raycaster and other important features like the handling of
	 the progress window (see ProgressWin).</li><br><br>
	<li><b>void do_NormalRayCasting(void)</b><br><br>
	 Interface function of the ordinary raycaster which uses only the general volume object and the basic
	 rendering models.<br><br>
	 \see IDVR::VolumeRenderer::SWVolumeRenderer::ClassificationModels<br><br>
	 \see IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels<br><br>
	 \see IDVR::VolumeRenderer::SWVolumeRenderer::CompositionModels</li><br><br>
    <li><b>void do_TwoLevelRayCasting(void)</b><br><br>
	 Interface function of the enhanced raycaster which uses all valid volume data objects with its own rendering
	 attriutes and models.<br><br>
	 \see IDVR::VolumeRenderer::SWVolumeRenderer::ClassificationModels<br><br>
	 \see IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels<br><br>
	 \see IDVR::VolumeRenderer::SWVolumeRenderer::CompositionModels</li><br><br>
	<li><b>void do_ImportanceDrivenRayCasting(Footprint *n_Footprint)</b><br><br>
	 Interface function of the additionally enhanced raycaster which uses not only two-level rendering but also the %IDVR 
	 rending methods to render the different volume data object depending on its importanve values.
	 attriutes and models.<br><br>
	 \param n_Footprint the footprint structure of the volume data set.<br><br>
	 \see IDVR::VolumeRenderer::SWVolumeRenderer::IDVRCompositionModels<br><br>
	 \see IDVR::VolumeRenderer::SWVolumeRenderer::IDVRCompositionModels</li><br><br>
	<li><b>__property void set_Data(VolumeData* n_DataSet)</b><br><br>
	 Interface function to link the current loaded volume data to the raycaster.<br><br>
	 \param n_DataSet the current loaded data structure.</li><br><br>
	<li><b>__property void set_TwoLevelRendering(bool n_IsTwoLevel)</b><br><br>
	 Interface function to switch between ordinary and two-level rendering.<br><br>
	 \param n_IsTwoLevel if true then two-level rendering will be activated, otherwise 
	  ordinary rendering will be used.</li><br><br>
	<li><b>__property void set_ImportanceDrivenRendering(bool n_IsImportance)</b><br><br>
	 Interface function to activate %IDVR rendering.<br><br>
	 \param n_IsImportance if true then %IDVR rendering will be used.</li><br><br>
	<li><b>__property void set_SampingDistance(double n_SamplingDistance)</b><br><br>
	 Interface function to set the specific opacity threshold.<br><br>
	 \param n_SamplingDistance the new sampling distance.</li><br><br>
	<li><b>__property void set_OpacityThreshold(double n_ThresholdOpacity)</b><br><br>
	 Interface function to set the specific opacity threshold.<br><br>
	 \param n_ThresholdOpacity the new opacity threshold.</li><br><br>
	<li><b>__property void set_GradientThreshold(double n_ThresholdGradient)</b><br><br>
	 Interface function to set the specific gradient threshold.<br><br>
	 \param n_ThresholdGradient the new gradient threshold.</li><br><br>
	<li><b>__property void set_ImagePlane(ImagePlane* n_ImagePlane)</b><br><br>
	 Interface function to set the specific image plane with its current size and orientation 
	 (depends on the view direction).<br><br>
	 \param n_ImagePlane the new image plane.</li><br><br>
	<li><b>__property void set_GradientEstimationMode(char n_GradientEstimationMode)</b><br><br>
	 Interface function to set the specific gradient calculation mode.<br><br>
	 \param n_GradientEstimationMode the new gradient calculation mode.</li><br><br>
	<li><b>__property void set_DensityInterpolationMode(char n_DensityInterpolationMode)</b><br><br>
	 Interface function to set the specific density interpolation mode.<br><br>
	 \param n_DensityInterpolationMode the new density interpolation mode.</li><br><br>
	<li><b>__property void set_GradientInterpolationMode(char n_GradientInterpolationMode)</b><br><br>
	 Interface function to set the specific gradient interpolation mode.<br><br>
	 \param n_GradientInterpolationMode the new gradient interpolation mode.</li><br><br>
	<li><b>__property void set_VoxelIDInterpolationMode(char n_VoxelIDInterpolationMode)</b><br><br>
	 Interface function to set the specific object ID interpolation mode.<br><br>
	 \param n_VoxelIDInterpolationMode the new object ID interpolation mode.</li><br><br>
	<li><b>__property RenderPrimitives::ImagePlane* get_ImagePlane(void)</b><br><br>
	 Interface function to get the current image plane.<br><br>
	 \return The current image plane which is linked to the raycaster.</li><br><br>
	<li><b>__property VolumeData* get_Data(void)</b><br><br>
	 Interface function to get the current volume data structure.<br><br>
	 \return The current volume data structure which is linked to the raycaster.</li><br><br>
   </ul>
 */