#pragma once

#include "IClassificationModel.h"
#include "ICompositingModel.h"
#include "IShadingModel.h"

#using <mscorlib.dll>

namespace IDVR {
namespace Interfaces {
	
using namespace IDVR::VolumeRenderer::RenderPrimitives;
using namespace System::Drawing;

/*
<B>Interface for the renderer.</B>

namespace: IDVR::Interfaces

This interface provides methods to render the scene. Each renderer consists of three parts. the first one is
a classification model. It assigns calculates an opacity value for a given raysample. The second one is a shading
model which calculates a color value for a given raysample. At last, the compositing model calculates the final
color and opacity values by blending the new color and opacity of a raysample with the already calcultaed ones.

@see IDVR::VolumeRenderer::SWVolumeRenderer::ClassificationModels::LinearClassificationModelSW - the linear classification model.
@see IDVR::VolumeRenderer::SWVolumeRenderer::ClassificationModels::TFClassificationModelSW - the transfer function classification model.
@see IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels::ContourShadingModelSW - the contour shading model
@see IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels::MipShadingModelSW - the mip shading model
@see IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels::PhongShadingModelSW - the phong shading model
@see IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels::ToneShadingModelSW - the tone shading model
@see IDVR::VolumeRenderer::SWVolumeRenderer::CompositingModels::DvrCompositingModelSW - the dvr compositing model
@see IDVR::VolumeRenderer::SWVolumeRenderer::CompositingModels::LMipCompositingModelSW - the LMip compositing model

@author <i>Christopher Thurnher</i>
@version 1.0
@date 02-19-2005
*/

public __gc __interface IRenderer
{
public:

	/*
	Calculates the opacity value for a given raysample.
	@param n_RaySample the given raysample.
	@return The modified raysample.
	*/
	RaySample* doClassification(RaySample *n_RaySample) = 0;
	/*
	Calculates the intensity value for a given raysample.
	@param n_RaySample the given raysample.
	@return The modified raysample.
	*/
	RaySample* doShading(RaySample *n_RaySample) = 0;
	/*
	Blends the final intensity and opacity values of a given raysample.
	@param n_RaySample the given raysample.
	*/
	void doCompositing(RaySample *n_RaySample) = 0;
	/*
	Blends the final pixelcolor with the given background color.
	@param n_BackColor the background color.
	*/
	void doCompositingBackground(float n_BackColor __gc[]) = 0;
	/*
	Sets the classification model.
	@param n_IClassificator the classification model.
	*/
    void setClassificator(IClassificationModel *n_IClassificator) = 0;
	/*
	Sets the shading model.
	@param n_IShader the shading model.
	*/
	void setShader(IShadingModel *n_IShader) = 0;
	/*
	Sets the composition model.
	@param n_ICompositor the composition model.
	*/
	void setCompositor(ICompositingModel *n_ICompositor) = 0;

	/*
	Returns the classification model.
	@return The classification model.
	*/
	IClassificationModel* getClassificator(void) = 0;
	/*
	Returns the shading model.
	@return The shading model.
	*/
	IShadingModel* getShader(void) = 0;
	/*
	Returns the composition model.
	@return The composition model.
	*/
	ICompositingModel* getCompositor(void) = 0;

	/*
	Returns true, if gradinet estimation is needed.
	@return True, if gradinet estimation is needed.
	*/
	bool need_GradientEstimation(void) = 0;
	/*
	Resets the values of the composition model.
	*/
	void reset_Compositor(void) = 0;
	/*
	Returns the current intensity array.
	@return The current intensity array.
	*/
	float getIntensity(void) __gc[] = 0;
	/*
	Returns the current opacity value.
	@return The current opacity value.
	*/
	float getOpacity(void) = 0;

	/*
	Sets the render mode.
	@param n_RenderMode the render mode.
	*/
	void setRenderMode(char n_RenderMode) = 0;
	/*
	Returns the render mode.
	@return The render mode.
	*/
	char getRenderMode(void) = 0;
};
}
}

/*! \interface IDVR::Interfaces::IRenderer
	\brief <B>Interface for the renderer.</B>

	namespace: IDVR::Interfaces

	This interface provides methods to render the scene. Each renderer consists of three parts. the first one is
	a classification model. It assigns calculates an opacity value for a given raysample. The second one is a shading
	model which calculates a color value for a given raysample. At last, the compositing model calculates the final
	color and opacity values by blending the new color and opacity of a raysample with the already calcultaed ones.

	@see IDVR::VolumeRenderer::SWVolumeRenderer::ClassificationModels::LinearClassificationModelSW - the linear classification model.
	@see IDVR::VolumeRenderer::SWVolumeRenderer::ClassificationModels::TFClassificationModelSW - the transfer function classification model.
	@see IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels::ContourShadingModelSW - the contour shading model
	@see IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels::MipShadingModelSW - the mip shading model
	@see IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels::PhongShadingModelSW - the phong shading model
	@see IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels::ToneShadingModelSW - the tone shading model
	@see IDVR::VolumeRenderer::SWVolumeRenderer::CompositingModels::DvrCompositingModelSW - the dvr compositing model
	@see IDVR::VolumeRenderer::SWVolumeRenderer::CompositingModels::LMipCompositingModelSW - the LMip compositing model

	@author <i>Christopher Thurnher</i>
	@version 1.0
	@date 02-19-2005

	<hr>
	<b>Public member function:</b><br><br><ul>
	<li><b>RaySample* doClassification(RaySample *n_RaySample)</b><br><br>
	 Calculates the opacity value for a given raysample.<br><br>
	 \param n_RaySample the given raysample.<br><br>
	 \return The modified raysample.</li><br><br>
	<li><b>RaySample* doShading(RaySample *n_RaySample)</b><br><br>
	 Calculates the intensity value for a given raysample.<br><br>
	 \param n_RaySample the given raysample.<br><br>
	 \return The modified raysample.</li><br><br>
	<li><b>void doCompositing(RaySample *n_RaySample)</b><br><br>
	 Blends the final intensity and opacity values of a given raysample.<br><br>
	 \param n_RaySample the given raysample.<br><br>
	 \return The modified raysample.</li><br><br>
	<li><b>void doCompositingBackground(float n_BackColor __gc[])</b><br><br>
	 Blends the final pixelcolor with the given background color.<br><br>
	 \param n_BackColor the background color.</li><br><br>
	<li><b>void setClassificator(IClassificationModel *n_IClassificator)</b><br><br>
	 Sets the classification model.<br><br>
	 \param n_IClassificator the classification model.</li><br><br>
	<li><b>void setShader(IShadingModel *n_IShader)</b><br><br>
	 Sets the shading model.<br><br>
	 \param n_IShader the shading model.</li><br><br>
	<li><b>void setCompositor(ICompositingModel *n_ICompositor)</b><br><br>
	 Sets the composition model.<br><br>
	 \param n_ICompositor the composition model.</li><br><br>
	<li><b>IClassificationModel* getClassificator(void)</b><br><br>
	 Returns the classification model.<br><br>
	 \return The classification model.</li><br><br>
	<li><b>IShadingModel* getShader(void)</b><br><br>
	 Returns the shading model.<br><br>
	 \return The shading model.</li><br><br>
    <li><b>ICompositingModel* getCompositor(void)</b><br><br>
	 Returns the composition model.<br><br>
	 \return The composition model.</li><br><br>
	<li><b>bool need_GradientEstimation(void)</b><br><br>
	 Returns true, if gradinet estimation is needed.<br><br>
	 \return True, if gradinet estimation is needed.</li><br><br>
    <li><b>void reset_Compositor(void)</b><br><br>
	 Resets the values of the composition model.</li><br><br>
    <li><b>float getIntensity(void) __gc[]</b><br><br>
	 Returns the current intensity array.<br><br>
	 \return The current intensity array.</li><br><br>
	<li><b>float getOpacity(void)</b><br><br>
	 Returns the current opacity value.<br><br>
	 \return The current opacity value.</li><br><br>
	<li><b>void setRenderMode(char n_RenderMode)</b><br><br>
	 Sets the render mode.<br><br>
	 \param n_RenderMode the render mode.</li><br><br>
	<li><b>char getRenderMode(void)</b><br><br>
	 Returns the render mode.<br><br>
	 \return The render mode.</li><br><br>
   </ul>
 */