#pragma once

#include "Matrix4.h"

namespace IDVR {
namespace VolumeRenderer {
namespace RenderPrimitives {

using namespace IDVR::Tools;

/**
<b>Class for the image plane data structure.</b>

This class contains a couple of methods for initalization and computing 
the image plane.    
@author <i>Christopher Thurnher</i>
@version 1.0
@date 02-17-2005
*/
public __gc class ImagePlane
{
private:

	double m_Width; /**< The width of the image plane. */
	double m_Heigth; /**< The height of the image plane. */
	double m_Resolution; /**< The resolution of the image plane. */
	double m_RayDistanceWidth; /**< The horizontal distance of two adjacent rays. */
	double m_RayDistanceHeigth; /**< The vertical distance of two adjacent rays. */
	unsigned short m_VolumeDimension __gc[]; /**< The dimension of the volume data. */

	Vector3 *m_DefaultPoints[]; /**< The default points of the image plane. */
	Vector3* m_ViewDirection; /**< The view direction of the image plane. */
	Vector3 *m_TransformedPoints[]; /**< The transformed points of the image plane. */
	Vector3 *m_NullPointMiddlePoint; /**< The vector between the null point and the middle point. */
	Vector3 *m_NullPoint; /**< The point on the image plane, which is the intersection point between the image plane and the straight line, which contains the middle point of the volume and is perpendicular to the image plane. */

	Matrix4 *m_TranslateMatrix; /**< The translation matrix of the image plane. */
	Matrix4 *m_RotMatrix; /**< The rotation matrix of the image plane. */
	Matrix4 *m_ScaleMatrix; /**< The scale matrix of the image plane. */
	
	static const char LBottom = 0, LUpper  = 1, RUpper = 2, RBottom = 3;
	

public:

	/**
	This is the standard constructor.
	This standard constructor initialzed all elements with zero.
	*/
	ImagePlane(void);
	/**
	This constructor initalized the image plane with the given parameters.
	@param n_LeftUpper the left upper point of the image plane.
	@param n_RightUpper the right upper point of the image plane.
	@param n_LeftBottom the left bottom point of the image plane.
	@param n_RightBottom the right bottom point of the image plane.
	@param n_Resolution the resolution of the image plane.
	@param n_VolumeDimension the volume dimension  of the dataset.
	*/
	ImagePlane(Vector3* n_LeftUpper, Vector3* n_RightUpper, Vector3* n_LeftBottom, Vector3* n_RightBottom, short n_Resolution, unsigned short n_VolumeDimension __gc[]);
	/**
	This constructor initalized the image plane with the given parameters.
	@param n_LeftUpper the left upper point of the image plane.
	@param n_RightUpper the right upper point of the image plane.
	@param n_LeftBottom the left bottom point of the image plane.
	@param n_RightBottom the right bottom point of the image plane.
	@param n_Resolution the resolution of the image plane.
	@param n_ViewDirection the view direction of the image plane.
	@param n_VolumeDimension the volume dimension  of the dataset.
	*/
	ImagePlane(Vector3* n_LeftUpper, Vector3* n_RightUpper, Vector3* n_LeftBottom, Vector3* n_RightBottom, short n_Resolution, Vector3* n_ViewDirection, unsigned short n_VolumeDimension __gc[]);
	/**
	This constructor initalized the image plane with a given image plane.
	@param n_ImagePlane the image plane.
	*/
	ImagePlane(ImagePlane* n_ImagePlane);

	/**
	Calculates the ray dstance.
	*/
	void calculateRayDistance(void);
	/**
	Calculates the vie direction.
	*/
	void calculateViewDirection(void);
	/**
	Calculates the view direction, the width and height of the image plane and the ray distance.
	*/
	void calculateNewParameters(void);
	/**
	Calculates the middle point of the image plane.
	@return The calculated middle point.
	*/
	Vector3* calculateMiddlePoint(void);
	/**
	calculates the default middle point of the image plane.
	@return The default middle point.
	*/
	Vector3* calculateDefaultMiddlePoint(void);
	/**
	calculates the distance of the image plane with a given point.
	@param n_Point the point which distance should be calculated.
	@return the calculated distance.
	*/
	double calculateDistance(Vector3 *n_Point);
	/**
	Calculates the null point.
	*/
	void calculateNullPoint(void);
	/**
	Calculates the vector between the null point and the middle point.
	*/
	void calculateNullPointMiddlePoint(void);
	/**
	Does all image plane transformations according to the transform-, rotate- and scale matrix.
	*/
	void transformImagePlane(void);
	/**
	Returns the width of the image plane.
	@return The width of the image plane.
	*/
	__property double get_Width(void);
	/**
	Returns the height of the image plane.
	@return The height of the image plane.
	*/
	__property double get_Heigth(void);
	/**
	Returns the resolution of the image plane.
	@return The resolution of the image plane.
	*/
	__property double get_Resolution(void);
	/**
	Returns the view direction of the image plane.
	@return The view direction of the image plane.
	*/
	__property Vector3* get_ViewDirection(void);
	/**
	Returns the horizontal distance of two adjacent rays.
	@return The horizontal distance of two adjacent rays.
	*/
	__property double get_RayDistanceWidth(void);
	/**
	Returns the vertical distance of two adjacent rays.
	@return The vertical distance of two adjacent rays.
	*/
	__property double get_RayDistanceHeigth(void);
	/**
	Returns the null point of the image plane.
	@return The null point of the image plane.
	*/
	__property Vector3* get_NullPoint(void);
	/**
	Returns the vector between the null point and the middle point of the image plane.
	@return The vector between the null point and the middle point of the image plane.
	*/
	__property Vector3* get_NullPointMiddlePoint(void);
	/**
	Returns the translate matrix of the image plane.
	@return The translate matrix of the image plane.
	*/
	__property Matrix4* get_TranslateMatrix(void);
	/**
	Returns the rotate matrix of the image plane.
	@return The rotate matrix of the image plane.
	*/
	__property Matrix4* get_RotMatrix(void);
	/**
	Returns the scale matrix of the image plane.
	@return The scale matrix of the image plane.
	*/
	__property Matrix4* get_ScaleMatrix(void);
	/**
	Returns the volume dimension of the dataset.
	@return The volume dimension of the dataset.
	*/
	__property unsigned short get_VolumeDimension(void) __gc[];
	/**
	Returns the default points of the image plane.
	@return The default points of the image plane.
	*/
	__property Vector3* get_DefaultPoints(void) [];
	/**
	Returns the transformed points of the image plane.
	@return The transformed points of the image plane.
	*/
	__property Vector3* get_TransformedPoints(void) [];
	/**
	Sets the width of the image plane.
	@param n_SideLength the sidelength of the image plane.
	*/
	__property void set_Width(double n_SideLength);
	/**
	Sets the height of the image plane.
	@param n_SideLength the sidelength of the image plane.
	*/
	__property void set_Heigth(double n_SideLength);
	/**
	Sets the resolution of the image plane.
	@param n_Resolution the resolution of the image plane.
	*/
	__property void set_Resolution(double n_Resolution);
	/**
	Sets the translate matrix of the image plane.
	@param n_TranslateMatrix the translate matrix of the image plane.
	*/
	__property void set_TranslateMatrix(Matrix4 *n_TranslateMatrix);
	/**
	Sets the rotate matrix of the image plane.
	@param n_RotMatrix the rotate matrix of the image plane.
	*/
	__property void set_RotMatrix(Matrix4 *n_RotMatrix);
	/**
	Sets the scale matrix of the image plane.
	@param n_ScaleMatrix the scale matrix of the image plane.
	*/
	__property void set_ScaleMatrix(Matrix4 *n_ScaleMatrix);
	/**
	Sets the view direction of the image plane.
	@param n_ViewDirection the view direction of the image plane.
	*/
	__property void set_ViewDirection(Vector3* n_ViewDirection);
	/**
	/**
	Sets the volume dimension of the dasaset.
	@param n_VolumeDimension the volume dimension of the dasaset.
	*/
	__property void set_VolumeDimension(unsigned short n_VolumeDimension __gc[]);

	/**
	Returns the horizontal view vector of the image plane.
	@return The horizontal view vector of the image plane.
	*/
	Vector3* getHorizontalViewVector(void);
	/**
	Returns the vertical view vector of the image plane.
	@return The vertical view vector of the image plane.
	*/
	Vector3* getVerticalViewVector(void);
	/**
	Returns the current left upper point of the image plane.
	@return The current left upper point of the image plane.
	*/
	Vector3* getCurrentLeftUpperPoint(void);
	/**
	Returns the current right bottom point of the image plane.
	@return The current right bottom point of the image plane.
	*/
	Vector3* getCurrentRightBottomPoint(void);
	/**
	Returns the current left bottom point of the image plane.
	@return The current left bottom point of the image plane.
	*/
	Vector3* getCurrentLeftBottomPoint(void);
	/**
	Returns the current right upper point of the image plane.
	@return The current right upper point of the image plane.
	*/
	Vector3* getCurrentRightUpperPoint(void);
	/**
	Returns the default left upper point of the image plane.
	@return The default left upper point of the image plane.
	*/
	Vector3* getDefaultLeftUpperPoint(void);
	/**
	Returns the default right bottom point of the image plane.
	@return The default right bottom point of the image plane.
	*/
	Vector3* getDefaultRightBottomPoint(void);
	/**
	Returns the default left bottom point of the image plane.
	@return The default left bottom point of the image plane.
	*/
	Vector3* getDefaultLeftBottomPoint(void);
	/**
	Returns the default right upper point of the image plane.
	@return The default right upper point of the image plane.
	*/
	Vector3* getDefaultRightUpperPoint(void);
	/**
	Returns the default points of the image plane.
	@return The default points of the image plane.
	*/
	Vector3* getDefaultPoints(void) __gc[];
	/**
	Returns the distance between the null point and the middle point of the image plane.
	@return The distance between the null point and the middle point of the image plane.
	*/
	Vector3* getDistanceFromOrigin(void);

	/**
	Sets the default left upper point.
	@param n_LeftUpper the default left upper point.
	*/
	void setDefaultLeftUpperPoint(Vector3* n_LeftUpper);
	/**
	Sets the default right bottom point.
	@param n_RightBottom the default right bottom point.
	*/
	void setDefaultRightBottomPoint(Vector3* n_RightBottom);
	/**
	Sets the default left bottom point.
	@param n_LeftBottom the default left bottom point.
	*/
	void setDefaultLeftBottomPoint(Vector3* n_LeftBottom);
	/**
	Sets the default right upper point.
	@param n_RightUpper right default left upper point.
	*/
	void setDefaultRightUpperPoint(Vector3* n_RightUpper);

	/**
	Multiplies the translate matrix of the image plane with the given translate matrix
	@param n_TranslateMatrix the translate matrix
	*/
	void modifyTranslateMatrix(Matrix4 *n_TranslateMatrix);
	/**
	Multiplies the rotate matrix of the image plane with the given rotate matrix
	@param n_RotMatrix the rotate matrix
	*/
	void modifyRotMatrix(Matrix4 *n_RotMatrix);
	/**
	Multiplies the scale matrix of the image plane with the given scale matrix
	@param n_ScaleMatrix the scale matrix
	*/
	void modifyScaleMatrix(Matrix4 *n_ScaleMatrix);

	/**
	Sets the identity matrix for the translate-, rotate- and scale martrix of the image plane
	*/
	void resetMatrices(void);
	/**
	Resets the transformed points to the default points of the image palne
	*/
	void resetPoints(void);

	/**
	Desides if two vectors are on the same side of the image plane
	@param n_Vector1 first vector
	@param n_Vector2 second vector
	@return True, if both vectors are on the same side of the image plane
	*/
	bool sameSide(Vector3* n_Vector1, Vector3 *n_Vector2);
};
}
}
}
