#include "StdAfx.h"
#include ".\light.h"
#using <mscorlib.dll>

using namespace System;
using namespace IDVR::Tools;
using namespace IDVR::VolumeRenderer::RenderPrimitives;

//This is the standard constructor.
Light::Light(void) 
{ 
	this->m_LightColor = 0; 
	this->m_DefaultLightDirection = new Vector3(); 
	this->m_OffsetLightDirection = new Vector3(); 
	this->m_BaseLightDirection = new Vector3();
	this->m_OffsetRotation = Matrix4::CreateIdentityMatrix();
	this->m_BaseRotation = Matrix4::CreateIdentityMatrix();
	this->m_LightViewMode = ABSOLUTELIGHTVIEW;
}

//This constructor initalized a light object with the given parameters.
Light::Light(int n_LightColor, Vector3* n_DefaultLightDirection)
{
	this->m_LightColor = n_LightColor; 
	this->m_DefaultLightDirection = new Vector3(n_DefaultLightDirection->X, n_DefaultLightDirection->Y, n_DefaultLightDirection->Z);
	this->m_OffsetLightDirection = new Vector3(n_DefaultLightDirection->X, n_DefaultLightDirection->Y, n_DefaultLightDirection->Z); 
	this->m_BaseLightDirection = new Vector3(n_DefaultLightDirection->X, n_DefaultLightDirection->Y, n_DefaultLightDirection->Z);
	this->m_OffsetRotation = Matrix4::CreateIdentityMatrix();
	this->m_BaseRotation = Matrix4::CreateIdentityMatrix();
	this->m_LightViewMode = ABSOLUTELIGHTVIEW;
}

//This constructor initalized a light object with the given light object.
Light::Light(Light *n_Light)
{
	this->m_LightColor = n_Light->LightColor;
	this->m_DefaultLightDirection = n_Light->DefaultLightDirection;
	this->m_BaseLightDirection = n_Light->BaseLightDirection;
	this->m_OffsetLightDirection = n_Light->OffsetLightDirection;
	this->m_OffsetRotation = n_Light->OffsetRotation;
	this->m_BaseRotation = n_Light->BaseRotation;
	this->m_LightViewMode = ABSOLUTELIGHTVIEW;
}

//Sets the light color.
void Light::set_LightColor(int n_LightColor)
{
	this->m_LightColor = n_LightColor;
}

//Sets the default light direction.
void Light::set_DefaultLightDirection(Vector3 *n_DefaultLightDirection)
{
	this->m_DefaultLightDirection->X = n_DefaultLightDirection->X;
	this->m_DefaultLightDirection->Y = n_DefaultLightDirection->Y;
	this->m_DefaultLightDirection->Z = n_DefaultLightDirection->Z;

	calculateLightDirection();
}

//Sets the offset light direction rotation matrix.
void Light::set_OffsetRotation(Matrix4 *n_OffsetRotation)
{
	this->m_OffsetRotation = new Matrix4(n_OffsetRotation);
}

//Sets the base light direction rotation matrix.
void Light::set_BaseRotation(Matrix4 *n_BaseRotation)
{
	this->m_BaseRotation = new Matrix4(n_BaseRotation);
}

//Sets the light view mode.
void Light::set_LightViewMode(unsigned char n_LightViewMode)
{
	this->m_LightViewMode = n_LightViewMode;
}

//Returns the light color.
int Light::get_LightColor(void)
{
	return this->m_LightColor;
}

//Returns the default light direction.
Vector3* Light::get_DefaultLightDirection(void)
{
	return new Vector3(this->m_DefaultLightDirection->X, this->m_DefaultLightDirection->Y, this->m_DefaultLightDirection->Z);
}	

//Returns the light direction.
Vector3* Light::get_OffsetLightDirection(void)
{
	return new Vector3(this->m_OffsetLightDirection->X, this->m_OffsetLightDirection->Y, this->m_OffsetLightDirection->Z);
}

//Returns the base light direction.
Vector3* Light::get_BaseLightDirection(void)
{
	return new Vector3(this->m_BaseLightDirection->X, this->m_BaseLightDirection->Y, this->m_BaseLightDirection->Z);
}

//Returns the offset light direction offset.
Matrix4* Light::get_OffsetRotation(void)
{
	return new Matrix4(this->m_OffsetRotation);
}

//Returns the base light direction offset.
Matrix4* Light::get_BaseRotation(void)
{
	return new Matrix4(this->m_BaseRotation);
}

//Returns the light view mode.
unsigned char Light::get_LightViewMode(void)
{
	return this->m_LightViewMode;
}

//Returns the float array of the light color (values between 0 and 1).
float Light::getNormalizedLightColorArray(void) __gc[]
{
	Color tempColor = Color::FromArgb(this->m_LightColor);
	float tempValues __gc[] = { tempColor.R / 255.0f, tempColor.G / 255.0f, tempColor.B / 255.0f, tempColor.A / 255.0f};

	return tempValues;
}

//Calculates new light direction.
void Light::calculateLightDirection(void)
{
	if (this->m_LightViewMode == ABSOLUTELIGHTVIEW)
	{
		Vector3 * newLightPoint = Vector4::GetVector3(this->m_BaseRotation->multiply_WithPoint(new Vector4(this->m_DefaultLightDirection, 1.0)));
		this->m_BaseLightDirection = Vector3::CreateNormalizedVector(newLightPoint);

		newLightPoint = Vector4::GetVector3(this->m_OffsetRotation->multiply_WithPoint(new Vector4(this->m_BaseLightDirection, 1.0)));
		this->m_OffsetLightDirection = Vector3::CreateNormalizedVector(newLightPoint);
	}
	else
	{
		this->m_BaseLightDirection = new Vector3(this->m_DefaultLightDirection);

		Vector3 *newLightPoint = Vector4::GetVector3(this->m_OffsetRotation->multiply_WithPoint(new Vector4(this->m_DefaultLightDirection, 1.0)));
		this->m_OffsetLightDirection = Vector3::CreateNormalizedVector(newLightPoint);
	}
}

//Add the given rotation to the current rotation offset.
void Light::modifyOffsetRotation(Matrix4* n_OffsetRotation)
{
	this->m_OffsetRotation = Matrix4::MultiplyMatices(this->m_OffsetRotation, n_OffsetRotation);
}

//Add the given rotation to the current rotation base.
void Light::modifyBaseRotation(Matrix4* n_BaseRotation)
{
	this->m_BaseRotation = Matrix4::MultiplyMatices(this->m_BaseRotation, n_BaseRotation);
}

//Aeturns the light direction but with the y component reflected about x-axis.
//Will be only used with reflection about x-axis!
Vector3* Light::getOpenGLLightDirection(void)
{
	Matrix4* reflectionMatrix = Matrix4::CreateReflectionAboutXMatrix();
	return Vector4::GetVector3(reflectionMatrix->multiply_WithPoint(new Vector4(this->m_OffsetLightDirection,1.0)));
}

//Resets the offset light direction to the base direction.
void Light::resetOffsetLightDirection(void)
{
	this->m_OffsetRotation = Matrix4::CreateIdentityMatrix();
	calculateLightDirection();
}

//Resets the base light direction to the current offset direction.
void Light::resetBaseLightDirection(void)
{
	this->m_BaseRotation   = Matrix4::CreateIdentityMatrix();
	calculateLightDirection();
}