#pragma once

#include ".\Matrix4.h"

namespace IDVR {
namespace VolumeRenderer {
namespace RenderPrimitives {

using namespace System::Drawing;
using namespace IDVR::Tools;

/**
<b>Class for the light data structure.</b>

This class contains a couple of methods for the light object.
@author <i>Christopher Thurnher</i>
@version 1.0
@date 02-17-2005
*/
public __gc class Light
{

private:

	int m_LightColor; /**< The color of the light. */
	Vector3 *m_OffsetLightDirection; /**< The offset direction of the light. */
	Vector3 *m_BaseLightDirection; /**< The base direction of the light. */
	Vector3 *m_DefaultLightDirection; /**< The default direction of the light. */
	Matrix4 *m_OffsetRotation; /**< The offset rotaion of the light. */
	Matrix4 *m_BaseRotation; /**< The base rotaion of the light. */

	unsigned char m_LightViewMode; /**< The view mode of the light. */

public:

	static const char ABSOLUTELIGHTVIEW = 0;
	static const char RELATIVLIGHTVIEW = 1;
	
	/**
	This is the standard constructor.
	*/
	Light(void);
	/**
	This constructor initalized a light object with the given parameters.
	@param n_LightColor the color of the light.
	@param n_DefaultLightDirection the default direction of the light.
	*/
	Light(int n_LightColor, Vector3* n_DefaultLightDirection);
	/**
	This constructor initalized a light object with the given light object.
	@param n_Light the light object.
	*/
	Light(Light *n_Light);

	/**
	Sets the light color.
	@param n_LightColor the color of the light.
	*/
	__property void set_LightColor(int n_LightColor);
	/**
	Sets the default light direction.
	@param n_DefaultLightDirection the default light direction.
	*/
	__property void set_DefaultLightDirection(Vector3 *n_DefaultLightDirection);
	/**
	Sets the offset light direction rotation matrix.
	@param n_OffsetRotation the offset rotation matrix.
	*/
	__property void set_OffsetRotation(Matrix4 *n_OffsetRotation);
	/**
	Sets the base light direction rotation matrix.
	@param n_BaseRotation the base rotation matrix.
	*/
	__property void set_BaseRotation(Matrix4 *n_BaseRotation);
	/**
	Sets the light view mode.
	@param n_LightViewMode the light view mode.
	*/
	__property void set_LightViewMode(unsigned char n_LightViewMode);
	
	/**
	Returns the light color.
	@return The color of the light.
	*/
	__property int get_LightColor(void);
	/**
	Returns the light direction.
	@return The offset direction of the light.
	*/
	__property Vector3* get_OffsetLightDirection(void);
	/**
	Returns the base light direction.
	@return  The base direction of the light.
	*/
	__property Vector3* get_BaseLightDirection(void);
	/**
	Returns the default light direction.
	@return The default direction of the light.
	*/
	__property Vector3* get_DefaultLightDirection(void);
	/**
	Returns the offset light direction offset.
	@return The offset rotation matrix of the light.
	*/
	__property Matrix4* get_OffsetRotation(void);
	/**
	Returns the base light direction offset.
	@return The base rotation matrix of the light.
	*/
	__property Matrix4* get_BaseRotation(void);
	/**
	Returns the light view mode.
	@return The view mode of the light.
	*/
	__property unsigned char get_LightViewMode(void);

	/**
	Add the given rotation to the current rotation offset.
	@param n_OffsetRotation The rotation matrix, with which the light's offset rotation matrix is multiplied with.
	*/
	void modifyOffsetRotation(Matrix4* n_OffsetRotation);
	/**
	Add the given rotation to the current rotation base.
	@param n_BaseRotation The rotation matrix, with which the light's base rotation matrix is multiplied with.
	*/
	void modifyBaseRotation(Matrix4* n_BaseRotation);
	/**
	Calculates new light direction.
	*/
	void calculateLightDirection(void);
	/**
	Returns the float array of the light color (values between 0 and 1).
	@return The normalized color array of the light.
	*/
	float getNormalizedLightColorArray(void) __gc[];
	/**
	Returns the light direction but with the y component reflected about x-axis.
	@return The direction of the light with the y component reflected about x-axis.
	*/
	Vector3* getOpenGLLightDirection(void);
	/**
	Resets the offset light direction to the base direction.
	*/
	void resetOffsetLightDirection(void);
	/**
	Resets the base light direction to the current offset direction.
	*/
	void resetBaseLightDirection(void);
};
}
}
}
