#pragma once

#include "Vector3.h"
#include "Vector4.h"

namespace IDVR {
namespace Tools {

	using namespace System;

	/**
	<B>Class for a 4x4 matrix.</B>

	This class is part of the Tools namespace and its target is the handling and computing
	of different linear transformations (e.g. transformations, rotations,etc.). For all operations including
	such linear transformations this class which based on homogenous coordinates should be used. 
	<br>
	With specific initialization function such as 
	<ul>
	<li>Matrix4::CreateTranslationMatrix</li> 
	<li>Matrix4::CreateRotXMatrix</li> 
	<li>Matrix4::CreateRotYMatrix</li> 
	<li>Matrix4::CreateRotZMatrix</li> 
	<li>Matrix4::CreateScaleUniformMatrix(double)</li> 
	<li>Matrix4::CreateReflectionAboutXMatrix</li> 
	<li>Matrix4::CreateReflectionAboutYMatrix</li> 
	<li>Matrix4::CreateReflectionAboutZMatrix</li> 
	</ul>
	all needed transformations can be created and with the Matrix4::MultiplyMatices function the accumulation of an
	arbitrary count of linear transformation will be possible.
	<br>
	Please notice that the logical order of the matrix elements is in ordinary (C++ like) alignment. The following
	brief schema visualizes the valid matix alignment: 
	<center>
	<table border="0">
	<tr>
	<td><i>m00</i></td>
	<td><i>m01</i></td>
	<td><i>m02</i></td>
	<td><i>m03</i></td>
	</tr><tr>
	<td><i>m10</i></td>
	<td><i>m11</i></td>
	<td><i>m12</i></td>
	<td><i>m13</i></td>
	</tr><tr>
	<td><i>m20</i></td>
	<td><i>m21</i></td>
	<td><i>m22</i></td>
	<td><i>m23</i></td>
	</tr><tr>
	<td><i>m30</i></td>
	<td><i>m31</i></td>
	<td><i>m32</i></td>
	<td><i>m33</i></td>
	</table>
	</center>
	<br>
	The implemented linear operations of this class are of course only a fraction of linear algebra, the minimum
	pool of operations which will be used in the IDVR program.
	@see Matrix3 - the 3x3 matrix class for specific linear operations.
	@author <i>Bernhard Pflugfelder</i>
	@version 1.0
	@date 02-17-2005
	*/
public __gc class Matrix4
{

private:

	double m_MatrixElements __gc[,]; /**<The array of the necessary matrix elements. All the needed 16 matrix elements are saved in a double array.*/

public:

	static const unsigned char ORDER = 4; /**<A constant that specifies the matrix order of matices of this class type. */

	/**
	This is the standard constructor.
	This standard constructor initialized all matrix elements with zero.
	*/
	Matrix4(void);
	/**
	Constructor initialization by a specified array of matrix elements.
	This constructor initialized all matrix elements with the corresponding elements of the specified 2 dimensional array.
	@param n_MatrixElements the values for the matrix elements.
	*/
	Matrix4(double n_MatrixElements __gc[,]);
	/**
	Constructor initialization by an existing %Matrix4 object.
	This constructor initialized all matrix elements with the corresponfing values of the specified %Matrix4 object. Please
	notice that the currently initialized object is not directly linked to the specified parameter by the same memory address
	but it has its own memory space.
	@param n_Matrix the %Matrix4 object containing the initialization values.
	*/
	Matrix4(Matrix4 *n_Matrix);

	/**
	Use this function for multiply matrices with a specified Vector4 object.
	A right-side matrix multiplication will be used so that the %Vector4 object always is the right sided <i>matrix</i>.
	@param v_Point the specific %Vector4 object to multiply the current matrix.
	@return The result as a newly created %Vector4 object.
	*/
	Vector4* multiply_WithPoint(Vector4 *v_Point);
	/**
	Multiply the current matrix with a skalar value.
	@param n_Scalar the skalar value to multiply the matrix.
	*/
	void multiply_WithScalar(double n_Scalar);
	/**
	This function returns the specific matrix element. This element is specified by the specified parameter which 
	are the row and column count.
	@param row the row count of the specific element.
	@param column the column count of the specific element.
	@return The matrix element which is defined by teh parameters.
	@exception Exception instance will be thrown if either the row or column count is not valid.
	*/
	double getItem(int row, int column);
	/**
	Get all matrix elements as two dimensional double array.
	@return The two dimensional array containing the corresponding matrix elements.
	*/
	double getArray(void) __gc[,];
	/**
	This function sets a specific matrix element by the specified value. 
	The matrix element which should be overwrited is specified by the specified parameter which 
	are the row and column count.
	@param row the row count of the specific element.
	@param column the column count of the specific element.
	@param value the new value of the specific element.
	@exception Exception instance will be thrown if either the row or column count is not valid.
	*/
	void setItem(int row, int column, double value);
	/**
	Get an one dimensional array of the matrix elements.
	@return The one dimensional array of all matrix elements.
	*/
	double getOneDimArray(void) __gc[];
	/**
	This function returns the negative matrix of the current %Matrix4 object. 
	This means that the current matrix is multiplied with the skalar -1 which is actually a skalar product.
	@return The negative matrix as a new %Matrix4 object.
	*/
	Matrix4* getNegativeMatrix(void);
	/**
	Calculates the determinant of the current matrix.
	For the calculation of the determinant the ordinary mathematical method will be used. This method
	is based on reducing the 4x4 matrix to a 3x3 matrix which determinant value can be easily calculated.
	@return The calculated determinant value.
	*/
	double calculateDeterminant(void);
	/**
	Returns the transpose matrix of the current matrix.
	For the transpose matrix a new instance of the %Matrix4 class will be created.
	@return The transposed matrix as a newly created %Matrix4 object.
	*/
	Matrix4* transpose(void);

	/**
	Creates a newly translation matrix based on the specified translation vector.
	With the values of the Vector3 instance a standard translation matrix will be created.
	@param n_Tanslation the translation values which should be used.
	@return The created translation matrix. 
	*/
	static Matrix4* CreateTranslationMatrix(Vector3* n_Tanslation);
	/**
	Creates a newly rotaion matrix around the x-axis based on the specified angle.
	Please notice that the angle parameter have to be only in radians and not in degrees. 
	@param n_angle the radian angle between 0 and PI.
	@return The created rotation matrix about the x-axis. 
	*/
	static Matrix4* CreateRotXMatrix(double n_angle);
	/**
	Creates a newly rotaion matrix around the y-axis based on the specified angle.
	Please notice that the angle parameter have to be only in radians and not in degrees. 
	@param n_angle the radian angle between 0 and PI.
	@return The created rotation matrix about the y-axis. 
	*/
	static Matrix4* CreateRotYMatrix(double n_angle);
	/**
	Creates a newly rotaion matrix around the z-axis based on the specified angle.
	Please notice that the angle parameter have to be only in radians and not in degrees. 
	@param n_angle the radian angle between 0 and PI.
	@return The created rotation matrix about the z-axis. 
	*/
	static Matrix4* CreateRotZMatrix(double n_angle);
	/**
	Calculates the inverted matrix of the specified 4x4 matrix.
	The calculation of the inverse matrix will be realized by multiply the Cofactor matrix with the
	inverse determinant (just an abrasive try to explain this approximation of the inverse matrix).
	@param n_Matrix the original 4x4 matrix.
	@return The calculated 4x4 inverse matrix. 
	@see Matrix3::CreateReducedMatrix - a main funtion to calculate the Cofactor matrix.
	*/
	static Matrix4* InvertMatrix(Matrix4 *n_Matrix);
	/**
	Creates an uniform scale matrix by the specified scale parameter around the origin..
	That means that all dimensions will be scaled by the same factor.
	@param n_ScaleFactor the scale factor.
	@return The created uniform scale matrix. 
	*/
	static Matrix4* CreateScaleUniformMatrix(double n_ScaleFactor);
	/**
	Creates an uniform scale matrix by the specified scale parameter around an arbitrary point.
	That means that all dimensions will be scaled by the same factor around a fix point.
	@param n_ScaleFactor the scale factor.
	@param n_FixedPoint the fix point of the scale transformation.
	@return The created uniform scale matrix. 
	*/
	static Matrix4* CreateScaleUniformMatrix(double n_ScaleFactor, Vector3 *n_FixedPoint);
	/**
	This static funtion multiplies the specified 4x4 matrices.
	An ordinary matrix multiplication will be used and the resultiin 3x3 matrix will be saved to a
	new created %Matrix4 object.
	@param leftMatrix the left sided matrix.
	@param rightMatrix the right sided matrix.
	@return The resulting 4x4 matrix of the multiplication.
	*/
	static Matrix4* MultiplyMatices(Matrix4* leftMatrix, Matrix4* rightMatrix);
	/**
	Creates a reflection matrix about the x-axis.
	@return The created standard reflection matrix around the x-axis.
	*/
	static Matrix4* CreateReflectionAboutXMatrix(void);
	/**
	Creates a reflection matrix about the y-axis.
	@return The created standard reflection matrix around the y-axis.
	*/
	static Matrix4* CreateReflectionAboutYMatrix(void);
	/**
	Creates a reflection matrix about the z-axis.
	@return The created standard reflection matrix around the z-axis.
	*/
	static Matrix4* CreateReflectionAboutZMatrix(void);
	/**
	Creates an newly 4x4 identity matrix.
	@return The 4x4 identity matrix.
	*/
	static Matrix4* CreateIdentityMatrix(void);
};
}
}
