#include "DataLoader.h"
#include "IRenderer.h"
#include "ProgressWin.h"

#pragma once

using namespace System;
using namespace System::ComponentModel;
using namespace System::Collections;
using namespace System::Windows::Forms;
using namespace System::Data;
using namespace System::Drawing;
using namespace System::Threading;
using namespace System::Globalization;
using namespace IDVR;
using namespace IDVR::Volume;
using namespace IDVR::Volume::DataStructures;

namespace IDVR
{
	/**
	<B>GUI class for creating the <i>Object Creation</i> dialog window.</B>.

	This class is part of the main application namespace IDVR and contains the definition of the <i>Object Creation</i> dialog window.
	<br><br>
	During the creation of a new volume data object by loading the corresponding *.mask file the user have to input a general description name
	for this new volume data object (see also class DataObject). Thus this class implements the input dialog for this needed object description.
	<br><br>
	After the input the actual reading of the mask data will be performed and a new instance of the class DataObject has to be initialized with the 
	loaded data. This instance will be also linked to the current volume data structure which is represented by the curren instance of the class 
	VolumeData.
	<br><br>
	The API documentation for this class shows only the most important member variables and function. Event handling functions and GUI
	components (e.g. labels, textboxes,...) will not be included but all essential parts of the class are surely documented.

	@see VolumeData - represents the entire loaded volume data structure.
	@see DataObject - is a particular volume data object.
	@author <i>Bernhard Pflugfelder</i>
	@version 1.0
	@date 02-21-2005
	*/
	public __gc class ObjectDescriptionInput : public System::Windows::Forms::Form
	{
	public: 

		/**
		This is the unique constructor of this class.
		It implements the standard initialization of the GUI window and actually start a 
		thread to create and control a specific dialog window.
		@param n_VolumeData the current loaded volume data structure.
		@param n_GUIElementHandler a thread handler for update several components the main GUI window.
		@param n_Renderer the specific render model for the new volume data object.
		@param n_ParentForm a direct pointer to the main GUI window.
		*/
		ObjectDescriptionInput(DataLoader *n_VolumeData, Thread *n_GUIElementHandler, IRenderer *n_Renderer, Form *n_ParentForm)
		{
			this->m_VolumeData = n_VolumeData;
			this->m_Renderer = n_Renderer;
			this->m_GUIElementHandler = n_GUIElementHandler;
			this->m_ProgressBarDisplay = NULL;
			this->m_ParentForm = n_ParentForm;
		
			InitializeComponent();

			this->set_ShowInTaskbar(false);
			String *fileName = this->m_VolumeData->get_FileName();
			this->teb_MaskFile->Text = fileName;
			int indexBegin = fileName->LastIndexOf('\\') + 1;
			int indexEnd = fileName->Length - fileName->LastIndexOf('.');
			this->teb_Description->Text = fileName->Substring(indexBegin, fileName->Length - indexBegin - indexEnd);
		}
        
	protected: 

		/**
		Destroys all GUI components before this current instance will be actually discarded.
		@param disposing is a flag variable which is true if the instance have to be discarded.
		*/
		void Dispose(Boolean disposing)
		{
			if (disposing && components)
			{
				components->Dispose();
			}
			__super::Dispose(disposing);
		}
	private: System::Windows::Forms::Label *  lab_MaskFile;
	private: System::Windows::Forms::TextBox *  teb_MaskFile;
	private: System::Windows::Forms::Label *  lab_Description;
	private: System::Windows::Forms::TextBox *  teb_Description;
	private: System::Windows::Forms::Label *  label1;
	private: System::Windows::Forms::Button *  but_OK;
	private: System::Windows::Forms::Button *  but_Cancel;

	private: DataLoader *m_VolumeData;    /**< Saves the current loaded volume data structure. */
			 IRenderer *m_Renderer;	      /**< Saves the render model for the new volume data object. */
			 Thread *m_GUIElementHandler; /**< Saves a thread handler to control several GUI components of the Main GUI window.
										  After the creation process have been successfully completed some GUI components must be updated which
										  are summarized in a thread function.
										  */
			 ProgressWin *m_ProgressBarDisplay; /**< Saves the current instance of the progressbar dialog.
												     This dialog will be shown during loading the mask data. */
			 Form *m_ParentForm;          /**< Saves a pointer to the current main GUI windon. */

	private:
		
		/// Erforderliche Designervariable.
		System::ComponentModel::Container* components;

		/**
		The main function to create the all needed GUI components.
		Furthermore, the linking of those created GUI components (e.g. labels, textboxes,...) to the current container
		is implemented. After the correct linking all GUI items will be shown on the screen.
		@note This function must not be changed in a code editor because otherwise the GUI builder could be working incorrectly.
		*/
		void InitializeComponent(void)
		{
			this->lab_MaskFile = new System::Windows::Forms::Label();
			this->teb_MaskFile = new System::Windows::Forms::TextBox();
			this->lab_Description = new System::Windows::Forms::Label();
			this->teb_Description = new System::Windows::Forms::TextBox();
			this->label1 = new System::Windows::Forms::Label();
			this->but_OK = new System::Windows::Forms::Button();
			this->but_Cancel = new System::Windows::Forms::Button();
			this->SuspendLayout();
			// 
			// lab_MaskFile
			// 
			this->lab_MaskFile->Location = System::Drawing::Point(16, 48);
			this->lab_MaskFile->Name = S"lab_MaskFile";
			this->lab_MaskFile->Size = System::Drawing::Size(104, 16);
			this->lab_MaskFile->TabIndex = 4;
			this->lab_MaskFile->Text = S"Choosen Mask File:";
			// 
			// teb_MaskFile
			// 
			this->teb_MaskFile->Location = System::Drawing::Point(136, 45);
			this->teb_MaskFile->Name = S"teb_MaskFile";
			this->teb_MaskFile->ReadOnly = true;
			this->teb_MaskFile->Size = System::Drawing::Size(208, 20);
			this->teb_MaskFile->TabIndex = 5;
			this->teb_MaskFile->TabStop = false;
			this->teb_MaskFile->Text = S"";
			// 
			// lab_Description
			// 
			this->lab_Description->Location = System::Drawing::Point(16, 96);
			this->lab_Description->Name = S"lab_Description";
			this->lab_Description->Size = System::Drawing::Size(88, 16);
			this->lab_Description->TabIndex = 3;
			this->lab_Description->Text = S"Description:";
			// 
			// teb_Description
			// 
			this->teb_Description->Location = System::Drawing::Point(136, 93);
			this->teb_Description->Name = S"teb_Description";
			this->teb_Description->Size = System::Drawing::Size(208, 20);
			this->teb_Description->TabIndex = 2;
			this->teb_Description->Text = S"";
			this->teb_Description->TextChanged += new System::EventHandler(this, perform_ActionTextboxTextChanged);
			// 
			// label1
			// 
			this->label1->Font = new System::Drawing::Font(S"Microsoft Sans Serif", 9.75F, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, (System::Byte)0);
			this->label1->Location = System::Drawing::Point(48, 16);
			this->label1->Name = S"label1";
			this->label1->Size = System::Drawing::Size(280, 16);
			this->label1->TabIndex = 6;
			this->label1->Text = S"Please enter a name for the choosen object";
			this->label1->TextAlign = System::Drawing::ContentAlignment::TopCenter;
			// 
			// but_OK
			// 
			this->but_OK->Location = System::Drawing::Point(88, 144);
			this->but_OK->Name = S"but_OK";
			this->but_OK->Size = System::Drawing::Size(75, 25);
			this->but_OK->TabIndex = 0;
			this->but_OK->Text = S"OK";
			this->but_OK->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// but_Cancel
			// 
			this->but_Cancel->Location = System::Drawing::Point(208, 144);
			this->but_Cancel->Name = S"but_Cancel";
			this->but_Cancel->Size = System::Drawing::Size(75, 25);
			this->but_Cancel->TabIndex = 1;
			this->but_Cancel->Text = S"Cancel";
			this->but_Cancel->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// ObjectDescriptionInput
			// 
			this->AutoScaleBaseSize = System::Drawing::Size(5, 13);
			this->ClientSize = System::Drawing::Size(384, 190);
			this->Controls->Add(this->but_Cancel);
			this->Controls->Add(this->but_OK);
			this->Controls->Add(this->label1);
			this->Controls->Add(this->teb_Description);
			this->Controls->Add(this->teb_MaskFile);
			this->Controls->Add(this->lab_Description);
			this->Controls->Add(this->lab_MaskFile);
			this->FormBorderStyle = System::Windows::Forms::FormBorderStyle::FixedSingle;
			this->MaximizeBox = false;
			this->MinimizeBox = false;
			this->Name = S"ObjectDescriptionInput";
			this->StartPosition = System::Windows::Forms::FormStartPosition::CenterScreen;
			this->Text = S"Open Mask (.MASK) File";
			this->ResumeLayout(false);

		}		


		/**
		Starts the actual loading precess of the mask data and creates the corresponding volume data object.
		This function is an ordinary event handler which means that it will be only called if the corresponding GUI button
		has been selected.
		@param sender the source component of the event.
		@param e the actual event.
		*/
	private: System::Void perform_ActionButtonClick(System::Object *  sender, System::EventArgs *  e)
	{
		if (dynamic_cast<Button*>(sender)->Equals( this->but_OK ))
		{
			try
			{
				Array *objectIDs = this->m_VolumeData->m_Dataset->getObjectIDs();
				for (int i = 0; i <  objectIDs->Length; i++)
				{
					String *newItem = this->m_VolumeData->m_Dataset->getObjectMember(Convert::ToUInt16(objectIDs->get_Item(i), NumberFormatInfo::get_InvariantInfo()))->ObjectDescription;
					if (0 == String::CompareOrdinal(this->teb_Description->get_Text(),newItem))
					{
						MessageBox::Show(this, S"The given object name is already used. Please choose a different name for this object." , S"Error by reading the .MASK File", MessageBoxButtons::OK, MessageBoxIcon::Error);
						setGUIElementsToObjectLoadedStatus();
						return;
					}
				}
				this->set_Cursor(System::Windows::Forms::Cursors::WaitCursor);
				this->m_ParentForm->set_Cursor(System::Windows::Forms::Cursors::WaitCursor);
						
				if (this->m_ProgressBarDisplay)
				{
					this->Hide();
					this->m_ParentForm->Refresh();

					this->m_ProgressBarDisplay->Show();
					this->m_ProgressBarDisplay->Refresh();
					
					this->m_VolumeData->setProgressbarHandler(this->m_ProgressBarDisplay);
				}
				else
					this->m_VolumeData->setProgressbarHandler(NULL);

				short objectID = this->m_VolumeData->readMaskFile(this->teb_Description->get_Text());
				this->m_VolumeData->m_Dataset->getObjectMember(objectID)->set_RenderModel(this->m_Renderer);

				setGUIElementsToObjectLoadedStatus();
			}
			catch (Exception* e)
			{
				MessageBox::Show(this, e->get_Message(), S"Error by reading the .MASK File", MessageBoxButtons::OK, MessageBoxIcon::Error);
			}
			return;
		}
				 
		if (dynamic_cast<Button*>(sender)->Equals( this->but_Cancel ))
		{
			this->m_GUIElementHandler = NULL;
			this->m_ProgressBarDisplay = NULL;
			this->Close();
			return;
		}
	}

	private: System::Void perform_ActionTextboxTextChanged(System::Object *  sender, System::EventArgs *  e)
	{
		if (teb_Description->get_TextLength() == 0)
			but_OK->set_Enabled(false);
		else
			but_OK->set_Enabled(true);
	}

	/**
	Starts the assigned thread handler to update several GUI components of the main GUI window.
	This function will be only used if the loading process has been finished successfully.
	*/
	private: System::Void setGUIElementsToObjectLoadedStatus(void)
	{
		this->set_Cursor(System::Windows::Forms::Cursors::Default);
		this->m_ParentForm->set_Cursor(System::Windows::Forms::Cursors::Default);
		if (this->m_GUIElementHandler)
			this->m_GUIElementHandler->Start();

		this->Close();
	}

	/**
	Sets a new instance of the progressbar dialog window.
	@param n_ProgressBarDisplay the new progressbar dialog window as an instanve of the class ProgessWin.
	*/
	public: void setProgressBarDisplay(ProgressWin *n_ProgressBarDisplay)
	{
		this->m_ProgressBarDisplay = n_ProgressBarDisplay;
	}
};
}
