#pragma once

#include "IOpenGLCanvas.h"
#include "DataLoader.h"
#include "ImagePlane.h"
#include "Matrix4.h"
#include "DrawModelGLSW.h"
#include "ProxyObjectGL.h"
#include "LightObjectGL.h"
#include "ClearCanvasGL.h"
#include "ArcBall.h"
#include "Light.h"
#include "DataLoader.h"

using namespace System;
using namespace System::ComponentModel;
using namespace System::Collections;
using namespace System::Windows::Forms;
using namespace System::Data;
using namespace System::Drawing;
using namespace IDVR::Tools;
using namespace IDVR::Tools::OpenGLTools;
using namespace IDVR::Volume;
using namespace IDVR::VolumeRenderer::RenderPrimitives;
using namespace IDVR::Interfaces;


namespace IDVR
{
	/**
	<B>Class for the openGL canvas.</B>

	namespace: IDVR

	Baseclass: System::Windows::Forms::Form, IDVR::Interfaces::IOpenGLCanvas

	This is the class, where the rendered image is drawn to. It also provides methos to draw a proxy object and to
	visualize the light direction.

	@author <i>Christopher Thurnher</i>
	@version 1.0
	@date 02-20-2005
	*/
	public __gc class OpenGLCanvas : public System::Windows::Forms::Form, public IOpenGLCanvas
	{
	public: 

		/**
		Constructor.
		@param n_LoactionX The x-location of the canvas.
		@param n_LocationY The Y-location of the canvas.
		@param n_Width The width of the openGL canvas.
		@param n_Heigth The height of the openGL canvas.
		@param n_CanvasWidth The width of the openGL panel.
		@param n_CanvasHeight The height of the openGL panel.
		@param n_CanvasBackColor The background color of the canvas.
		*/
		OpenGLCanvas(int n_LoactionX, int n_LocationY, int n_Width, int n_Heigth, int n_CanvasWidth, int n_CanvasHeight, Color n_CanvasBackColor)
		{
			InitializeComponent();

			this->m_ArcBallProxy = new ArcBall((float)n_CanvasWidth, (float)n_CanvasHeight);
			this->m_ArcBallWorld = new ArcBall((float)n_CanvasWidth, (float)n_CanvasHeight);
			
			this->m_ThisRotProxy = Matrix4::CreateIdentityMatrix();
			this->m_LastRotProxy = Matrix4::CreateIdentityMatrix();
			this->m_TransformMatrixProxy = Matrix4::CreateIdentityMatrix();

			this->m_ThisRotWorld = Matrix4::CreateIdentityMatrix();
			this->m_LastRotWorld = Matrix4::CreateIdentityMatrix();
			this->m_TransformMatrixWorld = Matrix4::CreateIdentityMatrix();

			this->m_CurrentTransformWorld = Matrix4::CreateIdentityMatrix();

			this->m_OpenGLPanel->set_Location(Point(n_LoactionX, n_LocationY));

			this->set_Width(n_Width);
			this->set_Height(n_Heigth);

			this->m_OpenGLPanel->set_Width(n_CanvasWidth);
			this->m_OpenGLPanel->set_Height(n_CanvasHeight);
			this->m_CanvasBackColor = n_CanvasBackColor;

			this->m_ImagePlane = new ImagePlane();
			this->m_DrawModel = new DrawModelGLSW(n_CanvasWidth, n_CanvasHeight, n_CanvasBackColor);
			this->m_ProxyObject = NULL;
			this->m_LightObject = NULL;
			this->m_ClearCanvasGL = new ClearCanvasGL();
			this->m_ClearCanvasGL->set_CanvasBackColor(this->m_CanvasBackColor);
			this->m_LightDirectionPanel = NULL;
			this->m_MainLight = m_ProxyLight = NULL;

			this->m_IsProxyObjectDrawn = this->m_IsResultDrawn = this->m_IsLightObjectDrawn = this->m_IsLeftMouseButton = this->m_IsRightMouseButton = false;
			this->m_Status = NODATALOADED;
			this->m_LightViewMode = LightObjectGL::ABSOLUTELIGHTVIEW;

			clearCanvas();
		}
        
	protected: 
		void Dispose(Boolean disposing)
		{
			if (disposing && components)
			{
				components->Dispose();
			}
			__super::Dispose(disposing);
		}
	private: OpenGLPanel::OpenGLPanel *  m_OpenGLPanel; /**< The openGL panel. */

	private:

		ImagePlane *m_ImagePlane; /**< The image plane. */
		Light *m_MainLight; /**< The main light */
		Light *m_ProxyLight; /**< The light object for the proxy objet. */
		DrawModelGLSW *m_DrawModel; /**< The draw model. */
		ProxyObjectGL *m_ProxyObject; /**< The proxy object. */
		LightObjectGL *m_LightObject; /**< The light object. */
		ClearCanvasGL *m_ClearCanvasGL; /**< Object to clear the canvas. */

		Color m_CanvasBackColor; /**< Background color of the canvas. */
		StatusBarPanel *m_LightDirectionPanel; /**< Panel for the light direction. */
		StatusBarPanel *m_ViewDirectionPanel; /**< Panel for the view direction. */
		bool m_IsProxyObjectDrawn; /**< flag for the proxy object. */
		bool m_IsResultDrawn; /**< flag for the result. */
		bool m_IsLightObjectDrawn; /**< flag for the light object. */
		bool m_IsLeftMouseButton; /**< flag for the left mouse button. */
		bool m_IsRightMouseButton; /**< flag for the right mouse button. */
		unsigned char m_Status; /**< Status. */
		unsigned char m_LightViewMode; /**< Light view mode. */

		ArcBall *m_ArcBallProxy; /**< Arcball for the proxy object. */
		ArcBall *m_ArcBallWorld; /**< Arcball for the world. */
		Matrix4 *m_ThisRotProxy; /**< New rotation matrix of the proxy object. */
		Matrix4 *m_LastRotProxy; /**< Last rotation matrix of the proxy object. */
		Matrix4 *m_TransformMatrixProxy; /**< Transform matrix of the proxy object. */
		Matrix4 *m_ThisRotWorld; /**< New rotation matrix of the world. */
		Matrix4 *m_LastRotWorld; /**< Last rotation matrix of the world. */
		Matrix4 *m_TransformMatrixWorld; /**< Transform matrix of the world. */
		Matrix4 *m_CurrentTransformWorld; /**< The current transform matrix. */

		PointF m_BeginPointLightRotation; /**< Point, that marks the begin of the light rotation. */
		PointF m_EndPointLightRotation; /**< Point, that marks the end of the light rotation. */

		static const char NODATALOADED = 0; /**< No data loaded. */
		static const char READY = 1;/**< Ready. */
		static const char RENDERING = 2;/**< Rendering. */

		
		System::ComponentModel::Container* components; 

		/**
		Initializes the GUI components
		*/
		void InitializeComponent(void)
		{
			this->m_OpenGLPanel = new OpenGLPanel::OpenGLPanel();
			this->SuspendLayout();
			// 
			// m_OpenGLPanel
			// 
			this->m_OpenGLPanel->Code = 0;
			this->m_OpenGLPanel->Location = System::Drawing::Point(128, 128);
			this->m_OpenGLPanel->Name = S"m_OpenGLPanel";
			this->m_OpenGLPanel->Size = System::Drawing::Size(512, 512);
			this->m_OpenGLPanel->TabIndex = 1;
			this->m_OpenGLPanel->SizeChanged += new System::EventHandler(this, sizeChanged_OpenGLCanvas);
			this->m_OpenGLPanel->MouseUp += new System::Windows::Forms::MouseEventHandler(this, mouseUp_OpenGLCanvas);
			this->m_OpenGLPanel->DoubleClick += new System::EventHandler(this, m_OpenGLPanel_DoubleClick);
			this->m_OpenGLPanel->MouseMove += new System::Windows::Forms::MouseEventHandler(this, mouseMove_OpenGLCanvas);
			this->m_OpenGLPanel->MouseWheel += new System::Windows::Forms::MouseEventHandler(this, mouseWheel_OpenGLCanvas);
			this->m_OpenGLPanel->MouseDown += new System::Windows::Forms::MouseEventHandler(this, mouseDown_OpenGLCanvas);
			// 
			// OpenGLCanvas
			// 
			this->AutoScaleBaseSize = System::Drawing::Size(5, 13);
			this->AutoScroll = true;
			this->ClientSize = System::Drawing::Size(768, 768);
			this->Controls->Add(this->m_OpenGLPanel);
			this->FormBorderStyle = System::Windows::Forms::FormBorderStyle::None;
			this->MaximizeBox = false;
			this->MinimizeBox = false;
			this->Name = S"OpenGLCanvas";
			this->StartPosition = System::Windows::Forms::FormStartPosition::Manual;
			this->Text = S"IDVR Canvas";
			this->SizeChanged += new System::EventHandler(this, sizeChanged);
			this->ResumeLayout(false);

		}		


	public:

		/**
		Sets the size of the openGL canvas.
		@param n_Width the width of the openGL canvas.
		@param n_Height the height of the openGL canvas.
		*/
		__property void set_Size(int n_Width, int n_Height);
		/**
		Sets the size of the openGL panel.
		@param n_Width the width of the openGL panel.
		@param n_Height the height of the openGL panel.
		*/
		__property void set_OpenGLCanvasSize(int n_Width, int n_Height);
		/**
		Sets the image plane.
		@param n_ImagePlane the image plane.
		*/
		__property void set_ImagePlaneModel(ImagePlane *n_ImagePlane);
		/**
		Sets the draw model.
		@param n_DrawModelGLSW the draw model.
		*/
		__property void set_DrawModel(DrawModelGLSW *n_DrawModelGLSW);
		/**
		Sets the light model.
		@param n_LightModel the light model.
		*/
		__property void set_LightModel(Light *n_LightModel);
		/**
		Sets the proxy object.
		@param n_ProxyObject the proxy object.
		*/
		__property void set_ProxyObject(ProxyObjectGL *n_ProxyObject);
		/**
		Sets the status.
		@param n_Status the status.
		*/
		__property void set_SystemStatus(unsigned char n_Status);
		/**
		Sets the background color of the openGL canvas.
		@param n_CanvasBackColor the background color of the openGL canvas.
		*/
		__property void set_CanvasBackColor(Color n_CanvasBackColor);

		/**
		Returns the image plane.
		@return The image plane.
		*/
		__property ImagePlane*    get_ImagePlaneModel(void);
		/**
		Returns the draw model.
		@return The draw model.
		*/
		__property DrawModelGLSW* get_DrawModel(void);
		/**
		Returns the proxy object.
		@return The proxy object.
		*/
		__property ProxyObjectGL* get_ProxyObject(void);

		/**
		Sets the light direction panel.
		@param n_LightDirectionPanel the light direction panel.
		*/
		void setLightDirectionPanel(StatusBarPanel *n_LightDirectionPanel);
		/**
		Sets the view direction panel.
		@param n_ViewDirectionPanel view direction panel.
		*/
		void setViewDirectionPanel(StatusBarPanel *n_ViewDirectionPanel);
		/**
		Sets the transform matrix of the proxy object.
		@param n_TransMatrix the transform matrix of the proxy object.
		*/
		void setProxyObjectTransMatrix(Matrix4 *n_TransMatrix);
		/**
		Sets the light view mode.
		@param n_LightViewMode the light view mode.
		*/
		void setLightViewMode(unsigned char n_LightViewMode);
		
		/**
		Returns the openGL canvas.
		@return The openGL canvas.
		*/
		OpenGLPanel::OpenGLPanel* getCanvas(void);

		/**
		Creates the proxy object.
		@param n_CanvasLength the canvas lenght.
		@param n_RelLength the realtive lenght of the proxy object.
		@param n_TransMatrix the transform matrix.
		*/
		void createProxyObjectGL(int n_CanvasLength, float n_RelLength __gc[], Matrix4 *n_TransMatrix);
		/**
		Creates the proxy object.
		@param n_CanvasLength the canvas lenght.
		@param n_RelLength the realtive lenght of the proxy object.
		*/
		void createProxyObjectGL(int n_CanvasLength, float n_RelLength __gc[]);
		/**
		Creates the draw object.
		@param n_Width the width of the openGL canvas.
		@param n_Height the height of the openGL canvas.
		@param n_Resolution the resolution of the openGL canvas.
		*/
		void createDrawModelGLSW(int n_Width, int n_Height, short n_Resolution);
		/**
		Creates the light object.
		@param n_CanvasLength the canvas lenght.
		@param n_LightModel the light model.
		*/
		void createLightObjectGL(int n_CanvasLength, Light *n_LightModel);
		/**
		Creates the light object.
		@param n_CanvasLength the canvas lenght.
		@param n_LightModel the light model.
		@param n_TransMatrix the transform matrix.
		*/
		void createLightObjectGL(int n_CanvasLength, Light *n_LightModel, Matrix4 *n_TransMatrix);

		/**
		Modifies the transform matrix of the proxy object.
		@param n_TransMatrix the transform matrix.
		*/
		void modifyProxyObjectTransMatrix(Matrix4 *n_TransMatrix);
		/**
		Modifies the offset rotation matrix of the light object.
		@param n_OffsetRotation the offset rotation matrix.
		*/
		void modifyLightObjectGLOffsetRotation(Matrix4 *n_OffsetRotation);
		/**
		Modifies the base rotation matrix of the light object.
		@param n_BaseRotation the base rotation matrix.
		*/
		void modifyLightObjectGLBaseRotation(Matrix4 *n_BaseRotation);

		/**
		Draws the framebuffer to the openGL canvas.
		@param n_FrameBuffer the framebuffer.
		*/
		virtual void drawImage(unsigned char n_FrameBuffer __gc[]);

		/**
		Draws the proxy object.
		*/
		void drawProxy(void);
		/**
		Clears the openGL canvas.
		*/
		void clearCanvas(void);
		/**
		Draws the light arrow.
		*/
		void drawLightArrow(void);

		/**
		Resets the view parameters to default values.
		*/
		void resetViewPointPosition(void);
		/**
		Resets the light direction to default values.
		*/
		void resetLightDirection(void);


/**
Eventhandler for size changed events.
@param sender the sender.
@param e the event.
*/
private: System::Void sizeChanged(System::Object *  sender, System::EventArgs *  e)
{
	Point newPoint = Point((this->get_Width() - this->m_OpenGLPanel->get_Width()) / 2,
						   (this->get_Height() - this->m_OpenGLPanel->get_Height()) / 2);
	this->m_OpenGLPanel->set_Location(newPoint);
}

/**
Eventhandler for size changed events.
@param sender the sender.
@param e the event.
*/
private: System::Void sizeChanged_OpenGLCanvas(System::Object *  sender, System::EventArgs *  e)
{
	Point newPoint = Point((this->get_Width() - this->m_OpenGLPanel->get_Width()) / 2,
						   (this->get_Height() - this->m_OpenGLPanel->get_Height()) / 2);
	this->m_OpenGLPanel->set_Location(newPoint);
}

/**
Eventhandler for mouse down events.
@param sender the sender.
@param e the event.
*/
private: System::Void mouseDown_OpenGLCanvas(System::Object *  sender, System::Windows::Forms::MouseEventArgs *  e)
{
	if (this->m_Status == READY)
	{
		if (e->Button == MouseButtons::Left)
		{
			PointF newMousePointProxy = PointF((float)e->get_X(), (float)e->get_Y());
			PointF newMousePointWorld = PointF((float)e->get_X(), this->m_OpenGLPanel->get_Size().Height - (float)e->get_Y());

			this->m_LastRotProxy = this->m_ThisRotProxy;
			this->m_ArcBallProxy->click(newMousePointProxy);

			this->m_LastRotWorld = this->m_ThisRotWorld;
			this->m_ArcBallWorld->click(newMousePointWorld);

			this->drawProxy();

			this->m_IsLeftMouseButton = true;
			this->m_IsRightMouseButton = false;
		}
		else if (e->Button == MouseButtons::Right)
		{
			this->m_BeginPointLightRotation = PointF((float)e->get_X(), (float)e->get_Y());

			drawLightArrow();

			this->m_IsLeftMouseButton = false;
			this->m_IsRightMouseButton = true;
		}
	}
}

/**
Eventhandler for mouse up events.
@param sender the sender.
@param e the event.
*/
private: System::Void mouseUp_OpenGLCanvas(System::Object *  sender, System::Windows::Forms::MouseEventArgs *  e)
{
	if (this->m_Status == READY)
	{
		if (e->Button == MouseButtons::Left)
		{
			this->m_ArcBallProxy->resetVectors();
			this->m_ArcBallWorld->resetVectors();

			this->m_ImagePlane->set_RotMatrix(this->m_CurrentTransformWorld);
			this->m_ImagePlane->transformImagePlane();

			this->m_ViewDirectionPanel->set_Text(String::Concat(S"View:   ", 
				Decimal::Round(Convert::ToDecimal(this->m_ImagePlane->ViewDirection->X), 1).ToString(), S"  |  ", 
				Decimal::Round(Convert::ToDecimal(this->m_ImagePlane->ViewDirection->Y), 1).ToString(), S"  |  ", 
				Decimal::Round(Convert::ToDecimal(this->m_ImagePlane->ViewDirection->Z), 1).ToString()));

			this->m_MainLight->set_BaseRotation(this->m_CurrentTransformWorld);
			this->m_MainLight->calculateLightDirection();
		
			this->m_LightDirectionPanel->set_Text(String::Concat(S"Light:   ", 
				Decimal::Round(Convert::ToDecimal(this->m_MainLight->OffsetLightDirection->X), 1).ToString(), S"  |  ", 
				Decimal::Round(Convert::ToDecimal(this->m_MainLight->OffsetLightDirection->Y), 1).ToString(), S"  |  ", 
				Decimal::Round(Convert::ToDecimal(this->m_MainLight->OffsetLightDirection->Z), 1).ToString()));

			this->m_ProxyLight->set_BaseRotation(this->ProxyObject->TransformationMatrix);
			this->m_ProxyLight->calculateLightDirection();

			this->m_LightObject->set_BaseRotation(Matrix4::InvertMatrix(this->ProxyObject->TransformationMatrix));
			drawLightArrow();
		}
	}
}

/**
Eventhandler for mouse move events.
@param sender the sender.
@param e the event.
*/
private: System::Void mouseMove_OpenGLCanvas(System::Object *  sender, System::Windows::Forms::MouseEventArgs *  e)
{
	if (this->m_Status == READY)
	{
		if (e->Button == MouseButtons::Left)
		{
			Vector4 *newSpherePoint = new Vector4(); 
			Vector4 *newSpherePoint2 = new Vector4(); 
			PointF newMousePoint = PointF((float)e->get_X(), (float)e->get_Y());
			PointF newMousePointWorld = PointF((float)e->get_X(), this->m_OpenGLPanel->get_Size().Height - (float)e->get_Y());
			

			this->m_ArcBallProxy->drag(newMousePoint, newSpherePoint);
			this->m_ArcBallProxy->CreateRotationMatrixFromVector4(this->m_ThisRotProxy, newSpherePoint);

			this->m_ThisRotProxy = Matrix4::MultiplyMatices(this->m_ThisRotProxy, this->m_LastRotProxy);
			//this->thisRot = Matrix4::MultiplyMatices(this->lastRot, this->thisRot);
			this->m_ArcBallProxy->SetRotationFromMatrix(this->m_TransformMatrixProxy, this->m_ThisRotProxy);
			//New: now it is an object based rotation because of left side multiplication !!!
			this->m_ProxyObject->set_TransformationMatrix(Matrix4::MultiplyMatices(this->m_TransformMatrixProxy, this->m_ProxyObject->TransformationMatrix));

			
			this->m_ArcBallWorld->drag(newMousePointWorld, newSpherePoint2);
			this->m_ArcBallWorld->CreateRotationMatrixFromVector4(this->m_ThisRotWorld, newSpherePoint2);
			this->m_ThisRotWorld = Matrix4::MultiplyMatices(this->m_ThisRotWorld, this->m_LastRotWorld);
			this->m_ArcBallWorld->SetRotationFromMatrix(this->m_TransformMatrixWorld, this->m_ThisRotWorld);
			this->m_CurrentTransformWorld = Matrix4::MultiplyMatices(this->m_TransformMatrixWorld, this->m_CurrentTransformWorld);
		
			this->drawProxy();

			this->m_ArcBallProxy->resetVectors();
			this->m_ArcBallProxy->click(newMousePoint);
			this->m_LastRotProxy = this->m_ThisRotProxy;

			this->m_ArcBallWorld->resetVectors();
			this->m_ArcBallWorld->click(newMousePointWorld);
			this->m_LastRotWorld = this->m_ThisRotWorld;
		}

		else if (e->Button == MouseButtons::Right)
		{
			this->m_EndPointLightRotation = PointF((float)e->get_X(), (float)e->get_Y());
			PointF diffPoint = PointF(this->m_EndPointLightRotation.X - this->m_BeginPointLightRotation.X, this->m_EndPointLightRotation.Y - this->m_BeginPointLightRotation.Y);

			double dragAngleX = ((double)diffPoint.Y / this->m_OpenGLPanel->get_Size().Height * Math::PI);
			double dragAngleY = ((double)diffPoint.X / this->m_OpenGLPanel->get_Size().Width * Math::PI);

			Matrix4 *rotateLightModelOpenGL = Matrix4::MultiplyMatices(Matrix4::CreateRotXMatrix(dragAngleX), Matrix4::CreateRotYMatrix(dragAngleY));
			Matrix4 *rotateLightModelWorld  = Matrix4::MultiplyMatices(Matrix4::CreateRotXMatrix(-dragAngleX), Matrix4::CreateRotYMatrix(dragAngleY));

			this->m_LightObject->modifyOffsetRotation(rotateLightModelOpenGL);
			
			this->m_MainLight->modifyOffsetRotation(rotateLightModelWorld);
			this->m_MainLight->calculateLightDirection();
			
			this->m_ProxyLight->modifyOffsetRotation(rotateLightModelOpenGL);
			this->m_ProxyLight->calculateLightDirection();

			this->m_LightDirectionPanel->set_Text(String::Concat(S"Light:   ", 
				Decimal::Round(Convert::ToDecimal(this->m_MainLight->OffsetLightDirection->X), 1).ToString(), S"  |  ", 
				Decimal::Round(Convert::ToDecimal(this->m_MainLight->OffsetLightDirection->Y), 1).ToString(), S"  |  ", 
				Decimal::Round(Convert::ToDecimal(this->m_MainLight->OffsetLightDirection->Z), 1).ToString()));

			drawLightArrow();

			this->m_BeginPointLightRotation.X = this->m_EndPointLightRotation.X;
			this->m_BeginPointLightRotation.Y = this->m_EndPointLightRotation.Y;
		}
	}
}

/**
Eventhandler for mouse wheel events.
@param sender the sender.
@param e the event.
*/
private: System::Void mouseWheel_OpenGLCanvas(System::Object *  sender, System::Windows::Forms::MouseEventArgs *  e)
{
	if (this->m_Status == READY)
	{
		int zoomFactor = e->Delta;

		if (zoomFactor > 0)
		{
			Vector3 *middlePoint = this->m_ImagePlane->get_NullPoint();

			//ImagePlane
			Matrix4 *scaleMatrixIP = Matrix4::CreateScaleUniformMatrix(0.9, middlePoint);
			this->m_ImagePlane->modifyScaleMatrix(scaleMatrixIP);

			this->m_ImagePlane->calculateNewParameters();

			//ProxyObject
			Matrix4 *scaleMatrixPO = Matrix4::CreateScaleUniformMatrix(1/0.9);

			modifyProxyObjectTransMatrix(scaleMatrixPO);
			drawProxy();
		}
		else if (zoomFactor < 0)
		{
			Vector3 *middlePoint = this->m_ImagePlane->get_NullPoint();

			//ImagePlane
			Matrix4 *scaleMatrixIP = Matrix4::CreateScaleUniformMatrix(1/0.9, middlePoint);
			this->m_ImagePlane->modifyScaleMatrix(scaleMatrixIP);

			this->m_ImagePlane->calculateNewParameters();

			//ProxyObject
			Matrix4 *scaleMatrixPO = Matrix4::CreateScaleUniformMatrix(0.9);

			modifyProxyObjectTransMatrix(scaleMatrixPO);
			drawProxy();
		}
	}
}

/**
Eventhandler for double click events.
@param sender the sender.
@param e the event.
*/
private: System::Void m_OpenGLPanel_DoubleClick(System::Object *  sender, System::EventArgs *e)
{
	if (this->m_IsLeftMouseButton)
		resetViewPointPosition();
	else 
		if (this->m_IsRightMouseButton)
		resetLightDirection();
}
};
}
