#pragma once

#include "IOpenGLCanvas.h"
#include "RenderMachineSW.h"
#include "Light.h"
#include "ImagePlane.h"
#include "ClearCanvasGL.h"
#include "DrawModelGLSW.h"
#include "InterpolationSW.h"
#include "GradientEstimationSW.h"
#include "IRenderer.h"
#include "PhongShadingModelSW.h"
#include "ContourShadingModelSW.h"
#include "ToneShadingModelSW.h"
#include "VolumeData.h"

using namespace System;
using namespace System::ComponentModel;
using namespace System::Collections;
using namespace System::Windows::Forms;
using namespace System::Data;
using namespace System::Drawing;
using namespace System::Threading;
using namespace IDVR::Volume::DataStructures;
using namespace IDVR::Tools::OpenGLTools;
using namespace IDVR::Interfaces;
using namespace IDVR::VolumeRenderer::RenderPrimitives;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer::RenderCalculations;

namespace IDVR
{
	/**
	<B>GUI class for creating the <i>Preview Canvas</i> window and actually crendering the specific preview.</B>.

	This class is part of the main application namespace IDVR and contains the definition of the <i>Preview Canvas</i> window.
	<br><br>
	To provide a preview look of the current specified rendering settings with the current loaded volume data set this class implements
	both the GUI window management and the actual rendering calculations for the preview.<br>
	Due the preview picture should be rendererd as fast as possible the size of the canvas and some basic rendering attrributes (e.g. higher sampling distance,
	lower gradient threshold) will be adapted to fulfill this important requirement.<br><br>
	Note that the preview canvas window is a seperate small sub window which will be shown next to the main GUI window.
	<br><br>
	The API documentation for this class shows only the most important member variables and function. Event handling functions and GUI
	components (e.g. labels, textboxes,...) will not be included but all essential parts of the class are surely documented.

	@see Neu - the main GUI window class of this application.
	@author <i>Bernhard Pflugfelder</i>
	@version 1.0
	@date 02-21-2005
	*/
	public __gc class OpenGLPreviewCanvas  : public System::Windows::Forms::Form, public IOpenGLCanvas
	{
	public: 

		/**
		This is a constructor of this class with specific parameters for correct initialization.
		It implements the standard initialization of the GUI window and actually start a 
		thread to create and control a specific dialog window.
		@param n_LoactionX the actual x coordinate of the upper left corner of the preview canvas window.
		@param n_LocationY the actual y coordinate of the upper left corner of the preview canvas window.
		@param n_CanvasBackColor the background color of the prieview canvas.
		@param n_OpacityThreshold a opacity threshold which will be used during the rendering process.
		@param n_GradientThreshold a gradient threshold which will be used during the rendering process.
		@param n_ParentForm a direct pointer to the main GUI window.
		*/
		OpenGLPreviewCanvas(int n_LoactionX, int n_LocationY, Color n_CanvasBackColor, 
				float n_OpacityThreshold, float n_GradientThreshold, Form *n_ParentForm)
		{
			InitializeComponent();

			Point newPoint = Point(n_LoactionX, n_LocationY);
			this->set_Location(newPoint);

			this->m_CanvasBackColor = n_CanvasBackColor;
			this->m_OriginalImagePlane = NULL;
			this->m_PreviewImagePlane = NULL;
			this->m_DrawModel = NULL;
			this->m_ClearCanvas = new ClearCanvasGL();
			this->m_ClearCanvas->set_CanvasBackColor(this->m_CanvasBackColor);
			this->m_ParentForm = n_ParentForm;
			this->m_Dataset = NULL;
			this->m_SubItem_PreviewCanvas = this->m_TbSubItem_PreviewCanvas = NULL;

			this->m_Status = NODATALOADED;

			this->m_OpacityThreshold = n_OpacityThreshold;
			this->m_GradientThreshold = n_GradientThreshold;

			clearCanvas();
		}

		/**
		This is a constructor of this class with specific parameters for correct initialization.
		It implements the standard initialization of the GUI window and actually start a 
		thread to create and control a specific dialog window.
		@param n_LoactionX the actual x coordinate of the upper left corner of the preview canvas window.
		@param n_LocationY the actual y coordinate of the upper left corner of the preview canvas window.
		@param n_CanvasBackColor the background color of the prieview canvas.
		@param n_Dataset a link to the current loaded volume data set.
		@param n_OpacityThreshold a opacity threshold which will be used during the rendering process.
		@param n_GradientThreshold a gradient threshold which will be used during the rendering process.
		@param n_ParentForm a direct pointer to the main GUI window.
		*/
		OpenGLPreviewCanvas(int n_LoactionX, int n_LocationY, Color n_CanvasBackColor, 
						VolumeData *n_Dataset, float n_OpacityThreshold, float n_GradientThreshold, Form *n_ParentForm)
		{
			InitializeComponent();

			Point newPoint = Point(n_LoactionX, n_LocationY);
			this->set_Location(newPoint);

			this->m_CanvasBackColor = n_CanvasBackColor;
			this->m_OriginalImagePlane = NULL;
			this->m_PreviewImagePlane = NULL;
			this->m_DrawModel = NULL;
			this->m_ClearCanvas = new ClearCanvasGL();
			this->m_ClearCanvas->set_CanvasBackColor(this->m_CanvasBackColor);
			this->m_Dataset = n_Dataset;
			this->m_ParentForm = n_ParentForm;

			this->m_Status = READY;
			this->m_TwoLevelRendering = this->m_IDVR = false;

			this->m_OpacityThreshold = n_OpacityThreshold;
			this->m_GradientThreshold = n_GradientThreshold;
			this->m_SubItem_PreviewCanvas = this->m_TbSubItem_PreviewCanvas = NULL;

			clearCanvas();
		}
        
	protected: 

		/**
		Destroys all GUI components before this current instance will be actually discarded.
		@param disposing is a flag variable which is true if the instance have to be discarded.
		*/
		void Dispose(Boolean disposing)
		{
			if (disposing && components)
			{
				components->Dispose();
				this->m_DrawModel = NULL;
				this->m_ClearCanvas = NULL;
				this->m_PreviewImagePlane = NULL;
			}
			__super::Dispose(disposing);
		}

	private: OpenGLPanel::OpenGLPanel *  panGL_Preview;
	private: System::Windows::Forms::Button *  but_Render;
	private: System::Windows::Forms::Button *  but_Clear;
	private: System::Windows::Forms::ToolTip *  tot_ToolTips;
	private: System::Windows::Forms::ImageList *  iml_Images;
	private: System::ComponentModel::IContainer *  components;

	private: 
			 Light *m_LightModel;		/**< Saves the assigned light model.
											 The instance includes all needed light model settings */ 
			 ImagePlane *m_OriginalImagePlane; /**< Pointer to the original image plane of the main canvas. */
			 ImagePlane *m_PreviewImagePlane;  /**< Saves the preview image plane for the adapted convas attributes. */
			 ClearCanvasGL *m_ClearCanvas;	   /**< Saves a deletion object for the preview canvas.
													This deletion object is implemented with OpenGL and clears the preview panel. */
			 DrawModelGLSW *m_DrawModel;	   /**< Saves a drawing object for the preview canvas.
													This drawing object is implemented with OpenGL and draws the preview results onto the preview panel. */
			 Color m_CanvasBackColor;		   /**< Saves the current background color of the preview canvas. */
			 VolumeData *m_Dataset;			   /**< Pointer to the current loaded volume data set. */
			 bool m_TwoLevelRendering;         /**< Saves whether two-level rendering is activated or not. */
			 bool m_IDVR;                      /**< Saves whether %IDVR rendering is activated or not. */
			 float m_OpacityThreshold;         /**< Saves the current opacity threshold for early ray termination. */
			 float m_GradientThreshold;        /**< Saves the current gradient threshold. */

			 TD::SandBar::MenuButtonItem *m_SubItem_PreviewCanvas, *m_TbSubItem_PreviewCanvas;
			 Form *m_ParentForm;               /**< Saves a pointer to the current main GUI windon. */

			 unsigned char m_Status;		   
			 static const char NODATALOADED = 0, READY = 1, RENDERING = 2;
			
	private:

		/**
		The main function to create the all needed GUI components.
		Furthermore, the linking of those created GUI components (e.g. labels, textboxes,...) to the current container
		is implemented. After the correct linking all GUI items will be shown on the screen.
		@note This function must not be changed in a code editor because otherwise the GUI builder could be working incorrectly.
		*/
		void InitializeComponent(void)
		{
			this->components = new System::ComponentModel::Container();
			System::Resources::ResourceManager *  resources = new System::Resources::ResourceManager(__typeof(IDVR::OpenGLPreviewCanvas));
			this->panGL_Preview = new OpenGLPanel::OpenGLPanel();
			this->but_Render = new System::Windows::Forms::Button();
			this->iml_Images = new System::Windows::Forms::ImageList(this->components);
			this->but_Clear = new System::Windows::Forms::Button();
			this->tot_ToolTips = new System::Windows::Forms::ToolTip(this->components);
			this->SuspendLayout();
			// 
			// panGL_Preview
			// 
			this->panGL_Preview->Code = 0;
			this->panGL_Preview->Cursor = System::Windows::Forms::Cursors::Default;
			this->panGL_Preview->Location = System::Drawing::Point(0, 0);
			this->panGL_Preview->Name = S"panGL_Preview";
			this->panGL_Preview->Size = System::Drawing::Size(200, 200);
			this->panGL_Preview->TabIndex = 0;
			this->tot_ToolTips->SetToolTip(this->panGL_Preview, S"Preview Canvas");
			// 
			// but_Render
			// 
			this->but_Render->FlatStyle = System::Windows::Forms::FlatStyle::Popup;
			this->but_Render->ImageAlign = System::Drawing::ContentAlignment::BottomCenter;
			this->but_Render->ImageIndex = 0;
			this->but_Render->ImageList = this->iml_Images;
			this->but_Render->Location = System::Drawing::Point(204, 4);
			this->but_Render->Name = S"but_Render";
			this->but_Render->Size = System::Drawing::Size(36, 36);
			this->but_Render->TabIndex = 1;
			this->tot_ToolTips->SetToolTip(this->but_Render, S"Start Rendering");
			this->but_Render->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// iml_Images
			// 
			this->iml_Images->ImageSize = System::Drawing::Size(28, 28);
			this->iml_Images->ImageStream = (__try_cast<System::Windows::Forms::ImageListStreamer *  >(resources->GetObject(S"iml_Images.ImageStream")));
			this->iml_Images->TransparentColor = System::Drawing::Color::Transparent;
			// 
			// but_Clear
			// 
			this->but_Clear->FlatStyle = System::Windows::Forms::FlatStyle::Popup;
			this->but_Clear->ImageAlign = System::Drawing::ContentAlignment::BottomCenter;
			this->but_Clear->ImageIndex = 1;
			this->but_Clear->ImageList = this->iml_Images;
			this->but_Clear->Location = System::Drawing::Point(204, 48);
			this->but_Clear->Name = S"but_Clear";
			this->but_Clear->Size = System::Drawing::Size(36, 36);
			this->but_Clear->TabIndex = 2;
			this->tot_ToolTips->SetToolTip(this->but_Clear, S"Delete Image");
			this->but_Clear->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// tot_ToolTips
			// 
			this->tot_ToolTips->AutoPopDelay = 5000;
			this->tot_ToolTips->InitialDelay = 500;
			this->tot_ToolTips->ReshowDelay = 100;
			// 
			// OpenGLPreviewCanvas
			// 
			this->AutoScaleBaseSize = System::Drawing::Size(5, 13);
			this->BackColor = System::Drawing::SystemColors::ControlDark;
			this->ClientSize = System::Drawing::Size(244, 200);
			this->Controls->Add(this->but_Clear);
			this->Controls->Add(this->but_Render);
			this->Controls->Add(this->panGL_Preview);
			this->FormBorderStyle = System::Windows::Forms::FormBorderStyle::FixedSingle;
			this->MaximizeBox = false;
			this->Name = S"OpenGLPreviewCanvas";
			this->StartPosition = System::Windows::Forms::FormStartPosition::Manual;
			this->Text = S"Preview Image";
			this->tot_ToolTips->SetToolTip(this, S"Preview Canvas Window");
			this->Closing += new System::ComponentModel::CancelEventHandler(this, OpenGLPreviewCanvas_Closing);
			this->MouseEnter += new System::EventHandler(this, OpenGLPreviewCanvas_MouseEnter);
			this->ResumeLayout(false);

		}		


	public:
		
			/**
			Sets a link to the new original image plane of the main canvas.
			@param n_OriginalImagePlane a link to the new original (main) image plane.
			*/
			__property void set_OriginalImagePlane(ImagePlane *n_OriginalImagePlane);
			/**
			Sets a link to the new light model which will be used for preview rendering.
			@param n_LightModel a link to the new light model.
			*/
			__property void set_LightModel(Light *n_LightModel);
			/**
			Sets the new light model which will be used for preview rendering.
			@param n_CanvasBackColor the background color of the prieview canvas.
			*/
			__property void set_CanvasBackColor(Color n_CanvasBackColor);
	
			//Sets the current status of the preview canvas (is it ready of rendering - inactive)
			__property void set_SystemStatus(unsigned char n_Status);
			/**
			Sets the new two-level rendering status which will be used for preview rendering.
			@param n_TwoLevelRendering teh status of two-level rendering.
			*/
			__property void set_TwoLevelRendering(bool n_TwoLevelRendering);
			/**
			Sets the new %IDVR rendering status which will be used for preview rendering.
			@param n_IDVR the status of %IDVR rendering.
			*/
			__property void set_IDVR(bool n_IDVR);
			/**
			Sets a link to the current loaded volume data set.
			@param n_Dataset a link to the current loaded volume data set.
			*/
			__property void set_Dataset(VolumeData *n_Dataset);
			/**
			Sets the new opacity threshold which is essential for early ray termination.
			@param n_OpacityThreshold the new value of the opacity threshold.
			*/
			__property void set_OpacityThreshold(float n_OpacityThreshold);
			/**
			Sets the new gradient threshold which is essential for the rendering process.
			@param n_GradientThreshold the new value of the gradient threshold.
			*/
			__property void set_GradientThreshold(float n_GradientThreshold);

			void setControlSubItem(TD::SandBar::MenuButtonItem *n_SubItem_PreviewCanvas);
			void setControlTbSubItem(TD::SandBar::MenuButtonItem *n_TbSubItem_PreviewCanvas);

			/**
			Clears the preview canvas and sets it to the current background color.
			*/
			void clearCanvas(void);
			/**
			Draws the the color values of the specified array onto the canvas.
			@param n_FrameBuffer the resulting RGB color values.
			*/
			virtual void drawImage(unsigned char n_FrameBuffer __gc[]);



	private: 
			
			/**
			Starts the actual rendering process for the preview image.
			*/
			void startRenderingProcess(void);
			void setReadyStatus(void);

	private: System::Void perform_ActionButtonClick(System::Object *  sender, System::EventArgs *  e)
	{
		if (this->m_Status != READY) return;

		if (dynamic_cast<Button*>(sender)->Equals( this->but_Render ))
		{
			startRenderingProcess();
			return;
		}
	
		if (dynamic_cast<Button*>(sender)->Equals( this->but_Clear ))
		{
			clearCanvas();
			return;
		}
	}

	private: System::Void OpenGLPreviewCanvas_MouseEnter(System::Object *  sender, System::EventArgs *  e)
	{
			 switch(this->m_Status)
			 {
				case NODATALOADED:
							this->set_Cursor(System::Windows::Forms::Cursors::No);
							break;
				case READY:
							this->set_Cursor(System::Windows::Forms::Cursors::Default);
							break;
				case RENDERING:
							this->set_Cursor(System::Windows::Forms::Cursors::WaitCursor);
							break;
			 }
	}

	private: System::Void OpenGLPreviewCanvas_Closing(System::Object *  sender, System::ComponentModel::CancelEventArgs *  e)
	{
		this->m_SubItem_PreviewCanvas->Checked = false;
		this->m_TbSubItem_PreviewCanvas->Checked = false;
	}
};
}

