#pragma once

#include "Matrix4.h"
#include "Light.h"

using namespace CsGL::OpenGL;
using namespace CsGL::Pointers;
using namespace CsGL::Util;
using namespace System;
using namespace IDVR::Tools;
using namespace IDVR::VolumeRenderer::RenderPrimitives;

namespace IDVR {
namespace Tools {
namespace OpenGLTools {

	using namespace System::Drawing;
	/**
	<b>Class for drawing the proxy object onto the cancas.</b>

	This class is part of the OpenGLTools namespace which is actually a subnamespace of the main namespace Tools. 
	<br><br>
	The task of this class is not included in the actual rendering process and the succeeding drawing, but to display a proxy
	object with the current rotaion status which will be also used for the actual rendering of the volume data set to help the 
	user during user-defined rotation.
	<br><br>
	As every other class of the subpackage OpenGLTools this class has the base class OpenGLCode of the CSOpenGL library
	to correctly implement the OpenGL code Which will be handled by the OpenGLPanel which will show the specific drawing 
	results on the screen.<br><br>
	
	@see OpenGLCanvas - the GUI class for the canvas with its draw control
	@author <i>Bernhard Pflugfelder</i>
	@version 1.0
	@date 02-20-2005
	*/
public __gc class ProxyObjectGL : public OpenGLPanel::OpenGLCode
{

private:

	float m_RelLength __gc[]; /**< Saves the relative length of the proxy object (independly from the current canvas size). */
	float m_AbsLength __gc[]; /**< Saves the absolute length of the proxy object (depends on the current canvas size). */
	float m_CanvasBackColor __gc[]; /**< Saves the current canvas background color. */
	int m_CanvasLength;             /**< Saves the width of the current canvas. */
	unsigned int m_CuboidObject;    /**< Saves the index of the used OpenGL texture object for the proxy object. */
	Matrix4 *m_TransMatrix;			/**< Saves the current universal transformation matrix of the proxy object. */
	Matrix4 *m_TranslateImagePlaneMatrix; /**< Saves the current translation matrix of the proxy object (the same for the image plane). */
	Matrix4 *m_RotImagePlaneMatrix;       /**< Saves the current rotation matrix of the proxy object (the same for the image plane). */
	Matrix4 *m_ScaleImagePlaneMatrix;	  /**< Saves the current scale matrix of the proxy object (the same for the image plane). */
	Light *m_LightModel; /**< Saves the current light model with all needed light attributes. 
								     This light attributes are direction and color. */


	static const float LENGTHRATIO = 1.0f / 3.0f;
	__value enum LengthDim {X = 0,Y,Z};

public:
	/**
	This is the standard constructor.
	All member variables of this new instance will be set to zero/null.
	*/
	ProxyObjectGL(void);
	/**
	Constructor initialization with the size of the canvas.
	All member variables will be set to zero besides the canvas length member variable and proxy object relative-length
	member variable.
	@param n_CanvasLength the width of the current canvas.
	@param n_RelLength the relative length of the proxy object.
	*/
	ProxyObjectGL(int n_CanvasLength, float n_RelLength __gc[]);
	/**
	Constructor initialization with the size of the canvas and the basic transformation matrix.
	All member variables will be set to zero besides the canvas length member variable and proxy object relative-length
	member variable. Additionally the basic transformation matrix of the proxy object will be set with the corresponding
	parameter.
	@param n_CanvasLength the width of the current canvas.
	@param n_RelLength the relative length of the proxy object.
	@param n_TransMatrix the basic transformation matrix of the proxy object.
	*/
	ProxyObjectGL(int n_CanvasLength, float n_RelLength __gc[], Matrix4 *n_TransMatrix);
	/**
	This is the standard deconstructor.
	All member variables will be deleted.
	*/
	~ProxyObjectGL(void);
	
	/**
	This function overwrites the base init funtion of the class OpenGLCode.
	It contains the actual initialization code in OpenGL and also performs the code.
	*/
	virtual void Init(void);
	/**
	This function overwrites the base draw funtion of the class OpenGLCode.
	It contains the actual drawing code in OpenGL and also performs the code.
	*/
	virtual void Draw(void);

	/**
	Sets a new transformation matrix for the proxy object.
	@param n_TransMatrix the transformation matrix of the proxy object.
	*/
	__property void set_TransformationMatrix(Matrix4 *n_TransMatrix);
	/**
	Sets a new transformation matrix for the proxy object.
	@param n_CanvasLength the transformation matrix of the proxy object.
	*/
	__property void set_NewCanvasSize(int n_CanvasLength);
	/**
	Sets a new canvas background color.
	@param n_CanvasBackColor the new canvas background color as an instance of the standard color class.
	*/
	__property void set_CanvasBackColor(Color n_CanvasBackColor);

	/**
	Returns the current transformation matrix of the proxy object.
	@return The transformation matrix as an instance of the class Matrix4.
	*/
	__property Matrix4* get_TransformationMatrix(void);

	/**
	Sets a new relative length values for the proxy object.
	@param n_RelLength the relative length of the proxy object.
	*/
	void setNewRelLengthValues(float n_RelLength __gc[]);
	/**
	Sets a new light user-defined light model.
	@param n_LightModel the new light model which conatins light direction and color.
	*/
	void setLightModel(Light *n_LightModel);

private:
	/**
	Defines the standard proxy object in OpenGL as an OpenGL display list.
	*/
	void defineCuboidObject(void);
};
}
}
}