#pragma once

#include ".\IRenderMachine.h"
#include ".\IIDVRCompositingModel.h"
#include ".\IIDVRModulationModel.h"
#include ".\OpenGLCanvas.h"
#include ".\GradientEstimationSW.h"
#include ".\Ray.h"
#include ".\RendererSW.h"
#include ".\LMipCompositingModelSW.h"
#include ".\DvrCompositingModelSW.h"
#include ".\Gradient.h"
#include ".\InterpolationSW.h"
#include ".\GradientInterpolationWithPreCalcSW.h"
#include ".\GradientInterpolationWithoutPreCalcSW.h"
#include ".\ProgressWin.h"
#include ".\FootprintElement.h"

namespace IDVR {
namespace VolumeRenderer {
namespace SWVolumeRenderer {

using namespace IDVR;
using namespace IDVR::Tools;
using namespace IDVR::VolumeRenderer::Interfaces;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer::CompositingModels;

/**
<B>Class for the actual rendering.</B>

namespace: IDVR::VolumeRenderer::SWVolumeRenderer

Baseclass: IDVR::Interfaces::IRenderMachine

This class provides methods to render the scene. It has methods for normal volume rendering (see RenderMachineSW::do_NormalRayCasting),
two-level rendering (see RenderMachineSW::do_TwoLevelRayCasting) and importance driven rendering (see RenderMachineSW::do_ImportanceDrivenRayCasting).

@author <i>Christopher Thurnher</i>
@version 1.0
@date 02-20-2005
*/

public __gc class RenderMachineSW : public IRenderMachine
{
private:

	VolumeData* m_DataSet; /**< The dataset. */
	VolumeRenderer::RenderPrimitives::ImagePlane* m_ImagePlane; /**< The image plane. */
	IRenderer* m_GlobalRenderer; /**< The global renderer. */
	IIDVRCompositingModel *m_IDVRCompositingModel; /**< The importance driven compositing model. */
	Color m_BackColor; /**< The background color. */
	ProgressWin *m_ProgressWin; /**< The progress bar. */
	Thread *m_GUIHandler; /**< Rendering thread. */
	IOpenGLCanvas *m_Canvas; /**< The openGL canvas. */

	bool m_IsTwoLevel; /**< Two level rendering flag. */
	bool m_IsImportance; /**< Importance driven rendering thread. */
	char m_GradientEstimationMode; /**< The gradient estimation mode. */
	char m_DensityInterpolationMode; /**< The density interpolation mode. */
	char m_GradientInterpolationMode; /**< The gradient interpolation mode. */
	char m_VoxelIDInterpolationMode; /**< The voxelID interpolation mode. */
	double m_SamplingDistance; /**< The sampling distance. */
	double m_ThresholdOpacity; /**< The opacity threshold. */
	double m_ThresholdGradient; /**< The gradient magnitude threshold. */
	unsigned char m_FrameBuffer __gc[]; /**< The framebuffer for the rendered image. */

	__value enum Indices{X = 0, Y, Z};
	__value enum ColorIndices{R = 0, G, B};

	static const int ENTRYPOINT = 0; /**< Entrypoint. */
	static const int EXITPOINT = 1; /**< Exitpoint. */

public:

	/**
	Standard constructor.
	*/
	RenderMachineSW(void);
	/**
	Constructor.
	@param n_Dataset the dataset.
	@param n_ImagePlane the image plane.
	@param n_BackColor the background color.
	@param n_Canvas the openGL canvas (rendering target).
	*/
	RenderMachineSW(VolumeData *n_Dataset, RenderPrimitives::ImagePlane* n_ImagePlane, Color n_BackColor, IOpenGLCanvas *n_Canvas);
	/**
	Standard destructor.
	*/
	~RenderMachineSW(void);

	/**
	Calculates the intersection points of a ray with the volume.
	@param startPoint the first point of the ray on the image plane.
	@return The intersection points.
	*/
	Vector3* calculate_IntersectionPoints(Vector3* startPoint) __gc[];
	
	/**
	Starts the rendering process.
	*/
	virtual void start_Raycasting(void);
	/**
	Does normal raycasting.
	*/
	virtual void do_NormalRayCasting(void);
	/**
	Does two-level raycasting.
	*/
	virtual void do_TwoLevelRayCasting(void);
	/**
	Does importance driven raycasting.
	@param n_Footprint the footprint object.
	*/
	virtual void do_ImportanceDrivenRayCasting(Footprint *n_Footprint);

	/**
	Sets the volume dataset.
	@param n_DataSet the volume dataset.
	*/
	__property virtual void set_Data(VolumeData* n_DataSet);
	/**
	Sets the two-level rendering flag.
	@param n_IsTwoLevel the two-level rendering flag.
	*/
	__property virtual void set_TwoLevelRendering(bool n_IsTwoLevel);
	/**
	Sets the importance driven rendering flag.
	@param n_IsImportance the importance driven rendering flag.
	*/
	__property virtual void set_ImportanceDrivenRendering(bool n_IsImportance); 
	/**
	Sets the sampling distance.
	@param n_SamplingDistance the sampling distance.
	*/
	__property virtual void set_SampingDistance(double n_SamplingDistance);
	/**
	Sets the opacity threshold.
	@param n_ThresholdOpacity the opacity threshold.
	*/
	__property virtual void set_OpacityThreshold(double n_ThresholdOpacity);
	/**
	Sets the gradient threshold.
	@param n_ThresholdGradient the gradient threshold.
	*/
	__property virtual void set_GradientThreshold(double n_ThresholdGradient);
	/**
	Sets the image plane.
	@param n_ImagePlane the image plane.
	*/
	__property virtual void set_ImagePlane(RenderPrimitives::ImagePlane* n_ImagePlane);
	/**
	Sets the gradient estimation mode.
	@param n_GradientEstimationMode the gradient estimation mode.
	*/
	__property virtual void set_GradientEstimationMode(char n_GradientEstimationMode);
	/**
	Sets the density interpolation mode.
	@param n_DensityInterpolationMode the density interpolation mode.
	*/
	__property virtual void set_DensityInterpolationMode(char n_DensityInterpolationMode);
	/**
	Sets the gradient interpolation mode.
	@param n_GradientInterpolationMode the gradient interpolation mode.
	*/
	__property virtual void set_GradientInterpolationMode(char n_GradientInterpolationMode);
	/**
	Sets the voxelID interpolation mode.
	@param n_VoxelIDInterpolationMode the voxelID interpolation mode.
	*/
	__property virtual void set_VoxelIDInterpolationMode(char n_VoxelIDInterpolationMode);

	/**
	Returns the image plane.
	@return The image plane.
	*/
	__property virtual RenderPrimitives::ImagePlane* get_ImagePlane(void);
	/**
	Returns the volume dataset.
	@return The volume dataset.
	*/
	__property virtual VolumeData* get_Data(void);

	/**
	Returns the framebuffer.
	@return The framebuffer.
	*/
	unsigned char getFrameBuffer(void) __gc[];
	/**
	Generates the footprint object.
	@return The footprint object.
	*/
	Footprint* generateFootprint(void);
	/**
	Sets the openGL canvas.
	@param n_Canvas The openGL canvas.
	*/
	void setOpenGLCanvas(IOpenGLCanvas *n_Canvas);
	/**
	Sets the progress bar.
	@param n_ProgressWin the progress bar.
	*/
	void setProgressWin(ProgressWin *n_ProgressWin);
	/**
	Sets the GUI handler.
	@param n_GUIHandler the GUI handler.
	*/
	void setGUIHandler(Thread *n_GUIHandler);
};
}
}
}