#pragma once

#include "IRenderer.h"
#include "MipShadingModelSW.h"

namespace IDVR {
namespace VolumeRenderer {
namespace SWVolumeRenderer {

using namespace IDVR::Interfaces;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels;

/**
<B>Class for the renderer.</B>

namespace: IDVR::VolumeRenderer::SWVolumeRenderer

Baseclass: IDVR::Interfaces::IRenderer

This class provides methods to render the scene. Each renderer consists of three parts. the first one is
a classification model. It assigns calculates an opacity value for a given raysample. The second one is a shading
model which calculates a color value for a given raysample. At last, the compositing model calculates the final
color and opacity values by blending the new color and opacity of a raysample with the already calcultaed ones.

@see IDVR::VolumeRenderer::SWVolumeRenderer::ClassificationModels::LinearClassificationModelSW - the linear classification model.
@see IDVR::VolumeRenderer::SWVolumeRenderer::ClassificationModels::TFClassificationModelSW - the transfer function classification model.
@see IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels::ContourShadingModelSW - the contour shading model
@see IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels::MipShadingModelSW - the mip shading model
@see IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels::PhongShadingModelSW - the phong shading model
@see IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels::ToneShadingModelSW - the tone shading model
@see IDVR::VolumeRenderer::SWVolumeRenderer::CompositingModels::DvrCompositingModelSW - the dvr compositing model
@see IDVR::VolumeRenderer::SWVolumeRenderer::CompositingModels::LMipCompositingModelSW - the LMip compositing model

@author <i>Christopher Thurnher</i>
@version 1.0
@date 02-19-2005
*/

public __gc class RendererSW : public IRenderer
{
private:

	IClassificationModel *m_IClassificator; /**< the classification model. */
	IShadingModel *m_IShader; /**< the shading model. */
	ICompositingModel *m_ICompositor; /**< the composition model. */
	char m_RenderMode; /**< the render mode. */

public:

	//render modes
	static const char TF_PHONG_DVR = 0; /**< classification: transfer function, shading: phong, copmpositing: dvr */
	static const char LIN_MIP_LMIP = 1; /**< classification: linear, shading: mip, copmpositing: LMip */
	static const char TF_CONTOUR_DVR = 2; /**< classification: transfer function, shading: contour, copmpositing: dvr */
	static const char LIN_CONTOUR_LMIP = 3; /**< classification: linear, shading: contour, copmpositing: LMip */
	static const char TF_TONE_DVR = 4; /**< classification: transfer function, shading: tone, copmpositing: dvr */
	static const char LIN_TONE_LMIP = 5; /**< classification: linear, shading: tone, copmpositing: LMip */
	static const char TF_MIP_LMIP = 6; /**< classification: transfer function, shading: mip, copmpositing: LMip */

	/**
	Standard constructor.
	*/
	RendererSW(void);
	/**
	Constructor.
	@param n_RenderMode the render mode.
	@param n_IClassificator the classification model.
	@param n_IShader the shading model.
	@param n_ICompositor the composition model.
	*/
	RendererSW(char n_RenderMode, IClassificationModel *n_IClassificator, IShadingModel *n_IShader, ICompositingModel *n_ICompositor);
	/**
	Standard destructor.
	*/
	~RendererSW(void);

	/**
	Calculates the opacity value for a given raysample.
	@param n_RaySample the given raysample.
	@return The modified raysample.
	*/
	virtual RaySample* doClassification(RaySample *n_RaySample);
	/**
	Calculates the intensity value for a given raysample.
	@param n_RaySample the given raysample.
	@return The modified raysample.
	*/
	virtual RaySample* doShading(RaySample *n_RaySample);
	/**
	Blends the final intensity and opacity values of a given raysample.
	@param n_RaySample the given raysample.
	*/
	virtual void doCompositing(RaySample *n_RaySample);
	/**
	Blends the final pixelcolor with the given background color.
	@param n_BackColor the background color.
	*/
	virtual void doCompositingBackground(float n_BackColor __gc[]);
	/**
	Sets the classification model.
	@param n_IClassificator the classification model.
	*/
    virtual void setClassificator(IClassificationModel *n_IClassificator);
	/**
	Sets the shading model.
	@param n_IShader the shading model.
	*/
	virtual void setShader(IShadingModel *n_IShader);
	/**
	Sets the composition model.
	@param n_ICompositor the composition model.
	*/
	virtual void setCompositor(ICompositingModel *n_ICompositor);

	/**
	Returns the classification model.
	@return The classification model.
	*/
	virtual IClassificationModel* getClassificator(void);
	/**
	Returns the shading model.
	@return The shading model.
	*/
	virtual IShadingModel* getShader(void);
	/**
	Returns the composition model.
	@return The composition model.
	*/
	virtual ICompositingModel* getCompositor(void);

	/**
	Returns true, if gradinet estimation is needed.
	@return True, if gradinet estimation is needed.
	*/
	virtual bool need_GradientEstimation(void);
	/**
	Resets the values of the composition model.
	*/
	virtual void reset_Compositor(void);
	/**
	Returns the current intensity array.
	@return The current intensity array.
	*/
	virtual float getIntensity(void) __gc[];
	/**
	Returns the current opacity value.
	@return The current opacity value.
	*/
	virtual float getOpacity(void);

	/**
	Sets the render mode.
	@param n_RenderMode the render mode.
	*/
	virtual void setRenderMode(char n_RenderMode);
	/**
	Returns the render mode.
	@return The render mode.
	*/
	virtual char getRenderMode(void);

};
}
}
}
