#pragma once

#include "IShadingmodel.h"
#include "Vector3.h"
#include "Transferfunction.h"

namespace IDVR {
namespace VolumeRenderer {
namespace SWVolumeRenderer {
namespace ShadingModels {

using namespace IDVR::Interfaces;
using namespace IDVR::VolumeRenderer::RenderPrimitives;
using namespace IDVR::Tools;
using namespace System::Drawing;
using namespace IDVR::Volume::DataStructures;

/**<B>Class for the contour shading model.</B>

Namespace: IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels

Baseclass: IDVR::Interfaces::IShadingModel

This class calculates the color value of a given raysample. If the gradient of the raysample points to the light,
the raysample gets a warm color, otherwise a cold one.

@see ContourShadingModelSW - the contour shading model
@see MipShadingModelSW - the mip shading model
@see PhongShadingModelSW - the phong shading model
@author <i>Christopher Thurnher</i>
@version 1.0
@date 02-18-2005
*/

public __gc class ToneShadingModelSW : public IShadingModel
{

private:

	__value enum colorIndices{RED, GREEN, BLUE};

	unsigned short m_DensityScope; /**< the scope of the density values of the dataset. */
	Vector3* m_LightDirection; /**< the light direction. */
	double m_GradientMagnitudeThreshold; /**< a big gradient magnitude threshold indicates a surface. */
	double m_KWarm; /**< ratio of the chosen warm color. */
	double m_KCold; /**< ratio of the chosen cold color. */
	float m_WarmColor __gc[]; /**< the warm color. */
	float m_ColdColor __gc[]; /**< the cold color. */
	float m_KTF; /**< ratio of the calculated color according to the transfer function. */
	Transferfunction* m_TransferFunc; /**< the transfer function. */

public:

	/**
	Standard constructor.
	*/
	ToneShadingModelSW(void);
	/**
	Constructor.
	@param n_DensityScope the scope of the density values of the dataset.
	@param n_GradientMagnitudeThreshold the gradient magnitude threshold.
	@param n_KWarm the ratio of the chosen warm color.
	@param n_CWarm the warm color.
	@param n_KCold the ratio of the chosen cold color.
	@param n_CCold the cold color.
	@param n_LightDirection the light direction.
	*/
	ToneShadingModelSW(unsigned short n_DensityScope, double n_GradientMagnitudeThreshold, double n_KWarm, Color n_CWarm, double n_KCold, Color n_CCold, Vector3* n_LightDirection);
	/**
	Constructor.
	@param n_DensityScope the scope of the density values of the dataset.
	@param n_GradientMagnitudeThreshold the gradient magnitude threshold.
	@param n_KWarm the ratio of the chosen warm color.
	@param n_CWarm the warm color.
	@param n_KCold the ratio of the chosen cold color.
	@param n_CCold the cold color.
	@param n_LightDirection the light direction.
	@param n_KTF the ration of the transfer function.
	@param n_TransferFunc the transfer function.
	*/
	ToneShadingModelSW(unsigned short n_DensityScope, double n_GradientMagnitudeThreshold, double n_KWarm, Color n_CWarm, double n_KCold, Color n_CCold, Vector3* n_LightDirection, float n_KTF, Transferfunction* n_TransferFunc);

	/**
	Calculates the Color value for a given raysample.
	@param n_RaySample the specified raysample.
	@return The modified raysample.
	*/
	virtual RaySample* doShading(RaySample* n_RaySample);

	/**
	Sets the current light direction.
	@param n_Light the light direction.
	*/
	__property void set_LightDirection(Vector3* n_Light);
	/**
	Returns the gradient magnitude threshold.
	@return The gradient magnitude threshold.
	*/
	__property double get_ToneGradientMagnitudeThreshold(void);
	/**
	Returns the ratio of the warm color.
	@return The ratio of the warm color.
	*/
	__property double get_ToneKWarm(void);
	/**
	Returns the ratio of the cold color.
	@return The ratio of the cold color.
	*/
	__property double get_ToneKCold(void);
	/**
	Returns the the warm color as Color object.
	@return The the warm color as Color object.
	*/
	__property Color get_ToneWarmColor(void);
	/**
	Returns the the cold color as Color object.
	@return The the cold color as Color object.
	*/
	__property Color get_ToneColdColor(void);
	/**
	Returns the ktf value.
	@return The ktf value.
	*/
	__property float get_ToneKTF(void);
};
}
}
}
}