#include "DataLoader.h"
#include "IRenderer.h"
#include "RendererSW.h"
#include "TFClassificationModelSW.h"
#include "DvrCompositingModelSW.h"
#include "PhongShadingModelSW.h"
#include "ToneShadingModelSW.h"
#include "ContourShadingModelSW.h"

#pragma once

using namespace System;
using namespace System::ComponentModel;
using namespace System::Collections;
using namespace System::Windows::Forms;
using namespace System::Data;
using namespace System::Drawing;
using namespace System::Drawing::Drawing2D;
using namespace IDVR;
using namespace IDVR::Interfaces;
using namespace IDVR::Volume;
using namespace IDVR::Volume::DataStructures;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer::ClassificationModels;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer::CompositingModels;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels;

namespace IDVR
{
	/**
	<B>GUI class for the transfer function editor.</B>

	namespace: IDVR

	Baseclass: System::Windows::Forms::Form

	This is the GUI class for the transfer function editor. It provides methods modify the opacity and color values
	of a certain density value of a given object and so design a user specified transfer function.

	@author <i>Christopher Thurnher</i>
	@version 1.0
	@date 02-21-2005
	*/
	public __gc class TransferFuncWin : public System::Windows::Forms::Form
	{
	public: 

		/**
		Constructor.
		@param n_VolumeData the volume dataset.
		@param objectID the objectID of the given object.
		@param n_RenderingMode the rendering mode.
		@param n_TransFunc the transfer function.
		*/
		TransferFuncWin(DataLoader *n_VolumeData, int objectID, char n_RenderingMode, Transferfunction *n_TransFunc)
		{
			this->m_VolumeData = n_VolumeData;
			this->m_SubItem_EditTransferFuntion = this->m_TbSubItem_EditTransferFuntion = NULL;
			this->m_PreviewPanel = NULL;

			this->pan_Histogram_Paint = new System::Windows::Forms::PaintEventHandler(this, perform_ActionPanelPaint); 
			this->pan_Density_Paint = new System::Windows::Forms::PaintEventHandler(this, perform_ActionPanelPaint); 
			this->pan_RGB_Paint = new System::Windows::Forms::PaintEventHandler(this, perform_ActionPanelPaint); 
			this->MouseDown = new System::Windows::Forms::MouseEventHandler(this, perform_ActionPanelMouseDown);
			this->MouseMove = new System::Windows::Forms::MouseEventHandler(this, perform_ActionPanelMouseMove);

			InitializeComponent();

			this->selectedObjectID = objectID;

			this->pan_Density->add_Paint(this->pan_Density_Paint);
			this->pan_Histogram->add_Paint(this->pan_Histogram_Paint);
			this->pan_RGB->add_Paint(this->pan_RGB_Paint);

			//MouseEventHandler
			this->pan_Histogram->add_MouseDown(MouseDown);
			this->pan_Histogram->add_MouseUp(MouseUp);
			this->pan_Histogram->add_MouseMove(MouseMove);		

			DataStructures::Transferfunction *TransFunc;

			this->m_RenderingMode = n_RenderingMode;
			this->m_TransFuncInput = n_TransFunc;
			this->set_Text(this->MAINTITLE);

			if (this->m_RenderingMode == TF_MIP_LMIP)
			{
				this->pan_RGB->set_Enabled(false);
				this->pan_RGB->set_BackColor(Color::LightGray);
				this->pan_RGB->set_BorderStyle(BorderStyle::Fixed3D);
				this->but_Color->set_Enabled(false);
				this->but_ResetPoints->set_Enabled(false);
				this->grb_Interpolation->set_Enabled(false);
				this->trb_Interpolation->set_Enabled(false);
			}

			IRenderer* renderer = this->m_VolumeData->m_Dataset->getObjectMember(selectedObjectID)->get_RenderModel();

			if (this->m_TransFuncInput->get_FunctionLength() != 0)
				TransFunc = this->m_TransFuncInput;
			else
			{
				switch (m_RenderingMode)
				{
				case TF_PHONG_DVR:	
					if ((renderer->getClassificator()->GetType() == __typeof(TFClassificationModelSW)) && 
							(renderer->getShader()->GetType() == __typeof(PhongShadingModelSW)))
						TransFunc = dynamic_cast<TFClassificationModelSW*>(renderer->getClassificator())->get_TransferFunc();
					else
						TransFunc = Transferfunction::CreateGeneralTransfer(DataLoader::DensityScope);
					break;

				case TF_MIP_LMIP:	
					if ((renderer->getClassificator()->GetType() == __typeof(TFClassificationModelSW)) && 
						(renderer->getShader()->GetType() == __typeof(MipShadingModelSW)))
						TransFunc = dynamic_cast<TFClassificationModelSW*>(renderer->getClassificator())->get_TransferFunc();
					else
						TransFunc = Transferfunction::CreateGeneralTransfer(DataLoader::DensityScope);
					break;

				case TF_CONTOUR_DVR:	
					if ((renderer->getClassificator()->GetType() == __typeof(TFClassificationModelSW)) && 
							(renderer->getShader()->GetType() == __typeof(ContourShadingModelSW)))
						TransFunc = dynamic_cast<TFClassificationModelSW*>(renderer->getClassificator())->get_TransferFunc();
					else
						TransFunc = Transferfunction::CreateGeneralTransfer(DataLoader::DensityScope);
					break;

				case TF_TONE_DVR:	
					if ((renderer->getClassificator()->GetType() == __typeof(TFClassificationModelSW)) && 
							(renderer->getShader()->GetType() == __typeof(ToneShadingModelSW)))
						TransFunc = dynamic_cast<TFClassificationModelSW*>(renderer->getClassificator())->get_TransferFunc();
					else
						TransFunc = Transferfunction::CreateGeneralTransfer(DataLoader::DensityScope);
					break;
				}
			}

			this->colorSamples = Transferfunction::convertColorSamplesToDrawMode(TransFunc->get_ColorSamples(),
																				  this->pan_Histogram->get_Size().Width);
			this->opacitySamples = Transferfunction::convertOpacitySamplesToDrawMode(TransFunc->get_OpacitySamples(),
																					 this->pan_Histogram->get_Size().Width,
																				     this->pan_Histogram->get_Size().Height);


			this->TransFunc_Length = this->pan_Histogram->get_Size().Width;
			this->c_TransferFunc = new Color __gc[this->TransFunc_Length];
			this->m_InterpolationFactor = TransFunc->get_InterpolationFactor();

			this->default_RGBValue = TransFunc->get_BackColor();
			
			this->colDia_ChoseColor->set_Color(default_RGBValue);
			this->trb_Interpolation->set_Value(Convert::ToInt32(Math::Round(this->trb_Interpolation->get_Maximum() * this->m_InterpolationFactor)));

			sort_OpacityIndices();
			if (this->m_RenderingMode != TF_MIP_LMIP)
			{
				sort_ColorIndices();
				do_Interpolation();
				pan_RGB_repaint();
			}
		}
        
	protected: 
		void Dispose(Boolean disposing)
		{
			if (disposing && components)
			{
				components->Dispose();
			}
			__super::Dispose(disposing);
		}

	private: System::Windows::Forms::GroupBox *  grb_Histogram;
	private: System::Windows::Forms::Panel *  pan_Density;
	private: System::Windows::Forms::Panel *  pan_Histogram;
	private: System::Windows::Forms::Button *  but_Color;
	private: System::Windows::Forms::Button *  but_Cancel;
	private: System::Windows::Forms::Button *  but_OK;
	private: System::Windows::Forms::ColorDialog *  colDia_ChoseColor;
	private: System::Windows::Forms::Panel *  pan_RGB;
	private: System::Windows::Forms::TrackBar *  trb_Interpolation;
	private: System::Windows::Forms::GroupBox *  grb_Interpolation;
	private: System::Windows::Forms::Button *  but_ResetPoints;
	private: System::Windows::Forms::Button *  but_InterpolationReset;
	private: System::Windows::Forms::Label *  lab_Description;
	private: System::Windows::Forms::GroupBox *  grp_ColorControl;
	private: System::Windows::Forms::Label *  lab_ColorControl;
	private: System::Windows::Forms::Button *  but_Opacity;
	
	//PaintEventHandler for the three panels
	private: System::Windows::Forms::PaintEventHandler* pan_Histogram_Paint;
	private: System::Windows::Forms::PaintEventHandler* pan_Density_Paint;
	private: System::Windows::Forms::PaintEventHandler* pan_RGB_Paint;
	
    //MouseEventHandler for drawing opacity and color functions
	private: System::Windows::Forms::MouseEventHandler* MouseDown;
	private: System::Windows::Forms::MouseEventHandler* MouseUp;
	private: System::Windows::Forms::MouseEventHandler* MouseMove;

	public:	static const char SAMPLE_FACTOR = 8; /**< Sample factor 8. */
			static const char POINT_SIZE_BIG = 8; /**< Pointsize of the lines (8). */
			static const char POINT_SIZE_HALF = 4; /**< Pointsize of the lines (4). */
			static const char TF_PHONG_DVR = 0; /**< classification: transfer function, shading: phong, copmpositing: dvr */
			static const char TF_CONTOUR_DVR = 1; /**< classification: transfer function, shading: contour, copmpositing: dvr */
			static const char TF_TONE_DVR = 2; /**< classification: transfer function, shading: tone, copmpositing: dvr */
			static const char TF_MIP_LMIP = 3; /**< classification: transfer function, shading: mip, copmpositing: LMip */

			static const char MOUSEBUTTON_DOWNRANGE = 5; /**< Range of the mouseclick (5). */
			static const char MOUSEBUTTON_MOVERANGE = 10;/**< Range of the mouseclick (10). */
			static String *MAINTITLE = S"Edit Transferfunction ";/**< Titel of the GUI class. */
					 

	private:
			 DataLoader *m_VolumeData; /**< The volume dataset. */
             int TransFunc_Length; /**< the length of the transfer function. */
			 int sortedIndicesColor __gc[]; /**< The color indices of the transfer function. */
			 int sortedIndicesOpacity __gc[]; /**< The opacity indices of the transfer function. */
			 Hashtable *colorSamples; /**< The color samples of the transfer function. */
			 Hashtable *opacitySamples; /**< The opacity samples of the transfer function. */
			 char m_RenderingMode; /**< The randering mode. */
			 double m_InterpolationFactor; /**< The interpolation factor. */
			 Transferfunction *m_TransFuncInput; /**< The transfer function input. */

			 Color default_RGBValue; /**< The defauld RGB values. */
			 Color c_TransferFunc __gc[]; /**< The color array of the transfer function. */
			 int selectedObjectID; /**< The selected objectID. */
			 Panel* m_PreviewPanel; /**< The preview panel. */
			 TD::SandBar::MenuButtonItem *m_SubItem_EditTransferFuntion; /**< the subitem for the GUI. */
			 TD::SandBar::MenuButtonItem *m_TbSubItem_EditTransferFuntion; /**< the toolbar subitem for the GUI. */

			 __value enum ColorIndex {RED = 0,GREEN,BLUE};

		
		System::ComponentModel::Container* components;

		/**
		Initializes the GUI components
		*/
		void InitializeComponent(void)
		{
			this->grb_Histogram = new System::Windows::Forms::GroupBox();
			this->lab_Description = new System::Windows::Forms::Label();
			this->pan_Density = new System::Windows::Forms::Panel();
			this->pan_Histogram = new System::Windows::Forms::Panel();
			this->pan_RGB = new System::Windows::Forms::Panel();
			this->but_Color = new System::Windows::Forms::Button();
			this->but_Cancel = new System::Windows::Forms::Button();
			this->but_OK = new System::Windows::Forms::Button();
			this->colDia_ChoseColor = new System::Windows::Forms::ColorDialog();
			this->trb_Interpolation = new System::Windows::Forms::TrackBar();
			this->grb_Interpolation = new System::Windows::Forms::GroupBox();
			this->but_InterpolationReset = new System::Windows::Forms::Button();
			this->but_ResetPoints = new System::Windows::Forms::Button();
			this->grp_ColorControl = new System::Windows::Forms::GroupBox();
			this->lab_ColorControl = new System::Windows::Forms::Label();
			this->but_Opacity = new System::Windows::Forms::Button();
			this->grb_Histogram->SuspendLayout();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->trb_Interpolation))->BeginInit();
			this->grb_Interpolation->SuspendLayout();
			this->grp_ColorControl->SuspendLayout();
			this->SuspendLayout();
			// 
			// grb_Histogram
			// 
			this->grb_Histogram->Controls->Add(this->lab_Description);
			this->grb_Histogram->Controls->Add(this->pan_Density);
			this->grb_Histogram->Controls->Add(this->pan_Histogram);
			this->grb_Histogram->Controls->Add(this->pan_RGB);
			this->grb_Histogram->Location = System::Drawing::Point(8, 8);
			this->grb_Histogram->Name = S"grb_Histogram";
			this->grb_Histogram->Size = System::Drawing::Size(528, 384);
			this->grb_Histogram->TabIndex = 1;
			this->grb_Histogram->TabStop = false;
			this->grb_Histogram->Text = S"Transfer Function";
			// 
			// lab_Description
			// 
			this->lab_Description->Location = System::Drawing::Point(8, 16);
			this->lab_Description->Name = S"lab_Description";
			this->lab_Description->Size = System::Drawing::Size(480, 32);
			this->lab_Description->TabIndex = 13;
			this->lab_Description->Text = S"To edit the opacity transfer function use the left mouse button and move along th" 
				S"e wished path. To set a new color sample point use the right mouse button.";
			// 
			// pan_Density
			// 
			this->pan_Density->BackColor = System::Drawing::SystemColors::ActiveCaptionText;
			this->pan_Density->BorderStyle = System::Windows::Forms::BorderStyle::FixedSingle;
			this->pan_Density->Location = System::Drawing::Point(8, 320);
			this->pan_Density->Name = S"pan_Density";
			this->pan_Density->Size = System::Drawing::Size(512, 24);
			this->pan_Density->TabIndex = 1;
			// 
			// pan_Histogram
			// 
			this->pan_Histogram->BackColor = System::Drawing::Color::Beige;
			this->pan_Histogram->Location = System::Drawing::Point(8, 48);
			this->pan_Histogram->Name = S"pan_Histogram";
			this->pan_Histogram->Size = System::Drawing::Size(512, 264);
			this->pan_Histogram->TabIndex = 0;
			// 
			// pan_RGB
			// 
			this->pan_RGB->BackColor = System::Drawing::SystemColors::ActiveCaptionText;
			this->pan_RGB->BorderStyle = System::Windows::Forms::BorderStyle::FixedSingle;
			this->pan_RGB->Location = System::Drawing::Point(8, 352);
			this->pan_RGB->Name = S"pan_RGB";
			this->pan_RGB->Size = System::Drawing::Size(512, 24);
			this->pan_RGB->TabIndex = 12;
			// 
			// but_Color
			// 
			this->but_Color->FlatStyle = System::Windows::Forms::FlatStyle::Flat;
			this->but_Color->Location = System::Drawing::Point(8, 88);
			this->but_Color->Name = S"but_Color";
			this->but_Color->Size = System::Drawing::Size(96, 28);
			this->but_Color->TabIndex = 7;
			this->but_Color->Text = S"Default Color";
			this->but_Color->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// but_Cancel
			// 
			this->but_Cancel->Location = System::Drawing::Point(544, 56);
			this->but_Cancel->Name = S"but_Cancel";
			this->but_Cancel->Size = System::Drawing::Size(112, 28);
			this->but_Cancel->TabIndex = 6;
			this->but_Cancel->Text = S"Cancel";
			this->but_Cancel->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// but_OK
			// 
			this->but_OK->Location = System::Drawing::Point(544, 16);
			this->but_OK->Name = S"but_OK";
			this->but_OK->Size = System::Drawing::Size(112, 28);
			this->but_OK->TabIndex = 5;
			this->but_OK->Text = S"OK";
			this->but_OK->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// trb_Interpolation
			// 
			this->trb_Interpolation->AutoSize = false;
			this->trb_Interpolation->Location = System::Drawing::Point(552, 328);
			this->trb_Interpolation->Maximum = 40;
			this->trb_Interpolation->Name = S"trb_Interpolation";
			this->trb_Interpolation->Size = System::Drawing::Size(96, 24);
			this->trb_Interpolation->SmallChange = 2;
			this->trb_Interpolation->TabIndex = 12;
			this->trb_Interpolation->TickStyle = System::Windows::Forms::TickStyle::None;
			this->trb_Interpolation->Value = 20;
			this->trb_Interpolation->Scroll += new System::EventHandler(this, perform_ActionTrackbarScroll);
			// 
			// grb_Interpolation
			// 
			this->grb_Interpolation->Controls->Add(this->but_InterpolationReset);
			this->grb_Interpolation->Location = System::Drawing::Point(544, 304);
			this->grb_Interpolation->Name = S"grb_Interpolation";
			this->grb_Interpolation->Size = System::Drawing::Size(112, 88);
			this->grb_Interpolation->TabIndex = 13;
			this->grb_Interpolation->TabStop = false;
			this->grb_Interpolation->Text = S"Interpolation";
			// 
			// but_InterpolationReset
			// 
			this->but_InterpolationReset->FlatStyle = System::Windows::Forms::FlatStyle::Flat;
			this->but_InterpolationReset->Location = System::Drawing::Point(24, 56);
			this->but_InterpolationReset->Name = S"but_InterpolationReset";
			this->but_InterpolationReset->Size = System::Drawing::Size(64, 24);
			this->but_InterpolationReset->TabIndex = 0;
			this->but_InterpolationReset->Text = S"Reset";
			this->but_InterpolationReset->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// but_ResetPoints
			// 
			this->but_ResetPoints->FlatStyle = System::Windows::Forms::FlatStyle::Flat;
			this->but_ResetPoints->Location = System::Drawing::Point(8, 128);
			this->but_ResetPoints->Name = S"but_ResetPoints";
			this->but_ResetPoints->Size = System::Drawing::Size(96, 28);
			this->but_ResetPoints->TabIndex = 14;
			this->but_ResetPoints->Text = S"Color Samples";
			this->but_ResetPoints->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// grp_ColorControl
			// 
			this->grp_ColorControl->Controls->Add(this->lab_ColorControl);
			this->grp_ColorControl->Controls->Add(this->but_Color);
			this->grp_ColorControl->Controls->Add(this->but_ResetPoints);
			this->grp_ColorControl->Controls->Add(this->but_Opacity);
			this->grp_ColorControl->Location = System::Drawing::Point(544, 128);
			this->grp_ColorControl->Name = S"grp_ColorControl";
			this->grp_ColorControl->Size = System::Drawing::Size(112, 168);
			this->grp_ColorControl->TabIndex = 15;
			this->grp_ColorControl->TabStop = false;
			this->grp_ColorControl->Text = S"Funtion Control";
			// 
			// lab_ColorControl
			// 
			this->lab_ColorControl->Location = System::Drawing::Point(8, 24);
			this->lab_ColorControl->Name = S"lab_ColorControl";
			this->lab_ColorControl->Size = System::Drawing::Size(96, 16);
			this->lab_ColorControl->TabIndex = 0;
			this->lab_ColorControl->Text = S"Reset of:";
			// 
			// but_Opacity
			// 
			this->but_Opacity->FlatStyle = System::Windows::Forms::FlatStyle::Flat;
			this->but_Opacity->Location = System::Drawing::Point(8, 48);
			this->but_Opacity->Name = S"but_Opacity";
			this->but_Opacity->Size = System::Drawing::Size(96, 28);
			this->but_Opacity->TabIndex = 8;
			this->but_Opacity->Text = S"Default Opacity";
			this->but_Opacity->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// TransferFuncWin
			// 
			this->AutoScaleBaseSize = System::Drawing::Size(5, 13);
			this->ClientSize = System::Drawing::Size(664, 406);
			this->Controls->Add(this->trb_Interpolation);
			this->Controls->Add(this->but_Cancel);
			this->Controls->Add(this->but_OK);
			this->Controls->Add(this->grb_Histogram);
			this->Controls->Add(this->grb_Interpolation);
			this->Controls->Add(this->grp_ColorControl);
			this->FormBorderStyle = System::Windows::Forms::FormBorderStyle::FixedSingle;
			this->MaximizeBox = false;
			this->Name = S"TransferFuncWin";
			this->StartPosition = System::Windows::Forms::FormStartPosition::CenterScreen;
			this->Closed += new System::EventHandler(this, TransferFuncWin_Closed);
			this->grb_Histogram->ResumeLayout(false);
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->trb_Interpolation))->EndInit();
			this->grb_Interpolation->ResumeLayout(false);
			this->grp_ColorControl->ResumeLayout(false);
			this->ResumeLayout(false);

		}	




	/**
	Eventhandler for mouse move events.
	@param sender the sender.
	@param e the event.
	*/
	private: System::Void perform_ActionPanelMouseMove(System::Object *  sender, System::Windows::Forms::MouseEventArgs * e)
	{
			if (e->Button == MouseButtons::Left)
			{
				int currentX = e->get_X(), currentY = e->get_Y();

				if (e->get_X() > this->pan_Histogram->get_Size().Width - 1)
					currentX = this->pan_Histogram->get_Size().Width - 1;
				else if (e->get_X() < 0)
					currentX = 0;

				if (e->get_Y() > this->pan_Histogram->get_Size().Height - 1)
					currentY = this->pan_Histogram->get_Size().Height - 1;
				else if (e->get_Y() < 0)
					currentY = 0;

				bool isOpacitySampleSelected = false;
				int selectedX, selectedY;

				IDictionaryEnumerator *nextPos = this->opacitySamples->GetEnumerator();
				
				while (nextPos->MoveNext())
				{
					int x1 = Convert::ToInt32(nextPos->get_Key());
					int y1 = Convert::ToInt32(nextPos->get_Value());
					
					if (Math::Abs(x1 - currentX) < MOUSEBUTTON_MOVERANGE && Math::Abs(y1 - currentY) < MOUSEBUTTON_MOVERANGE)
					{
						isOpacitySampleSelected = true;
						selectedX = x1;
						selectedY = y1;
					}
				 }

				 if (isOpacitySampleSelected)
				 {
					 if (selectedX == 0)
					 {
						this->opacitySamples->set_Item(__box(0), __box(currentY));
					 }
					 else if (selectedX == this->pan_Histogram->get_Size().Width)
					 {
						this->opacitySamples->set_Item(__box(this->pan_Histogram->get_Size().Width), __box(currentY));
					 }
					 else
					 {
						try
						{
							this->opacitySamples->Remove(__box(selectedX));
							this->opacitySamples->Add(__box(currentX), __box(currentY));
						}
						catch(ArgumentException*)
						{
							if (currentX == 0)
								this->opacitySamples->set_Item(__box(1), __box(currentY));
							else if (currentX == this->pan_Histogram->get_Size().Width)
								this->opacitySamples->set_Item(__box(this->pan_Histogram->get_Size().Height - 1), __box(currentY));
							else if (currentX > selectedX)
								this->opacitySamples->set_Item(__box(currentX + 1), __box(currentY));
							else if (currentX <= selectedX)
								this->opacitySamples->set_Item(__box(currentX - 1), __box(currentY));
						}
					 }
					 sort_OpacityIndices();
					 pan_Histogram_repaint();
				 }
				 
			}

			else if (e->Button == MouseButtons::Right)
			{
				int currentX = e->get_X();

				if (e->get_X() > this->pan_Histogram->get_Size().Width - 1)
					currentX = this->pan_Histogram->get_Size().Width - 1;
				else if (e->get_X() < 0)
					currentX = 0;

				Object *sampleColor; bool isColorSampleSelected = false; int selectedX;
				IDictionaryEnumerator *nextPos = this->colorSamples->GetEnumerator();
				
				while (nextPos->MoveNext())
				{
					int x1 = Convert::ToInt32(nextPos->get_Key());
						
					if (Math::Abs(x1 - currentX) < MOUSEBUTTON_MOVERANGE)
					{
						isColorSampleSelected = true;
						selectedX = x1;
						sampleColor = nextPos->get_Value();
					}
				}

				if (isColorSampleSelected)
				{
					try
					{
						this->colorSamples->Remove(__box(selectedX));
						this->colorSamples->Add(__box(currentX), sampleColor);
					}
					catch(ArgumentException*)
					{
						if (currentX > selectedX)
							this->colorSamples->set_Item(__box(currentX + 1), sampleColor);
						else
							this->colorSamples->set_Item(__box(currentX - 1), sampleColor);
					}
					
					if (this->m_RenderingMode != TF_MIP_LMIP)
					{
						sort_ColorIndices();
						do_Interpolation();
						pan_RGB_repaint();
					}
					pan_Histogram_repaint();
					
				}
			}
	}

	/**
	Eventhandler for mouse down events.
	@param sender the sender.
	@param e the event.
	*/
	private: System::Void perform_ActionPanelMouseDown(System::Object *  sender, System::Windows::Forms::MouseEventArgs * e)
	{
			 if (e->Button == MouseButtons::Left)
			 {
				bool isOpacitySampleSelected = false;
				IDictionaryEnumerator* nextPos = this->opacitySamples->GetEnumerator();
				
				while (nextPos->MoveNext())
				{
					int x1 = Convert::ToInt32(nextPos->get_Key());
					int y1 = Convert::ToInt32(nextPos->get_Value());
					
					if (Math::Abs(x1 - e->get_X()) < MOUSEBUTTON_DOWNRANGE && Math::Abs(y1 - e->get_Y()) < MOUSEBUTTON_DOWNRANGE)
						isOpacitySampleSelected = true;
				}

				if (!isOpacitySampleSelected)
				 {
					 //String *newKey = Convert::ToString(e->get_X());
					 //String *newValue = Convert::ToString(e->get_Y());
					 this->opacitySamples->set_Item(__box(e->get_X()), __box(e->get_Y()));
				
					 sort_OpacityIndices();
					 pan_Histogram_repaint();	
				 }
			 }
			 else if (e->Button == MouseButtons::Right && this->m_RenderingMode != TF_MIP_LMIP)
			 {
				bool isColorSampleSelected = false;
				IDictionaryEnumerator* nextPos = this->colorSamples->GetEnumerator();
				
				while (nextPos->MoveNext())
				{
					int x1 = Convert::ToInt32(nextPos->get_Key());
	
					if (Math::Abs(x1 - e->get_X()) < MOUSEBUTTON_DOWNRANGE)
						isColorSampleSelected = true;
				}

				if (!isColorSampleSelected)
				{
					this->colDia_ChoseColor->FullOpen = true;
					if (this->colDia_ChoseColor->ShowDialog() == DialogResult::OK)
					{
						colorSamples->set_Item(__box(e->get_X()), __box(colDia_ChoseColor->get_Color().ToArgb()));

						sort_ColorIndices();
						do_Interpolation();
						pan_RGB_repaint();	
						pan_Histogram_repaint();
					}
				}
			 }
		 }

	/**
	Eventhandler for panel paint events.
	@param sender the sender.
	@param e the event.
	*/
	private: System::Void perform_ActionPanelPaint(System::Object *  sender, System::Windows::Forms::PaintEventArgs *  e)
	{
		 if (__try_cast<Panel*>(sender)->Equals( this->pan_Histogram ))
		 {
			 pan_Histogram_repaint();
			 return;
		 }

		 if (__try_cast<Panel*>(sender)->Equals( this->pan_Density ))
		 {
			 pan_Density_repaint();
			 return;
		 }

		 if (__try_cast<Panel*>(sender)->Equals( this->pan_RGB ))
		 {
			if (this->m_RenderingMode != TF_MIP_LMIP)
				pan_RGB_repaint();
			return;
		}
	 }

	private:

		/**
		Repaints the density values.
		*/
		void pan_Density_repaint(void);
		/**
		Interpolates all values.
		*/
		void do_Interpolation(void);
		/**
		Interpolated between two color values.
		@param x1 first x-value.
		@param x2 second x-value.
		@param c1 first color value.
		@param c2 second color value.
		*/
		void interpolate_ColorScope(int x1, int x2, Color c1, Color c2);
		/**
		Repaints the histogram.
		*/
		void pan_Histogram_repaint(void);
		/**
		Repaints the RGB histogram.
		*/
		void pan_RGB_repaint(void);

		/**
		Sorts all color indices.
		*/
		void sort_ColorIndices(void);
		/**
		Sorts all opacity indices.
		*/
		void sort_OpacityIndices(void);

	public: 

		/**
		Sets the objectID of the transfer function.
		@param n_SelectedObjectID the objectID of the transfer function.
		*/
		void set_selectedObjectID(unsigned short n_SelectedObjectID);
		/**
		Sets the preview panel.
		@param n_Pan_PreView the preview panel.
		*/
		void setPreViewPanel(Panel* n_Pan_PreView);
		/**
		Expands the title.
		@param n_TitleAddition the title addition.
		*/
		void setTitleAddition(String *n_TitleAddition);
		/**
		Sets the subitem for the GUI.
		@param n_SubItem_EditTransferFuntion the subitem for the GUI.
		*/
		void setControlSubItem(TD::SandBar::MenuButtonItem *n_SubItem_EditTransferFuntion);
		/**
		Sets the toolbar subitem for the GUI.
		@param n_TbSubItem_EditTransferFuntion the toolbar subitem for the GUI.
		*/
		void setControlTbSubItem(TD::SandBar::MenuButtonItem *n_TbSubItem_EditTransferFuntion);

		/**
		Sorts the color indices.
		@param n_ColorSamples the color samples.
		@return The sorted array.
		*/
		static int SortColorIndices(Hashtable *n_ColorSamples) __gc[];
		/**
		Sorts the opacity indices.
		@param n_OpacitySamples the opacity samples.
		@return The sorted array.
		*/
		static int SortOpacityIndices(Hashtable *n_OpacitySamples) __gc[];
		/**
		Interpolates all values of the transfer function.
		@param n_TransFunc the transfer function.
		@param n_PanelWidth the width of the panel.
		@return The interpolated color array.
		*/
		static Color DoInterpolation(Transferfunction *n_TransFunc, int n_PanelWidth) __gc[];
		/**
		Interpolates between two color values.
		@param x1 first x-value.
		@param x2 second x-value.
		@param c1 first color value.
		@param c2 second color value.
		@param n_ColorFrequency the color frequency.
		@param n_InterpolationFactor the interpolation factor.
		@return The interpolated color array.
		*/
		static Color InterpolateColorScope(int x1, int x2, Color c1, Color c2, Color n_ColorFrequency __gc[], float n_InterpolationFactor) __gc[];



	/**
	Eventhandler for trackbar scroll events.
	@param sender the sender.
	@param e the event.
	*/
	private: System::Void perform_ActionTrackbarScroll(System::Object *  sender, System::EventArgs *  e)
	{
		 if (this->sortedIndicesColor == NULL)
			 sort_ColorIndices();
		 do_Interpolation();
		 pan_Histogram_repaint();
		 pan_RGB_repaint();
	}

	/**
	Eventhandler for button click events.
	@param sender the sender.
	@param e the event.
	*/
	private: System::Void perform_ActionButtonClick(System::Object *  sender, System::EventArgs *  e)
	{
		if (dynamic_cast<Button*>(sender)->Equals( this->but_OK ))
		{
			this->m_TransFuncInput->set_BackColor(this->default_RGBValue);
			this->m_TransFuncInput->set_FunctionLength(DataLoader::DensityScope);
			this->m_TransFuncInput->set_ColorSamples(this->colorSamples);
			this->m_TransFuncInput->set_ColorSamples(
						Transferfunction::convertColorSamplesToSaveMode(this->colorSamples,
																		this->pan_Histogram->get_Size().Width));	
			this->m_TransFuncInput->set_OpacitySamples(
						Transferfunction::convertOpacitySamplesToSaveMode(this->opacitySamples,
																		  this->pan_Histogram->get_Size().Width,
																		  this->pan_Histogram->get_Size().Height));	
																										
			this->m_TransFuncInput->set_InterpolationFactor((float)this->m_InterpolationFactor);
			this->m_TransFuncInput->calculateOpacityFrequency(DataLoader::DensityScope);
			this->m_TransFuncInput->calculateColorFrequency(DataLoader::DensityScope);
			
			if (this->m_RenderingMode == TF_PHONG_DVR)
				this->m_PreviewPanel->Invalidate();

			this->m_SubItem_EditTransferFuntion->set_Checked(false);
			this->m_TbSubItem_EditTransferFuntion->set_Checked(false);

			this->Dispose(true);
			return;
		}

		if (dynamic_cast<Button*>(sender)->Equals( this->but_Cancel ))
		{
			if (this->m_RenderingMode == TF_PHONG_DVR)
				this->m_PreviewPanel->Invalidate();

			this->m_SubItem_EditTransferFuntion->set_Checked(false);
			this->m_TbSubItem_EditTransferFuntion->set_Checked(false);

			this->Dispose(true);
			return;
		}

		if (dynamic_cast<Button*>(sender)->Equals( this->but_Color ))
		{
			this->colDia_ChoseColor->set_Color(this->default_RGBValue);
			this->colDia_ChoseColor->FullOpen = true;
			if (this->colDia_ChoseColor->ShowDialog(this) == DialogResult::OK)
			{
				this->default_RGBValue = colDia_ChoseColor->Color;
				do_Interpolation();

				pan_Histogram_repaint();
				pan_RGB_repaint();
			}
			return;
		}

		if (dynamic_cast<Button*>(sender)->Equals( this->but_InterpolationReset ))
		{
			this->trb_Interpolation->set_Value(this->trb_Interpolation->get_Maximum() / 2);
			this->m_InterpolationFactor = 0.5;
		    do_Interpolation();
			pan_RGB_repaint();
			pan_Histogram_repaint();
			return;
		}

		if (dynamic_cast<Button*>(sender)->Equals( this->but_Opacity ))
		{
			Transferfunction *transFunc = Transferfunction::CreateGeneralTransfer(DataLoader::DensityScope);

			this->opacitySamples = NULL;
			this->opacitySamples = Transferfunction::convertOpacitySamplesToDrawMode(transFunc->get_OpacitySamples(),
																					 this->pan_Histogram->get_Size().Width,
																					 this->pan_Histogram->get_Size().Height);
			sort_OpacityIndices();
			pan_Histogram_repaint();
			return;
		}

		if (dynamic_cast<Button*>(sender)->Equals( this->but_ResetPoints ))
		{
			this->colorSamples->Clear();

			 pan_RGB_repaint();

			 for (int i = 0; i < this->TransFunc_Length; i++)
			 {
				 this->c_TransferFunc[i]   = this->default_RGBValue;    
			 }

			 pan_Histogram_repaint();
			 return;
		}
	}

	/**
	Eventhandler for window closed events.
	@param sender the sender.
	@param e the event.
	*/
	private: System::Void TransferFuncWin_Closed(System::Object *  sender, System::EventArgs *  e)
	{
		this->m_SubItem_EditTransferFuntion->set_Checked(false);
		this->m_TbSubItem_EditTransferFuntion->set_Checked(false);
		Dispose(true);
	}

};
}
