#pragma once

namespace IDVR {
namespace Volume {
namespace DataStructures {

		using namespace System;
		using namespace System::Collections;	
		using namespace System::Drawing;

	/**
	<b>Class for representing a complete transfer funtion.</b>

	This class is part of the DataStructure namespace which is actually a subnamespace of the main namespace Volume. 
	Furthermore the standard class Object is inherted to the %Transferfunction class for better controlling of a particular 
	quantity of %Transferfunction instances.
	<br><br>
	The main target of this class is the storage of the complete transfer function which will be defined by the user on the GUI 
	(see TransferFuncWin). This transfer function is an important attribute for each volume data object and decides the quality 
	of the rendering process. Thus, each volume data object will have its own transfer function which is realized by the implemention
	of an own instance of this class into the IRenderer object of the specific DataObject instance.
	<br><br>
	Because the user defines only some sample points of the transfer function the remaining funtion values has to be interpolated with
	the member functions Transferfunction::calculateOpacityFrequency and Transferfunction::calculateColorFrequency.
	<br><br>
	The following member variables stores all necessary data attributes:
	<br><ul>
	<li>Transferfunction::m_OpacityFunction saves the whole opacity function. Note that for each density value the 
	    corresponding opacity value will be saved in the member array.</li>
	<li>Transferfunction::m_ColorFunction saves the whole color function. Each color value contains three components for red, green anb blue (RGB).
	    Note that for each density value the corresponding color value will be saved in the member array.</li>
	<li>Transferfunction::m_OpacitySamples saves the user-defined sample points of the opacity transfer function. </li>
	<li>Transferfunction::m_ColorSamples saves the user-defined sample points of the color transfer function. </li>
	</ul>
	@see VolumeData - the main datastructure class.
	@see DataObject - the volume data object class.
	@see IRenderer - the abstract render model interface for controlling a specific render model.
	@author <i>Bernhard Pflugfelder</i>
	@version 1.0
	@date 02-18-2005
	*/
public __gc class Transferfunction : public System::Object
	{
	public: 
			/**
			This is the standard constructor.
			All member variables of this new instance will be set to zero/null.
			*/
			Transferfunction(void);
			/**
			Constructor initialization with the maximum number of valid gray (<i>density</i>) values.
			All member variables will be set to zero, the Transferfunction::m_FunctionLength member will be initialized with
			the given maximum number of gray values.
			@param numOfDensityValues the maximum valid number of gray values.
			*/
			Transferfunction(int numOfDensityValues);
			/**
			Constructor initialization with an user-defined transfer function.
			All member variables will be set to the corresponding parameters.
			@param n_OpacityFunction the user-defined opacity transfer funtion at each density value.
			@param n_ColorFunction the user-defined color transfer funtion at each density value.
			@param n_DensityScope the maximum valid number of gray values.
			@param n_OpacitySamples the user-defined opacity transfer function samples as an instance of the Hashtable class.
			@param n_ColorSamples the user-defined color transfer function samples as an instance of the Hashtable class.
			*/
			Transferfunction(float n_OpacityFunction __gc[], float n_ColorFunction __gc[,], unsigned short n_DensityScope, Hashtable *n_ColorSamples, Hashtable *n_OpacitySamples);
			/**
			This is the standard deconstructor.
			Espacially the Histogram::m_OpacityFunction, Histogram::m_ColorFunction, Histogram::m_OpacitySamples and
			Histogram::m_ColorSamples members will be deleted.
			*/
			~Transferfunction(void);

			/**
			Adds a new opacity function value to a specified density value.
			The remaining opacity transfer function will not be changed.
			@param n_Density the choosen density value.
			@param n_Opacity the new opacity value.
			*/
			void addNewOpacityValue(int n_Density,  float n_Opacity); 
			/**
			Adds a new color function value to a specified density value.
			The remaining color transfer function will not be changed.
			@param n_Density the choosen density value.
			@param n_Color the new color value as an 3 element array for RGB components.
			*/
			void addNewColorValue(int n_Density,  float n_Color __gc[]);
			/**
			Returns the opacity function value to the specified density value.
			@param n_Density the choosen density value.
			@return The corresponding opacity value.
			*/
			float getOpacity(int n_Density);
			/**
			Returns the coor function value to the specified density value.
			@param n_Density the choosen density value.
			@return The corresponding color RGB value.
			*/
			float getColor(int n_Density) __gc[];
			/**
			This function interpolates the opacity function for each density value.
			Based upon the sample points of the Transferfunction::m_OpacitySamples member the opacity transfer function
			can be linearly interpolated.
			@param n_DensityScope the maximum valid number of gray values.
			*/
			void calculateOpacityFrequency(unsigned short n_DensityScope);
			/**
			This function interpolates the color function for each density value.
			Based upon the sample points of the Transferfunction::m_ColorSamples member the color transfer function
			can be linearly interpolated.
			@param n_DensityScope the maximum valid number of gray values.
			*/
			void calculateColorFrequency(unsigned short n_DensityScope);

			/**
			Returns the current opacity transfer function.
			@return The current opacity transfer funtion as an one dimensional float array.
			*/
			__property float get_OpacityFrequency(void) __gc[]; 
			/**
			Returns the current color transfer function.
			@return The current color transfer funtion as an three dimensional float array.
			*/
			__property float get_ColorFrequency(void) __gc[,];
			/**
			Returns the current length of the transfer function.
			@return The transfer function length
			@note The transfer function length is actually the maximum density scope.
			*/
			__property int get_FunctionLength(void);
			/**
			Returns the current color sample points.
			@return The current color sample points as an instance of the Hashtable class.
			*/
			__property Hashtable* get_ColorSamples(void);
			/**
			Returns the current opacity sample points.
			@return The current opacity sample points as an instance of the Hashtable class.
			*/
			__property Hashtable* get_OpacitySamples(void);
			/**
			Returns the current interpolation factor.
			@return The interpolation factor.
			*/
			__property float get_InterpolationFactor(void);
			/**
			Returns the current back color of the transfer function.
			@return The back color as an instance of the standard Color class.
			*/
			__property Color get_BackColor(void);
			/**
			Sets a new interpolation factor.
			@param n_InterpolationFactor the new interpolation factor which will be used for further sampling of the transfer function.
			*/
			__property void set_InterpolationFactor(float n_InterpolationFactor);
			/**
			Sets a new color transfer function.
			@param n_ColorFrequency the new color transfer funtion as an three (RGB) dimensional array.
			*/
			__property void set_ColorFrequency(float n_ColorFrequency __gc[,]);
			/**
			Sets a new opacity transfer function.
			@param n_OpacityFrequency the new opacity transfer funtion as an one dimensional array.
			*/
			__property void set_OpacityFrequency(float n_OpacityFrequency __gc[]);
			/**
			Sets a new function length.
			@param n_functionLength the new function length.
			@note The current transfer function will not be deleted. 
			*/
			__property void set_FunctionLength(int n_functionLength);
			/**
			Sets new color sample points.
			@param n_ColorSamples the new color sample points as an instance of the Hashtable class.
			*/
			__property void set_ColorSamples(Hashtable* n_ColorSamples);
			/**
			Sets new opacity sample points.
			@param n_OpacitySamples the new opacity sample points as an instance of the Hashtable class.
			*/
			__property void set_OpacitySamples(Hashtable* n_OpacitySamples);
			/**
			Sets a new back color for the current transfer function.
			@param n_BackColor the new back color as an instance of the standard Color class.
			*/
			__property void set_BackColor(Color n_BackColor);

			/**
			This static function converts the opacity sample point to drawable values.
			This means that all opacity sample point (exactly the coordinates of each sample points) will be converted to the
			original value scope between 0 and maximum height/width of the transfer function panel.
			@param n_OpacitySamples the current opacity sample points.
			@param panelWidth the current width of the transfer function panel.
			@param panelHeight the current height of the transfer function panel.
			@return The converted opacity sample points
			*/
			static Hashtable* convertOpacitySamplesToDrawMode(Hashtable *n_OpacitySamples, int panelWidth, int panelHeight);
			/**
			This static function converts the opacity sample point to normalzed values between 0 and 1.
			This means that all opacity sample point (exactly the coordinates of each sample points) will be converted to the
			value scope between 0 and 1.
			@param n_OpacitySamples the current opacity sample points.
			@param panelWidth the current width of the transfer function panel.
			@param panelHeight the current height of the transfer function panel.
			@return The converted opacity sample points
			*/
			static Hashtable* convertOpacitySamplesToSaveMode(Hashtable *n_OpacitySamples, int panelWidth, int panelHeight);
			/**
			This static function converts the color sample point to drawable values.
			This means that all color sample point (exactly the coordinates of each sample points) will be converted to the
			original value scope between 0 and maximum width of the transfer function panel.
			@param n_ColorSamples the current color sample points.
			@param panelWidth the current width of the transfer function panel.
			@return The converted opacity sample points
			*/
			static Hashtable* convertColorSamplesToDrawMode(Hashtable *n_ColorSamples, int panelWidth);
			/**
			This static function converts the color sample point to normalzed values between 0 and 1.
			This means that all color sample point (exactly the coordinates of each sample points) will be converted to the
			value scope between 0 and 1.
			@param n_ColorSamples the current opacity sample points.
			@param panelWidth the current width of the transfer function panel.
			@return The converted opacity sample points
			*/
			static Hashtable* convertColorSamplesToSaveMode(Hashtable *n_ColorSamples, int panelWidth);
		
			/**
			Calculates the downsampled color transfer function.
			@param sampleFactor the user-defined sampling factor.
			@param colorValues the color transfer function values.
			@return The downsampled color values.
			*/
			static float getDownSampledColors(int sampleFactor, float colorValues __gc[,]) __gc[,];
			/**
			Calculates the downsampled opacity transfer function.
			@param sampleFactor the user-defined sampling factor.
			@param opacityValues the opacity transfer function values.
			@return The downsampled opacity values.
			*/
			static float getDownSampledOpacity(int sampleFactor, float opacityValues __gc[]) __gc[];
			/**
			Calculates the upsampled color transfer function.
			@param n_DensityScope the maximum valid number of gray values.
			@param sampleFactor the user-defined sampling factor.
			@param colorValues the color transfer function values.
			@return The upsampled color values.
			*/
			static float getUpSampledColors(unsigned short n_DensityScope, int sampleFactor, float colorValues __gc[,]) __gc[,];
			/**
			Calculates the upsampled opacity transfer function.
			@param n_DensityScope the maximum valid number of gray values.
			@param sampleFactor the user-defined sampling factor.
			@param opacityValues the opacity transfer function values.
			@return The upsampled color values.
			*/
			static float getUpSampledOpacity(unsigned short n_DensityScope, int sampleFactor, float opacityValues __gc[]) __gc[];

			/**
			Creates a standard transfer function.
			This means an linear opacity transfer function and a contant color transfer function with blue color.
			@param n_DensityScope the maximum valid number of gray values.
			*/
			static Transferfunction* CreateGeneralTransfer(unsigned short n_DensityScope);

	private:
			/**
			This function interpolates the color transfer function linearly.
			@param x1 x coordinate of the left interpolation boundary point.
			@param x2 x coordinate of the right interpolation boundary point.
			@param c1 color value of the left interpolation boundary point.
			@param c2 color value of the right interpolation boundary point.
			*/
			void interpolate_ColorScope(int x1, int x2, Color c1, Color c2);

	private: float m_OpacityFunction __gc[]; /**< Saves the opacity transfer function for each density value. */
			 float m_ColorFunction __gc[,];  /**< Saves the color transfer function for each density value. */
			 int m_FunctionLength; /**< Saves the transfer function length. */
			 Hashtable *m_ColorSamples;		/**< Saves all user-defined color sample points with its actual x 
												 coordinate as hash key and the color value as hash value */
			 Hashtable *m_OpacitySamples;	/**< Saves all user-defined opacity sample points with its actual x 
												 coordinate as hash key and the opacity value as hash value */
			 float m_InterpolationFactor;  /**< Saves the interpolation factor which gives information over the linearity of the
										   interpolation: 
										   <br><ul>
										   <li>0.5 means absolut linearity</li>
										   <li>0 means use only the left interpolation color</li>
										   <li>1 means use only the right interpolation color</li>
										   <li>all other values between 0 and 1 will be use as interpolation weight for
										   the left and right interpolation color.</li>
										   </ul> */
										   
			 Color m_BackColor; /**< Saves tje background color. */

			 /** Constant index values for the three color components red, green and blue. */
			 __value enum ColorIndex {RED= 0,GREEN,BLUE};

	};
}
}
}