#pragma once

using namespace System;

namespace IDVR {
namespace Tools {

	/**
	<B>Class for a 3 dimensional vector.</B>

	This class is part of the basic Tools namespace and contains a couple of methods for initialization and computing 
	various 3-element vectors (3 dimensional vectors). Espacially the inner (see Vector3::DotProd) and outer 
	(see Vector3::CrossProd) product methods.  

	For using vector with more or less elements (e.g. 2 or 4 elements) please see the following classes
	Vector2 and Vector4.

	@see Vector2 the two dimensional vector.
	@see Vector4 the four dimensional vector.
	@author <i>Bernhard Pflugfelder</i>
	@version 1.0
	@date 02-17-2005
	*/
public __gc class Vector3 : public System::Object
{
public:

	double m_X; /**< The x element of the vector.  The x element is the first element and is based on a double variable. */
	double m_Y; /**< The y element of the vector.  The y element is the second element and is based on a double variable. */
	double m_Z; /**< The z element of the vector.  The z element is the third element and is based on a double variable. */

public:
	/**
	This is the standard constructor.
	This standard constructor initialzed all three elements with zero.
	*/
	Vector3(void);
	/**
	Constructor initialization by three parameters.
	This constructor initialized all three element of the vector with the specified parameters.
	@param n_X the value for the x element.
	@param n_Y the value for the y element.
	@param n_Z the value for the z element.
	*/
	Vector3(double n_X, double n_Y, double n_Z);
	/**
	Constructor initialization by an array parameters.
	This constructor initialized all three element of the vector with the specified array parameter.
	@param n_Value a three-element array for all three elements.
	*/
	Vector3(double n_Value __gc[]);
	/**
	Constructor initialization by an already existing %Vector3 object.
	This constructor initialized all three element of the vector with the values of the specified %Vector3 object.
	@param n_Vector3 the object containing thespecific element values.
	*/
	Vector3(Vector3 *n_Vector3);

	/**
	Returns the x element of the specific vector.
	@return The x element of the vector.
	*/
	__property double get_X(void);
	/**
	Returns the y element of the specific vector.
	@return The y element of the vector.
	*/
	__property double get_Y(void);
	/**
	Returns the z element of the specific vector.
	@return The z element of the vector.
	*/
	__property double get_Z(void);
	/**
	Sets the x element of the existing vector.
	@param n_X the x element of the vector.
	*/
	__property void set_X(double n_X);
	/**
	Sets the y element of the existing vector.
	@param n_Y the y element of the vector.
	*/
	__property void set_Y(double n_Y);
	/**
	Sets the z element of the existing vector.
	@param n_Z the z element of the vector.
	*/
	__property void set_Z(double n_Z);
	
	/**
	This function returns the dot product (inner product) of the current %Vector3 object and the specified vector parameter.
	@param v the second vector of the the dot product.
	@return The the dot product as a bouble value.
	@see Vector3::DotProd the static method for the dot product.
	*/
	double dotProd(Vector3* v);
	/**
	This function returns the skalar product as a new %Vector3 object of the current %Vector3 object and the specified skalar parameter.
	@param d the skalar value to multiply the vector.
	@return The the skalar product as a new %Vector3 object.
	*/
	Vector3* multSkalar(double d);
	/**
	Returns the magnitude of the specific vector.
	@return The magnitude of the vector.
	*/
	double magnitude(void);
	/**
	This function normalizes the current vector.
	@see Vector3::CreateNormalizedVector the static method for vector normalization. 
	*/
	void normalize(void);
	/**
	This function returns the specific element of the current vector. This element is specified by the specified parameter whereas
	the following index enumeration is valid:
	<br>
	<ul>
		<li>0 means the x elements</li>
		<li>1 means the y elements</li>
		<li>2 means the z elements</li>
	</ul>
	@param index the index for the specific element.
	@return The choosen vector element.
	*/
	double getItem(int index);
	/**
	This function returns the negative vector of the current %Vector3 object. This means that the current vector is multiplied 
	with -1 which is actually a skalar product.
	@return The negative vector as a new %Vector3 object.
	*/
	Vector3* getNegativeVector(void);
	/**
	This function returns the all vector elements as an three element float array.
	@return The three element float array containing all vector elements.
	@see Vector3::getDoubleArrayOfVector getting a double array containing all vector elements.
	*/
	float getFloatArrayOfVector(void) __gc[];
	/**
	This function returns the all vector elements as an three element double array.
	@return The three element double array containing all vector elements.
	@see Vector3::getFloatArrayOfVector getting a float array containing all vector elements.
	*/
	double getDoubleArrayOfVector(void) __gc[];
	/**
	This function converts the current vector to an OpenGL compatible vector form.
	For correct drawing in OpenGL the x and y elements of an existing %Vector3 object must
	be inverted, only after this invertion a correct drawing in OpenGL is guaranteed (is basically a
	reflextion about the y and z axis).
	@return The inverted vector as a new %Vector3 object.
	*/
	Vector3* convertToOpenGLAlignedVector(void);

	/**
	This static function created a normalized vector of the specified vector parameter.
	@param v the unnormalized vector.
	@return The normalized vector as a new instance.
	@see Vector3::normalize the non-static method for vector normalization. 
	*/
	static Vector3* CreateNormalizedVector(Vector3* v);
	/**
	This static function returns the sum vector of the two specified vector parameters.
	@param v1 the first vector for addition.
	@param v2 the second vector for addition.
	@return The sum vectoras a new instance.
	*/
	static Vector3* Addition(Vector3* v1, Vector3* v2);
	/**
	This static function returns the subtraction vector of the two specified vector parameters.
	@param v1 the first vector for subtraction.
	@param v2 the second vector for subtraction.
	@return The subtraction vector as a new instance.
	*/
	static Vector3* Subtraction(Vector3* v1, Vector3* v2);
	/**
	This static function returns the cross product (outer product) vector of the two specified vector parameters.
	@param v1 the first vector of the cross product.
	@param v2 the second vector of the cross product.
	@return The cross product vector as a new instance.
	*/
	static Vector3* CrossProd(Vector3* v1, Vector3* v2);
	/**
	This static function returns the dot product (inner product) of the two specified vector parameters.
	@param v1 the first vector of the the dot product.
	@param v2 the second vector of the the dot product.
	@return The the dot product as a double value.
	@see Vector3::dotProd the non-static method for the dot product.
	*/
	static double DotProd(Vector3* v1, Vector3* v2);
};
	
}
}