#pragma once

#include "Vector3.h"

using namespace System;

namespace IDVR {
namespace Tools {

	/**
	<B>Class for a 4 dimensional vector.</B>

	This class is part of the basic Tools namespace and contains a couple of methods for initialization and computing 
	various 4-element vectors (4 dimensional vectors). Espacially the inner (see Vector4::DotProd) and outer 
	(see Vector4::CrossProd) product methods.  

	For using vector with more or less elements (e.g. 2 or 3 elements) please see the following classes
	Vector2 and Vector3.

	@see Vector2 the two dimensional vector.
	@see Vector3 the four dimensional vector.
	@author <i>Bernhard Pflugfelder</i>
	@version 1.0
	@date 02-17-2005
	*/
public __gc class Vector4
{
private:

	double m_X; /**< The x element of the vector.  The x element is the first element and is based on a double variable. */
	double m_Y; /**< The y element of the vector.  The y element is the second element and is based on a double variable. */
	double m_Z; /**< The z element of the vector.  The z element is the third element and is based on a double variable. */
	double m_W; /**< The w element of the vector.  The w element is the fourth element and is based on a double variable. */

public:

	/**
	This is the standard constructor.
	This standard constructor initialized all four elements with zero.
	*/
	Vector4(void);
	/**
	Constructor initialization by three parameters.
	This constructor initialized all four element of the vector with the specified parameters.
	@param n_X the value for the x element.
	@param n_Y the value for the y element.
	@param n_Z the value for the z element.
	@param n_W the value for the w element.
	*/
	Vector4(double n_X, double n_Y, double n_Z, double n_W);
	/**
	Constructor initialization by an array parameters.
	This constructor initialized all four element of the vector with the specified array parameter.
	@param n_Value a four-element array for all three elements.
	*/
	Vector4(double n_Value __gc[]);
	/**
	Constructor initialization by an already existing %Vector4 object.
	This constructor initialized all three element of the vector with the values of the specified %Vector4 object.
	@param n_Vector3 the object containing thespecific element values.
	@param n_W the fourth element of the vector.
	*/
	Vector4(Vector3 *n_Vector3, double n_W);

	/**
	Returns the x element of the specific vector.
	@return The x element of the vector.
	*/
	__property double get_X(void);
	/**
	Returns the y element of the specific vector.
	@return The y element of the vector.
	*/
	__property double get_Y(void);
	/**
	Returns the z element of the specific vector.
	@return The z element of the vector.
	*/
	__property double get_Z(void);
	/**
	Returns the w element of the specific vector.
	@return The w element of the vector.
	*/
	__property double get_W(void);
	/**
	Sets the x element of the existing vector.
	@param n_X the x element of the vector.
	*/
	__property void set_X(double n_X);
	/**
	Sets the y element of the existing vector.
	@param n_Y the y element of the vector.
	*/
	__property void set_Y(double n_Y);
	/**
	Sets the z element of the existing vector.
	@param n_Z the z element of the vector.
	*/
	__property void set_Z(double n_Z);
	/**
	Sets the w element of the existing vector.
	@param n_W the w element of the vector.
	*/
	__property void set_W(double n_W);

	/**
	This function returns the dot product (inner product) of the current %Vector4 object and the specified vector parameter.
	@param v the second vector of the the dot product.
	@return The the dot product as a bouble value.
	@see Vector4::DotProd the static method for the dot product.
	*/
	double dotProd(Vector4 *v);
	/**
	This function returns the skalar product as a new %Vector4 object of the current %Vector4 object and the specified skalar parameter.
	@param d the skalar value to multiply the vector.
	@return The the skalar product as a new %Vector4 object.
	*/
	Vector4* multSkalar(double d);
	/**
	Returns the magnitude of the specific vector.
	@return The magnitude of the vector.
	*/
	double magnitude(void);
	/**
	This function normalizes the current vector.
	@see Vector3::CreateNormalizedVector the static method for vector normalization. 
	*/
	void normalize(void);
	/**
	This function returns the specific element of the current vector. This element is specified by the specified parameter whereas
	the following index enumeration is valid:
	<br>
	<ul>
		<li>0 means the x elements</li>
		<li>1 means the y elements</li>
		<li>2 means the z elements</li>
		<li>3 means the w elements</li>
	</ul>
	@param index the index for the specific element.
	@return The choosen vector element.
	*/
	double getItem(int index);
	/**
	This function returns the negative vector of the current %Vector4 object. This means that the current vector is multiplied 
	with -1 which is actually a skalar product.
	@return The negative vector as a new %Vector4 object.
	*/
	Vector4* getNegativeVector(void);
	/**
	This function returns the all vector elements as an three element float array.
	@return The three element float array containing all vector elements.
	@see Vector4::getDoubleArrayOfVector getting a double array containing all vector elements.
	*/
	float getFloatArrayOfVector(void) __gc[];
	/**
	This function returns the all vector elements as an three element double array.
	@return The three element double array containing all vector elements.
	@see Vector4::getFloatArrayOfVector getting a float array containing all vector elements.
	*/
	double getDoubleArrayOfVector(void) __gc[];
	/**
	This function returns a new instance of the Vector3 class which contains the firest three elements of the
	current %Vector4 object divided by the w element. This means that the x,y and z element will be divided by the w element 
	and afterwards copied to the new %Vector3 instance.
	@return The new %Vector3 instance containing the first thrre elements.
	@see Vector4::GetVector3 the static method for extacting a %Vector3 instance
	*/
	Vector3* getVector3(void);
	
	/**
	This static function created a normalized vector of the specified vector parameter.
	@param v the unnormalized vector.
	@return The normalized vector as a new instance.
	@see Vector4::normalize the non-static method for vector normalization. 
	*/
	static Vector4* CreateNormalizedVector(Vector4 *v);
	/**
	This static function returns the sum vector of the two specified vector parameters.
	@param v1 the first vector for addition.
	@param v2 the second vector for addition.
	@return The sum vectoras a new instance.
	*/
	static Vector4* Addition(Vector4 *v1, Vector4 *v2);
	/**
	This static function returns the subtraction vector of the two specified vector parameters.
	@param v1 the first vector for subtraction.
	@param v2 the second vector for subtraction.
	@return The subtraction vector as a new instance.
	*/
	static Vector4* Subtraction(Vector4 *v1, Vector4 *v2);
	/**
	This static function returns the cross product (outer product) vector of the two specified vector parameters.
	@param v1 the first vector of the cross product.
	@param v2 the second vector of the cross product.
	@return The cross product vector as a new instance.
	*/
	static Vector4* CrossProd(Vector4 *v1, Vector4 *v2);
	/**
	This static function returns the dot product (inner product) of the two specified vector parameters.
	@param v1 the first vector of the the dot product.
	@param v2 the second vector of the the dot product.
	@return The the dot product as a double value.
	@see Vector3::dotProd the non-static method for the dot product.
	*/
	static double DotProd(Vector4 *v1, Vector4 *v2);
	/**
	This static function returns a new instance of the Vector3 class which contains the firest three elements of the
	current %Vector4 object divided by the w element. This means that the x,y and z element will be divided by the w element 
	and afterwards copied to the new %Vector3 instance.
	@return The new %Vector3 instance containing the first thrre elements.
	@see Vector4::getVector3 the non-static method for extacting a %Vector3 instance
	*/
	static Vector3* GetVector3(Vector4 *n_Vector4);
	/**
	This static function initalizied a new %Vector4 object by the specified Vector3 instance.
	Hereby should be noted that the fourth element will be invariably set to zero.
	@param n_Vector3 the %Vector3 instance containing the initialization values.
	*/
	static Vector4* GetVector4OfVector3(Vector3 *n_Vector3);
};
}
}
	
