#pragma once

#include "VoxelData.h"
#include "DataObject.h"
#include "Histogram.h"
#include "Transferfunction.h"
#include "IRenderer.h"
#include "IIDVRCompositingModel.h"
#include "IDVRMImPCompositingModelSW.h"
#include "IDVRColAndOpModulatorSW.h"

namespace IDVR {
namespace Volume {
namespace DataStructures {

using namespace System;
using namespace System::Collections;
using namespace IDVR::Interfaces;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer::IDVRCompositingModels;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer::IDVRModulationModels;

	/**
	<b>Class for controlling the whole volume data set containing all needed attributes.</b>

	This class is part of the DataStructure namespace which is actually a subnamespace of the main namespace Volume. 
	Furthermore the standard class Object is inherted to the %VolumeData class for better controlling of a particular 
	quantity of %VolumeData instances.
	<br><br>
	With an instance of this class it is possible to represent the whole volume data set with all its components and attributes.
	Furthermore, a set of different member function builds up an interfave with which the access and the controll of the current
	load volume data will be easily managed.
	<br><br>
	The following main components are essential for the correct representation of the colume data and, thus, is part of this class:
	<br><ul>
	<li>VolumeData::m_VoxelData: this instance of the class VoxelData actually stores the loaded voxel data.</li>
	<li>VolumeData::m_ObjectList: this instance of the standard class Hashtable actually stores all defined volume data objects 
	    which are instances of the class DataObject.</li>
	</ul>
	@see DataObject - the volume data object class.
	@author <i>Bernhard Pflugfelder</i>
	@version 1.0
	@date 02-18-2005
	*/
public __gc class VolumeData : public System::Object 
{
	
private: 

	unsigned short m_DensityScope; /**< Saves the maximum valid density scope (e.g. 4096) */
	unsigned short m_NextFreeObjectKey;	/**< Stores the next number which is not been assigned to a specific data object. */
	bool m_GradientsPreCalculated;	/**< Gives information about whether the gradient values are already calcualeted or not. */
	VoxelData *m_VoxelData; /**< Saves the actual voxel data. */
	Hashtable *m_ObjectList;  /**< Saves the actual volume data objects whereas the hash key is an unique number and the hash value is
							   an instance of the DataObject class. */
	IIDVRCompositingModel *m_IIDVRCompositingModel; /**< Saves the calculated data for IDVR compositing. */

	/** Constant index values for the three components of the three dimensional volume data set. */
	__value enum Indices{X = 0, Y, Z};

public:

	/**
	This is the standard constructor.
	All member variables of this new instance will be set to zero/null.
	*/
	VolumeData(void);
	/**
	Constructor initialization with the valid volume data dimension.
	All member variables will be set to zero.
	@param n_VolumeDimension the valid size of the volume data set.
	@param n_DensityScope the maximum valid number of gray values.
	*/
	VolumeData(unsigned short n_VolumeDimension __gc[], unsigned short n_DensityScope);
	/**
	This is the standard deconstructor.
	All member variables will be deleted.
	*/
	~VolumeData(void);

	/** 
	Returns the current number of objects.
	@return The current number of stored data objects.
	*/
	__property unsigned short get_ObjectCount(void);
	/** 
	Returns the current volume dimension.
	@return The current volume dimension.
	*/
	__property unsigned short get_VolumeDimension(void) __gc[];
	/** 
	Returns the overall count of voxel for the current dataset.
	@return The voxel count of the current loaded data set.
	*/
	__property unsigned int get_VoxelCountOverall(void);
	/** 
	Returns the current voxel data set.
	@return The voxel data set as an instance of the Voxeldata class.
	*/
	__property VoxelData* get_VoxelDataset(void);
	/** 
	Returns the flag value whether all gradient values are already calculated and saved or not.
	@return The actual flag as a bool value.
	*/
	__property bool get_GradientsPreCalculated(void);
	/** 
	Sets the flag value whether all gradient values are already calculated and saved or not.
	@param n_GradientsPreCalculated the choosen flag as a bool value.
	*/
	__property void set_GradientsPreCalculated(bool n_GradientsPreCalculated);

	/**
	Returns whether volume data objects are stored in the current data structure.
	@return True, if volume data contains at least one object.
	*/
	bool haveObjects(void);
	/**
	Creates the general volume data object (always with objectID = 0).
	This general data object represents the entire voume data. Thus, this data object must be always generated
	for correct representation of the data set.
	@param n_ObjectDescription the description of the general volume data object.
	@param n_Importance the importance value of the general volume data object.
	@param n_Renderer the render model of the general volume data object.
	*/
	void addGeneralObject(String *n_ObjectDescription, unsigned short n_Importance, IRenderer* n_Renderer);
	/**
	Adds a new  volume data object object to the current data structure.
	@param n_ObjectDescription the description of the new volume data object.
	@param n_Importance the importance value of the general new data object.
	@param n_Renderer the render model of the general new data object.
	*/
	unsigned short addObject(String *n_ObjectDescription, unsigned short n_Importance, IRenderer* n_Renderer);
	/**
	Removes the specified volume data object of the current volume data set.
	@param n_ObjectID the uniquely assigned number which is actually the ID for the choosen volume data object.
	*/
	void removeObject(unsigned short n_ObjectID);
	/**
	Sets the specified voxel element with the choosen density value and object ID.
	@param n_VoxelCoordinates the uniquely address of the actual voxel.
	@param n_Density the new density value.
	@param n_ObjectID the new object ID which links the voxel to the associated volume data object.
	@exception An instance of the Exception class will be thrown if the voxel coordinates is not valid.
	*/
	void addVoxel(unsigned short n_VoxelCoordinates __gc[], unsigned short n_Density, unsigned short n_ObjectID);		
	/**
	Sets the specified voxel element with the choosen density value.
	The associated volume data object is the current general data object (ID = 0).
	@param n_VoxelCoordinates the uniquely address of the actual voxel.
	@param n_Density the new density value.
	@exception An instance of the Exception class will be thrown if the voxel coordinates is not valid.
	*/
	void addVoxel(unsigned short n_VoxelCoordinates __gc[], unsigned short n_Density);
	/**
	Returns the data object width the specified objectID.
	@param n_ObjectID the new object ID which links the voxel to the associated volume data object.
	@return The choosen volume data object.
	*/
	DataObject* getObjectMember(unsigned short n_ObjectID);	
	/**
	Returns all used object IDs of the current data set.
	@return An instance of the standard class Array containing all valid object IDs.
	*/
	Array* getObjectIDs(void);
	/**
	Returns the data object to which the specified voxel belongs.
	@param n_VoxelCoordinates the uniquely address of the actual voxel.
	@return The choosen volume data object.
	*/
	DataObject* getObjectMember(unsigned short n_VoxelCoordinates __gc[]);
	/**
	Returns the general data object (objectID = 0).
	@return The general volume data object.
	*/
	DataObject* getGeneralDataObject(void);
	/**
	Returns the overall numer of voxels of the current volume data set.
	@return The voxel count of the current volume data set.
	*/
	int getMaxVoxelCount(void);
	/**
	Returns the slice of the volume data set with the constant specified x-value.
	@param n_SliceX the x-value which defines the actual data slice.
	@return An two dimensional array containing all density values of the actual slice.
	*/
	unsigned short getVoxelSliceX(unsigned short n_SliceX) __gc[,];
	/**
	Returns the slice of the volume data set with the constant specified y-value.
	@param n_SliceY the y-value which defines the actual data slice.
	@return An two dimensional array containing all density values of the actual slice.
	*/
	unsigned short getVoxelSliceY(unsigned short n_SliceY) __gc[,];
	/**
	Returns the slice of the volume data set with the constant specified z-value.
	@param n_SliceZ the z-value which defines the actual data slice.
	@return An two dimensional array containing all density values of the actual slice.
	*/
	unsigned short getVoxelSliceZ(unsigned short n_SliceZ) __gc[,];
	/**
	Sets a new IIDVRCompositingModel.
	@param n_IIDVRCompositingModel an instance of the class IIDVRCompositingModel.
	*/
	void setIIDVRCompositingModel(IIDVRCompositingModel *n_IIDVRCompositingModel);
	/**
	Returns the IIDVRCompositingModel of the current volume data set.
	@return An instance of the class IIDVRCompositingModel.
	*/
	IIDVRCompositingModel* getIIDVRCompositingModel(void);
	/**
	Creates the general IIDVRCompositingModel (MImP with ColAndOp).
	*/
	void createGeneralIDVRObjects(void);
	/**
	Resets the current volume data set to default status. 
	That means all objectIDs will be set to GENERALOBJECT and all stored volume data objects will be removed.
	*/
	void resetDataset(void);

	/**
	Returns the importance value of the choosen object ID.
	@param n_DataItem a uniquely number which is linked to an existing volume data object.
	@exception An instance of the Exception class will be thrown if the object ID is not valid.
	*/
	unsigned short getVoxelImportance(unsigned short n_DataItem);
	/**
	Returns the importance value of the choosen voxel coordinates.
	@param n_VoxelCoordinates the uniquely address of the actual voxel.
	@exception An instance of the Exception class will be thrown if the voxel coordinates is not valid.
	*/
	unsigned short getVoxelImportance(unsigned short n_VoxelCoordinates __gc[]);
	/**
	Returns the importance value of the choosen voxel coordinates.
	@param n_Column the x element of the voxel coordinates.
	@param n_Row the y element of the voxel coordinates.
	@param n_Depth the z element of the voxel coordinates.
	@exception An instance of the Exception class will be thrown if the voxel coordinates is not valid.
	*/
	unsigned short getVoxelImportance(unsigned short n_Column, unsigned short n_Row, unsigned short n_Depth);

	static const unsigned short GENERALOBJECT = 0, NO_OBJECT = 0;
};
}
}
}