#include "stdafx.h"
#include "VoxelData.h"

using namespace System;
using namespace System::Collections;
using namespace IDVR::Volume::DataStructures;
 
/*
This is the standard constructor.
All member variables of this new instance will be set to zero/null.
*/
VoxelData::VoxelData(void) 
{
	this->m_Dataset = this->m_VolumeDimension = NULL;
	this->m_GradientValues = NULL;
	this->m_VoxelCountOfSlice = 0;
}

/*
Constructor initialization with the valid volume data dimension.
All member variables will be set to zero and the VoxelData::m_Dataset array will be initialized with the
specified volume dimension.
*/
VoxelData::VoxelData(unsigned short n_VolumeDimension __gc[])
{
	this->m_VolumeDimension = new unsigned short __gc[3];
	this->m_VolumeDimension[X] = n_VolumeDimension[X];
	this->m_VolumeDimension[Y] = n_VolumeDimension[Y];
	this->m_VolumeDimension[Z] = n_VolumeDimension[Z];

	this->m_VoxelCountOfSlice = this->m_VolumeDimension[X] * this->m_VolumeDimension[Y];
	this->m_Dataset = new unsigned short __gc[this->m_VolumeDimension[Z] * this->m_VoxelCountOfSlice];
	this->m_GradientValues = new float __gc[this->m_Dataset->Length * 3];	
}

/*
This is the standard deconstructor.
All member variables will be deleted.
*/
void VoxelData::~VoxelData(void)
{
	this->m_VolumeDimension = NULL;
	this->m_Dataset = NULL;
	this->m_GradientValues = NULL;
}

/**********************************************************************************************************************/
/**********************************************************************************************************************/

//Returns the current volume dimension of the data set.
unsigned short VoxelData::get_VolumeDimension(void) __gc[]
{
	return this->m_VolumeDimension;
}

/*
Returns the voxel count per data slice of the current data set.
For each voxel of the data slice the z coordinate is constant.
*/
unsigned int VoxelData::get_VoxelCountOfSlice(void)
{
	return this->m_VoxelCountOfSlice;
}

//Returns the total count of voxel for current dataset.
unsigned int VoxelData::get_VoxelCountOverall(void)
{
	return this->m_VolumeDimension[X] * this->m_VolumeDimension[Y] * this->m_VolumeDimension[Z];
}

//Sets the basic voxel data to a voxel specified by uniquely voxel coordinates.
void VoxelData::setVoxelValues(unsigned short n_VoxelCoordinates __gc[], unsigned short n_ObjectDensity, unsigned short n_ObjectID)
{
	int index = (n_VoxelCoordinates[Z] * this->m_VoxelCountOfSlice) +  (n_VoxelCoordinates[Y] * this->m_VolumeDimension[X]) + n_VoxelCoordinates[X];
	this->m_Dataset[index] = ((n_ObjectID & 0x000F) << 12) + (n_ObjectDensity & 0x0FFF);

}

//Sets the basic voxel data to a voxel specified by uniquely voxel coordinates.
void VoxelData::setVoxelValues(unsigned short n_Column, unsigned short n_Row, unsigned short n_Depth, unsigned short n_ObjectDensity, unsigned short n_ObjectID)
{
	int index = (n_Depth * this->m_VoxelCountOfSlice) +  (n_Row * this->m_VolumeDimension[X]) + n_Column;
	this->m_Dataset[index] = ((n_ObjectID & 0x000F) << 12) + (n_ObjectDensity & 0x0FFF);

}

//Sets only a new object ID value to a voxel specified by uniquely voxel coordinates.
void VoxelData::setVoxelObjectID(unsigned short n_VoxelCoordinates __gc[], unsigned short n_ObjectID)
{
	int index = (n_VoxelCoordinates[Z] * this->m_VoxelCountOfSlice) +  (n_VoxelCoordinates[Y] * this->m_VolumeDimension[X]) + n_VoxelCoordinates[X];	
	this->m_Dataset[index] = ((n_ObjectID & 0x000F) << 12) + (this->m_Dataset[index] & 0x0FFF);
}

/*
Sets only a new object ID value to a voxel specified by uniquely voxel coordinates.
The remaining voxel data will not be changed.
*/
void VoxelData::setVoxelObjectID(unsigned short n_Column, unsigned short n_Row, unsigned short n_Depth, unsigned short n_ObjectID)
{
	int index = (n_Depth * this->m_VoxelCountOfSlice) +  (n_Row * this->m_VolumeDimension[X]) + n_Column;
	this->m_Dataset[index] = ((n_ObjectID & 0x000F) << 12) + (this->m_Dataset[index] & 0x0FFF);
}

/*
Sets only a new density value to a voxel specified by uniquely voxel coordinates.
The remaining voxel data will not be changed.
*/
void VoxelData::setVoxelDensity(unsigned short n_VoxelCoordinates __gc[], unsigned short n_Density)
{
	int index = (n_VoxelCoordinates[Z] * this->m_VoxelCountOfSlice) +  (n_VoxelCoordinates[Y] * this->m_VolumeDimension[X]) + n_VoxelCoordinates[X];	
	this->m_Dataset[index] = (n_Density & 0x0FFF) + (this->m_Dataset[index] & 0xF000);
}

/*
Sets only a new density value to a voxel specified by uniquely voxel coordinates.
The remaining voxel data will not be changed.
*/
void VoxelData::setVoxelDensity(unsigned short n_Column, unsigned short n_Row, unsigned short n_Depth, unsigned short n_Density)
{
	int index = (n_Depth * this->m_VoxelCountOfSlice) +  (n_Row * this->m_VolumeDimension[X]) + n_Column;
	this->m_Dataset[index] = (n_Density & 0x0FFF) + (this->m_Dataset[index] & 0xF000);
}

/*
Sets a new gradient vector to the corresponding voxel specified by uniquely voxel coordinates.
The remaining voxel data will not be changed. A valid gradient vector consists of three elements for the x-, y-
and z-component (thus, a three dimensional vector structure).
*/
void VoxelData::setVoxelGradient(unsigned short n_VoxelCoordinates __gc[], float n_GradientValues __gc[])
{
	//calculate current array postion for gradient values
	//remember the gradient array structure: x_0,y_0,z_0,x_1,Y_2,z_2,....,x_n-1,y_n-1,z_n-1 
	int index = 3 * ((n_VoxelCoordinates[Z] * this->m_VoxelCountOfSlice) +  (n_VoxelCoordinates[Y] * this->m_VolumeDimension[X]) + n_VoxelCoordinates[X]);	
	this->m_GradientValues[index + X] = n_GradientValues[X];
	this->m_GradientValues[index + Y] = n_GradientValues[Y];
	this->m_GradientValues[index + Z] = n_GradientValues[Z];
}

/*
Sets a new gradient vector to the corresponding voxel specified by uniquely voxel coordinates.
The remaining voxel data will not be changed. A valid gradient vector consists of three elements for the x-, y-
and z-component (thus, a three dimensional vector structure).
*/
void VoxelData::setVoxelGradient(unsigned short n_Column, unsigned short n_Row, unsigned short n_Depth, float n_GradientValues __gc[])
{
	//calculate current array postion for gradient values
	//remember the gradient array structure: x_0,y_0,z_0,x_1,Y_2,z_2,....,x_n-1,y_n-1,z_n-1 
	int index = 3 * ((n_Depth * this->m_VoxelCountOfSlice) +  (n_Row * this->m_VolumeDimension[X]) + n_Column);
	this->m_GradientValues[index + X] = n_GradientValues[X];
	this->m_GradientValues[index + Y] = n_GradientValues[Y];
	this->m_GradientValues[index + Z] = n_GradientValues[Z];
}

//Returns the density value the given voxel data variable.
unsigned short VoxelData::getVoxelDensity(unsigned short n_VoxelData)
{
	return (n_VoxelData & 0x0FFF);
}

//Returns the density value to the specified voxel.
unsigned short VoxelData::getVoxelDensity(unsigned short n_VoxelCoordinates __gc[])
{
	if ( (n_VoxelCoordinates[X] < 0 || n_VoxelCoordinates[X] >= this->m_VolumeDimension[X]) || 
		 (n_VoxelCoordinates[Y] < 0 || n_VoxelCoordinates[Y] >= this->m_VolumeDimension[Y]) ||
		 (n_VoxelCoordinates[Z] < 0 || n_VoxelCoordinates[Z] >= this->m_VolumeDimension[Z]) )
			return 0;

	int index = (n_VoxelCoordinates[Z] * this->m_VoxelCountOfSlice) +  (n_VoxelCoordinates[Y] * this->m_VolumeDimension[X]) + n_VoxelCoordinates[X];	
	return getVoxelDensity(this->m_Dataset[index]);
}

//Returns the density value to the specified voxel.
unsigned short VoxelData::getVoxelDensity(unsigned short n_Column, unsigned short n_Row, unsigned short n_Depth)
{
	if ( (n_Column < 0 || n_Column >= this->m_VolumeDimension[X]) ||
		 (n_Row    < 0 || n_Row    >= this->m_VolumeDimension[Y]) ||
		 (n_Depth  < 0 || n_Depth  >= this->m_VolumeDimension[Z]) )
			return 0;

	int index = (n_Depth * this->m_VoxelCountOfSlice) +  (n_Row * this->m_VolumeDimension[X]) + n_Column;
	return getVoxelDensity(this->m_Dataset[index]);
}

//Returns the object ID key value the given voxel data variable.
unsigned short VoxelData::getVoxelObjectID(unsigned short n_VoxelData)
{
	return ((n_VoxelData & 0xF000) >> 12);
}

//Returns the object ID key to the specified voxel.
unsigned short VoxelData::getVoxelObjectID(unsigned short n_VoxelCoordinates __gc[])
{
	if ( (n_VoxelCoordinates[X] < 0 || n_VoxelCoordinates[X] >= this->m_VolumeDimension[X]) || 
		 (n_VoxelCoordinates[Y] < 0 || n_VoxelCoordinates[Y] >= this->m_VolumeDimension[Y]) ||
		 (n_VoxelCoordinates[Z] < 0 || n_VoxelCoordinates[Z] >= this->m_VolumeDimension[Z]) )
			return 0;

	int index = (n_VoxelCoordinates[Z] * this->m_VoxelCountOfSlice) +  (n_VoxelCoordinates[Y] * this->m_VolumeDimension[X]) + n_VoxelCoordinates[X];	
	return getVoxelObjectID(this->m_Dataset[index]);
}

//Returns the object ID key to the specified voxel.
unsigned short VoxelData::getVoxelObjectID(unsigned short n_Column, unsigned short n_Row, unsigned short n_Depth)
{
	if ( (n_Column < 0 || n_Column >= this->m_VolumeDimension[X]) ||
		 (n_Row    < 0 || n_Row    >= this->m_VolumeDimension[Y]) ||
		 (n_Depth  < 0 || n_Depth  >= this->m_VolumeDimension[Z]) )
		 return 0;

	int index = (n_Depth * this->m_VoxelCountOfSlice) +  (n_Row * this->m_VolumeDimension[X]) + n_Column;	
	return getVoxelObjectID(this->m_Dataset[index]);
}

//Returns the gradient vector elements to the corresponding voxel coordinates.
float VoxelData::getVoxelGradient(unsigned short n_VoxelCoordinates __gc[]) __gc[]
{
	if ( (n_VoxelCoordinates[X] < 0 || n_VoxelCoordinates[X] >= this->m_VolumeDimension[X]) || 
		 (n_VoxelCoordinates[Y] < 0 || n_VoxelCoordinates[Y] >= this->m_VolumeDimension[Y]) ||
		 (n_VoxelCoordinates[Z] < 0 || n_VoxelCoordinates[Z] >= this->m_VolumeDimension[Z]) )
	{
		float tempValue __gc[] = {0,0,0};
		return tempValue;
	}


	//calculate current array postion for gradient values
	//remember the gradient array structure: x_0,y_0,z_0,x_1,Y_2,z_2,....,x_n-1,y_n-1,z_n-1 
	int index = 3 * ((n_VoxelCoordinates[Z] * this->m_VoxelCountOfSlice) +  (n_VoxelCoordinates[Y] * this->m_VolumeDimension[X]) + n_VoxelCoordinates[X]);	
	float tempValue __gc[] = {this->m_GradientValues[index + X], this->m_GradientValues[index + Y], this->m_GradientValues[index + Z]};

	return tempValue;
}

//Returns the gradient vector elements to the corresponding voxel coordinates.
float VoxelData::getVoxelGradient(unsigned short n_Column, unsigned short n_Row, unsigned short n_Depth) __gc[]
{
	if ( (n_Column < 0 || n_Column >= this->m_VolumeDimension[X]) ||
		 (n_Row    < 0 || n_Row    >= this->m_VolumeDimension[Y]) ||
		 (n_Depth  < 0 || n_Depth  >= this->m_VolumeDimension[Z]) )
	{
		float tempValue __gc[] = {0,0,0};
		return tempValue;
	}

	//calculate current array postion for gradient values
	//remember the gradient array structure: x_0,y_0,z_0,x_1,Y_2,z_2,....,x_n-1,y_n-1,z_n-1 
	int index = 3 * ((n_Depth * this->m_VoxelCountOfSlice) +  (n_Row * this->m_VolumeDimension[X]) + n_Column);
	float tempValue __gc[] = {this->m_GradientValues[index + X], this->m_GradientValues[index + Y], this->m_GradientValues[index + Z]};

	return tempValue;
}

//Returns the basic voxel data in form of a voxel data variable.
unsigned short VoxelData::getVoxelData(unsigned short n_VoxelCoordinates __gc[])
{
	if ( (n_VoxelCoordinates[X] < 0 || n_VoxelCoordinates[X] >= this->m_VolumeDimension[X]) || 
		 (n_VoxelCoordinates[Y] < 0 || n_VoxelCoordinates[Y] >= this->m_VolumeDimension[Y]) ||
		 (n_VoxelCoordinates[Z] < 0 || n_VoxelCoordinates[Z] >= this->m_VolumeDimension[Z]) )
			return 0;

	int index = (n_VoxelCoordinates[Z] * this->m_VoxelCountOfSlice) +  (n_VoxelCoordinates[Y] * this->m_VolumeDimension[X]) + n_VoxelCoordinates[X];	
	return this->m_Dataset[index];
}

//Returns the basic voxel data in form of a voxel data variable.
unsigned short VoxelData::getVoxelData(unsigned short n_Column, unsigned short n_Row, unsigned short n_Depth)
{
	if ( (n_Column < 0 || n_Column >= this->m_VolumeDimension[X]) ||
		 (n_Row    < 0 || n_Row    >= this->m_VolumeDimension[Y]) ||
		 (n_Depth  < 0 || n_Depth  >= this->m_VolumeDimension[Z]) )
		 return 0;

	int index = (n_Depth * this->m_VoxelCountOfSlice) +  (n_Row * this->m_VolumeDimension[X]) + n_Column;	
	return this->m_Dataset[index];
}

/*
Resets the whole gradient data set to zero.
That means each element of the gradient vector will be set ot zero but the current volume data size
will not be changed.
*/
void VoxelData::resetVoxelGradients(void)
{
	this->m_GradientValues = NULL;
	this->m_GradientValues = new float __gc[this->m_Dataset->Length * 3];
}