#pragma once

#include "DataObject.h"
#include "Transferfunction.h"
#include "Vector3.h"

namespace IDVR {
namespace Volume {
namespace DataStructures {

using namespace System;
using namespace System::Collections;
using namespace IDVR::Tools;

	/**
	<b>Class for controlling the basic voxel data of the volume data set.</b>

	This class is part of the DataStructure namespace which is actually a subnamespace of the main namespace Volume. 
	Furthermore the standard class Object is inherted to the %VoxelData class for better controlling of a particular 
	quantity of %VoxelData instances.
	<br><br>
	After loading the entire volume data set (see DataLoader class) which actually consists of one density value for each voxel element an
	instance of this class will be created to store all those voxel data. Furthermore, a couple of function are provided to help managing and controlling
	the basic voxel data.
	<br>
	Basically, the voxel data will be saved into an one dimensional unsigned short (16 bits per voxel value), but the interface function to
	access this data is adapted to a three dimensional (in fact a volume data structure) access by the user for better handling of this mostly huge voxel
	data set.
	<br>
	Thus, the indexing process offset of working with three dimensional arrays will be completely avoided without losing good access handling.
	<br><br>
	Most important are the two member arrays VoxelData::m_Dataset and VoxelData::m_GradientValues which save the actual voxel data and the corresponding 
	gradient vector data. The voxel data consist of 4 bits for the object ID number (linked to an existing volume data object - see DataObject class) and
	more 12 bits for actual density data. Alltogether  16 bits basic data are needed for each voxel and additionally 3 * 32 bit for the gradient vector data. 
	Please note that only on instance of this class can be created for each instance of the actual data structure class VolumeData. Thus every data structure
	object encapsulates one voxel data structure which makes surely most sense.
	@see VolumeData - the main datastructure class.
	@see DataObject - the volume data object class.
	@author <i>Bernhard Pflugfelder</i>
	@version 1.0
	@date 02-19-2005
	*/
	public __gc class VoxelData : public System::Object 
	{
	public: 
			/**
			This is the standard constructor.
			All member variables of this new instance will be set to zero/null.
			*/
			VoxelData(void);
			/**
			Constructor initialization with the valid volume data dimension.
			All member variables will be set to zero and the VoxelData::m_Dataset array will be initialized with the
			specified volume dimension.
			@param n_VolumeDimension the valid size of the volume data set.
			*/
			VoxelData(unsigned short n_VolumeDimension __gc[]);
			/**
			This is the standard deconstructor.
			All member variables will be deleted.
			*/
			~VoxelData(void);

			/**
			Sets the basic voxel data to a voxel specified by uniquely voxel coordinates.
			@param n_VoxelCoordinates the exact voxel coordinates.
			@param n_ObjectDensity the new voxel density value.
			@param n_ObjectID the new object ID key for the voxel.
			@exception An instance of the class Exception will be thrown if the voxel coordinates are not valid.
			*/
			void setVoxelValues(unsigned short n_VoxelCoordinates __gc[], unsigned short n_ObjectDensity, unsigned short n_ObjectID);
			/**
			Sets the basic voxel data to a voxel specified by uniquely voxel coordinates.
			@param n_Column the x element of the exact voxel coordinates.
			@param n_Row the x element of the exact voxel coordinates.
			@param n_Depth the x element of the exact voxel coordinates.
			@param n_ObjectDensity the new voxel density value.
			@param n_ObjectID the new object ID key for the voxel.
			@exception An instance of the class Exception will be thrown if the voxel coordinates are not valid.
			*/
			void setVoxelValues(unsigned short n_Column, unsigned short n_Row, unsigned short n_Depth, unsigned short n_ObjectDensity, unsigned short n_ObjectID);

			/**
			Sets only a new object ID value to a voxel specified by uniquely voxel coordinates.
			The remaining voxel data will not be changed.
			@param n_VoxelCoordinates the exact voxel coordinates.
			@param n_ObjectID the new object ID key for the voxel.
			@exception An instance of the class Exception will be thrown if the voxel coordinates are not valid.
			*/
			void setVoxelObjectID(unsigned short n_VoxelCoordinates __gc[], unsigned short n_ObjectID);
			/**
			Sets only a new object ID value to a voxel specified by uniquely voxel coordinates.
			The remaining voxel data will not be changed.
			@param n_Column the x element of the exact voxel coordinates.
			@param n_Row the x element of the exact voxel coordinates.
			@param n_Depth the x element of the exact voxel coordinates.
			@param n_ObjectID the new object ID key for the voxel.
			@exception An instance of the class Exception will be thrown if the voxel coordinates are not valid.
			*/
			void setVoxelObjectID(unsigned short n_Column, unsigned short n_Row, unsigned short n_Depth, unsigned short n_ObjectID);

			/**
			Sets only a new density value to a voxel specified by uniquely voxel coordinates.
			The remaining voxel data will not be changed.
			@param n_VoxelCoordinates the exact voxel coordinates.
			@param n_Density the new voxel density value.
			@exception An instance of the class Exception will be thrown if the voxel coordinates are not valid.
			*/
			void setVoxelDensity(unsigned short n_VoxelCoordinates __gc[], unsigned short n_Density);
			/**
			Sets only a new density value to a voxel specified by uniquely voxel coordinates.
			The remaining voxel data will not be changed.
			@param n_Column the x element of the exact voxel coordinates.
			@param n_Row the x element of the exact voxel coordinates.
			@param n_Depth the x element of the exact voxel coordinates.
			@param n_ObjectID the new object ID key for the voxel.
			@exception An instance of the class Exception will be thrown if the voxel coordinates are not valid.
			*/
			void setVoxelDensity(unsigned short n_Column, unsigned short n_Row, unsigned short n_Depth, unsigned short n_ObjectID);

			/**
			Sets a new gradient vector to the corresponding voxel specified by uniquely voxel coordinates.
			The remaining voxel data will not be changed. A valid gradient vector consists of three elements for the x-, y-
			and z-component (thus, a three dimensional vector structure).
			@param n_VoxelCoordinates the exact voxel coordinates.
			@param n_GradientValues the new grdient vector.
			@exception An instance of the class Exception will be thrown if the voxel coordinates are not valid.
			*/
			void setVoxelGradient(unsigned short n_VoxelCoordinates __gc[], float n_GradientValues __gc[]);
			/**
			Sets a new gradient vector to the corresponding voxel specified by uniquely voxel coordinates.
			The remaining voxel data will not be changed. A valid gradient vector consists of three elements for the x-, y-
			and z-component (thus, a three dimensional vector structure).
			@param n_Column the x element of the exact voxel coordinates.
			@param n_Row the x element of the exact voxel coordinates.
			@param n_Depth the x element of the exact voxel coordinates.
			@param n_GradientValues the new grdient vector.
			@exception An instance of the class Exception will be thrown if the voxel coordinates are not valid.
			*/
			void setVoxelGradient(unsigned short n_Column, unsigned short n_Row, unsigned short n_Depth, float n_GradientValues __gc[]);

			/**
			Returns the density value the given voxel data variable.
			@param n_DataItem is the actual data variable for each voxel.
			@return the current density value of the voxel.
			*/
			unsigned short getVoxelDensity(unsigned short n_DataItem);
			/**
			Returns the density value to the specified voxel.
			@param n_VoxelCoordinates the exact voxel coordinates.
			@return the current density value of the voxel.
			@exception An instance of the class Exception will be thrown if the voxel coordinates are not valid.
			*/
			unsigned short getVoxelDensity(unsigned short n_VoxelCoordinates __gc[]);
			/**
			Returns the density value to the specified voxel.
			@param n_Column the x element of the exact voxel coordinates.
			@param n_Row the x element of the exact voxel coordinates.
			@param n_Depth the x element of the exact voxel coordinates.
			@return the current density value of the voxel.
			@exception An instance of the class Exception will be thrown if the voxel coordinates are not valid.
			*/
			unsigned short getVoxelDensity(unsigned short n_Column, unsigned short n_Row, unsigned short n_Depth);

			/**
			Returns the object ID key value the given voxel data variable.
			@param n_DataItem is the actual data variable for each voxel.
			@return the current ID key of the voxel.
			*/
			unsigned short getVoxelObjectID(unsigned short n_DataItem);
			/**
			Returns the object ID key to the specified voxel.
			@param n_VoxelCoordinates the exact voxel coordinates.
			@return the current ID key of the voxel.
			@exception An instance of the class Exception will be thrown if the voxel coordinates are not valid.
			*/
			unsigned short getVoxelObjectID(unsigned short n_VoxelCoordinates __gc[]);
			/**
			Returns the object ID key to the specified voxel.
			@param n_Column the x element of the exact voxel coordinates.
			@param n_Row the x element of the exact voxel coordinates.
			@param n_Depth the x element of the exact voxel coordinates.
			@return the current ID key of the voxel.
			@exception An instance of the class Exception will be thrown if the voxel coordinates are not valid.
			*/
			unsigned short getVoxelObjectID(unsigned short n_Column, unsigned short n_Row, unsigned short n_Depth);

			/**
			Returns the gradient vector elements to the corresponding voxel coordinates.
			@param n_VoxelCoordinates the exact voxel coordinates.
			@return the current gradient vector as an one dimensional array with three elements.
			@exception An instance of the class Exception will be thrown if the voxel coordinates are not valid.
			*/
			float getVoxelGradient(unsigned short n_VoxelCoordinates __gc[]) __gc[];
			/**
			Returns the gradient vector elements to the corresponding voxel coordinates.
			@param n_Column the x element of the exact voxel coordinates.
			@param n_Row the x element of the exact voxel coordinates.
			@param n_Depth the x element of the exact voxel coordinates.
			@return the current gradient vector as an one dimensional array with three elements.
			@exception An instance of the class Exception will be thrown if the voxel coordinates are not valid.
			*/
			float getVoxelGradient(unsigned short n_Column, unsigned short n_Row, unsigned short n_Depth) __gc[];

			/**
			Returns the basic voxel data in form of a voxel data variable.
			@param n_VoxelCoordinates the exact voxel coordinates.
			@return the current voxel data varible.
			@exception An instance of the class Exception will be thrown if the voxel coordinates are not valid.
			*/
			unsigned short getVoxelData(unsigned short n_VoxelCoordinates __gc[]);
			/**
			Returns the basic voxel data in form of a voxel data variable.
			@param n_Column the x element of the exact voxel coordinates.
			@param n_Row the x element of the exact voxel coordinates.
			@param n_Depth the x element of the exact voxel coordinates.
			@return the current voxel data varible.
			@exception An instance of the class Exception will be thrown if the voxel coordinates are not valid.
			*/
			unsigned short getVoxelData(unsigned short n_Column, unsigned short n_Row, unsigned short n_Depth);

			/**
			Returns the current volume dimension of the data set.
			@return the volume dimension as an one dimensional array with three elements.
			*/
			__property unsigned short get_VolumeDimension(void) __gc[];
			/**
			Returns the voxel count per data slice of the current data set.
			For each voxel of the data slice the z coordinate is constant.
			@return the volume dimension as an one dimensional array with three elements.
			*/
			__property unsigned int get_VoxelCountOfSlice(void);
			/**
			Returns the total count of voxel for current dataset.
			@return the overall voxel count.
			*/
			__property unsigned int get_VoxelCountOverall(void);

			/**
			Resets the whole gradient data set to zero.
			That means each element of the gradient vector will be set ot zero but the current volume data size
			will not be changed.
			*/
			void resetVoxelGradients(void);
		
	private:
			
			unsigned short m_Dataset __gc[]; /**< Saves the basic voxel data structure which is the object ID and the density value. */
			unsigned short m_VolumeDimension __gc[]; /**< Saves the valid volume dimension of the current data set. */
			float m_GradientValues __gc[]; /** Saves the corresponding gradient vector values. */
			int m_VoxelCountOfSlice; /**< The voxel count per slice. For each voxel of the data slice the z coordinate is constant. */

			/** Constant index values for the three components of the three dimensional volume data set. */
			__value enum Indices{X = 0, Y, Z};
	public:

			static const unsigned short GENERALOBJECT = 0;
	};
}
}
}