#pragma once

#include ".\DataLoader.h"

namespace IDVR {
namespace Tools {

	using namespace System::Xml;
	using namespace IDVR::Volume;
	using namespace IDVR::Volume::DataStructures;
	/**
	<b>Class for loading and saving the application sttings into a XML-file.</b>

	This class is part of the Tools namespace and its task is the handling of the loading
	and saving of the application sttings.<br><br>
	To saving several application settings which are choosen by the user on the main GUI window (see class Neu)
	this class implements a XML parser which actually saves those setting in a predefined XML data structure. The 
	file name and path will be choosen by the user by a standard file save dialog and is assigned to the main saving
	function XMLParser::save_ConfigFile.<br>
	Furthermore, the user has the possiblity to save a complete opacity and color transfer function which is been defined
	in the class TransferFuncWin. Like before, the file name and path will be assigned to the function XMLParser::save_Transferfunction
	which implements the saving of the transfer function into a XML data structure.<br><br>
	To load the application setting or the user-defined transferfunctions the member function XMLParser::load_ConfigFile and 
	XMLParser::load_Transferfunction are included in this class.
	
	@see Neu - the main GUI window class
	@see TransferFuncWin - the transfer function editor class
	@author <i>Bernhard Pflugfelder</i>
	@version 1.0
	@date 02-20-2005
	*/
public __gc class XMLParser
{
private:
	/** Constant variables for xml attribute identification. */
	__value enum attributes{NAME = 0, /**< The constant value for xml attribute name. */
		                    VALUE     /**< The constant value for xml attribute value. */};
	/** Constant n_Index values for the three color components red, green and blue. */
	__value enum colorIndices{RED = 0, GREEN, BLUE};
	String* m_FilePath;				 /**< Saves the current file path of the XML file. */
	String* m_FileName;				 /**< Saves the current file name of the XML file. */

public:
	/**
	Constructor initialization with the valid XML file name and path.
	All member variables will be set to the corresponding parameters.
	@param n_FilePath the new file name and path of an existing XML file.
	*/
	XMLParser(String* n_FilePath);

	/**
	Loads the entire application settings from an already specified XML file.
	Select the XML file by calling the constructor XMLParser(String*). Thus, a new instance
	of this class must be created if a new file nam and path is choosen.
	*/
	void load_ConfigFile(void);
	/**
	Loads the entire opacity and color transfer function  from an already specified XML file.
	Select the XML file by calling the constructor XMLParser(String*). Thus, a new instance
	of this class must be created if a new file nam and path is choosen.
	@return An instance of the class Transferfunction which contains the loaded copacity and color functions.
	*/
	Transferfunction* load_Transferfunction(void);
	/**
	Saves the assigned values of the parameter into the already specified XML file.
	Select the XML file by calling the constructor XMLParser(String*). Thus, a new instance
	of this class must be created if a new file nam and path is choosen.
	@param n_NewValues contains all needed application settings.
	*/
	void save_ConfigFile(Array *n_NewValues);
	/**
	Saves the assigned transfer function into the already specified XML file.
	Select the XML file by calling the constructor XMLParser(String*). Thus, a new instance
	of this class must be created if a new file nam and path is choosen.
	@param n_TransFunc contains the opacity and color transfer functions.
	*/
	void save_Transferfunction(Transferfunction* n_TransFunc);

private:
	/**
	Reads the values of teh setting group <i>Datafile</i> from the XML file.
	@param n_XMLReader is the acutal standard XML parser as an instance of the standard class XmlNodeReader.
	*/
	void read_DatafileSettings(XmlNodeReader *n_XMLReader);
	/**
	Reads the values of teh setting group <i>PhongShading</i> from the XML file.
	@param n_XMLReader is the acutal standard XML parser as an instance of the standard class XmlNodeReader.
	*/
	void read_PhongShadingSettings(XmlNodeReader *n_XMLReader);
	/**
	Reads the values of teh setting group <i>Optimations</i> from the XML file.
	@param n_XMLReader is the acutal standard XML parser as an instance of the standard class XmlNodeReader.
	*/
	void read_OptimationSettings(XmlNodeReader *n_XMLReader);
	/**
	Reads the values of teh setting group <i>DepthCueing</i> from the XML file.
	@param n_XMLReader is the acutal standard XML parser as an instance of the standard class XmlNodeReader.
	*/
	void read_DepthCueingSettings(XmlNodeReader *n_XMLReader);
	/**
	Reads the values of teh setting group <i>MIP</i> from the XML file.
	@param n_XMLReader is the acutal standard XML parser as an instance of the standard class XmlNodeReader.
	*/
	void read_MIPSettings(XmlNodeReader *n_XMLReader);
	/**
	Reads the values of teh setting group <i>Contour</i> from the XML file.
	@param n_XMLReader is the acutal standard XML parser as an instance of the standard class XmlNodeReader.
	*/
	void read_ContourSettings(XmlNodeReader *n_XMLReader);
	/**
	Reads the values of teh setting group <i>Tone</i> from the XML file.
	@param n_XMLReader is the acutal standard XML parser as an instance of the standard class XmlNodeReader.
	*/
	void read_ToneSettings(XmlNodeReader *n_XMLReader);

	/**
	Saves the assigned values of the setting group <i>Datafile</i> into the specified XML file.
	@param n_SaveConfigFile is the actual XML parser which creates a valid XML data structure.
	@param n_NewValues contains the setting values.
	@param n_Index is the current XML node number.
	*/
	void save_DatafileSettings(XmlTextWriter *n_SaveConfigFile, Array *n_NewValues, int *n_Index);
	/**
	Saves the assigned values of the setting group <i>PhongShading</i> into the specified XML file.
	@param n_SaveConfigFile is the actual XML parser which creates a valid XML data structure.
	@param n_NewValues contains the setting values.
	@param n_Index is the current XML node number.
	*/
	void save_PhongShadingSettings(XmlTextWriter *n_SaveConfigFile, Array *n_NewValues, int *n_Index);
	/**
	Saves the assigned values of the setting group <i>Optimations</i> into the specified XML file.
	@param n_SaveConfigFile is the actual XML parser which creates a valid XML data structure.
	@param n_NewValues contains the setting values.
	@param n_Index is the current XML node number.
	*/
	void save_OptimationSettings(XmlTextWriter *n_SaveConfigFile, Array *n_NewValues, int *n_Index);
	/**
	Saves the assigned values of the setting group <i>DeothCueing</i> into the specified XML file.
	@param n_SaveConfigFile is the actual XML parser which creates a valid XML data structure.
	@param n_NewValues contains the setting values.
	@param n_Index is the current XML node number.
	*/
	void save_DepthCueingSettings(XmlTextWriter *n_SaveConfigFile, Array *n_NewValues, int *n_Index);
	/**
	Saves the assigned values of the setting group <i>MIP</i> into the specified XML file.
	@param n_SaveConfigFile is the actual XML parser which creates a valid XML data structure.
	@param n_NewValues contains the setting values.
	@param n_Index is the current XML node number.
	*/
	void save_MIPSettings(XmlTextWriter *n_SaveConfigFile, Array *n_NewValues, int *n_Index);
	/**
	Saves the assigned values of the setting group <i>Contour</i> into the specified XML file.
	@param n_SaveConfigFile is the actual XML parser which creates a valid XML data structure.
	@param n_NewValues contains the setting values.
	@param n_Index is the current XML node number.
	*/
	void save_ContourSettings(XmlTextWriter *n_SaveConfigFile, Array *n_NewValues, int *n_Index);
	/**
	Saves the assigned values of the setting group <i>Tone</i> into the specified XML file.
	@param n_SaveConfigFile is the actual XML parser which creates a valid XML data structure.
	@param n_NewValues contains the setting values.
	@param n_Index is the current XML node number.
	*/
	void save_ToneSettings(XmlTextWriter *n_SaveConfigFile, Array *n_NewValues, int *n_Index);
};
}
}
