#pragma once

#include "TransferFuncWin.h"
#include "ObjectDescriptionInput.h"
#include "ProgressWin.h"
#include "AboutWin.h"
#include "DataLoader.h"
#include "RenderMachineSW.h"
#include "RendererSW.h"
#include "MipShadingModelSW.h"
#include "PhongShadingModelSW.h"
#include "ContourShadingModelSW.h"
#include "ToneShadingModelSW.h"
#include "TFClassificationModelSW.h"
#include "LinearClassificationModelSW.h"
#include "DvrCompositingModelSW.h"
#include "LMipCompositingModelSW.h"
#include "IDVRMImPCompositingModelSW.h"
#include "IDVRColAndOpModulatorSW.h"
#include "ImagePlane.h"
#include "InterpolationSW.h"
#include "XMLParser.h"
#include "IOpenGLCanvas.h"
#include "OpenGLCanvas.h"
#include "OpenGLPreviewCanvas.h"
#include "SliceViewer.h"
#include "AnimationWin.h"

using namespace System;
using namespace System::ComponentModel;
using namespace System::Collections;
using namespace System::Windows::Forms;
using namespace System::Data;
using namespace System::Drawing;
using namespace System::Drawing::Imaging;
using namespace TD;
using namespace IDVR;
using namespace IDVR::Volume;
using namespace IDVR::Tools;
using namespace IDVR::VolumeRenderer::RenderPrimitives;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer::IDVRCompositingModels;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer::IDVRModulationModels;

namespace IDVR
{
	/**
	<B>GUI class for creating the <i>Main GUI</i> window.</B>.

	This class is part of the main application namespace IDVR and contains the definition of the <i>Main GUI</i> window and is the actual main entrance
	class of the %IDVR application.
	<br><br>
	All main settings and actions of the %IDVR application wiil be managed by this class and it creates the main GUI window with which the user will handle
	the application. The loading of the volume data file and possible mask files, the selction of rendering models and almost any other main funtion of this 
	application will be controlled by this class. Also the rendering settings, tansformation settings of the image plane are provided.<br>
	Furthermore, all essential data memeber of the entire application are included here which are the volume data structure, the image plane and the light model.
	<br><br>
	The API documentation for this class shows only the most important member variables and function. Event handling functions and GUI
	components (e.g. labels, textboxes,...) will not be included but all essential parts of the class are surely documented.

	@see VolumeData - represents the volume data structure.
	@see ImagePlane - the image plane with all needed attributes.
	@see Light - the light model and its attributes.
	@author <i>Bernhard Pflugfelder</i>
	@version 1.0
	@date 02-21-2005
	*/
	public __gc class Neu : public System::Windows::Forms::Form
	{
	public: 

		/**
		This is the standard constructor.
		It implements the standard initialization of the GUI window and actually start a 
		thread to create and control a specific dialog window.
		*/
		Neu(void)
		{
			try 
			{
				XMLParser *loadConfigFile = new XMLParser(DataLoader::configFileName);
				loadConfigFile->load_ConfigFile();
			}
			catch (Exception* e)
			{
				MessageBox::Show(this, e->get_Message(), S"Error by reading the Config-File", MessageBoxButtons::OK, MessageBoxIcon::Error);
				exit(1);
			}

			InitializeComponent();

			initialize_Elements();
		}
        
	protected: 

		/**
		Destroys all GUI components before this current instance will be actually discarded.
		@param disposing is a flag variable which is true if the instance have to be discarded.
		*/
		void Dispose(Boolean disposing)
		{
			if (disposing && components)
			{
				components->Dispose();
			}
			__super::Dispose(disposing);
		}

	private:

		 unsigned char m_GradientEstimationMode;    /**< Saves the current gradient calculation mode. */
		 unsigned char m_RenderType;			    /**< Saves the current render type. This flag variable will not be used actually. */
		 unsigned char m_SystemStatus;			
		 unsigned char m_LightViewMode;			    /**< Saves the current light view mode. */
	     unsigned char m_DensityInterpolationMode;  /**< Saves the current interpolation mode of the density values. */ 
		 unsigned char m_GradientInterpolationMode; /**< Saves the current interpolation mode of the gradient values. */ 
		 unsigned char m_VoxelIDInterpolationMode;  /**< Saves the current interpolation mode of the object ID values. */ 

		 ImagePlane* m_ImagePlane;					/**< Saves the current image plane of the main canvas. */
		 Light* m_Light;                            /**< Saves the assigned light model.
													     The instance includes all needed light model settings */ 
		 Transferfunction *m_TransFuncInput;        /**< Saves the current choosen transfer function of the transfer funtion preview. */
		 DataLoader *m_VolumeData;					/**< Saves the current loaded volume data set. */
		 OpenGLCanvas *m_Canvas;					/**< Saves the current used canvas window */
		 OpenGLPreviewCanvas *m_PreviewCanvas;		/**< Saves the current used preview canvas window */
		 float stepLength;

		 /** Constant index values for the three components of the three dimensional volume data set. */
		 __value enum Indices{X = 0, Y, Z};

		 static const unsigned char SOFTWARE = 0, HARDWARE = 1, NODATALOADED = 0, READY = 1, RENDERING = 2,
						            SUPERSAMPLING4 = 0, SUPERSAMPLING2 = 1, NORMALSAMPLING = 2, SUBSAMPLING2 = 3, SUBSAMPLING4 = 4;
		 static const double ROTATIONSTEP = Math::PI /16.0;
		 static const short PREVIEWCANVAS_DEFAULTPOSX = 0, PREVIEWCANVAS_DEFAULTPOSY = 0; 

		 static String *RENDERINGTHREAD = S"RenderingHandler", *MAINTITLE = S"IDVR RayCaster";

	private: TD::SandBar::SandBarManager *  sbM_GUIManager;
	private: TD::SandBar::ToolBarContainer *  leftSandBarDock;
	private: TD::SandBar::ToolBarContainer *  rightSandBarDock;
	private: TD::SandBar::ToolBarContainer *  bottomSandBarDock;
	private: TD::SandBar::ToolBarContainer *  topSandBarDock;
	private: TD::SandBar::MenuBar *  mam_MenueBar;
	private: TD::SandBar::MenuBarItem *  mItem_Help;
	private: TD::SandBar::ButtonItem *  tbb_LoadData;
	private: TD::SandBar::ButtonItem *  tbb_LoadMask;
	private: System::Windows::Forms::ImageList *  iml_ToolBarIcons;
	private: TD::SandBar::ComboBoxItem *  tbc_FilePath;
	private: TD::SandBar::ButtonItem *  tbb_BackColor;
	private: TD::SandBar::ComboBoxItem *  tbc_RenderLevel;

	private: TD::SandBar::ButtonItem *  tbb_LightColor;
	private: TD::SandBar::LabelItem *  tbl_LightCoords;
	private: TD::SandBar::LabelItem *  tbl_Transformation;
	private: TD::SandBar::ButtonItem *  tbb_TranslateLeft;
	private: TD::SandBar::ButtonItem *  tbb_TranslateRight;
	private: TD::SandBar::ButtonItem *  tbb_TranslateUp;
	private: TD::SandBar::ButtonItem *  tbb_TranslateDown;
	private: TD::SandBar::ButtonItem *  tbb_ZoomIn;
	private: TD::SandBar::ButtonItem *  tbb_ZoomOut;
	private: System::Windows::Forms::StatusBar *  stb_StatusBar;
	private: TD::SandBar::ContainerBar *  ctb_RenderControl;
	private: TD::SandBar::ContainerBarClientPanel *  containerBarClientPanel1;
	private: TD::SandBar::DropDownMenuItem *  mItem_Interpolation;
	private: TD::SandBar::DropDownMenuItem *  mItem_Gradient;


	private: TD::SandBar::MenuButtonItem *  subItem_Central;
	private: TD::SandBar::MenuButtonItem *  subItem_Regression;
	private: System::Windows::Forms::GroupBox *  grb_Rendering;
	private: System::Windows::Forms::Button *  but_Render;
	private: System::Windows::Forms::NumericUpDown *  nud_DepthK2;
	private: System::Windows::Forms::Label *  lab_DepthK2;
	private: System::Windows::Forms::Label *  lab_DepthK1;
	private: System::Windows::Forms::NumericUpDown *  nud_DepthK1;
	private: System::Windows::Forms::Label *  lab_Specular;
	private: System::Windows::Forms::Label *  lab_Diffuse;
	private: System::Windows::Forms::Label *  lab_Ambient;
	private: System::Windows::Forms::Label *  lab_SpExponent;
	private: System::Windows::Forms::Label *  lab_DepthCueing;
	private: TD::SandBar::DropDownMenuItem *  mItem_RendererType;
	private: TD::SandBar::MenuButtonItem *  subItem_Software;
	private: System::Windows::Forms::StatusBarPanel *  sbp_VolumeDimension;
	private: System::Windows::Forms::StatusBarPanel *  sbp_ViewDirection;
	private: System::Windows::Forms::StatusBarPanel *  sbp_LightDirection;
	private: System::Windows::Forms::StatusBarPanel *  sbp_RenderMode;
	private: System::Windows::Forms::StatusBarPanel *  sbp_Gradient;

	private: TD::SandBar::LabelItem *  tbl_BackColor;
	private: TD::SandBar::LabelItem *  tbl_LightColor;
	private: TD::SandBar::ToolBar *  tob_FirstBar;
	private: TD::SandBar::ToolBar *  tob_SecondBar;
	private: System::Windows::Forms::ComboBox *  cob_TransObj;
	private: System::Windows::Forms::CheckBox *  chb_TwoLevelVR;
	private: System::Windows::Forms::Label *  lab_TransObj;
	private: System::Windows::Forms::Panel *  pan_TFPreview;
	private: System::Windows::Forms::Label *  lab_DVRTitle;
	private: System::Windows::Forms::TabControl *  tab_RenderControl;

	private: System::Windows::Forms::Button *  but_MipAssign;
	private: System::Windows::Forms::Button *  but_MipReset;
	private: System::Windows::Forms::TabPage *  tabPage_DVR;
	private: System::Windows::Forms::TabPage *  tabPage_MIP;
	private: System::Windows::Forms::TabPage *  tabPage_Contour;
	private: System::Windows::Forms::GroupBox *  grb_RenderControl;
	private: TD::SandBar::MenuButtonItem *  subItem_Hardware;
	private: TD::SandBar::MenuBarItem *  mItem_File;
	private: System::Windows::Forms::ColorDialog *  colDia_BackColor;
	private: System::Windows::Forms::ColorDialog *  colDia_LightColor;
	private: TD::SandBar::MenuButtonItem *  subItem_About;
	private: TD::SandBar::MenuButtonItem *  subItem_LoadTransFunc;
	private: TD::SandBar::MenuButtonItem *  subItem_SaveTransFunc;
	private: TD::SandBar::MenuBarItem *  mItem_Settings;
	private: TD::SandBar::MenuButtonItem *  subItem_LightColor;
	private: TD::SandBar::MenuButtonItem *  subItem_BackColor;
	private: TD::SandBar::ComboBoxItem *  tbc_ViewDirection;
	private: System::Windows::Forms::Button *  but_ContourAssign;
	private: System::Windows::Forms::Button *  but_ContourReset;
	private: System::Windows::Forms::TextBox *  tex_ContourThreshold;
	private: System::Windows::Forms::Label *  lab_ContourThreshold;
	private: System::Windows::Forms::TrackBar *  trb_ContourThreshold;
	private: System::Windows::Forms::Label *  lab_ContourGradient;
	private: System::Windows::Forms::Label *  lab_ContourExponent;
	private: System::Windows::Forms::NumericUpDown *  nud_ContourExponent;
	private: System::Windows::Forms::NumericUpDown *  nud_ContourGradient;
	private: System::Windows::Forms::Button *  but_MipColor;
	private: System::Windows::Forms::Label *  lab_MipColor;
	private: System::Windows::Forms::Label *  lab_ContourColor;
	private: System::Windows::Forms::Button *  but_ContourColor;
	private: System::Windows::Forms::ColorDialog *  colDia_MipColor;
	private: System::Windows::Forms::ColorDialog *  colDia_ContourColor;
	private: System::Windows::Forms::TabPage *  tabPage_Tone;
	private: System::Windows::Forms::StatusBarPanel *  sbp_Status;
	private: System::Windows::Forms::Label *  lab_ToneColWarm;
	private: System::Windows::Forms::Button *  but_ToneColWarm;
	private: System::Windows::Forms::Label *  lab_ToneGradient;
	private: System::Windows::Forms::NumericUpDown *  nud_ToneGradient;
	private: System::Windows::Forms::TextBox *  tex_ToneThreshold;
	private: System::Windows::Forms::Label *  lab_ToneThreshold;
	private: System::Windows::Forms::TrackBar *  trb_ToneThreshold;
	private: System::Windows::Forms::Button *  but_ToneReset;
	private: System::Windows::Forms::NumericUpDown *  nud_ToneKW;
	private: System::Windows::Forms::NumericUpDown *  nud_ToneKC;
	private: System::Windows::Forms::ColorDialog *  colDia_ToneColorWarm;
	private: System::Windows::Forms::ColorDialog *  colDia_ToneColorCold;
	private: System::Windows::Forms::Button *  but_ToneAssign;
	private: System::Windows::Forms::Button *  but_ToneColCold;
	private: System::Windows::Forms::Label *  lab_ToneKC;
	private: System::Windows::Forms::Label *  lab_ToneKW;
	private: System::Windows::Forms::Label *  lab_ToneColCold;
	private: System::Windows::Forms::Button *  but_DVRAssign;
	private: System::Windows::Forms::Panel *  pan_Dvr;
	private: System::Windows::Forms::NumericUpDown *  nud_Ambient;
	private: System::Windows::Forms::NumericUpDown *  nud_Diffuse;
	private: System::Windows::Forms::NumericUpDown *  nud_Specular;
	private: System::Windows::Forms::NumericUpDown *  nud_SpExponent;
	private: System::Windows::Forms::Button *  but_DVRReset;
	private: System::Windows::Forms::Panel *  pan_Tone;
	private: System::Windows::Forms::RadioButton *  rab_ToneLMip;
	private: System::Windows::Forms::RadioButton *  rab_ToneDvr;
	private: System::Windows::Forms::Button *  but_ToneTransferFunc;
	private: System::Windows::Forms::NumericUpDown *  nud_ToneKTF;
	private: System::Windows::Forms::Label *  lab_ToneKTF;
	private: System::Windows::Forms::Panel *  pan_Contour;
	private: System::Windows::Forms::RadioButton *  rab_ContourLMip;
	private: System::Windows::Forms::Label *  lab_ContourKTF;
	private: System::Windows::Forms::NumericUpDown *  nud_ContourKTF;
	private: System::Windows::Forms::Button *  but_ContourTransferFunc;
	private: System::Windows::Forms::RadioButton *  rab_ContourDvr;
	private: System::Windows::Forms::TabPage *  tabPage_Basic;
	private: System::Windows::Forms::Label *  lab_ImageRes;
	private: System::Windows::Forms::TrackBar *  trb_ImageRes;
	private: System::Windows::Forms::TrackBar *  trb_CanvasSize;
	private: System::Windows::Forms::Label *  lab_CanvasSize;
	private: System::Windows::Forms::TabPage *  tabPage_Optimation;
	private: System::Windows::Forms::Label *  lab_SamplingDist;
	private: System::Windows::Forms::TrackBar *  trb_SamplingDist;
	private: System::Windows::Forms::Label *  lab_OpacityThreshold;
	private: System::Windows::Forms::NumericUpDown *  nud_OpacityThreshold;
	private: System::Windows::Forms::Label *  lab_GradientTreshold;
	private: System::Windows::Forms::NumericUpDown *  nud_GradientThreshold;
	private: System::Windows::Forms::TextBox *  tex_SamplingDist;
	private: System::Windows::Forms::TabControl *  tabControl1;
	private: System::Windows::Forms::StatusBarPanel *  sbp_RayDistance;
	private: System::Windows::Forms::OpenFileDialog *  openDia_LoadDataFile;
	private: System::Windows::Forms::OpenFileDialog *  openDia_LoadConfigFile;
	private: System::Windows::Forms::OpenFileDialog *  openDia_LoadTransFunc;
	private: TD::SandBar::MenuButtonItem *  subItem_LoadDataFile;
	private: TD::SandBar::MenuButtonItem *  subItem_LoadMaskFile;
	private: TD::SandBar::MenuButtonItem *  subItem_SaveConfigFile;
	private: System::Windows::Forms::SaveFileDialog *  saveDia_SaveConfigFile;
	private: System::Windows::Forms::SaveFileDialog *  saveDia_SaveTransFunc;
	private: TD::SandBar::MenuButtonItem *  subItem_LoadConfigFile;
	private: System::Windows::Forms::OpenFileDialog *  openDia_LoadMaskFile;
	private: System::Windows::Forms::ToolTip *  tot_ToolTips;
	private: System::Windows::Forms::ComboBox *  cob_SamplingFactor;
	private: TD::SandBar::MenuButtonItem *  subItem_PreviewWindow;
	private: System::Windows::Forms::RadioButton *  rab_MipLinClassification;
	private: System::Windows::Forms::RadioButton *  rab_MipTFClassification;
	private: System::Windows::Forms::Label *  lab_MipNote;
	private: System::Windows::Forms::Button *  but_MipTransferFunc;
	private: System::Windows::Forms::SaveFileDialog *  saveDia_SaveRenderedPicture;
	private: TD::SandBar::MenuButtonItem *  subItem_SaveRenderedPicture;
	private: System::Windows::Forms::Panel *  pan_TFPreviewColor;
	private: System::Windows::Forms::GroupBox *  grb_IDVR;
	private: System::Windows::Forms::Label *  lab_IDVRObject;
	private: System::Windows::Forms::ComboBox *  cob_IDVRObject;
	private: System::Windows::Forms::CheckBox *  chb_IDVR;
	private: System::Windows::Forms::Button *  but_IDVRReset;
	private: System::Windows::Forms::Label *  lab_IDVRCompositing;
	private: System::Windows::Forms::Label *  lab_IDVRModulation;
	private: System::Windows::Forms::ComboBox *  cob_IDVRCompositing;
	private: System::Windows::Forms::ComboBox *  cob_IDVRModulation;
	private: System::Windows::Forms::NumericUpDown *  nud_IDVRObjectImportance;
	private: System::Windows::Forms::Button *  but_IDVRAssign;
	private: TD::SandBar::ComboBoxItem *  tbc_VPRotationAxis;
	private: TD::SandBar::ButtonItem *  tbb_VPCWRotation;
	private: TD::SandBar::ButtonItem *  tbb_VPCCWRotation;
	private: TD::SandBar::ComboBoxItem *  comboBoxItem1;
	private: TD::SandBar::ComboBoxItem *  tbc_LightRotationAxis;
	private: TD::SandBar::ButtonItem *  tbb_LightCWRotation;
	private: TD::SandBar::ButtonItem *  tbb_LightCCWRotation;
	private: TD::SandBar::DropDownMenuItem *  tbMenueItem_CanvasControl;
	private: TD::SandBar::MenuButtonItem *  tbSubItem_ClearCanvas;
	private: TD::SandBar::MenuButtonItem *  tbSubItem_ResetLight;
	private: System::Windows::Forms::Button *  but_LoadMaskFile;
	private: System::Windows::Forms::Button *  but_DeleteVolumeObject;
	private: TD::SandBar::DropDownMenuItem *  tbMenuItem_Rendering;
	private: TD::SandBar::MenuButtonItem *  tbSubItem_StartRendering;
	private: TD::SandBar::MenuButtonItem *  tbSubItem_StopRendering;
	private: System::Windows::Forms::TextBox *  tex_MipThreshold;
	private: System::Windows::Forms::Label *  lab_MipThreshold;
	private: System::Windows::Forms::TrackBar *  trb_MipThreshold;
	private: TD::SandBar::MenuButtonItem *  subItem_PreviewCanvasDefaultPosition;
	private: System::Windows::Forms::Label *  lab_IDVRCountersinkSlope;
	private: System::Windows::Forms::NumericUpDown *  nud_IDVRCountersinkSlope;
	private: TD::SandBar::MenuButtonItem *  subItem_Density;
	private: TD::SandBar::MenuButtonItem *  subItem_Gradient;
	private: TD::SandBar::MenuButtonItem *  subItem_VoxelID;
	private: TD::SandBar::MenuButtonItem *  subItem_DensityNN;
	private: TD::SandBar::MenuButtonItem *  subItem_DensityTL;
	private: TD::SandBar::MenuButtonItem *  subItem_AllNN;
	private: TD::SandBar::MenuButtonItem *  subItem_AllTL;
	private: TD::SandBar::MenuButtonItem *  subItem_GradientNN;
	private: TD::SandBar::MenuButtonItem *  subItem_GradientTL;
	private: TD::SandBar::MenuButtonItem *  subItem_VoxelIDNN;
	private: TD::SandBar::MenuButtonItem *  subItem_VoxelIDTL;
	private: System::Windows::Forms::StatusBarPanel *  sbp_DenityMode;
	private: System::Windows::Forms::StatusBarPanel *  sbp_GradientMode;
	private: System::Windows::Forms::StatusBarPanel *  sbp_VoxelIDMode;
	private: TD::SandBar::MenuButtonItem *  subItem_EditTransferFunction;
	private: TD::SandBar::MenuBarItem *  mItem_Tools;
	private: TD::SandBar::MenuButtonItem *  subItem_SliceViewer;
	private: TD::SandBar::DropDownMenuItem *  tbMenuItem_Tools;
	private: TD::SandBar::MenuButtonItem *  tbSubItem_PreviewWindow;
	private: TD::SandBar::MenuButtonItem *  tbSubItem_SliceViewer;
	private: TD::SandBar::MenuButtonItem *  tbSubItem_EditTransferfunction;
	private: TD::SandBar::MenuButtonItem *  tbSubItem_ResetVD;
	private: TD::SandBar::MenuButtonItem *  tbSubItem_ResetLigthAndView;
	private: TD::SandBar::MenuButtonItem *  subItem_LightProps;
	private: TD::SandBar::MenuButtonItem *  subItem_LightViewMode;
	private: TD::SandBar::MenuButtonItem *  subItem_LightViewModeABS;
	private: TD::SandBar::MenuButtonItem *  subItem_LightViewModeREL;
	private: System::Windows::Forms::Label *  lab_IDVRGScale;
	private: System::Windows::Forms::NumericUpDown *  nud_IDVRGScale;
	private: TD::SandBar::MenuButtonItem *  tbSubItem_AnimationEditor;
	private: TD::SandBar::MenuButtonItem *  subItem_AnimationEditor;

	private: System::ComponentModel::IContainer *  components;

	private:

		/**
		The main function to create the all needed GUI components.
		Furthermore, the linking of those created GUI components (e.g. labels, textboxes,...) to the current container
		is implemented. After the correct linking all GUI items will be shown on the screen.
		@note This function must not be changed in a code editor because otherwise the GUI builder could be working incorrectly.
		*/
		void InitializeComponent(void)
		{
			this->components = new System::ComponentModel::Container();
			System::Resources::ResourceManager *  resources = new System::Resources::ResourceManager(__typeof(IDVR::Neu));
			this->sbM_GUIManager = new TD::SandBar::SandBarManager();
			this->bottomSandBarDock = new TD::SandBar::ToolBarContainer();
			this->leftSandBarDock = new TD::SandBar::ToolBarContainer();
			this->ctb_RenderControl = new TD::SandBar::ContainerBar();
			this->mItem_Interpolation = new TD::SandBar::DropDownMenuItem();
			this->subItem_Density = new TD::SandBar::MenuButtonItem();
			this->subItem_DensityNN = new TD::SandBar::MenuButtonItem();
			this->subItem_DensityTL = new TD::SandBar::MenuButtonItem();
			this->subItem_Gradient = new TD::SandBar::MenuButtonItem();
			this->subItem_GradientNN = new TD::SandBar::MenuButtonItem();
			this->subItem_GradientTL = new TD::SandBar::MenuButtonItem();
			this->subItem_VoxelID = new TD::SandBar::MenuButtonItem();
			this->subItem_VoxelIDNN = new TD::SandBar::MenuButtonItem();
			this->subItem_VoxelIDTL = new TD::SandBar::MenuButtonItem();
			this->subItem_AllNN = new TD::SandBar::MenuButtonItem();
			this->subItem_AllTL = new TD::SandBar::MenuButtonItem();
			this->mItem_Gradient = new TD::SandBar::DropDownMenuItem();
			this->subItem_Central = new TD::SandBar::MenuButtonItem();
			this->subItem_Regression = new TD::SandBar::MenuButtonItem();
			this->mItem_RendererType = new TD::SandBar::DropDownMenuItem();
			this->subItem_Software = new TD::SandBar::MenuButtonItem();
			this->subItem_Hardware = new TD::SandBar::MenuButtonItem();
			this->containerBarClientPanel1 = new TD::SandBar::ContainerBarClientPanel();
			this->grb_IDVR = new System::Windows::Forms::GroupBox();
			this->nud_IDVRGScale = new System::Windows::Forms::NumericUpDown();
			this->lab_IDVRGScale = new System::Windows::Forms::Label();
			this->nud_IDVRCountersinkSlope = new System::Windows::Forms::NumericUpDown();
			this->lab_IDVRCountersinkSlope = new System::Windows::Forms::Label();
			this->cob_IDVRModulation = new System::Windows::Forms::ComboBox();
			this->cob_IDVRCompositing = new System::Windows::Forms::ComboBox();
			this->lab_IDVRModulation = new System::Windows::Forms::Label();
			this->lab_IDVRCompositing = new System::Windows::Forms::Label();
			this->but_IDVRAssign = new System::Windows::Forms::Button();
			this->but_IDVRReset = new System::Windows::Forms::Button();
			this->nud_IDVRObjectImportance = new System::Windows::Forms::NumericUpDown();
			this->lab_IDVRObject = new System::Windows::Forms::Label();
			this->cob_IDVRObject = new System::Windows::Forms::ComboBox();
			this->chb_IDVR = new System::Windows::Forms::CheckBox();
			this->grb_RenderControl = new System::Windows::Forms::GroupBox();
			this->but_DeleteVolumeObject = new System::Windows::Forms::Button();
			this->iml_ToolBarIcons = new System::Windows::Forms::ImageList(this->components);
			this->but_LoadMaskFile = new System::Windows::Forms::Button();
			this->lab_TransObj = new System::Windows::Forms::Label();
			this->cob_TransObj = new System::Windows::Forms::ComboBox();
			this->tab_RenderControl = new System::Windows::Forms::TabControl();
			this->tabPage_DVR = new System::Windows::Forms::TabPage();
			this->pan_Dvr = new System::Windows::Forms::Panel();
			this->pan_TFPreviewColor = new System::Windows::Forms::Panel();
			this->nud_SpExponent = new System::Windows::Forms::NumericUpDown();
			this->nud_Specular = new System::Windows::Forms::NumericUpDown();
			this->nud_Diffuse = new System::Windows::Forms::NumericUpDown();
			this->nud_Ambient = new System::Windows::Forms::NumericUpDown();
			this->lab_DVRTitle = new System::Windows::Forms::Label();
			this->pan_TFPreview = new System::Windows::Forms::Panel();
			this->lab_Ambient = new System::Windows::Forms::Label();
			this->lab_Diffuse = new System::Windows::Forms::Label();
			this->lab_Specular = new System::Windows::Forms::Label();
			this->lab_SpExponent = new System::Windows::Forms::Label();
			this->nud_DepthK2 = new System::Windows::Forms::NumericUpDown();
			this->lab_DepthK2 = new System::Windows::Forms::Label();
			this->nud_DepthK1 = new System::Windows::Forms::NumericUpDown();
			this->lab_DepthK1 = new System::Windows::Forms::Label();
			this->lab_DepthCueing = new System::Windows::Forms::Label();
			this->but_DVRAssign = new System::Windows::Forms::Button();
			this->but_DVRReset = new System::Windows::Forms::Button();
			this->tabPage_MIP = new System::Windows::Forms::TabPage();
			this->but_MipTransferFunc = new System::Windows::Forms::Button();
			this->lab_MipNote = new System::Windows::Forms::Label();
			this->rab_MipTFClassification = new System::Windows::Forms::RadioButton();
			this->rab_MipLinClassification = new System::Windows::Forms::RadioButton();
			this->lab_MipColor = new System::Windows::Forms::Label();
			this->but_MipColor = new System::Windows::Forms::Button();
			this->tex_MipThreshold = new System::Windows::Forms::TextBox();
			this->lab_MipThreshold = new System::Windows::Forms::Label();
			this->trb_MipThreshold = new System::Windows::Forms::TrackBar();
			this->but_MipAssign = new System::Windows::Forms::Button();
			this->but_MipReset = new System::Windows::Forms::Button();
			this->tabPage_Tone = new System::Windows::Forms::TabPage();
			this->pan_Tone = new System::Windows::Forms::Panel();
			this->lab_ToneKTF = new System::Windows::Forms::Label();
			this->nud_ToneKTF = new System::Windows::Forms::NumericUpDown();
			this->but_ToneTransferFunc = new System::Windows::Forms::Button();
			this->rab_ToneDvr = new System::Windows::Forms::RadioButton();
			this->rab_ToneLMip = new System::Windows::Forms::RadioButton();
			this->lab_ToneKW = new System::Windows::Forms::Label();
			this->nud_ToneKC = new System::Windows::Forms::NumericUpDown();
			this->nud_ToneKW = new System::Windows::Forms::NumericUpDown();
			this->lab_ToneColWarm = new System::Windows::Forms::Label();
			this->lab_ToneGradient = new System::Windows::Forms::Label();
			this->nud_ToneGradient = new System::Windows::Forms::NumericUpDown();
			this->tex_ToneThreshold = new System::Windows::Forms::TextBox();
			this->lab_ToneThreshold = new System::Windows::Forms::Label();
			this->lab_ToneKC = new System::Windows::Forms::Label();
			this->trb_ToneThreshold = new System::Windows::Forms::TrackBar();
			this->but_ToneColWarm = new System::Windows::Forms::Button();
			this->but_ToneColCold = new System::Windows::Forms::Button();
			this->lab_ToneColCold = new System::Windows::Forms::Label();
			this->but_ToneReset = new System::Windows::Forms::Button();
			this->but_ToneAssign = new System::Windows::Forms::Button();
			this->tabPage_Contour = new System::Windows::Forms::TabPage();
			this->pan_Contour = new System::Windows::Forms::Panel();
			this->lab_ContourKTF = new System::Windows::Forms::Label();
			this->nud_ContourKTF = new System::Windows::Forms::NumericUpDown();
			this->but_ContourTransferFunc = new System::Windows::Forms::Button();
			this->rab_ContourDvr = new System::Windows::Forms::RadioButton();
			this->rab_ContourLMip = new System::Windows::Forms::RadioButton();
			this->but_ContourColor = new System::Windows::Forms::Button();
			this->lab_ContourColor = new System::Windows::Forms::Label();
			this->lab_ContourExponent = new System::Windows::Forms::Label();
			this->lab_ContourGradient = new System::Windows::Forms::Label();
			this->nud_ContourExponent = new System::Windows::Forms::NumericUpDown();
			this->nud_ContourGradient = new System::Windows::Forms::NumericUpDown();
			this->lab_ContourThreshold = new System::Windows::Forms::Label();
			this->tex_ContourThreshold = new System::Windows::Forms::TextBox();
			this->trb_ContourThreshold = new System::Windows::Forms::TrackBar();
			this->but_ContourAssign = new System::Windows::Forms::Button();
			this->but_ContourReset = new System::Windows::Forms::Button();
			this->chb_TwoLevelVR = new System::Windows::Forms::CheckBox();
			this->grb_Rendering = new System::Windows::Forms::GroupBox();
			this->but_Render = new System::Windows::Forms::Button();
			this->tabControl1 = new System::Windows::Forms::TabControl();
			this->tabPage_Basic = new System::Windows::Forms::TabPage();
			this->lab_ImageRes = new System::Windows::Forms::Label();
			this->trb_ImageRes = new System::Windows::Forms::TrackBar();
			this->trb_CanvasSize = new System::Windows::Forms::TrackBar();
			this->lab_CanvasSize = new System::Windows::Forms::Label();
			this->cob_SamplingFactor = new System::Windows::Forms::ComboBox();
			this->tabPage_Optimation = new System::Windows::Forms::TabPage();
			this->lab_SamplingDist = new System::Windows::Forms::Label();
			this->trb_SamplingDist = new System::Windows::Forms::TrackBar();
			this->lab_OpacityThreshold = new System::Windows::Forms::Label();
			this->nud_OpacityThreshold = new System::Windows::Forms::NumericUpDown();
			this->lab_GradientTreshold = new System::Windows::Forms::Label();
			this->nud_GradientThreshold = new System::Windows::Forms::NumericUpDown();
			this->tex_SamplingDist = new System::Windows::Forms::TextBox();
			this->rightSandBarDock = new TD::SandBar::ToolBarContainer();
			this->topSandBarDock = new TD::SandBar::ToolBarContainer();
			this->mam_MenueBar = new TD::SandBar::MenuBar();
			this->mItem_File = new TD::SandBar::MenuBarItem();
			this->subItem_LoadDataFile = new TD::SandBar::MenuButtonItem();
			this->subItem_LoadMaskFile = new TD::SandBar::MenuButtonItem();
			this->subItem_LoadTransFunc = new TD::SandBar::MenuButtonItem();
			this->subItem_SaveTransFunc = new TD::SandBar::MenuButtonItem();
			this->subItem_LoadConfigFile = new TD::SandBar::MenuButtonItem();
			this->subItem_SaveConfigFile = new TD::SandBar::MenuButtonItem();
			this->subItem_SaveRenderedPicture = new TD::SandBar::MenuButtonItem();
			this->mItem_Settings = new TD::SandBar::MenuBarItem();
			this->subItem_LightProps = new TD::SandBar::MenuButtonItem();
			this->subItem_LightColor = new TD::SandBar::MenuButtonItem();
			this->subItem_LightViewMode = new TD::SandBar::MenuButtonItem();
			this->subItem_LightViewModeABS = new TD::SandBar::MenuButtonItem();
			this->subItem_LightViewModeREL = new TD::SandBar::MenuButtonItem();
			this->subItem_BackColor = new TD::SandBar::MenuButtonItem();
			this->mItem_Tools = new TD::SandBar::MenuBarItem();
			this->subItem_PreviewWindow = new TD::SandBar::MenuButtonItem();
			this->subItem_PreviewCanvasDefaultPosition = new TD::SandBar::MenuButtonItem();
			this->subItem_EditTransferFunction = new TD::SandBar::MenuButtonItem();
			this->subItem_SliceViewer = new TD::SandBar::MenuButtonItem();
			this->mItem_Help = new TD::SandBar::MenuBarItem();
			this->subItem_About = new TD::SandBar::MenuButtonItem();
			this->tob_FirstBar = new TD::SandBar::ToolBar();
			this->tbb_LoadData = new TD::SandBar::ButtonItem();
			this->tbb_LoadMask = new TD::SandBar::ButtonItem();
			this->tbc_FilePath = new TD::SandBar::ComboBoxItem();
			this->tbb_BackColor = new TD::SandBar::ButtonItem();
			this->tbl_BackColor = new TD::SandBar::LabelItem();
			this->tbc_RenderLevel = new TD::SandBar::ComboBoxItem();
			this->tbMenuItem_Rendering = new TD::SandBar::DropDownMenuItem();
			this->tbSubItem_StartRendering = new TD::SandBar::MenuButtonItem();
			this->tbSubItem_StopRendering = new TD::SandBar::MenuButtonItem();
			this->tbMenuItem_Tools = new TD::SandBar::DropDownMenuItem();
			this->tbSubItem_PreviewWindow = new TD::SandBar::MenuButtonItem();
			this->tbSubItem_EditTransferfunction = new TD::SandBar::MenuButtonItem();
			this->tbSubItem_SliceViewer = new TD::SandBar::MenuButtonItem();
			this->tob_SecondBar = new TD::SandBar::ToolBar();
			this->tbb_LightColor = new TD::SandBar::ButtonItem();
			this->tbl_LightColor = new TD::SandBar::LabelItem();
			this->tbl_LightCoords = new TD::SandBar::LabelItem();
			this->tbb_LightCWRotation = new TD::SandBar::ButtonItem();
			this->tbb_LightCCWRotation = new TD::SandBar::ButtonItem();
			this->tbc_LightRotationAxis = new TD::SandBar::ComboBoxItem();
			this->tbc_ViewDirection = new TD::SandBar::ComboBoxItem();
			this->tbl_Transformation = new TD::SandBar::LabelItem();
			this->tbb_TranslateLeft = new TD::SandBar::ButtonItem();
			this->tbb_TranslateRight = new TD::SandBar::ButtonItem();
			this->tbb_TranslateUp = new TD::SandBar::ButtonItem();
			this->tbb_TranslateDown = new TD::SandBar::ButtonItem();
			this->tbb_ZoomIn = new TD::SandBar::ButtonItem();
			this->tbb_ZoomOut = new TD::SandBar::ButtonItem();
			this->tbb_VPCWRotation = new TD::SandBar::ButtonItem();
			this->tbb_VPCCWRotation = new TD::SandBar::ButtonItem();
			this->tbc_VPRotationAxis = new TD::SandBar::ComboBoxItem();
			this->tbMenueItem_CanvasControl = new TD::SandBar::DropDownMenuItem();
			this->tbSubItem_ClearCanvas = new TD::SandBar::MenuButtonItem();
			this->tbSubItem_ResetVD = new TD::SandBar::MenuButtonItem();
			this->tbSubItem_ResetLight = new TD::SandBar::MenuButtonItem();
			this->tbSubItem_ResetLigthAndView = new TD::SandBar::MenuButtonItem();
			this->stb_StatusBar = new System::Windows::Forms::StatusBar();
			this->sbp_VolumeDimension = new System::Windows::Forms::StatusBarPanel();
			this->sbp_ViewDirection = new System::Windows::Forms::StatusBarPanel();
			this->sbp_LightDirection = new System::Windows::Forms::StatusBarPanel();
			this->sbp_RayDistance = new System::Windows::Forms::StatusBarPanel();
			this->sbp_DenityMode = new System::Windows::Forms::StatusBarPanel();
			this->sbp_GradientMode = new System::Windows::Forms::StatusBarPanel();
			this->sbp_VoxelIDMode = new System::Windows::Forms::StatusBarPanel();
			this->sbp_Gradient = new System::Windows::Forms::StatusBarPanel();
			this->sbp_RenderMode = new System::Windows::Forms::StatusBarPanel();
			this->sbp_Status = new System::Windows::Forms::StatusBarPanel();
			this->openDia_LoadMaskFile = new System::Windows::Forms::OpenFileDialog();
			this->openDia_LoadDataFile = new System::Windows::Forms::OpenFileDialog();
			this->colDia_BackColor = new System::Windows::Forms::ColorDialog();
			this->colDia_LightColor = new System::Windows::Forms::ColorDialog();
			this->colDia_MipColor = new System::Windows::Forms::ColorDialog();
			this->colDia_ContourColor = new System::Windows::Forms::ColorDialog();
			this->colDia_ToneColorWarm = new System::Windows::Forms::ColorDialog();
			this->colDia_ToneColorCold = new System::Windows::Forms::ColorDialog();
			this->openDia_LoadConfigFile = new System::Windows::Forms::OpenFileDialog();
			this->openDia_LoadTransFunc = new System::Windows::Forms::OpenFileDialog();
			this->saveDia_SaveConfigFile = new System::Windows::Forms::SaveFileDialog();
			this->saveDia_SaveTransFunc = new System::Windows::Forms::SaveFileDialog();
			this->tot_ToolTips = new System::Windows::Forms::ToolTip(this->components);
			this->saveDia_SaveRenderedPicture = new System::Windows::Forms::SaveFileDialog();
			this->comboBoxItem1 = new TD::SandBar::ComboBoxItem();
			this->tbSubItem_AnimationEditor = new TD::SandBar::MenuButtonItem();
			this->subItem_AnimationEditor = new TD::SandBar::MenuButtonItem();
			this->leftSandBarDock->SuspendLayout();
			this->ctb_RenderControl->SuspendLayout();
			this->containerBarClientPanel1->SuspendLayout();
			this->grb_IDVR->SuspendLayout();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_IDVRGScale))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_IDVRCountersinkSlope))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_IDVRObjectImportance))->BeginInit();
			this->grb_RenderControl->SuspendLayout();
			this->tab_RenderControl->SuspendLayout();
			this->tabPage_DVR->SuspendLayout();
			this->pan_Dvr->SuspendLayout();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_SpExponent))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_Specular))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_Diffuse))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_Ambient))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_DepthK2))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_DepthK1))->BeginInit();
			this->tabPage_MIP->SuspendLayout();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->trb_MipThreshold))->BeginInit();
			this->tabPage_Tone->SuspendLayout();
			this->pan_Tone->SuspendLayout();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_ToneKTF))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_ToneKC))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_ToneKW))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_ToneGradient))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->trb_ToneThreshold))->BeginInit();
			this->tabPage_Contour->SuspendLayout();
			this->pan_Contour->SuspendLayout();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_ContourKTF))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_ContourExponent))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_ContourGradient))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->trb_ContourThreshold))->BeginInit();
			this->grb_Rendering->SuspendLayout();
			this->tabControl1->SuspendLayout();
			this->tabPage_Basic->SuspendLayout();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->trb_ImageRes))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->trb_CanvasSize))->BeginInit();
			this->tabPage_Optimation->SuspendLayout();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->trb_SamplingDist))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_OpacityThreshold))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_GradientThreshold))->BeginInit();
			this->topSandBarDock->SuspendLayout();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->sbp_VolumeDimension))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->sbp_ViewDirection))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->sbp_LightDirection))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->sbp_RayDistance))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->sbp_DenityMode))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->sbp_GradientMode))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->sbp_VoxelIDMode))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->sbp_Gradient))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->sbp_RenderMode))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->sbp_Status))->BeginInit();
			this->SuspendLayout();
			// 
			// sbM_GUIManager
			// 
			this->sbM_GUIManager->BottomContainer = this->bottomSandBarDock;
			this->sbM_GUIManager->LeftContainer = this->leftSandBarDock;
			this->sbM_GUIManager->OwnerForm = this;
			this->sbM_GUIManager->Renderer = new TD::SandBar::WhidbeyRenderer();
			this->sbM_GUIManager->RightContainer = this->rightSandBarDock;
			this->sbM_GUIManager->TopContainer = this->topSandBarDock;
			// 
			// bottomSandBarDock
			// 
			this->bottomSandBarDock->Dock = System::Windows::Forms::DockStyle::Bottom;
			this->bottomSandBarDock->Location = System::Drawing::Point(0, 902);
			this->bottomSandBarDock->Manager = this->sbM_GUIManager;
			this->bottomSandBarDock->Name = S"bottomSandBarDock";
			this->bottomSandBarDock->Size = System::Drawing::Size(1016, 0);
			this->bottomSandBarDock->TabIndex = 2;
			// 
			// leftSandBarDock
			// 
			this->leftSandBarDock->Controls->Add(this->ctb_RenderControl);
			this->leftSandBarDock->Dock = System::Windows::Forms::DockStyle::Left;
			this->leftSandBarDock->Location = System::Drawing::Point(0, 76);
			this->leftSandBarDock->Manager = this->sbM_GUIManager;
			this->leftSandBarDock->Name = S"leftSandBarDock";
			this->leftSandBarDock->Size = System::Drawing::Size(225, 804);
			this->leftSandBarDock->TabIndex = 0;
			// 
			// ctb_RenderControl
			// 
			TD::SandBar::ToolbarItemBase* __mcTemp__1[] = new TD::SandBar::ToolbarItemBase*[3];
			__mcTemp__1[0] = this->mItem_Interpolation;
			__mcTemp__1[1] = this->mItem_Gradient;
			__mcTemp__1[2] = this->mItem_RendererType;
			this->ctb_RenderControl->Buttons->AddRange(__mcTemp__1);
			this->ctb_RenderControl->Closable = false;
			this->ctb_RenderControl->Controls->Add(this->containerBarClientPanel1);
			this->ctb_RenderControl->DockLine = 1;
			this->ctb_RenderControl->Guid = System::Guid(S"8f041be6-dcb9-4b57-aca3-e16e43eb437e");
			this->ctb_RenderControl->Location = System::Drawing::Point(0, 2);
			this->ctb_RenderControl->MinimumSize = System::Drawing::Size(225, 284);
			this->ctb_RenderControl->Movable = false;
			this->ctb_RenderControl->Name = S"ctb_RenderControl";
			this->ctb_RenderControl->Resizable = false;
			this->ctb_RenderControl->Size = System::Drawing::Size(225, 802);
			this->ctb_RenderControl->TabIndex = 0;
			this->ctb_RenderControl->Text = S"Render Controls";
			// 
			// mItem_Interpolation
			// 
			TD::SandBar::MenuButtonItem* __mcTemp__2[] = new TD::SandBar::MenuButtonItem*[5];
			__mcTemp__2[0] = this->subItem_Density;
			__mcTemp__2[1] = this->subItem_Gradient;
			__mcTemp__2[2] = this->subItem_VoxelID;
			__mcTemp__2[3] = this->subItem_AllNN;
			__mcTemp__2[4] = this->subItem_AllTL;
			this->mItem_Interpolation->MenuItems->AddRange(__mcTemp__2);
			this->mItem_Interpolation->Text = S"Interpolation";
			// 
			// subItem_Density
			// 
			TD::SandBar::MenuButtonItem* __mcTemp__3[] = new TD::SandBar::MenuButtonItem*[2];
			__mcTemp__3[0] = this->subItem_DensityNN;
			__mcTemp__3[1] = this->subItem_DensityTL;
			this->subItem_Density->MenuItems->AddRange(__mcTemp__3);
			this->subItem_Density->Text = S"Density Interpolation by";
			// 
			// subItem_DensityNN
			// 
			this->subItem_DensityNN->Text = S"Nearest Neighbour";
			this->subItem_DensityNN->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// subItem_DensityTL
			// 
			this->subItem_DensityTL->Checked = true;
			this->subItem_DensityTL->Text = S"TriLinear";
			this->subItem_DensityTL->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// subItem_Gradient
			// 
			TD::SandBar::MenuButtonItem* __mcTemp__4[] = new TD::SandBar::MenuButtonItem*[2];
			__mcTemp__4[0] = this->subItem_GradientNN;
			__mcTemp__4[1] = this->subItem_GradientTL;
			this->subItem_Gradient->MenuItems->AddRange(__mcTemp__4);
			this->subItem_Gradient->Text = S"Gradient Interpolation by";
			this->subItem_Gradient->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// subItem_GradientNN
			// 
			this->subItem_GradientNN->Text = S"Nearest Neighbour";
			this->subItem_GradientNN->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// subItem_GradientTL
			// 
			this->subItem_GradientTL->Checked = true;
			this->subItem_GradientTL->Text = S"TriLinear";
			this->subItem_GradientTL->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// subItem_VoxelID
			// 
			TD::SandBar::MenuButtonItem* __mcTemp__5[] = new TD::SandBar::MenuButtonItem*[2];
			__mcTemp__5[0] = this->subItem_VoxelIDNN;
			__mcTemp__5[1] = this->subItem_VoxelIDTL;
			this->subItem_VoxelID->MenuItems->AddRange(__mcTemp__5);
			this->subItem_VoxelID->Text = S"VoxelID Interpolation by";
			this->subItem_VoxelID->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// subItem_VoxelIDNN
			// 
			this->subItem_VoxelIDNN->Checked = true;
			this->subItem_VoxelIDNN->Text = S"Nearest Neighbour";
			this->subItem_VoxelIDNN->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// subItem_VoxelIDTL
			// 
			this->subItem_VoxelIDTL->Text = S"TriLinear";
			this->subItem_VoxelIDTL->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// subItem_AllNN
			// 
			this->subItem_AllNN->Text = S"Nearest Neighbour for all";
			this->subItem_AllNN->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// subItem_AllTL
			// 
			this->subItem_AllTL->Text = S"TriLinear for all";
			this->subItem_AllTL->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// mItem_Gradient
			// 
			TD::SandBar::MenuButtonItem* __mcTemp__6[] = new TD::SandBar::MenuButtonItem*[2];
			__mcTemp__6[0] = this->subItem_Central;
			__mcTemp__6[1] = this->subItem_Regression;
			this->mItem_Gradient->MenuItems->AddRange(__mcTemp__6);
			this->mItem_Gradient->Text = S"Gradient";
			// 
			// subItem_Central
			// 
			this->subItem_Central->Checked = true;
			this->subItem_Central->Text = S"by Central Differences";
			this->subItem_Central->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// subItem_Regression
			// 
			this->subItem_Regression->Text = S"by 4D Regression";
			this->subItem_Regression->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// mItem_RendererType
			// 
			TD::SandBar::MenuButtonItem* __mcTemp__7[] = new TD::SandBar::MenuButtonItem*[2];
			__mcTemp__7[0] = this->subItem_Software;
			__mcTemp__7[1] = this->subItem_Hardware;
			this->mItem_RendererType->MenuItems->AddRange(__mcTemp__7);
			this->mItem_RendererType->Text = S"Type";
			// 
			// subItem_Software
			// 
			this->subItem_Software->Checked = true;
			this->subItem_Software->Text = S"Software Renderer";
			this->subItem_Software->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// subItem_Hardware
			// 
			this->subItem_Hardware->Text = S"Hardware Renderer";
			this->subItem_Hardware->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// containerBarClientPanel1
			// 
			this->containerBarClientPanel1->BorderStyle = System::Windows::Forms::BorderStyle::Fixed3D;
			this->containerBarClientPanel1->Controls->Add(this->grb_IDVR);
			this->containerBarClientPanel1->Controls->Add(this->grb_RenderControl);
			this->containerBarClientPanel1->Controls->Add(this->grb_Rendering);
			this->containerBarClientPanel1->Location = System::Drawing::Point(3, 50);
			this->containerBarClientPanel1->Name = S"containerBarClientPanel1";
			this->containerBarClientPanel1->Size = System::Drawing::Size(219, 749);
			this->containerBarClientPanel1->TabIndex = 0;
			// 
			// grb_IDVR
			// 
			this->grb_IDVR->Controls->Add(this->nud_IDVRGScale);
			this->grb_IDVR->Controls->Add(this->lab_IDVRGScale);
			this->grb_IDVR->Controls->Add(this->nud_IDVRCountersinkSlope);
			this->grb_IDVR->Controls->Add(this->lab_IDVRCountersinkSlope);
			this->grb_IDVR->Controls->Add(this->cob_IDVRModulation);
			this->grb_IDVR->Controls->Add(this->cob_IDVRCompositing);
			this->grb_IDVR->Controls->Add(this->lab_IDVRModulation);
			this->grb_IDVR->Controls->Add(this->lab_IDVRCompositing);
			this->grb_IDVR->Controls->Add(this->but_IDVRAssign);
			this->grb_IDVR->Controls->Add(this->but_IDVRReset);
			this->grb_IDVR->Controls->Add(this->nud_IDVRObjectImportance);
			this->grb_IDVR->Controls->Add(this->lab_IDVRObject);
			this->grb_IDVR->Controls->Add(this->cob_IDVRObject);
			this->grb_IDVR->Controls->Add(this->chb_IDVR);
			this->grb_IDVR->Location = System::Drawing::Point(1, 592);
			this->grb_IDVR->Name = S"grb_IDVR";
			this->grb_IDVR->Size = System::Drawing::Size(215, 156);
			this->grb_IDVR->TabIndex = 2;
			this->grb_IDVR->TabStop = false;
			this->grb_IDVR->Text = S"Importance Control";
			// 
			// nud_IDVRGScale
			// 
			this->nud_IDVRGScale->DecimalPlaces = 1;
			System::Int32 __mcTemp__8[] = new System::Int32[4];
			__mcTemp__8[0] = 1;
			__mcTemp__8[1] = 0;
			__mcTemp__8[2] = 0;
			__mcTemp__8[3] = 65536;
			this->nud_IDVRGScale->Increment = System::Decimal(__mcTemp__8);
			this->nud_IDVRGScale->Location = System::Drawing::Point(164, 108);
			System::Int32 __mcTemp__9[] = new System::Int32[4];
			__mcTemp__9[0] = 1;
			__mcTemp__9[1] = 0;
			__mcTemp__9[2] = 0;
			__mcTemp__9[3] = 65536;
			this->nud_IDVRGScale->Minimum = System::Decimal(__mcTemp__9);
			this->nud_IDVRGScale->Name = S"nud_IDVRGScale";
			this->nud_IDVRGScale->Size = System::Drawing::Size(44, 20);
			this->nud_IDVRGScale->TabIndex = 47;
			this->nud_IDVRGScale->TextAlign = System::Windows::Forms::HorizontalAlignment::Right;
			System::Int32 __mcTemp__10[] = new System::Int32[4];
			__mcTemp__10[0] = 1;
			__mcTemp__10[1] = 0;
			__mcTemp__10[2] = 0;
			__mcTemp__10[3] = 0;
			this->nud_IDVRGScale->Value = System::Decimal(__mcTemp__10);
			// 
			// lab_IDVRGScale
			// 
			this->lab_IDVRGScale->Location = System::Drawing::Point(112, 108);
			this->lab_IDVRGScale->Name = S"lab_IDVRGScale";
			this->lab_IDVRGScale->Size = System::Drawing::Size(48, 20);
			this->lab_IDVRGScale->TabIndex = 46;
			this->lab_IDVRGScale->Text = S"G_Scale";
			this->lab_IDVRGScale->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
			// 
			// nud_IDVRCountersinkSlope
			// 
			this->nud_IDVRCountersinkSlope->DecimalPlaces = 1;
			System::Int32 __mcTemp__11[] = new System::Int32[4];
			__mcTemp__11[0] = 1;
			__mcTemp__11[1] = 0;
			__mcTemp__11[2] = 0;
			__mcTemp__11[3] = 65536;
			this->nud_IDVRCountersinkSlope->Increment = System::Decimal(__mcTemp__11);
			this->nud_IDVRCountersinkSlope->Location = System::Drawing::Point(64, 108);
			System::Int32 __mcTemp__12[] = new System::Int32[4];
			__mcTemp__12[0] = 1;
			__mcTemp__12[1] = 0;
			__mcTemp__12[2] = 0;
			__mcTemp__12[3] = 65536;
			this->nud_IDVRCountersinkSlope->Minimum = System::Decimal(__mcTemp__12);
			this->nud_IDVRCountersinkSlope->Name = S"nud_IDVRCountersinkSlope";
			this->nud_IDVRCountersinkSlope->Size = System::Drawing::Size(44, 20);
			this->nud_IDVRCountersinkSlope->TabIndex = 45;
			this->nud_IDVRCountersinkSlope->TextAlign = System::Windows::Forms::HorizontalAlignment::Right;
			System::Int32 __mcTemp__13[] = new System::Int32[4];
			__mcTemp__13[0] = 1;
			__mcTemp__13[1] = 0;
			__mcTemp__13[2] = 0;
			__mcTemp__13[3] = 0;
			this->nud_IDVRCountersinkSlope->Value = System::Decimal(__mcTemp__13);
			// 
			// lab_IDVRCountersinkSlope
			// 
			this->lab_IDVRCountersinkSlope->Location = System::Drawing::Point(9, 108);
			this->lab_IDVRCountersinkSlope->Name = S"lab_IDVRCountersinkSlope";
			this->lab_IDVRCountersinkSlope->Size = System::Drawing::Size(51, 20);
			this->lab_IDVRCountersinkSlope->TabIndex = 44;
			this->lab_IDVRCountersinkSlope->Text = S"C_Slope:";
			this->lab_IDVRCountersinkSlope->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
			// 
			// cob_IDVRModulation
			// 
			this->cob_IDVRModulation->Location = System::Drawing::Point(80, 80);
			this->cob_IDVRModulation->Name = S"cob_IDVRModulation";
			this->cob_IDVRModulation->Size = System::Drawing::Size(128, 21);
			this->cob_IDVRModulation->TabIndex = 43;
			// 
			// cob_IDVRCompositing
			// 
			this->cob_IDVRCompositing->Location = System::Drawing::Point(80, 60);
			this->cob_IDVRCompositing->Name = S"cob_IDVRCompositing";
			this->cob_IDVRCompositing->Size = System::Drawing::Size(128, 21);
			this->cob_IDVRCompositing->TabIndex = 42;
			// 
			// lab_IDVRModulation
			// 
			this->lab_IDVRModulation->Location = System::Drawing::Point(9, 84);
			this->lab_IDVRModulation->Name = S"lab_IDVRModulation";
			this->lab_IDVRModulation->Size = System::Drawing::Size(72, 20);
			this->lab_IDVRModulation->TabIndex = 41;
			this->lab_IDVRModulation->Text = S"Modulation:";
			this->lab_IDVRModulation->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
			// 
			// lab_IDVRCompositing
			// 
			this->lab_IDVRCompositing->Location = System::Drawing::Point(9, 60);
			this->lab_IDVRCompositing->Name = S"lab_IDVRCompositing";
			this->lab_IDVRCompositing->Size = System::Drawing::Size(71, 20);
			this->lab_IDVRCompositing->TabIndex = 40;
			this->lab_IDVRCompositing->Text = S"Compositing:";
			this->lab_IDVRCompositing->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
			// 
			// but_IDVRAssign
			// 
			this->but_IDVRAssign->BackColor = System::Drawing::SystemColors::Control;
			this->but_IDVRAssign->FlatStyle = System::Windows::Forms::FlatStyle::Popup;
			this->but_IDVRAssign->Location = System::Drawing::Point(4, 132);
			this->but_IDVRAssign->Name = S"but_IDVRAssign";
			this->but_IDVRAssign->Size = System::Drawing::Size(128, 20);
			this->but_IDVRAssign->TabIndex = 38;
			this->but_IDVRAssign->Text = S"Assign Features";
			this->tot_ToolTips->SetToolTip(this->but_IDVRAssign, S"Set new features to DVR Renderer");
			this->but_IDVRAssign->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// but_IDVRReset
			// 
			this->but_IDVRReset->BackColor = System::Drawing::SystemColors::Control;
			this->but_IDVRReset->FlatStyle = System::Windows::Forms::FlatStyle::Popup;
			this->but_IDVRReset->Location = System::Drawing::Point(132, 132);
			this->but_IDVRReset->Name = S"but_IDVRReset";
			this->but_IDVRReset->Size = System::Drawing::Size(80, 20);
			this->but_IDVRReset->TabIndex = 39;
			this->but_IDVRReset->Text = S"Reset";
			this->tot_ToolTips->SetToolTip(this->but_IDVRReset, S"Set DVR Renderer to default status");
			this->but_IDVRReset->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// nud_IDVRObjectImportance
			// 
			this->nud_IDVRObjectImportance->Location = System::Drawing::Point(164, 36);
			System::Int32 __mcTemp__14[] = new System::Int32[4];
			__mcTemp__14[0] = 20;
			__mcTemp__14[1] = 0;
			__mcTemp__14[2] = 0;
			__mcTemp__14[3] = 0;
			this->nud_IDVRObjectImportance->Maximum = System::Decimal(__mcTemp__14);
			this->nud_IDVRObjectImportance->Name = S"nud_IDVRObjectImportance";
			this->nud_IDVRObjectImportance->Size = System::Drawing::Size(44, 20);
			this->nud_IDVRObjectImportance->TabIndex = 37;
			this->nud_IDVRObjectImportance->TextAlign = System::Windows::Forms::HorizontalAlignment::Right;
			this->nud_IDVRObjectImportance->ValueChanged += new System::EventHandler(this, perform_ActionNudValueChanged);
			// 
			// lab_IDVRObject
			// 
			this->lab_IDVRObject->Location = System::Drawing::Point(9, 36);
			this->lab_IDVRObject->Name = S"lab_IDVRObject";
			this->lab_IDVRObject->Size = System::Drawing::Size(43, 20);
			this->lab_IDVRObject->TabIndex = 35;
			this->lab_IDVRObject->Text = S"Object:";
			this->lab_IDVRObject->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
			// 
			// cob_IDVRObject
			// 
			this->cob_IDVRObject->Location = System::Drawing::Point(52, 36);
			this->cob_IDVRObject->Name = S"cob_IDVRObject";
			this->cob_IDVRObject->Size = System::Drawing::Size(108, 21);
			this->cob_IDVRObject->TabIndex = 34;
			this->cob_IDVRObject->SelectedIndexChanged += new System::EventHandler(this, perform_ActionComboBoxSelectedIndexChanged);
			// 
			// chb_IDVR
			// 
			this->chb_IDVR->Location = System::Drawing::Point(9, 16);
			this->chb_IDVR->Name = S"chb_IDVR";
			this->chb_IDVR->Size = System::Drawing::Size(196, 20);
			this->chb_IDVR->TabIndex = 36;
			this->chb_IDVR->Text = S"Activate IDVR";
			this->chb_IDVR->CheckedChanged += new System::EventHandler(this, perform_ActionCheckBoxCheckedChanged);
			// 
			// grb_RenderControl
			// 
			this->grb_RenderControl->Controls->Add(this->but_DeleteVolumeObject);
			this->grb_RenderControl->Controls->Add(this->but_LoadMaskFile);
			this->grb_RenderControl->Controls->Add(this->lab_TransObj);
			this->grb_RenderControl->Controls->Add(this->cob_TransObj);
			this->grb_RenderControl->Controls->Add(this->tab_RenderControl);
			this->grb_RenderControl->Controls->Add(this->chb_TwoLevelVR);
			this->grb_RenderControl->Location = System::Drawing::Point(1, 208);
			this->grb_RenderControl->Name = S"grb_RenderControl";
			this->grb_RenderControl->Size = System::Drawing::Size(215, 380);
			this->grb_RenderControl->TabIndex = 1;
			this->grb_RenderControl->TabStop = false;
			this->grb_RenderControl->Text = S"Rendering Control";
			// 
			// but_DeleteVolumeObject
			// 
			this->but_DeleteVolumeObject->FlatStyle = System::Windows::Forms::FlatStyle::Popup;
			this->but_DeleteVolumeObject->ImageIndex = 11;
			this->but_DeleteVolumeObject->ImageList = this->iml_ToolBarIcons;
			this->but_DeleteVolumeObject->Location = System::Drawing::Point(182, 40);
			this->but_DeleteVolumeObject->Name = S"but_DeleteVolumeObject";
			this->but_DeleteVolumeObject->Size = System::Drawing::Size(28, 28);
			this->but_DeleteVolumeObject->TabIndex = 35;
			this->tot_ToolTips->SetToolTip(this->but_DeleteVolumeObject, S"Delete the selected object");
			this->but_DeleteVolumeObject->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// iml_ToolBarIcons
			// 
			this->iml_ToolBarIcons->ImageSize = System::Drawing::Size(16, 16);
			this->iml_ToolBarIcons->ImageStream = (__try_cast<System::Windows::Forms::ImageListStreamer *  >(resources->GetObject(S"iml_ToolBarIcons.ImageStream")));
			this->iml_ToolBarIcons->TransparentColor = System::Drawing::Color::Transparent;
			// 
			// but_LoadMaskFile
			// 
			this->but_LoadMaskFile->FlatStyle = System::Windows::Forms::FlatStyle::Popup;
			this->but_LoadMaskFile->ImageIndex = 0;
			this->but_LoadMaskFile->ImageList = this->iml_ToolBarIcons;
			this->but_LoadMaskFile->Location = System::Drawing::Point(182, 12);
			this->but_LoadMaskFile->Name = S"but_LoadMaskFile";
			this->but_LoadMaskFile->Size = System::Drawing::Size(28, 28);
			this->but_LoadMaskFile->TabIndex = 34;
			this->tot_ToolTips->SetToolTip(this->but_LoadMaskFile, S"Add a new object");
			this->but_LoadMaskFile->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// lab_TransObj
			// 
			this->lab_TransObj->Location = System::Drawing::Point(8, 48);
			this->lab_TransObj->Name = S"lab_TransObj";
			this->lab_TransObj->Size = System::Drawing::Size(40, 21);
			this->lab_TransObj->TabIndex = 1;
			this->lab_TransObj->Text = S"Object:";
			this->lab_TransObj->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
			// 
			// cob_TransObj
			// 
			this->cob_TransObj->Location = System::Drawing::Point(48, 48);
			this->cob_TransObj->Name = S"cob_TransObj";
			this->cob_TransObj->Size = System::Drawing::Size(132, 21);
			this->cob_TransObj->TabIndex = 0;
			this->cob_TransObj->SelectedIndexChanged += new System::EventHandler(this, perform_ActionComboBoxSelectedIndexChanged);
			// 
			// tab_RenderControl
			// 
			this->tab_RenderControl->Controls->Add(this->tabPage_DVR);
			this->tab_RenderControl->Controls->Add(this->tabPage_MIP);
			this->tab_RenderControl->Controls->Add(this->tabPage_Tone);
			this->tab_RenderControl->Controls->Add(this->tabPage_Contour);
			this->tab_RenderControl->Location = System::Drawing::Point(5, 72);
			this->tab_RenderControl->Name = S"tab_RenderControl";
			this->tab_RenderControl->SelectedIndex = 0;
			this->tab_RenderControl->Size = System::Drawing::Size(206, 304);
			this->tab_RenderControl->TabIndex = 0;
			this->tab_RenderControl->SelectedIndexChanged += new System::EventHandler(this, perform_ActionTabPanelSelectedIndexChanged);
			// 
			// tabPage_DVR
			// 
			this->tabPage_DVR->Controls->Add(this->pan_Dvr);
			this->tabPage_DVR->Controls->Add(this->but_DVRAssign);
			this->tabPage_DVR->Controls->Add(this->but_DVRReset);
			this->tabPage_DVR->Location = System::Drawing::Point(4, 22);
			this->tabPage_DVR->Name = S"tabPage_DVR";
			this->tabPage_DVR->Size = System::Drawing::Size(198, 278);
			this->tabPage_DVR->TabIndex = 1;
			this->tabPage_DVR->Text = S"DVR";
			this->tabPage_DVR->ToolTipText = S"Use the DVR Shader model";
			// 
			// pan_Dvr
			// 
			this->pan_Dvr->AutoScroll = true;
			this->pan_Dvr->Controls->Add(this->pan_TFPreviewColor);
			this->pan_Dvr->Controls->Add(this->nud_SpExponent);
			this->pan_Dvr->Controls->Add(this->nud_Specular);
			this->pan_Dvr->Controls->Add(this->nud_Diffuse);
			this->pan_Dvr->Controls->Add(this->nud_Ambient);
			this->pan_Dvr->Controls->Add(this->lab_DVRTitle);
			this->pan_Dvr->Controls->Add(this->pan_TFPreview);
			this->pan_Dvr->Controls->Add(this->lab_Ambient);
			this->pan_Dvr->Controls->Add(this->lab_Diffuse);
			this->pan_Dvr->Controls->Add(this->lab_Specular);
			this->pan_Dvr->Controls->Add(this->lab_SpExponent);
			this->pan_Dvr->Controls->Add(this->nud_DepthK2);
			this->pan_Dvr->Controls->Add(this->lab_DepthK2);
			this->pan_Dvr->Controls->Add(this->nud_DepthK1);
			this->pan_Dvr->Controls->Add(this->lab_DepthK1);
			this->pan_Dvr->Controls->Add(this->lab_DepthCueing);
			this->pan_Dvr->Location = System::Drawing::Point(4, 0);
			this->pan_Dvr->Name = S"pan_Dvr";
			this->pan_Dvr->Size = System::Drawing::Size(200, 256);
			this->pan_Dvr->TabIndex = 7;
			// 
			// pan_TFPreviewColor
			// 
			this->pan_TFPreviewColor->BackColor = System::Drawing::SystemColors::ControlDark;
			this->pan_TFPreviewColor->BorderStyle = System::Windows::Forms::BorderStyle::FixedSingle;
			this->pan_TFPreviewColor->Location = System::Drawing::Point(10, 104);
			this->pan_TFPreviewColor->Name = S"pan_TFPreviewColor";
			this->pan_TFPreviewColor->Size = System::Drawing::Size(170, 15);
			this->pan_TFPreviewColor->TabIndex = 41;
			this->pan_TFPreviewColor->Paint += new System::Windows::Forms::PaintEventHandler(this, perform_ActionPanelPaint);
			// 
			// nud_SpExponent
			// 
			this->nud_SpExponent->Location = System::Drawing::Point(140, 184);
			System::Int32 __mcTemp__15[] = new System::Int32[4];
			__mcTemp__15[0] = 999;
			__mcTemp__15[1] = 0;
			__mcTemp__15[2] = 0;
			__mcTemp__15[3] = 0;
			this->nud_SpExponent->Maximum = System::Decimal(__mcTemp__15);
			this->nud_SpExponent->Name = S"nud_SpExponent";
			this->nud_SpExponent->Size = System::Drawing::Size(44, 20);
			this->nud_SpExponent->TabIndex = 40;
			this->nud_SpExponent->TextAlign = System::Windows::Forms::HorizontalAlignment::Right;
			this->tot_ToolTips->SetToolTip(this->nud_SpExponent, S"Edit Specular Exponent");
			System::Int32 __mcTemp__16[] = new System::Int32[4];
			__mcTemp__16[0] = 100;
			__mcTemp__16[1] = 0;
			__mcTemp__16[2] = 0;
			__mcTemp__16[3] = 0;
			this->nud_SpExponent->Value = System::Decimal(__mcTemp__16);
			// 
			// nud_Specular
			// 
			this->nud_Specular->DecimalPlaces = 2;
			System::Int32 __mcTemp__17[] = new System::Int32[4];
			__mcTemp__17[0] = 1;
			__mcTemp__17[1] = 0;
			__mcTemp__17[2] = 0;
			__mcTemp__17[3] = 131072;
			this->nud_Specular->Increment = System::Decimal(__mcTemp__17);
			this->nud_Specular->Location = System::Drawing::Point(140, 164);
			System::Int32 __mcTemp__18[] = new System::Int32[4];
			__mcTemp__18[0] = 1;
			__mcTemp__18[1] = 0;
			__mcTemp__18[2] = 0;
			__mcTemp__18[3] = 0;
			this->nud_Specular->Maximum = System::Decimal(__mcTemp__18);
			this->nud_Specular->Name = S"nud_Specular";
			this->nud_Specular->Size = System::Drawing::Size(44, 20);
			this->nud_Specular->TabIndex = 39;
			this->nud_Specular->TextAlign = System::Windows::Forms::HorizontalAlignment::Right;
			this->tot_ToolTips->SetToolTip(this->nud_Specular, S"Edit Specular coefficient");
			System::Int32 __mcTemp__19[] = new System::Int32[4];
			__mcTemp__19[0] = 50;
			__mcTemp__19[1] = 0;
			__mcTemp__19[2] = 0;
			__mcTemp__19[3] = 131072;
			this->nud_Specular->Value = System::Decimal(__mcTemp__19);
			// 
			// nud_Diffuse
			// 
			this->nud_Diffuse->DecimalPlaces = 2;
			System::Int32 __mcTemp__20[] = new System::Int32[4];
			__mcTemp__20[0] = 1;
			__mcTemp__20[1] = 0;
			__mcTemp__20[2] = 0;
			__mcTemp__20[3] = 131072;
			this->nud_Diffuse->Increment = System::Decimal(__mcTemp__20);
			this->nud_Diffuse->Location = System::Drawing::Point(140, 144);
			System::Int32 __mcTemp__21[] = new System::Int32[4];
			__mcTemp__21[0] = 1;
			__mcTemp__21[1] = 0;
			__mcTemp__21[2] = 0;
			__mcTemp__21[3] = 0;
			this->nud_Diffuse->Maximum = System::Decimal(__mcTemp__21);
			this->nud_Diffuse->Name = S"nud_Diffuse";
			this->nud_Diffuse->Size = System::Drawing::Size(44, 20);
			this->nud_Diffuse->TabIndex = 38;
			this->nud_Diffuse->TextAlign = System::Windows::Forms::HorizontalAlignment::Right;
			this->tot_ToolTips->SetToolTip(this->nud_Diffuse, S"Edit Diffuse coefficient");
			System::Int32 __mcTemp__22[] = new System::Int32[4];
			__mcTemp__22[0] = 80;
			__mcTemp__22[1] = 0;
			__mcTemp__22[2] = 0;
			__mcTemp__22[3] = 131072;
			this->nud_Diffuse->Value = System::Decimal(__mcTemp__22);
			// 
			// nud_Ambient
			// 
			this->nud_Ambient->DecimalPlaces = 2;
			System::Int32 __mcTemp__23[] = new System::Int32[4];
			__mcTemp__23[0] = 1;
			__mcTemp__23[1] = 0;
			__mcTemp__23[2] = 0;
			__mcTemp__23[3] = 131072;
			this->nud_Ambient->Increment = System::Decimal(__mcTemp__23);
			this->nud_Ambient->Location = System::Drawing::Point(140, 124);
			System::Int32 __mcTemp__24[] = new System::Int32[4];
			__mcTemp__24[0] = 1;
			__mcTemp__24[1] = 0;
			__mcTemp__24[2] = 0;
			__mcTemp__24[3] = 0;
			this->nud_Ambient->Maximum = System::Decimal(__mcTemp__24);
			this->nud_Ambient->Name = S"nud_Ambient";
			this->nud_Ambient->Size = System::Drawing::Size(44, 20);
			this->nud_Ambient->TabIndex = 37;
			this->nud_Ambient->TextAlign = System::Windows::Forms::HorizontalAlignment::Right;
			this->tot_ToolTips->SetToolTip(this->nud_Ambient, S"Edit Ambient coefficient");
			System::Int32 __mcTemp__25[] = new System::Int32[4];
			__mcTemp__25[0] = 5;
			__mcTemp__25[1] = 0;
			__mcTemp__25[2] = 0;
			__mcTemp__25[3] = 65536;
			this->nud_Ambient->Value = System::Decimal(__mcTemp__25);
			// 
			// lab_DVRTitle
			// 
			this->lab_DVRTitle->Font = new System::Drawing::Font(S"Microsoft Sans Serif", 8.25F, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, (System::Byte)0);
			this->lab_DVRTitle->Location = System::Drawing::Point(16, 4);
			this->lab_DVRTitle->Name = S"lab_DVRTitle";
			this->lab_DVRTitle->Size = System::Drawing::Size(152, 12);
			this->lab_DVRTitle->TabIndex = 0;
			this->lab_DVRTitle->Text = S"Preview of Transferfunction:";
			// 
			// pan_TFPreview
			// 
			this->pan_TFPreview->BackColor = System::Drawing::Color::Beige;
			this->pan_TFPreview->BorderStyle = System::Windows::Forms::BorderStyle::Fixed3D;
			this->pan_TFPreview->ForeColor = System::Drawing::SystemColors::ControlText;
			this->pan_TFPreview->Location = System::Drawing::Point(10, 20);
			this->pan_TFPreview->Name = S"pan_TFPreview";
			this->pan_TFPreview->Size = System::Drawing::Size(170, 88);
			this->pan_TFPreview->TabIndex = 3;
			this->tot_ToolTips->SetToolTip(this->pan_TFPreview, S"Click for edit Transfer Function");
			this->pan_TFPreview->Click += new System::EventHandler(this, perform_ActionPanelClick);
			this->pan_TFPreview->Paint += new System::Windows::Forms::PaintEventHandler(this, perform_ActionPanelPaint);
			// 
			// lab_Ambient
			// 
			this->lab_Ambient->Location = System::Drawing::Point(12, 128);
			this->lab_Ambient->Name = S"lab_Ambient";
			this->lab_Ambient->Size = System::Drawing::Size(98, 16);
			this->lab_Ambient->TabIndex = 0;
			this->lab_Ambient->Text = S"Ambient reflection:";
			this->tot_ToolTips->SetToolTip(this->lab_Ambient, S"Edit Ambient coefficient");
			// 
			// lab_Diffuse
			// 
			this->lab_Diffuse->Location = System::Drawing::Point(12, 148);
			this->lab_Diffuse->Name = S"lab_Diffuse";
			this->lab_Diffuse->Size = System::Drawing::Size(96, 16);
			this->lab_Diffuse->TabIndex = 1;
			this->lab_Diffuse->Text = S"Diffuse reflection:";
			this->tot_ToolTips->SetToolTip(this->lab_Diffuse, S"Edit Diffuse coefficient");
			// 
			// lab_Specular
			// 
			this->lab_Specular->Location = System::Drawing::Point(12, 168);
			this->lab_Specular->Name = S"lab_Specular";
			this->lab_Specular->Size = System::Drawing::Size(104, 16);
			this->lab_Specular->TabIndex = 2;
			this->lab_Specular->Text = S"Specular reflection:";
			this->tot_ToolTips->SetToolTip(this->lab_Specular, S"Edit Specular coefficient");
			// 
			// lab_SpExponent
			// 
			this->lab_SpExponent->Location = System::Drawing::Point(12, 188);
			this->lab_SpExponent->Name = S"lab_SpExponent";
			this->lab_SpExponent->Size = System::Drawing::Size(104, 16);
			this->lab_SpExponent->TabIndex = 1;
			this->lab_SpExponent->Text = S"Specular exponent:";
			this->tot_ToolTips->SetToolTip(this->lab_SpExponent, S"Edit Specular Exponent");
			// 
			// nud_DepthK2
			// 
			this->nud_DepthK2->DecimalPlaces = 2;
			System::Int32 __mcTemp__26[] = new System::Int32[4];
			__mcTemp__26[0] = 1;
			__mcTemp__26[1] = 0;
			__mcTemp__26[2] = 0;
			__mcTemp__26[3] = 65536;
			this->nud_DepthK2->Increment = System::Decimal(__mcTemp__26);
			this->nud_DepthK2->Location = System::Drawing::Point(140, 232);
			System::Int32 __mcTemp__27[] = new System::Int32[4];
			__mcTemp__27[0] = 1;
			__mcTemp__27[1] = 0;
			__mcTemp__27[2] = 0;
			__mcTemp__27[3] = 0;
			this->nud_DepthK2->Maximum = System::Decimal(__mcTemp__27);
			this->nud_DepthK2->Name = S"nud_DepthK2";
			this->nud_DepthK2->Size = System::Drawing::Size(44, 20);
			this->nud_DepthK2->TabIndex = 36;
			this->nud_DepthK2->TextAlign = System::Windows::Forms::HorizontalAlignment::Right;
			this->tot_ToolTips->SetToolTip(this->nud_DepthK2, S"Edit second constant for Depth Cueing");
			// 
			// lab_DepthK2
			// 
			this->lab_DepthK2->Location = System::Drawing::Point(104, 232);
			this->lab_DepthK2->Name = S"lab_DepthK2";
			this->lab_DepthK2->Size = System::Drawing::Size(32, 20);
			this->lab_DepthK2->TabIndex = 35;
			this->lab_DepthK2->Text = S"K2:";
			this->lab_DepthK2->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
			// 
			// nud_DepthK1
			// 
			this->nud_DepthK1->DecimalPlaces = 2;
			System::Int32 __mcTemp__28[] = new System::Int32[4];
			__mcTemp__28[0] = 1;
			__mcTemp__28[1] = 0;
			__mcTemp__28[2] = 0;
			__mcTemp__28[3] = 65536;
			this->nud_DepthK1->Increment = System::Decimal(__mcTemp__28);
			this->nud_DepthK1->Location = System::Drawing::Point(48, 232);
			System::Int32 __mcTemp__29[] = new System::Int32[4];
			__mcTemp__29[0] = 1;
			__mcTemp__29[1] = 0;
			__mcTemp__29[2] = 0;
			__mcTemp__29[3] = 0;
			this->nud_DepthK1->Maximum = System::Decimal(__mcTemp__29);
			System::Int32 __mcTemp__30[] = new System::Int32[4];
			__mcTemp__30[0] = 1;
			__mcTemp__30[1] = 0;
			__mcTemp__30[2] = 0;
			__mcTemp__30[3] = 65536;
			this->nud_DepthK1->Minimum = System::Decimal(__mcTemp__30);
			this->nud_DepthK1->Name = S"nud_DepthK1";
			this->nud_DepthK1->Size = System::Drawing::Size(44, 20);
			this->nud_DepthK1->TabIndex = 33;
			this->nud_DepthK1->TextAlign = System::Windows::Forms::HorizontalAlignment::Right;
			this->tot_ToolTips->SetToolTip(this->nud_DepthK1, S"Edit first constant for Depth Cueing");
			System::Int32 __mcTemp__31[] = new System::Int32[4];
			__mcTemp__31[0] = 10;
			__mcTemp__31[1] = 0;
			__mcTemp__31[2] = 0;
			__mcTemp__31[3] = 65536;
			this->nud_DepthK1->Value = System::Decimal(__mcTemp__31);
			// 
			// lab_DepthK1
			// 
			this->lab_DepthK1->Location = System::Drawing::Point(12, 232);
			this->lab_DepthK1->Name = S"lab_DepthK1";
			this->lab_DepthK1->Size = System::Drawing::Size(32, 20);
			this->lab_DepthK1->TabIndex = 34;
			this->lab_DepthK1->Text = S"K1:";
			this->lab_DepthK1->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
			// 
			// lab_DepthCueing
			// 
			this->lab_DepthCueing->Location = System::Drawing::Point(12, 212);
			this->lab_DepthCueing->Name = S"lab_DepthCueing";
			this->lab_DepthCueing->Size = System::Drawing::Size(136, 16);
			this->lab_DepthCueing->TabIndex = 32;
			this->lab_DepthCueing->Text = S"Depth Cueing Properties:";
			// 
			// but_DVRAssign
			// 
			this->but_DVRAssign->BackColor = System::Drawing::SystemColors::Control;
			this->but_DVRAssign->FlatStyle = System::Windows::Forms::FlatStyle::Popup;
			this->but_DVRAssign->Location = System::Drawing::Point(0, 256);
			this->but_DVRAssign->Name = S"but_DVRAssign";
			this->but_DVRAssign->Size = System::Drawing::Size(128, 20);
			this->but_DVRAssign->TabIndex = 5;
			this->but_DVRAssign->Text = S"Assign Features";
			this->tot_ToolTips->SetToolTip(this->but_DVRAssign, S"Set new features to DVR Renderer");
			this->but_DVRAssign->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// but_DVRReset
			// 
			this->but_DVRReset->BackColor = System::Drawing::SystemColors::Control;
			this->but_DVRReset->FlatStyle = System::Windows::Forms::FlatStyle::Popup;
			this->but_DVRReset->Location = System::Drawing::Point(128, 256);
			this->but_DVRReset->Name = S"but_DVRReset";
			this->but_DVRReset->Size = System::Drawing::Size(70, 20);
			this->but_DVRReset->TabIndex = 8;
			this->but_DVRReset->Text = S"Reset";
			this->tot_ToolTips->SetToolTip(this->but_DVRReset, S"Set DVR Renderer to default status");
			this->but_DVRReset->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// tabPage_MIP
			// 
			this->tabPage_MIP->Controls->Add(this->but_MipTransferFunc);
			this->tabPage_MIP->Controls->Add(this->lab_MipNote);
			this->tabPage_MIP->Controls->Add(this->rab_MipTFClassification);
			this->tabPage_MIP->Controls->Add(this->rab_MipLinClassification);
			this->tabPage_MIP->Controls->Add(this->lab_MipColor);
			this->tabPage_MIP->Controls->Add(this->but_MipColor);
			this->tabPage_MIP->Controls->Add(this->tex_MipThreshold);
			this->tabPage_MIP->Controls->Add(this->lab_MipThreshold);
			this->tabPage_MIP->Controls->Add(this->trb_MipThreshold);
			this->tabPage_MIP->Controls->Add(this->but_MipAssign);
			this->tabPage_MIP->Controls->Add(this->but_MipReset);
			this->tabPage_MIP->Location = System::Drawing::Point(4, 22);
			this->tabPage_MIP->Name = S"tabPage_MIP";
			this->tabPage_MIP->Size = System::Drawing::Size(198, 278);
			this->tabPage_MIP->TabIndex = 2;
			this->tabPage_MIP->Text = S"MIP";
			this->tabPage_MIP->ToolTipText = S"Use the MIP (LMIP) Shader model";
			this->tabPage_MIP->Visible = false;
			// 
			// but_MipTransferFunc
			// 
			this->but_MipTransferFunc->FlatStyle = System::Windows::Forms::FlatStyle::Popup;
			this->but_MipTransferFunc->Location = System::Drawing::Point(8, 100);
			this->but_MipTransferFunc->Name = S"but_MipTransferFunc";
			this->but_MipTransferFunc->Size = System::Drawing::Size(184, 20);
			this->but_MipTransferFunc->TabIndex = 9;
			this->but_MipTransferFunc->Text = S"Edit Transfer Function";
			this->but_MipTransferFunc->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// lab_MipNote
			// 
			this->lab_MipNote->Location = System::Drawing::Point(8, 56);
			this->lab_MipNote->Name = S"lab_MipNote";
			this->lab_MipNote->Size = System::Drawing::Size(184, 40);
			this->lab_MipNote->TabIndex = 8;
			this->lab_MipNote->Text = S"Note: Only the opacity of the transfer function modifies the final image.";
			// 
			// rab_MipTFClassification
			// 
			this->rab_MipTFClassification->Location = System::Drawing::Point(8, 32);
			this->rab_MipTFClassification->Name = S"rab_MipTFClassification";
			this->rab_MipTFClassification->Size = System::Drawing::Size(184, 20);
			this->rab_MipTFClassification->TabIndex = 7;
			this->rab_MipTFClassification->Text = S"Transfer Function Classification";
			this->rab_MipTFClassification->CheckedChanged += new System::EventHandler(this, perform_ActionRadioButtonCheckedChanged);
			// 
			// rab_MipLinClassification
			// 
			this->rab_MipLinClassification->Location = System::Drawing::Point(8, 8);
			this->rab_MipLinClassification->Name = S"rab_MipLinClassification";
			this->rab_MipLinClassification->Size = System::Drawing::Size(184, 20);
			this->rab_MipLinClassification->TabIndex = 6;
			this->rab_MipLinClassification->Text = S"Linear Classification";
			this->rab_MipLinClassification->CheckedChanged += new System::EventHandler(this, perform_ActionRadioButtonCheckedChanged);
			// 
			// lab_MipColor
			// 
			this->lab_MipColor->BorderStyle = System::Windows::Forms::BorderStyle::Fixed3D;
			this->lab_MipColor->Location = System::Drawing::Point(128, 236);
			this->lab_MipColor->Name = S"lab_MipColor";
			this->lab_MipColor->Size = System::Drawing::Size(70, 20);
			this->lab_MipColor->TabIndex = 5;
			this->tot_ToolTips->SetToolTip(this->lab_MipColor, S"Display of current Base Color");
			// 
			// but_MipColor
			// 
			this->but_MipColor->FlatStyle = System::Windows::Forms::FlatStyle::Popup;
			this->but_MipColor->Location = System::Drawing::Point(0, 236);
			this->but_MipColor->Name = S"but_MipColor";
			this->but_MipColor->Size = System::Drawing::Size(128, 20);
			this->but_MipColor->TabIndex = 4;
			this->but_MipColor->Text = S"Choose Color";
			this->tot_ToolTips->SetToolTip(this->but_MipColor, S"Choose MIP Base Color");
			this->but_MipColor->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// tex_MipThreshold
			// 
			this->tex_MipThreshold->Location = System::Drawing::Point(136, 144);
			this->tex_MipThreshold->Name = S"tex_MipThreshold";
			this->tex_MipThreshold->Size = System::Drawing::Size(48, 20);
			this->tex_MipThreshold->TabIndex = 2;
			this->tex_MipThreshold->Text = S"";
			this->tex_MipThreshold->TextAlign = System::Windows::Forms::HorizontalAlignment::Right;
			this->tot_ToolTips->SetToolTip(this->tex_MipThreshold, S"Edit LMIP threshold");
			this->tex_MipThreshold->KeyDown += new System::Windows::Forms::KeyEventHandler(this, perform_ActionTextboxKeydown);
			// 
			// lab_MipThreshold
			// 
			this->lab_MipThreshold->Location = System::Drawing::Point(8, 144);
			this->lab_MipThreshold->Name = S"lab_MipThreshold";
			this->lab_MipThreshold->Size = System::Drawing::Size(120, 20);
			this->lab_MipThreshold->TabIndex = 1;
			this->lab_MipThreshold->Text = S"LMDensP - Threshold:";
			this->lab_MipThreshold->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
			// 
			// trb_MipThreshold
			// 
			this->trb_MipThreshold->AutoSize = false;
			this->trb_MipThreshold->Location = System::Drawing::Point(8, 168);
			this->trb_MipThreshold->Maximum = 4095;
			this->trb_MipThreshold->Name = S"trb_MipThreshold";
			this->trb_MipThreshold->Size = System::Drawing::Size(184, 28);
			this->trb_MipThreshold->TabIndex = 0;
			this->trb_MipThreshold->TickFrequency = 128;
			this->tot_ToolTips->SetToolTip(this->trb_MipThreshold, S"Choose LMIP threshold");
			this->trb_MipThreshold->Value = 4095;
			this->trb_MipThreshold->Scroll += new System::EventHandler(this, perform_ActionTrackBarScroll);
			// 
			// but_MipAssign
			// 
			this->but_MipAssign->FlatStyle = System::Windows::Forms::FlatStyle::Popup;
			this->but_MipAssign->Location = System::Drawing::Point(0, 256);
			this->but_MipAssign->Name = S"but_MipAssign";
			this->but_MipAssign->Size = System::Drawing::Size(128, 20);
			this->but_MipAssign->TabIndex = 2;
			this->but_MipAssign->Text = S"Assign Features";
			this->tot_ToolTips->SetToolTip(this->but_MipAssign, S"Set new features to MIP Renderer");
			this->but_MipAssign->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// but_MipReset
			// 
			this->but_MipReset->FlatStyle = System::Windows::Forms::FlatStyle::Popup;
			this->but_MipReset->Location = System::Drawing::Point(128, 256);
			this->but_MipReset->Name = S"but_MipReset";
			this->but_MipReset->Size = System::Drawing::Size(70, 20);
			this->but_MipReset->TabIndex = 3;
			this->but_MipReset->Text = S"Reset";
			this->tot_ToolTips->SetToolTip(this->but_MipReset, S"Set MIP Renderer to default status");
			this->but_MipReset->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// tabPage_Tone
			// 
			this->tabPage_Tone->Controls->Add(this->pan_Tone);
			this->tabPage_Tone->Controls->Add(this->but_ToneReset);
			this->tabPage_Tone->Controls->Add(this->but_ToneAssign);
			this->tabPage_Tone->Location = System::Drawing::Point(4, 22);
			this->tabPage_Tone->Name = S"tabPage_Tone";
			this->tabPage_Tone->Size = System::Drawing::Size(198, 278);
			this->tabPage_Tone->TabIndex = 4;
			this->tabPage_Tone->Text = S"Tone";
			this->tabPage_Tone->ToolTipText = S"Use the Tone Shader model";
			// 
			// pan_Tone
			// 
			this->pan_Tone->Controls->Add(this->lab_ToneKTF);
			this->pan_Tone->Controls->Add(this->nud_ToneKTF);
			this->pan_Tone->Controls->Add(this->but_ToneTransferFunc);
			this->pan_Tone->Controls->Add(this->rab_ToneDvr);
			this->pan_Tone->Controls->Add(this->rab_ToneLMip);
			this->pan_Tone->Controls->Add(this->lab_ToneKW);
			this->pan_Tone->Controls->Add(this->nud_ToneKC);
			this->pan_Tone->Controls->Add(this->nud_ToneKW);
			this->pan_Tone->Controls->Add(this->lab_ToneColWarm);
			this->pan_Tone->Controls->Add(this->lab_ToneGradient);
			this->pan_Tone->Controls->Add(this->nud_ToneGradient);
			this->pan_Tone->Controls->Add(this->tex_ToneThreshold);
			this->pan_Tone->Controls->Add(this->lab_ToneThreshold);
			this->pan_Tone->Controls->Add(this->lab_ToneKC);
			this->pan_Tone->Controls->Add(this->trb_ToneThreshold);
			this->pan_Tone->Controls->Add(this->but_ToneColWarm);
			this->pan_Tone->Controls->Add(this->but_ToneColCold);
			this->pan_Tone->Controls->Add(this->lab_ToneColCold);
			this->pan_Tone->Location = System::Drawing::Point(0, 0);
			this->pan_Tone->Name = S"pan_Tone";
			this->pan_Tone->Size = System::Drawing::Size(204, 256);
			this->pan_Tone->TabIndex = 0;
			// 
			// lab_ToneKTF
			// 
			this->lab_ToneKTF->Location = System::Drawing::Point(8, 184);
			this->lab_ToneKTF->Name = S"lab_ToneKTF";
			this->lab_ToneKTF->Size = System::Drawing::Size(76, 20);
			this->lab_ToneKTF->TabIndex = 36;
			this->lab_ToneKTF->Text = S"Anteil TF:";
			this->lab_ToneKTF->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
			this->tot_ToolTips->SetToolTip(this->lab_ToneKTF, S"Edit influence of choosen Transfer Function");
			// 
			// nud_ToneKTF
			// 
			this->nud_ToneKTF->DecimalPlaces = 2;
			System::Int32 __mcTemp__32[] = new System::Int32[4];
			__mcTemp__32[0] = 5;
			__mcTemp__32[1] = 0;
			__mcTemp__32[2] = 0;
			__mcTemp__32[3] = 131072;
			this->nud_ToneKTF->Increment = System::Decimal(__mcTemp__32);
			this->nud_ToneKTF->Location = System::Drawing::Point(84, 184);
			System::Int32 __mcTemp__33[] = new System::Int32[4];
			__mcTemp__33[0] = 1;
			__mcTemp__33[1] = 0;
			__mcTemp__33[2] = 0;
			__mcTemp__33[3] = 0;
			this->nud_ToneKTF->Maximum = System::Decimal(__mcTemp__33);
			this->nud_ToneKTF->Name = S"nud_ToneKTF";
			this->nud_ToneKTF->Size = System::Drawing::Size(44, 20);
			this->nud_ToneKTF->TabIndex = 35;
			this->tot_ToolTips->SetToolTip(this->nud_ToneKTF, S"Edit influence of choosen Transfer Function");
			System::Int32 __mcTemp__34[] = new System::Int32[4];
			__mcTemp__34[0] = 50;
			__mcTemp__34[1] = 0;
			__mcTemp__34[2] = 0;
			__mcTemp__34[3] = 131072;
			this->nud_ToneKTF->Value = System::Decimal(__mcTemp__34);
			// 
			// but_ToneTransferFunc
			// 
			this->but_ToneTransferFunc->FlatStyle = System::Windows::Forms::FlatStyle::Popup;
			this->but_ToneTransferFunc->Location = System::Drawing::Point(128, 184);
			this->but_ToneTransferFunc->Name = S"but_ToneTransferFunc";
			this->but_ToneTransferFunc->Size = System::Drawing::Size(70, 20);
			this->but_ToneTransferFunc->TabIndex = 34;
			this->but_ToneTransferFunc->Text = S"Edit TF";
			this->tot_ToolTips->SetToolTip(this->but_ToneTransferFunc, S"Edit Transfer Function for Composition");
			this->but_ToneTransferFunc->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// rab_ToneDvr
			// 
			this->rab_ToneDvr->Location = System::Drawing::Point(8, 160);
			this->rab_ToneDvr->Name = S"rab_ToneDvr";
			this->rab_ToneDvr->Size = System::Drawing::Size(116, 24);
			this->rab_ToneDvr->TabIndex = 33;
			this->rab_ToneDvr->Text = S"DVR Compositing:";
			this->tot_ToolTips->SetToolTip(this->rab_ToneDvr, S"Choose DVR Composition for current Tone Renderer");
			this->rab_ToneDvr->CheckedChanged += new System::EventHandler(this, perform_ActionRadioButtonCheckedChanged);
			// 
			// rab_ToneLMip
			// 
			this->rab_ToneLMip->Location = System::Drawing::Point(8, 80);
			this->rab_ToneLMip->Name = S"rab_ToneLMip";
			this->rab_ToneLMip->Size = System::Drawing::Size(120, 20);
			this->rab_ToneLMip->TabIndex = 32;
			this->rab_ToneLMip->Text = S"LMIP Compositing:";
			this->tot_ToolTips->SetToolTip(this->rab_ToneLMip, S"Choose MIP Composition for current Tone Renderer");
			this->rab_ToneLMip->CheckedChanged += new System::EventHandler(this, perform_ActionRadioButtonCheckedChanged);
			// 
			// lab_ToneKW
			// 
			this->lab_ToneKW->Location = System::Drawing::Point(8, 28);
			this->lab_ToneKW->Name = S"lab_ToneKW";
			this->lab_ToneKW->Size = System::Drawing::Size(88, 20);
			this->lab_ToneKW->TabIndex = 30;
			this->lab_ToneKW->Text = S"Influence K_W:";
			this->lab_ToneKW->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
			this->tot_ToolTips->SetToolTip(this->lab_ToneKW, S"Edit infuluence of Warm Base Color");
			// 
			// nud_ToneKC
			// 
			this->nud_ToneKC->DecimalPlaces = 2;
			System::Int32 __mcTemp__35[] = new System::Int32[4];
			__mcTemp__35[0] = 5;
			__mcTemp__35[1] = 0;
			__mcTemp__35[2] = 0;
			__mcTemp__35[3] = 131072;
			this->nud_ToneKC->Increment = System::Decimal(__mcTemp__35);
			this->nud_ToneKC->Location = System::Drawing::Point(148, 52);
			System::Int32 __mcTemp__36[] = new System::Int32[4];
			__mcTemp__36[0] = 1;
			__mcTemp__36[1] = 0;
			__mcTemp__36[2] = 0;
			__mcTemp__36[3] = 0;
			this->nud_ToneKC->Maximum = System::Decimal(__mcTemp__36);
			this->nud_ToneKC->Name = S"nud_ToneKC";
			this->nud_ToneKC->Size = System::Drawing::Size(44, 20);
			this->nud_ToneKC->TabIndex = 29;
			this->tot_ToolTips->SetToolTip(this->nud_ToneKC, S"Edit infuluence of Cold Base Color");
			System::Int32 __mcTemp__37[] = new System::Int32[4];
			__mcTemp__37[0] = 50;
			__mcTemp__37[1] = 0;
			__mcTemp__37[2] = 0;
			__mcTemp__37[3] = 131072;
			this->nud_ToneKC->Value = System::Decimal(__mcTemp__37);
			// 
			// nud_ToneKW
			// 
			this->nud_ToneKW->DecimalPlaces = 2;
			System::Int32 __mcTemp__38[] = new System::Int32[4];
			__mcTemp__38[0] = 5;
			__mcTemp__38[1] = 0;
			__mcTemp__38[2] = 0;
			__mcTemp__38[3] = 131072;
			this->nud_ToneKW->Increment = System::Decimal(__mcTemp__38);
			this->nud_ToneKW->Location = System::Drawing::Point(148, 28);
			System::Int32 __mcTemp__39[] = new System::Int32[4];
			__mcTemp__39[0] = 1;
			__mcTemp__39[1] = 0;
			__mcTemp__39[2] = 0;
			__mcTemp__39[3] = 0;
			this->nud_ToneKW->Maximum = System::Decimal(__mcTemp__39);
			this->nud_ToneKW->Name = S"nud_ToneKW";
			this->nud_ToneKW->Size = System::Drawing::Size(44, 20);
			this->nud_ToneKW->TabIndex = 28;
			this->tot_ToolTips->SetToolTip(this->nud_ToneKW, S"Edit infuluence of Warm Base Color");
			System::Int32 __mcTemp__40[] = new System::Int32[4];
			__mcTemp__40[0] = 50;
			__mcTemp__40[1] = 0;
			__mcTemp__40[2] = 0;
			__mcTemp__40[3] = 131072;
			this->nud_ToneKW->Value = System::Decimal(__mcTemp__40);
			// 
			// lab_ToneColWarm
			// 
			this->lab_ToneColWarm->BorderStyle = System::Windows::Forms::BorderStyle::Fixed3D;
			this->lab_ToneColWarm->Location = System::Drawing::Point(128, 216);
			this->lab_ToneColWarm->Name = S"lab_ToneColWarm";
			this->lab_ToneColWarm->Size = System::Drawing::Size(70, 20);
			this->lab_ToneColWarm->TabIndex = 25;
			this->tot_ToolTips->SetToolTip(this->lab_ToneColWarm, S"Display of current Warm Base Color");
			// 
			// lab_ToneGradient
			// 
			this->lab_ToneGradient->Location = System::Drawing::Point(8, 4);
			this->lab_ToneGradient->Name = S"lab_ToneGradient";
			this->lab_ToneGradient->Size = System::Drawing::Size(108, 20);
			this->lab_ToneGradient->TabIndex = 22;
			this->lab_ToneGradient->Text = S"Gradient Magnitude:";
			this->lab_ToneGradient->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
			this->tot_ToolTips->SetToolTip(this->lab_ToneGradient, S"Edit Tone Gradient magnitude threshold");
			// 
			// nud_ToneGradient
			// 
			this->nud_ToneGradient->DecimalPlaces = 2;
			System::Int32 __mcTemp__41[] = new System::Int32[4];
			__mcTemp__41[0] = 2;
			__mcTemp__41[1] = 0;
			__mcTemp__41[2] = 0;
			__mcTemp__41[3] = 131072;
			this->nud_ToneGradient->Increment = System::Decimal(__mcTemp__41);
			this->nud_ToneGradient->Location = System::Drawing::Point(148, 4);
			System::Int32 __mcTemp__42[] = new System::Int32[4];
			__mcTemp__42[0] = 1;
			__mcTemp__42[1] = 0;
			__mcTemp__42[2] = 0;
			__mcTemp__42[3] = 0;
			this->nud_ToneGradient->Maximum = System::Decimal(__mcTemp__42);
			this->nud_ToneGradient->Name = S"nud_ToneGradient";
			this->nud_ToneGradient->Size = System::Drawing::Size(44, 20);
			this->nud_ToneGradient->TabIndex = 20;
			this->nud_ToneGradient->TextAlign = System::Windows::Forms::HorizontalAlignment::Right;
			this->tot_ToolTips->SetToolTip(this->nud_ToneGradient, S"Edit Tone Gradient magnitude threshold");
			System::Int32 __mcTemp__43[] = new System::Int32[4];
			__mcTemp__43[0] = 20;
			__mcTemp__43[1] = 0;
			__mcTemp__43[2] = 0;
			__mcTemp__43[3] = 131072;
			this->nud_ToneGradient->Value = System::Decimal(__mcTemp__43);
			// 
			// tex_ToneThreshold
			// 
			this->tex_ToneThreshold->Location = System::Drawing::Point(132, 100);
			this->tex_ToneThreshold->Name = S"tex_ToneThreshold";
			this->tex_ToneThreshold->Size = System::Drawing::Size(60, 20);
			this->tex_ToneThreshold->TabIndex = 19;
			this->tex_ToneThreshold->Text = S"";
			this->tex_ToneThreshold->TextAlign = System::Windows::Forms::HorizontalAlignment::Right;
			this->tot_ToolTips->SetToolTip(this->tex_ToneThreshold, S"Edit LMIP threshold");
			this->tex_ToneThreshold->KeyDown += new System::Windows::Forms::KeyEventHandler(this, perform_ActionTextboxKeydown);
			// 
			// lab_ToneThreshold
			// 
			this->lab_ToneThreshold->Location = System::Drawing::Point(8, 104);
			this->lab_ToneThreshold->Name = S"lab_ToneThreshold";
			this->lab_ToneThreshold->Size = System::Drawing::Size(108, 20);
			this->lab_ToneThreshold->TabIndex = 18;
			this->lab_ToneThreshold->Text = S"LMIP - Threshold:";
			this->lab_ToneThreshold->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
			// 
			// lab_ToneKC
			// 
			this->lab_ToneKC->Location = System::Drawing::Point(8, 52);
			this->lab_ToneKC->Name = S"lab_ToneKC";
			this->lab_ToneKC->Size = System::Drawing::Size(88, 20);
			this->lab_ToneKC->TabIndex = 31;
			this->lab_ToneKC->Text = S"Influence K_C:";
			this->lab_ToneKC->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
			this->tot_ToolTips->SetToolTip(this->lab_ToneKC, S"Edit infuluence of Cold Base Color");
			// 
			// trb_ToneThreshold
			// 
			this->trb_ToneThreshold->AutoSize = false;
			this->trb_ToneThreshold->Location = System::Drawing::Point(8, 120);
			this->trb_ToneThreshold->Maximum = 100;
			this->trb_ToneThreshold->Name = S"trb_ToneThreshold";
			this->trb_ToneThreshold->Size = System::Drawing::Size(184, 28);
			this->trb_ToneThreshold->TabIndex = 17;
			this->trb_ToneThreshold->TickFrequency = 5;
			this->tot_ToolTips->SetToolTip(this->trb_ToneThreshold, S"Choose LMIP threshold");
			this->trb_ToneThreshold->Value = 100;
			this->trb_ToneThreshold->Scroll += new System::EventHandler(this, perform_ActionTrackBarScroll);
			// 
			// but_ToneColWarm
			// 
			this->but_ToneColWarm->FlatStyle = System::Windows::Forms::FlatStyle::Popup;
			this->but_ToneColWarm->Location = System::Drawing::Point(0, 216);
			this->but_ToneColWarm->Name = S"but_ToneColWarm";
			this->but_ToneColWarm->Size = System::Drawing::Size(128, 20);
			this->but_ToneColWarm->TabIndex = 24;
			this->but_ToneColWarm->Text = S"Choose Warm Color";
			this->tot_ToolTips->SetToolTip(this->but_ToneColWarm, S"Choose Warm Base Color");
			this->but_ToneColWarm->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// but_ToneColCold
			// 
			this->but_ToneColCold->FlatStyle = System::Windows::Forms::FlatStyle::Popup;
			this->but_ToneColCold->Location = System::Drawing::Point(0, 236);
			this->but_ToneColCold->Name = S"but_ToneColCold";
			this->but_ToneColCold->Size = System::Drawing::Size(128, 20);
			this->but_ToneColCold->TabIndex = 26;
			this->but_ToneColCold->Text = S"Choose Cold Color";
			this->tot_ToolTips->SetToolTip(this->but_ToneColCold, S"Choose Cold Base Color");
			this->but_ToneColCold->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// lab_ToneColCold
			// 
			this->lab_ToneColCold->BorderStyle = System::Windows::Forms::BorderStyle::Fixed3D;
			this->lab_ToneColCold->Location = System::Drawing::Point(128, 236);
			this->lab_ToneColCold->Name = S"lab_ToneColCold";
			this->lab_ToneColCold->Size = System::Drawing::Size(70, 20);
			this->lab_ToneColCold->TabIndex = 27;
			this->tot_ToolTips->SetToolTip(this->lab_ToneColCold, S"Display of current Cold Base Color");
			// 
			// but_ToneReset
			// 
			this->but_ToneReset->FlatStyle = System::Windows::Forms::FlatStyle::Popup;
			this->but_ToneReset->Location = System::Drawing::Point(128, 256);
			this->but_ToneReset->Name = S"but_ToneReset";
			this->but_ToneReset->Size = System::Drawing::Size(70, 20);
			this->but_ToneReset->TabIndex = 16;
			this->but_ToneReset->Text = S"Reset";
			this->tot_ToolTips->SetToolTip(this->but_ToneReset, S"Set Tone Renderer to default status");
			this->but_ToneReset->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// but_ToneAssign
			// 
			this->but_ToneAssign->FlatStyle = System::Windows::Forms::FlatStyle::Popup;
			this->but_ToneAssign->Location = System::Drawing::Point(0, 256);
			this->but_ToneAssign->Name = S"but_ToneAssign";
			this->but_ToneAssign->Size = System::Drawing::Size(128, 20);
			this->but_ToneAssign->TabIndex = 15;
			this->but_ToneAssign->Text = S"Assign Features";
			this->tot_ToolTips->SetToolTip(this->but_ToneAssign, S"Set new features to Tone Renderer");
			this->but_ToneAssign->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// tabPage_Contour
			// 
			this->tabPage_Contour->Controls->Add(this->pan_Contour);
			this->tabPage_Contour->Controls->Add(this->but_ContourAssign);
			this->tabPage_Contour->Controls->Add(this->but_ContourReset);
			this->tabPage_Contour->Location = System::Drawing::Point(4, 22);
			this->tabPage_Contour->Name = S"tabPage_Contour";
			this->tabPage_Contour->Size = System::Drawing::Size(198, 278);
			this->tabPage_Contour->TabIndex = 3;
			this->tabPage_Contour->Text = S"Contour";
			this->tabPage_Contour->ToolTipText = S"Use the Contour Shader model";
			this->tabPage_Contour->Visible = false;
			// 
			// pan_Contour
			// 
			this->pan_Contour->AutoScroll = true;
			this->pan_Contour->Controls->Add(this->lab_ContourKTF);
			this->pan_Contour->Controls->Add(this->nud_ContourKTF);
			this->pan_Contour->Controls->Add(this->but_ContourTransferFunc);
			this->pan_Contour->Controls->Add(this->rab_ContourDvr);
			this->pan_Contour->Controls->Add(this->rab_ContourLMip);
			this->pan_Contour->Controls->Add(this->but_ContourColor);
			this->pan_Contour->Controls->Add(this->lab_ContourColor);
			this->pan_Contour->Controls->Add(this->lab_ContourExponent);
			this->pan_Contour->Controls->Add(this->lab_ContourGradient);
			this->pan_Contour->Controls->Add(this->nud_ContourExponent);
			this->pan_Contour->Controls->Add(this->nud_ContourGradient);
			this->pan_Contour->Controls->Add(this->lab_ContourThreshold);
			this->pan_Contour->Controls->Add(this->tex_ContourThreshold);
			this->pan_Contour->Controls->Add(this->trb_ContourThreshold);
			this->pan_Contour->Location = System::Drawing::Point(0, 0);
			this->pan_Contour->Name = S"pan_Contour";
			this->pan_Contour->Size = System::Drawing::Size(204, 256);
			this->pan_Contour->TabIndex = 6;
			// 
			// lab_ContourKTF
			// 
			this->lab_ContourKTF->Location = System::Drawing::Point(8, 184);
			this->lab_ContourKTF->Name = S"lab_ContourKTF";
			this->lab_ContourKTF->Size = System::Drawing::Size(76, 20);
			this->lab_ContourKTF->TabIndex = 40;
			this->lab_ContourKTF->Text = S"Anteil TF:";
			this->lab_ContourKTF->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
			this->tot_ToolTips->SetToolTip(this->lab_ContourKTF, S"Edit influence of choosen Transfer Function");
			// 
			// nud_ContourKTF
			// 
			this->nud_ContourKTF->DecimalPlaces = 2;
			System::Int32 __mcTemp__44[] = new System::Int32[4];
			__mcTemp__44[0] = 5;
			__mcTemp__44[1] = 0;
			__mcTemp__44[2] = 0;
			__mcTemp__44[3] = 131072;
			this->nud_ContourKTF->Increment = System::Decimal(__mcTemp__44);
			this->nud_ContourKTF->Location = System::Drawing::Point(84, 184);
			System::Int32 __mcTemp__45[] = new System::Int32[4];
			__mcTemp__45[0] = 1;
			__mcTemp__45[1] = 0;
			__mcTemp__45[2] = 0;
			__mcTemp__45[3] = 0;
			this->nud_ContourKTF->Maximum = System::Decimal(__mcTemp__45);
			this->nud_ContourKTF->Name = S"nud_ContourKTF";
			this->nud_ContourKTF->Size = System::Drawing::Size(44, 20);
			this->nud_ContourKTF->TabIndex = 39;
			this->tot_ToolTips->SetToolTip(this->nud_ContourKTF, S"Edit influence of choosen Transfer Function");
			System::Int32 __mcTemp__46[] = new System::Int32[4];
			__mcTemp__46[0] = 50;
			__mcTemp__46[1] = 0;
			__mcTemp__46[2] = 0;
			__mcTemp__46[3] = 131072;
			this->nud_ContourKTF->Value = System::Decimal(__mcTemp__46);
			// 
			// but_ContourTransferFunc
			// 
			this->but_ContourTransferFunc->FlatStyle = System::Windows::Forms::FlatStyle::Popup;
			this->but_ContourTransferFunc->Location = System::Drawing::Point(128, 184);
			this->but_ContourTransferFunc->Name = S"but_ContourTransferFunc";
			this->but_ContourTransferFunc->Size = System::Drawing::Size(70, 20);
			this->but_ContourTransferFunc->TabIndex = 38;
			this->but_ContourTransferFunc->Text = S"Edit TF";
			this->tot_ToolTips->SetToolTip(this->but_ContourTransferFunc, S"Edit Transfer Function for Composition");
			this->but_ContourTransferFunc->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// rab_ContourDvr
			// 
			this->rab_ContourDvr->Location = System::Drawing::Point(8, 160);
			this->rab_ContourDvr->Name = S"rab_ContourDvr";
			this->rab_ContourDvr->Size = System::Drawing::Size(116, 24);
			this->rab_ContourDvr->TabIndex = 37;
			this->rab_ContourDvr->Text = S"DVR Compositing:";
			this->tot_ToolTips->SetToolTip(this->rab_ContourDvr, S"Choose DVR Composition for current Contour Renderer");
			this->rab_ContourDvr->CheckedChanged += new System::EventHandler(this, perform_ActionRadioButtonCheckedChanged);
			// 
			// rab_ContourLMip
			// 
			this->rab_ContourLMip->Location = System::Drawing::Point(8, 80);
			this->rab_ContourLMip->Name = S"rab_ContourLMip";
			this->rab_ContourLMip->Size = System::Drawing::Size(120, 20);
			this->rab_ContourLMip->TabIndex = 33;
			this->rab_ContourLMip->Text = S"LMIP Compositing:";
			this->tot_ToolTips->SetToolTip(this->rab_ContourLMip, S"Choose MIP Composition for current Contour Renderer");
			this->rab_ContourLMip->CheckedChanged += new System::EventHandler(this, perform_ActionRadioButtonCheckedChanged);
			// 
			// but_ContourColor
			// 
			this->but_ContourColor->FlatStyle = System::Windows::Forms::FlatStyle::Popup;
			this->but_ContourColor->Location = System::Drawing::Point(0, 236);
			this->but_ContourColor->Name = S"but_ContourColor";
			this->but_ContourColor->Size = System::Drawing::Size(128, 20);
			this->but_ContourColor->TabIndex = 13;
			this->but_ContourColor->Text = S"Choose Color";
			this->tot_ToolTips->SetToolTip(this->but_ContourColor, S"Choose Contour Base Color");
			this->but_ContourColor->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// lab_ContourColor
			// 
			this->lab_ContourColor->BorderStyle = System::Windows::Forms::BorderStyle::Fixed3D;
			this->lab_ContourColor->Location = System::Drawing::Point(128, 236);
			this->lab_ContourColor->Name = S"lab_ContourColor";
			this->lab_ContourColor->Size = System::Drawing::Size(70, 20);
			this->lab_ContourColor->TabIndex = 14;
			this->tot_ToolTips->SetToolTip(this->lab_ContourColor, S"Display of current Base Color");
			// 
			// lab_ContourExponent
			// 
			this->lab_ContourExponent->Location = System::Drawing::Point(8, 36);
			this->lab_ContourExponent->Name = S"lab_ContourExponent";
			this->lab_ContourExponent->Size = System::Drawing::Size(108, 20);
			this->lab_ContourExponent->TabIndex = 12;
			this->lab_ContourExponent->Text = S"Exponent:";
			this->lab_ContourExponent->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
			this->tot_ToolTips->SetToolTip(this->lab_ContourExponent, S"Edit Contour Gradient Expontent");
			// 
			// lab_ContourGradient
			// 
			this->lab_ContourGradient->Location = System::Drawing::Point(8, 4);
			this->lab_ContourGradient->Name = S"lab_ContourGradient";
			this->lab_ContourGradient->Size = System::Drawing::Size(108, 20);
			this->lab_ContourGradient->TabIndex = 11;
			this->lab_ContourGradient->Text = S"Gradient Magnitude:";
			this->lab_ContourGradient->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
			this->tot_ToolTips->SetToolTip(this->lab_ContourGradient, S"Edit Contour Gradient magnitude threshold");
			// 
			// nud_ContourExponent
			// 
			this->nud_ContourExponent->Location = System::Drawing::Point(148, 32);
			System::Int32 __mcTemp__47[] = new System::Int32[4];
			__mcTemp__47[0] = 20;
			__mcTemp__47[1] = 0;
			__mcTemp__47[2] = 0;
			__mcTemp__47[3] = 0;
			this->nud_ContourExponent->Maximum = System::Decimal(__mcTemp__47);
			System::Int32 __mcTemp__48[] = new System::Int32[4];
			__mcTemp__48[0] = 1;
			__mcTemp__48[1] = 0;
			__mcTemp__48[2] = 0;
			__mcTemp__48[3] = 0;
			this->nud_ContourExponent->Minimum = System::Decimal(__mcTemp__48);
			this->nud_ContourExponent->Name = S"nud_ContourExponent";
			this->nud_ContourExponent->Size = System::Drawing::Size(44, 20);
			this->nud_ContourExponent->TabIndex = 10;
			this->nud_ContourExponent->TextAlign = System::Windows::Forms::HorizontalAlignment::Right;
			this->tot_ToolTips->SetToolTip(this->nud_ContourExponent, S"Edit Contour Gradient Expontent");
			System::Int32 __mcTemp__49[] = new System::Int32[4];
			__mcTemp__49[0] = 4;
			__mcTemp__49[1] = 0;
			__mcTemp__49[2] = 0;
			__mcTemp__49[3] = 0;
			this->nud_ContourExponent->Value = System::Decimal(__mcTemp__49);
			// 
			// nud_ContourGradient
			// 
			this->nud_ContourGradient->DecimalPlaces = 2;
			System::Int32 __mcTemp__50[] = new System::Int32[4];
			__mcTemp__50[0] = 2;
			__mcTemp__50[1] = 0;
			__mcTemp__50[2] = 0;
			__mcTemp__50[3] = 131072;
			this->nud_ContourGradient->Increment = System::Decimal(__mcTemp__50);
			this->nud_ContourGradient->Location = System::Drawing::Point(148, 4);
			System::Int32 __mcTemp__51[] = new System::Int32[4];
			__mcTemp__51[0] = 1;
			__mcTemp__51[1] = 0;
			__mcTemp__51[2] = 0;
			__mcTemp__51[3] = 0;
			this->nud_ContourGradient->Maximum = System::Decimal(__mcTemp__51);
			this->nud_ContourGradient->Name = S"nud_ContourGradient";
			this->nud_ContourGradient->Size = System::Drawing::Size(44, 20);
			this->nud_ContourGradient->TabIndex = 9;
			this->nud_ContourGradient->TextAlign = System::Windows::Forms::HorizontalAlignment::Right;
			this->tot_ToolTips->SetToolTip(this->nud_ContourGradient, S"Edit Contour Gradient magnitude threshold");
			System::Int32 __mcTemp__52[] = new System::Int32[4];
			__mcTemp__52[0] = 5;
			__mcTemp__52[1] = 0;
			__mcTemp__52[2] = 0;
			__mcTemp__52[3] = 65536;
			this->nud_ContourGradient->Value = System::Decimal(__mcTemp__52);
			// 
			// lab_ContourThreshold
			// 
			this->lab_ContourThreshold->Location = System::Drawing::Point(8, 104);
			this->lab_ContourThreshold->Name = S"lab_ContourThreshold";
			this->lab_ContourThreshold->Size = System::Drawing::Size(108, 20);
			this->lab_ContourThreshold->TabIndex = 7;
			this->lab_ContourThreshold->Text = S"LMIP - Threshold:";
			this->lab_ContourThreshold->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
			// 
			// tex_ContourThreshold
			// 
			this->tex_ContourThreshold->Location = System::Drawing::Point(132, 100);
			this->tex_ContourThreshold->Name = S"tex_ContourThreshold";
			this->tex_ContourThreshold->Size = System::Drawing::Size(60, 20);
			this->tex_ContourThreshold->TabIndex = 8;
			this->tex_ContourThreshold->Text = S"";
			this->tex_ContourThreshold->TextAlign = System::Windows::Forms::HorizontalAlignment::Right;
			this->tot_ToolTips->SetToolTip(this->tex_ContourThreshold, S"Edit LMIP threshold");
			this->tex_ContourThreshold->KeyDown += new System::Windows::Forms::KeyEventHandler(this, perform_ActionTextboxKeydown);
			// 
			// trb_ContourThreshold
			// 
			this->trb_ContourThreshold->AutoSize = false;
			this->trb_ContourThreshold->Location = System::Drawing::Point(8, 120);
			this->trb_ContourThreshold->Maximum = 100;
			this->trb_ContourThreshold->Name = S"trb_ContourThreshold";
			this->trb_ContourThreshold->Size = System::Drawing::Size(184, 28);
			this->trb_ContourThreshold->TabIndex = 6;
			this->trb_ContourThreshold->TickFrequency = 5;
			this->tot_ToolTips->SetToolTip(this->trb_ContourThreshold, S"Choose LMIP threshold");
			this->trb_ContourThreshold->Value = 100;
			this->trb_ContourThreshold->Scroll += new System::EventHandler(this, perform_ActionTrackBarScroll);
			// 
			// but_ContourAssign
			// 
			this->but_ContourAssign->FlatStyle = System::Windows::Forms::FlatStyle::Popup;
			this->but_ContourAssign->Location = System::Drawing::Point(0, 256);
			this->but_ContourAssign->Name = S"but_ContourAssign";
			this->but_ContourAssign->Size = System::Drawing::Size(128, 20);
			this->but_ContourAssign->TabIndex = 4;
			this->but_ContourAssign->Text = S"Assign Features";
			this->tot_ToolTips->SetToolTip(this->but_ContourAssign, S"Set new features to Contour Renderer");
			this->but_ContourAssign->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// but_ContourReset
			// 
			this->but_ContourReset->FlatStyle = System::Windows::Forms::FlatStyle::Popup;
			this->but_ContourReset->Location = System::Drawing::Point(128, 256);
			this->but_ContourReset->Name = S"but_ContourReset";
			this->but_ContourReset->Size = System::Drawing::Size(70, 20);
			this->but_ContourReset->TabIndex = 5;
			this->but_ContourReset->Text = S"Reset";
			this->tot_ToolTips->SetToolTip(this->but_ContourReset, S"Set Contour Renderer to default status");
			this->but_ContourReset->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// chb_TwoLevelVR
			// 
			this->chb_TwoLevelVR->Location = System::Drawing::Point(8, 20);
			this->chb_TwoLevelVR->Name = S"chb_TwoLevelVR";
			this->chb_TwoLevelVR->Size = System::Drawing::Size(176, 24);
			this->chb_TwoLevelVR->TabIndex = 33;
			this->chb_TwoLevelVR->Text = S"Activate Two-Level-Rendering";
			this->chb_TwoLevelVR->CheckedChanged += new System::EventHandler(this, perform_ActionCheckBoxCheckedChanged);
			// 
			// grb_Rendering
			// 
			this->grb_Rendering->BackColor = System::Drawing::SystemColors::Control;
			this->grb_Rendering->Controls->Add(this->but_Render);
			this->grb_Rendering->Controls->Add(this->tabControl1);
			this->grb_Rendering->Location = System::Drawing::Point(1, 0);
			this->grb_Rendering->Name = S"grb_Rendering";
			this->grb_Rendering->Size = System::Drawing::Size(215, 208);
			this->grb_Rendering->TabIndex = 0;
			this->grb_Rendering->TabStop = false;
			this->grb_Rendering->Text = S"Rendering Quality";
			// 
			// but_Render
			// 
			this->but_Render->BackColor = System::Drawing::SystemColors::Control;
			this->but_Render->Image = (__try_cast<System::Drawing::Image *  >(resources->GetObject(S"but_Render.Image")));
			this->but_Render->ImageAlign = System::Drawing::ContentAlignment::BottomLeft;
			this->but_Render->Location = System::Drawing::Point(5, 172);
			this->but_Render->Name = S"but_Render";
			this->but_Render->Size = System::Drawing::Size(207, 32);
			this->but_Render->TabIndex = 32;
			this->but_Render->Text = S"Render Volume";
			this->tot_ToolTips->SetToolTip(this->but_Render, S"Start rendering");
			this->but_Render->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// tabControl1
			// 
			this->tabControl1->Controls->Add(this->tabPage_Basic);
			this->tabControl1->Controls->Add(this->tabPage_Optimation);
			this->tabControl1->ItemSize = System::Drawing::Size(43, 18);
			this->tabControl1->Location = System::Drawing::Point(5, 16);
			this->tabControl1->Name = S"tabControl1";
			this->tabControl1->SelectedIndex = 0;
			this->tabControl1->Size = System::Drawing::Size(206, 156);
			this->tabControl1->TabIndex = 0;
			// 
			// tabPage_Basic
			// 
			this->tabPage_Basic->Controls->Add(this->lab_ImageRes);
			this->tabPage_Basic->Controls->Add(this->trb_ImageRes);
			this->tabPage_Basic->Controls->Add(this->trb_CanvasSize);
			this->tabPage_Basic->Controls->Add(this->lab_CanvasSize);
			this->tabPage_Basic->Controls->Add(this->cob_SamplingFactor);
			this->tabPage_Basic->Location = System::Drawing::Point(4, 22);
			this->tabPage_Basic->Name = S"tabPage_Basic";
			this->tabPage_Basic->Size = System::Drawing::Size(198, 130);
			this->tabPage_Basic->TabIndex = 0;
			this->tabPage_Basic->Text = S"Basics";
			this->tabPage_Basic->ToolTipText = S"Use Basic rendering settings";
			// 
			// lab_ImageRes
			// 
			this->lab_ImageRes->Location = System::Drawing::Point(8, 28);
			this->lab_ImageRes->Name = S"lab_ImageRes";
			this->lab_ImageRes->Size = System::Drawing::Size(168, 16);
			this->lab_ImageRes->TabIndex = 21;
			this->lab_ImageRes->Text = S"Image Resolution:";
			// 
			// trb_ImageRes
			// 
			this->trb_ImageRes->AutoSize = false;
			this->trb_ImageRes->Location = System::Drawing::Point(8, 40);
			this->trb_ImageRes->Maximum = 32;
			this->trb_ImageRes->Minimum = 1;
			this->trb_ImageRes->Name = S"trb_ImageRes";
			this->trb_ImageRes->Size = System::Drawing::Size(184, 32);
			this->trb_ImageRes->TabIndex = 2;
			this->tot_ToolTips->SetToolTip(this->trb_ImageRes, S"Choose image plane resolution");
			this->trb_ImageRes->Value = 16;
			this->trb_ImageRes->Scroll += new System::EventHandler(this, perform_ActionTrackBarScroll);
			// 
			// trb_CanvasSize
			// 
			this->trb_CanvasSize->AutoSize = false;
			this->trb_CanvasSize->Location = System::Drawing::Point(8, 92);
			this->trb_CanvasSize->Maximum = 32;
			this->trb_CanvasSize->Minimum = 1;
			this->trb_CanvasSize->Name = S"trb_CanvasSize";
			this->trb_CanvasSize->Size = System::Drawing::Size(184, 32);
			this->trb_CanvasSize->TabIndex = 23;
			this->tot_ToolTips->SetToolTip(this->trb_CanvasSize, S"Choose size of canvas");
			this->trb_CanvasSize->Value = 16;
			this->trb_CanvasSize->Scroll += new System::EventHandler(this, perform_ActionTrackBarScroll);
			// 
			// lab_CanvasSize
			// 
			this->lab_CanvasSize->Location = System::Drawing::Point(4, 80);
			this->lab_CanvasSize->Name = S"lab_CanvasSize";
			this->lab_CanvasSize->Size = System::Drawing::Size(160, 16);
			this->lab_CanvasSize->TabIndex = 22;
			this->lab_CanvasSize->Text = S"Canvas Size:";
			// 
			// cob_SamplingFactor
			// 
			this->cob_SamplingFactor->DropDownStyle = System::Windows::Forms::ComboBoxStyle::DropDownList;
			System::Object* __mcTemp__53[] = new System::Object*[5];
			__mcTemp__53[0] = S"Supersampling 4";
			__mcTemp__53[1] = S"Supersampling 2";
			__mcTemp__53[2] = S"Normalsampling";
			__mcTemp__53[3] = S"Subsampling 2";
			__mcTemp__53[4] = S"Subsampling 4";
			this->cob_SamplingFactor->Items->AddRange(__mcTemp__53);
			this->cob_SamplingFactor->Location = System::Drawing::Point(4, 4);
			this->cob_SamplingFactor->Name = S"cob_SamplingFactor";
			this->cob_SamplingFactor->Size = System::Drawing::Size(190, 21);
			this->cob_SamplingFactor->TabIndex = 23;
			this->tot_ToolTips->SetToolTip(this->cob_SamplingFactor, S"Choose specific Sampling Factor");
			this->cob_SamplingFactor->SelectedIndexChanged += new System::EventHandler(this, perform_ActionComboBoxSelectedIndexChanged);
			// 
			// tabPage_Optimation
			// 
			this->tabPage_Optimation->Controls->Add(this->lab_SamplingDist);
			this->tabPage_Optimation->Controls->Add(this->trb_SamplingDist);
			this->tabPage_Optimation->Controls->Add(this->lab_OpacityThreshold);
			this->tabPage_Optimation->Controls->Add(this->nud_OpacityThreshold);
			this->tabPage_Optimation->Controls->Add(this->lab_GradientTreshold);
			this->tabPage_Optimation->Controls->Add(this->nud_GradientThreshold);
			this->tabPage_Optimation->Controls->Add(this->tex_SamplingDist);
			this->tabPage_Optimation->Location = System::Drawing::Point(4, 22);
			this->tabPage_Optimation->Name = S"tabPage_Optimation";
			this->tabPage_Optimation->Size = System::Drawing::Size(198, 130);
			this->tabPage_Optimation->TabIndex = 2;
			this->tabPage_Optimation->Text = S"Optimation";
			this->tabPage_Optimation->ToolTipText = S"Use optimation rendering settings";
			this->tabPage_Optimation->Visible = false;
			// 
			// lab_SamplingDist
			// 
			this->lab_SamplingDist->Location = System::Drawing::Point(8, 8);
			this->lab_SamplingDist->Name = S"lab_SamplingDist";
			this->lab_SamplingDist->Size = System::Drawing::Size(104, 20);
			this->lab_SamplingDist->TabIndex = 27;
			this->lab_SamplingDist->Text = S"Sampling Distance:";
			this->lab_SamplingDist->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
			// 
			// trb_SamplingDist
			// 
			this->trb_SamplingDist->AutoSize = false;
			this->trb_SamplingDist->Location = System::Drawing::Point(8, 32);
			this->trb_SamplingDist->Maximum = 20;
			this->trb_SamplingDist->Minimum = 1;
			this->trb_SamplingDist->Name = S"trb_SamplingDist";
			this->trb_SamplingDist->Size = System::Drawing::Size(184, 32);
			this->trb_SamplingDist->TabIndex = 28;
			this->tot_ToolTips->SetToolTip(this->trb_SamplingDist, S"Choose Sampling Distance");
			this->trb_SamplingDist->Value = 7;
			this->trb_SamplingDist->Scroll += new System::EventHandler(this, perform_ActionTrackBarScroll);
			// 
			// lab_OpacityThreshold
			// 
			this->lab_OpacityThreshold->Location = System::Drawing::Point(8, 72);
			this->lab_OpacityThreshold->Name = S"lab_OpacityThreshold";
			this->lab_OpacityThreshold->Size = System::Drawing::Size(112, 20);
			this->lab_OpacityThreshold->TabIndex = 1;
			this->lab_OpacityThreshold->Text = S"Opacity Threshold:";
			this->lab_OpacityThreshold->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
			this->tot_ToolTips->SetToolTip(this->lab_OpacityThreshold, S"Edit general Opacity threshold");
			// 
			// nud_OpacityThreshold
			// 
			this->nud_OpacityThreshold->DecimalPlaces = 2;
			System::Int32 __mcTemp__54[] = new System::Int32[4];
			__mcTemp__54[0] = 1;
			__mcTemp__54[1] = 0;
			__mcTemp__54[2] = 0;
			__mcTemp__54[3] = 131072;
			this->nud_OpacityThreshold->Increment = System::Decimal(__mcTemp__54);
			this->nud_OpacityThreshold->Location = System::Drawing::Point(144, 72);
			System::Int32 __mcTemp__55[] = new System::Int32[4];
			__mcTemp__55[0] = 1;
			__mcTemp__55[1] = 0;
			__mcTemp__55[2] = 0;
			__mcTemp__55[3] = 0;
			this->nud_OpacityThreshold->Maximum = System::Decimal(__mcTemp__55);
			this->nud_OpacityThreshold->Name = S"nud_OpacityThreshold";
			this->nud_OpacityThreshold->RightToLeft = System::Windows::Forms::RightToLeft::No;
			this->nud_OpacityThreshold->Size = System::Drawing::Size(48, 20);
			this->nud_OpacityThreshold->TabIndex = 3;
			this->tot_ToolTips->SetToolTip(this->nud_OpacityThreshold, S"Edit general Opacity threshold");
			System::Int32 __mcTemp__56[] = new System::Int32[4];
			__mcTemp__56[0] = 1;
			__mcTemp__56[1] = 0;
			__mcTemp__56[2] = 0;
			__mcTemp__56[3] = 0;
			this->nud_OpacityThreshold->Value = System::Decimal(__mcTemp__56);
			this->nud_OpacityThreshold->ValueChanged += new System::EventHandler(this, perform_ActionNudValueChanged);
			// 
			// lab_GradientTreshold
			// 
			this->lab_GradientTreshold->Location = System::Drawing::Point(8, 104);
			this->lab_GradientTreshold->Name = S"lab_GradientTreshold";
			this->lab_GradientTreshold->Size = System::Drawing::Size(112, 20);
			this->lab_GradientTreshold->TabIndex = 2;
			this->lab_GradientTreshold->Text = S"Gradient Treshold:";
			this->lab_GradientTreshold->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
			this->tot_ToolTips->SetToolTip(this->lab_GradientTreshold, S"Edit general Gradient magnitude threshold");
			// 
			// nud_GradientThreshold
			// 
			this->nud_GradientThreshold->DecimalPlaces = 2;
			System::Int32 __mcTemp__57[] = new System::Int32[4];
			__mcTemp__57[0] = 1;
			__mcTemp__57[1] = 0;
			__mcTemp__57[2] = 0;
			__mcTemp__57[3] = 131072;
			this->nud_GradientThreshold->Increment = System::Decimal(__mcTemp__57);
			this->nud_GradientThreshold->Location = System::Drawing::Point(144, 104);
			System::Int32 __mcTemp__58[] = new System::Int32[4];
			__mcTemp__58[0] = 1;
			__mcTemp__58[1] = 0;
			__mcTemp__58[2] = 0;
			__mcTemp__58[3] = 0;
			this->nud_GradientThreshold->Maximum = System::Decimal(__mcTemp__58);
			System::Int32 __mcTemp__59[] = new System::Int32[4];
			/*__mcTemp__59[0] = 1;
			__mcTemp__59[1] = 0;
			__mcTemp__59[2] = 0;
			__mcTemp__59[3] = 131072;*/
			__mcTemp__59[0] = 0;
			__mcTemp__59[1] = 0;
			__mcTemp__59[2] = 0;
			__mcTemp__59[3] = 0;
			this->nud_GradientThreshold->Minimum = System::Decimal(__mcTemp__59);
			this->nud_GradientThreshold->Name = S"nud_GradientThreshold";
			this->nud_GradientThreshold->Size = System::Drawing::Size(48, 20);
			this->nud_GradientThreshold->TabIndex = 4;
			this->tot_ToolTips->SetToolTip(this->nud_GradientThreshold, S"Edit general Gradient magnitude threshold");
			System::Int32 __mcTemp__60[] = new System::Int32[4];
			__mcTemp__60[0] = 1;
			__mcTemp__60[1] = 0;
			__mcTemp__60[2] = 0;
			__mcTemp__60[3] = 131072;
			this->nud_GradientThreshold->Value = System::Decimal(__mcTemp__60);
			this->nud_GradientThreshold->ValueChanged += new System::EventHandler(this, perform_ActionNudValueChanged);
			// 
			// tex_SamplingDist
			// 
			this->tex_SamplingDist->Location = System::Drawing::Point(144, 8);
			this->tex_SamplingDist->Name = S"tex_SamplingDist";
			this->tex_SamplingDist->RightToLeft = System::Windows::Forms::RightToLeft::Yes;
			this->tex_SamplingDist->Size = System::Drawing::Size(48, 20);
			this->tex_SamplingDist->TabIndex = 0;
			this->tex_SamplingDist->Text = S"";
			this->tot_ToolTips->SetToolTip(this->tex_SamplingDist, S"Edit Sampling Distance");
			this->tex_SamplingDist->KeyDown += new System::Windows::Forms::KeyEventHandler(this, perform_ActionTextboxKeydown);
			// 
			// rightSandBarDock
			// 
			this->rightSandBarDock->Dock = System::Windows::Forms::DockStyle::Right;
			this->rightSandBarDock->Location = System::Drawing::Point(1016, 76);
			this->rightSandBarDock->Manager = this->sbM_GUIManager;
			this->rightSandBarDock->Name = S"rightSandBarDock";
			this->rightSandBarDock->Size = System::Drawing::Size(0, 826);
			this->rightSandBarDock->TabIndex = 1;
			// 
			// topSandBarDock
			// 
			this->topSandBarDock->Controls->Add(this->mam_MenueBar);
			this->topSandBarDock->Controls->Add(this->tob_FirstBar);
			this->topSandBarDock->Controls->Add(this->tob_SecondBar);
			this->topSandBarDock->Dock = System::Windows::Forms::DockStyle::Top;
			this->topSandBarDock->Location = System::Drawing::Point(0, 0);
			this->topSandBarDock->Manager = this->sbM_GUIManager;
			this->topSandBarDock->Name = S"topSandBarDock";
			this->topSandBarDock->Size = System::Drawing::Size(1016, 76);
			this->topSandBarDock->TabIndex = 3;
			// 
			// mam_MenueBar
			// 
			TD::SandBar::ToolbarItemBase* __mcTemp__61[] = new TD::SandBar::ToolbarItemBase*[4];
			__mcTemp__61[0] = this->mItem_File;
			__mcTemp__61[1] = this->mItem_Settings;
			__mcTemp__61[2] = this->mItem_Tools;
			__mcTemp__61[3] = this->mItem_Help;
			this->mam_MenueBar->Buttons->AddRange(__mcTemp__61);
			this->mam_MenueBar->Guid = System::Guid(S"f0e4b316-653c-4c3e-89ee-8bb6cbcd3a6a");
			this->mam_MenueBar->Location = System::Drawing::Point(2, 0);
			this->mam_MenueBar->Name = S"mam_MenueBar";
			this->mam_MenueBar->Size = System::Drawing::Size(1014, 24);
			this->mam_MenueBar->TabIndex = 0;
			// 
			// mItem_File
			// 
			TD::SandBar::MenuButtonItem* __mcTemp__62[] = new TD::SandBar::MenuButtonItem*[7];
			__mcTemp__62[0] = this->subItem_LoadDataFile;
			__mcTemp__62[1] = this->subItem_LoadMaskFile;
			__mcTemp__62[2] = this->subItem_LoadTransFunc;
			__mcTemp__62[3] = this->subItem_SaveTransFunc;
			__mcTemp__62[4] = this->subItem_LoadConfigFile;
			__mcTemp__62[5] = this->subItem_SaveConfigFile;
			__mcTemp__62[6] = this->subItem_SaveRenderedPicture;
			this->mItem_File->MenuItems->AddRange(__mcTemp__62);
			this->mItem_File->Text = S"&File";
			// 
			// subItem_LoadDataFile
			// 
			this->subItem_LoadDataFile->Shortcut = System::Windows::Forms::Shortcut::CtrlD;
			this->subItem_LoadDataFile->Text = S"Load Data File";
			this->subItem_LoadDataFile->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// subItem_LoadMaskFile
			// 
			this->subItem_LoadMaskFile->Shortcut = System::Windows::Forms::Shortcut::CtrlM;
			this->subItem_LoadMaskFile->Text = S"Load Mask File";
			this->subItem_LoadMaskFile->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// subItem_LoadTransFunc
			// 
			this->subItem_LoadTransFunc->BeginGroup = true;
			this->subItem_LoadTransFunc->Shortcut = System::Windows::Forms::Shortcut::CtrlT;
			this->subItem_LoadTransFunc->Text = S"Load  Transferfunction";
			this->subItem_LoadTransFunc->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// subItem_SaveTransFunc
			// 
			this->subItem_SaveTransFunc->Text = S"Save Transferfunction";
			this->subItem_SaveTransFunc->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// subItem_LoadConfigFile
			// 
			this->subItem_LoadConfigFile->BeginGroup = true;
			this->subItem_LoadConfigFile->Shortcut = System::Windows::Forms::Shortcut::CtrlC;
			this->subItem_LoadConfigFile->Text = S"Load Configuration File";
			this->subItem_LoadConfigFile->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// subItem_SaveConfigFile
			// 
			this->subItem_SaveConfigFile->Text = S"Save Configuration File";
			this->subItem_SaveConfigFile->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// subItem_SaveRenderedPicture
			// 
			this->subItem_SaveRenderedPicture->BeginGroup = true;
			this->subItem_SaveRenderedPicture->Shortcut = System::Windows::Forms::Shortcut::CtrlS;
			this->subItem_SaveRenderedPicture->Text = S"Save Rendered Picture";
			this->subItem_SaveRenderedPicture->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// mItem_Settings
			// 
			TD::SandBar::MenuButtonItem* __mcTemp__63[] = new TD::SandBar::MenuButtonItem*[2];
			__mcTemp__63[0] = this->subItem_LightProps;
			__mcTemp__63[1] = this->subItem_BackColor;
			this->mItem_Settings->MenuItems->AddRange(__mcTemp__63);
			this->mItem_Settings->Text = S"Settings";
			// 
			// subItem_LightProps
			// 
			TD::SandBar::MenuButtonItem* __mcTemp__64[] = new TD::SandBar::MenuButtonItem*[2];
			__mcTemp__64[0] = this->subItem_LightColor;
			__mcTemp__64[1] = this->subItem_LightViewMode;
			this->subItem_LightProps->MenuItems->AddRange(__mcTemp__64);
			this->subItem_LightProps->Text = S"Light Settings";
			// 
			// subItem_LightColor
			// 
			this->subItem_LightColor->Shortcut = System::Windows::Forms::Shortcut::CtrlL;
			this->subItem_LightColor->Text = S"Light Color";
			this->subItem_LightColor->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// subItem_LightViewMode
			// 
			TD::SandBar::MenuButtonItem* __mcTemp__65[] = new TD::SandBar::MenuButtonItem*[2];
			__mcTemp__65[0] = this->subItem_LightViewModeABS;
			__mcTemp__65[1] = this->subItem_LightViewModeREL;
			this->subItem_LightViewMode->MenuItems->AddRange(__mcTemp__65);
			this->subItem_LightViewMode->Text = S"Light View Mode";
			// 
			// subItem_LightViewModeABS
			// 
			this->subItem_LightViewModeABS->Text = S"Absolute Light View";
			this->subItem_LightViewModeABS->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// subItem_LightViewModeREL
			// 
			this->subItem_LightViewModeREL->Checked = true;
			this->subItem_LightViewModeREL->Text = S"Relative Light View";
			this->subItem_LightViewModeREL->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// subItem_BackColor
			// 
			this->subItem_BackColor->BeginGroup = true;
			this->subItem_BackColor->Shortcut = System::Windows::Forms::Shortcut::CtrlB;
			this->subItem_BackColor->Text = S"Background Color";
			this->subItem_BackColor->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// mItem_Tools
			// 
			TD::SandBar::MenuButtonItem* __mcTemp__66[] = new TD::SandBar::MenuButtonItem*[5];
			__mcTemp__66[0] = this->subItem_PreviewWindow;
			__mcTemp__66[1] = this->subItem_PreviewCanvasDefaultPosition;
			__mcTemp__66[2] = this->subItem_EditTransferFunction;
			__mcTemp__66[3] = this->subItem_SliceViewer;
			__mcTemp__66[4] = this->subItem_AnimationEditor;
			this->mItem_Tools->MenuItems->AddRange(__mcTemp__66);
			this->mItem_Tools->Text = S"Tools";
			// 
			// subItem_PreviewWindow
			// 
			this->subItem_PreviewWindow->BeginGroup = true;
			this->subItem_PreviewWindow->Checked = true;
			this->subItem_PreviewWindow->Shortcut = System::Windows::Forms::Shortcut::CtrlP;
			this->subItem_PreviewWindow->Text = S"Preview Canvas";
			this->subItem_PreviewWindow->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// subItem_PreviewCanvasDefaultPosition
			// 
			this->subItem_PreviewCanvasDefaultPosition->Text = S"    Default Position";
			this->subItem_PreviewCanvasDefaultPosition->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// subItem_EditTransferFunction
			// 
			this->subItem_EditTransferFunction->BeginGroup = true;
			this->subItem_EditTransferFunction->Shortcut = System::Windows::Forms::Shortcut::CtrlE;
			this->subItem_EditTransferFunction->Text = S"Transferfunction Editor";
			this->subItem_EditTransferFunction->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// subItem_SliceViewer
			// 
			this->subItem_SliceViewer->BeginGroup = true;
			this->subItem_SliceViewer->Shortcut = System::Windows::Forms::Shortcut::CtrlV;
			this->subItem_SliceViewer->Text = S"Slice Viewer";
			this->subItem_SliceViewer->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// mItem_Help
			// 
			TD::SandBar::MenuButtonItem* __mcTemp__67[] = new TD::SandBar::MenuButtonItem*[1];
			__mcTemp__67[0] = this->subItem_About;
			this->mItem_Help->MenuItems->AddRange(__mcTemp__67);
			this->mItem_Help->Text = S"&Help";
			// 
			// subItem_About
			// 
			this->subItem_About->Text = S"About...";
			this->subItem_About->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// tob_FirstBar
			// 
			TD::SandBar::ToolbarItemBase* __mcTemp__68[] = new TD::SandBar::ToolbarItemBase*[8];
			__mcTemp__68[0] = this->tbb_LoadData;
			__mcTemp__68[1] = this->tbb_LoadMask;
			__mcTemp__68[2] = this->tbc_FilePath;
			__mcTemp__68[3] = this->tbb_BackColor;
			__mcTemp__68[4] = this->tbl_BackColor;
			__mcTemp__68[5] = this->tbc_RenderLevel;
			__mcTemp__68[6] = this->tbMenuItem_Rendering;
			__mcTemp__68[7] = this->tbMenuItem_Tools;
			this->tob_FirstBar->Buttons->AddRange(__mcTemp__68);
			this->tob_FirstBar->Closable = false;
			this->tob_FirstBar->DockLine = 1;
			this->tob_FirstBar->Guid = System::Guid(S"8fd12a6a-b546-4edd-9659-f38db281e291");
			this->tob_FirstBar->ImageList = this->iml_ToolBarIcons;
			this->tob_FirstBar->Location = System::Drawing::Point(2, 24);
			this->tob_FirstBar->Movable = false;
			this->tob_FirstBar->Name = S"tob_FirstBar";
			this->tob_FirstBar->Size = System::Drawing::Size(1014, 26);
			this->tob_FirstBar->Stretch = true;
			this->tob_FirstBar->TabIndex = 1;
			this->tob_FirstBar->Text = S"";
			// 
			// tbb_LoadData
			// 
			this->tbb_LoadData->BuddyMenu = this->subItem_LoadDataFile;
			this->tbb_LoadData->ImageIndex = 0;
			this->tbb_LoadData->Text = S"Load Datafile";
			this->tbb_LoadData->ToolTipText = S"Load a DataSet File (*.dat)";
			// 
			// tbb_LoadMask
			// 
			this->tbb_LoadMask->BuddyMenu = this->subItem_LoadMaskFile;
			this->tbb_LoadMask->ImageIndex = 0;
			this->tbb_LoadMask->Text = S"Load Maskfile";
			this->tbb_LoadMask->ToolTipText = S"Load a mask file (*.msk) for an volume object";
			// 
			// tbc_FilePath
			// 
			this->tbc_FilePath->BeginGroup = true;
			this->tbc_FilePath->ControlWidth = 310;
			this->tbc_FilePath->Padding->Bottom = 0;
			this->tbc_FilePath->Padding->Left = 1;
			this->tbc_FilePath->Padding->Right = 1;
			this->tbc_FilePath->Padding->Top = 0;
			this->tbc_FilePath->Text = S"Current Dataset:";
			// 
			// tbb_BackColor
			// 
			this->tbb_BackColor->BeginGroup = true;
			this->tbb_BackColor->BuddyMenu = this->subItem_BackColor;
			this->tbb_BackColor->ImageIndex = 10;
			this->tbb_BackColor->Text = S"Back Color";
			this->tbb_BackColor->ToolTipText = S"Choose Background Color";
			// 
			// tbl_BackColor
			// 
			this->tbl_BackColor->ForeColor = System::Drawing::Color::Black;
			this->tbl_BackColor->Text = S"|||";
			// 
			// tbc_RenderLevel
			// 
			this->tbc_RenderLevel->BeginGroup = true;
			this->tbc_RenderLevel->ControlWidth = 110;
			System::Object* __mcTemp__69[] = new System::Object*[3];
			__mcTemp__69[0] = S"High-quality (slow)";
			__mcTemp__69[1] = S"Standard";
			__mcTemp__69[2] = S"Low-quality (fast)";
			this->tbc_RenderLevel->Items->AddRange(__mcTemp__69);
			this->tbc_RenderLevel->Padding->Bottom = 0;
			this->tbc_RenderLevel->Padding->Left = 1;
			this->tbc_RenderLevel->Padding->Right = 1;
			this->tbc_RenderLevel->Padding->Top = 0;
			// 
			// tbMenuItem_Rendering
			// 
			this->tbMenuItem_Rendering->BeginGroup = true;
			this->tbMenuItem_Rendering->ImageIndex = 1;
			this->tbMenuItem_Rendering->MenuImageList = this->iml_ToolBarIcons;
			TD::SandBar::MenuButtonItem* __mcTemp__70[] = new TD::SandBar::MenuButtonItem*[2];
			__mcTemp__70[0] = this->tbSubItem_StartRendering;
			__mcTemp__70[1] = this->tbSubItem_StopRendering;
			this->tbMenuItem_Rendering->MenuItems->AddRange(__mcTemp__70);
			this->tbMenuItem_Rendering->Text = S"Rendering";
			this->tbMenuItem_Rendering->ToolTipText = S"Rendering control";
			// 
			// tbSubItem_StartRendering
			// 
			this->tbSubItem_StartRendering->Text = S"Start Rendering";
			this->tbSubItem_StartRendering->Activate += new System::EventHandler(this, perform_ActionTbMenueButtonItemActivate);
			// 
			// tbSubItem_StopRendering
			// 
			this->tbSubItem_StopRendering->Text = S"Stop Rendering";
			this->tbSubItem_StopRendering->Activate += new System::EventHandler(this, perform_ActionTbMenueButtonItemActivate);
			// 
			// tbMenuItem_Tools
			// 
			TD::SandBar::MenuButtonItem* __mcTemp__71[] = new TD::SandBar::MenuButtonItem*[4];
			__mcTemp__71[0] = this->tbSubItem_PreviewWindow;
			__mcTemp__71[1] = this->tbSubItem_EditTransferfunction;
			__mcTemp__71[2] = this->tbSubItem_SliceViewer;
			__mcTemp__71[3] = this->tbSubItem_AnimationEditor;
			this->tbMenuItem_Tools->MenuItems->AddRange(__mcTemp__71);
			this->tbMenuItem_Tools->Text = S"Tools";
			// 
			// tbSubItem_PreviewWindow
			// 
			this->tbSubItem_PreviewWindow->Checked = true;
			this->tbSubItem_PreviewWindow->Text = S"Preview Canvas";
			this->tbSubItem_PreviewWindow->Activate += new System::EventHandler(this, perform_ActionTbMenueButtonItemActivate);
			// 
			// tbSubItem_EditTransferfunction
			// 
			this->tbSubItem_EditTransferfunction->Text = S"Transferfunction Editor";
			this->tbSubItem_EditTransferfunction->Activate += new System::EventHandler(this, perform_ActionTbMenueButtonItemActivate);
			// 
			// tbSubItem_SliceViewer
			// 
			this->tbSubItem_SliceViewer->Text = S"Slice Viewer";
			this->tbSubItem_SliceViewer->Activate += new System::EventHandler(this, perform_ActionTbMenueButtonItemActivate);
			// 
			// tob_SecondBar
			// 
			TD::SandBar::ToolbarItemBase* __mcTemp__72[] = new TD::SandBar::ToolbarItemBase*[18];
			__mcTemp__72[0] = this->tbb_LightColor;
			__mcTemp__72[1] = this->tbl_LightColor;
			__mcTemp__72[2] = this->tbl_LightCoords;
			__mcTemp__72[3] = this->tbb_LightCWRotation;
			__mcTemp__72[4] = this->tbb_LightCCWRotation;
			__mcTemp__72[5] = this->tbc_LightRotationAxis;
			__mcTemp__72[6] = this->tbc_ViewDirection;
			__mcTemp__72[7] = this->tbl_Transformation;
			__mcTemp__72[8] = this->tbb_TranslateLeft;
			__mcTemp__72[9] = this->tbb_TranslateRight;
			__mcTemp__72[10] = this->tbb_TranslateUp;
			__mcTemp__72[11] = this->tbb_TranslateDown;
			__mcTemp__72[12] = this->tbb_ZoomIn;
			__mcTemp__72[13] = this->tbb_ZoomOut;
			__mcTemp__72[14] = this->tbb_VPCWRotation;
			__mcTemp__72[15] = this->tbb_VPCCWRotation;
			__mcTemp__72[16] = this->tbc_VPRotationAxis;
			__mcTemp__72[17] = this->tbMenueItem_CanvasControl;
			this->tob_SecondBar->Buttons->AddRange(__mcTemp__72);
			this->tob_SecondBar->Closable = false;
			this->tob_SecondBar->DockLine = 2;
			this->tob_SecondBar->Guid = System::Guid(S"9865ae3b-d277-45da-9c0e-2f5f4f4672dc");
			this->tob_SecondBar->ImageList = this->iml_ToolBarIcons;
			this->tob_SecondBar->Location = System::Drawing::Point(2, 50);
			this->tob_SecondBar->Movable = false;
			this->tob_SecondBar->Name = S"tob_SecondBar";
			this->tob_SecondBar->Size = System::Drawing::Size(1014, 26);
			this->tob_SecondBar->Stretch = true;
			this->tob_SecondBar->TabIndex = 2;
			this->tob_SecondBar->Text = S"";
			// 
			// tbb_LightColor
			// 
			this->tbb_LightColor->BuddyMenu = this->subItem_LightColor;
			this->tbb_LightColor->ImageIndex = 10;
			this->tbb_LightColor->Text = S"Light Color";
			this->tbb_LightColor->ToolTipText = S"Choose Light Color";
			// 
			// tbl_LightColor
			// 
			this->tbl_LightColor->ForeColor = System::Drawing::Color::White;
			this->tbl_LightColor->Text = S"|||";
			// 
			// tbl_LightCoords
			// 
			this->tbl_LightCoords->Text = S"Lightposition:";
			this->tbl_LightCoords->ToolTipText = S"Change the Light position";
			// 
			// tbb_LightCWRotation
			// 
			this->tbb_LightCWRotation->ImageIndex = 8;
			this->tbb_LightCWRotation->ToolTipText = S"Clockwise Rotation  of Light Direction";
			this->tbb_LightCWRotation->Activate += new System::EventHandler(this, perform_ActionTbButtonActivate);
			// 
			// tbb_LightCCWRotation
			// 
			this->tbb_LightCCWRotation->ImageIndex = 9;
			this->tbb_LightCCWRotation->ToolTipText = S"Counterclockwise Rotation  of Light Direction";
			this->tbb_LightCCWRotation->Activate += new System::EventHandler(this, perform_ActionTbButtonActivate);
			// 
			// tbc_LightRotationAxis
			// 
			this->tbc_LightRotationAxis->ControlWidth = 30;
			System::Object* __mcTemp__73[] = new System::Object*[3];
			__mcTemp__73[0] = S"X";
			__mcTemp__73[1] = S"Y";
			__mcTemp__73[2] = S"Z";
			this->tbc_LightRotationAxis->Items->AddRange(__mcTemp__73);
			this->tbc_LightRotationAxis->Padding->Bottom = 0;
			this->tbc_LightRotationAxis->Padding->Left = 1;
			this->tbc_LightRotationAxis->Padding->Right = 1;
			this->tbc_LightRotationAxis->Padding->Top = 0;
			this->tbc_LightRotationAxis->Text = S"About Axis";
			// 
			// tbc_ViewDirection
			// 
			this->tbc_ViewDirection->BeginGroup = true;
			this->tbc_ViewDirection->ControlWidth = 90;
			System::Object* __mcTemp__74[] = new System::Object*[3];
			__mcTemp__74[0] = S"along pos. X";
			__mcTemp__74[1] = S"along pos. Y";
			__mcTemp__74[2] = S"along pos. Z";
			this->tbc_ViewDirection->Items->AddRange(__mcTemp__74);
			this->tbc_ViewDirection->Padding->Bottom = 0;
			this->tbc_ViewDirection->Padding->Left = 1;
			this->tbc_ViewDirection->Padding->Right = 1;
			this->tbc_ViewDirection->Padding->Top = 0;
			this->tbc_ViewDirection->Text = S"Standard View:";
			// 
			// tbl_Transformation
			// 
			this->tbl_Transformation->Text = S"Transform View:";
			this->tbl_Transformation->ToolTipText = S"Change the View position";
			// 
			// tbb_TranslateLeft
			// 
			this->tbb_TranslateLeft->ImageIndex = 2;
			this->tbb_TranslateLeft->ToolTipText = S"One step left";
			this->tbb_TranslateLeft->Activate += new System::EventHandler(this, perform_ActionTbButtonActivate);
			// 
			// tbb_TranslateRight
			// 
			this->tbb_TranslateRight->ImageIndex = 3;
			this->tbb_TranslateRight->ToolTipText = S"One step right";
			this->tbb_TranslateRight->Activate += new System::EventHandler(this, perform_ActionTbButtonActivate);
			// 
			// tbb_TranslateUp
			// 
			this->tbb_TranslateUp->ImageIndex = 4;
			this->tbb_TranslateUp->ToolTipText = S"One step up";
			this->tbb_TranslateUp->Activate += new System::EventHandler(this, perform_ActionTbButtonActivate);
			// 
			// tbb_TranslateDown
			// 
			this->tbb_TranslateDown->ImageIndex = 5;
			this->tbb_TranslateDown->ToolTipText = S"One step down";
			this->tbb_TranslateDown->Activate += new System::EventHandler(this, perform_ActionTbButtonActivate);
			// 
			// tbb_ZoomIn
			// 
			this->tbb_ZoomIn->BeginGroup = true;
			this->tbb_ZoomIn->ImageIndex = 6;
			this->tbb_ZoomIn->ToolTipText = S"Zoom in";
			this->tbb_ZoomIn->Activate += new System::EventHandler(this, perform_ActionTbButtonActivate);
			// 
			// tbb_ZoomOut
			// 
			this->tbb_ZoomOut->ImageIndex = 7;
			this->tbb_ZoomOut->ToolTipText = S"Zoom out";
			this->tbb_ZoomOut->Activate += new System::EventHandler(this, perform_ActionTbButtonActivate);
			// 
			// tbb_VPCWRotation
			// 
			this->tbb_VPCWRotation->BeginGroup = true;
			this->tbb_VPCWRotation->ImageIndex = 8;
			this->tbb_VPCWRotation->ToolTipText = S"Clockwise rotation of View Point";
			this->tbb_VPCWRotation->Activate += new System::EventHandler(this, perform_ActionTbButtonActivate);
			// 
			// tbb_VPCCWRotation
			// 
			this->tbb_VPCCWRotation->ImageIndex = 9;
			this->tbb_VPCCWRotation->ToolTipText = S"Counterclockwise Rotation  of View Point";
			this->tbb_VPCCWRotation->Activate += new System::EventHandler(this, perform_ActionTbButtonActivate);
			// 
			// tbc_VPRotationAxis
			// 
			this->tbc_VPRotationAxis->ControlWidth = 30;
			System::Object* __mcTemp__75[] = new System::Object*[3];
			__mcTemp__75[0] = S"X";
			__mcTemp__75[1] = S"Y";
			__mcTemp__75[2] = S"Z";
			this->tbc_VPRotationAxis->Items->AddRange(__mcTemp__75);
			this->tbc_VPRotationAxis->Padding->Bottom = 0;
			this->tbc_VPRotationAxis->Padding->Left = 1;
			this->tbc_VPRotationAxis->Padding->Right = 1;
			this->tbc_VPRotationAxis->Padding->Top = 0;
			this->tbc_VPRotationAxis->Text = S"About axis:";
			// 
			// tbMenueItem_CanvasControl
			// 
			this->tbMenueItem_CanvasControl->BeginGroup = true;
			TD::SandBar::MenuButtonItem* __mcTemp__76[] = new TD::SandBar::MenuButtonItem*[4];
			__mcTemp__76[0] = this->tbSubItem_ClearCanvas;
			__mcTemp__76[1] = this->tbSubItem_ResetVD;
			__mcTemp__76[2] = this->tbSubItem_ResetLight;
			__mcTemp__76[3] = this->tbSubItem_ResetLigthAndView;
			this->tbMenueItem_CanvasControl->MenuItems->AddRange(__mcTemp__76);
			this->tbMenueItem_CanvasControl->Text = S"Canvas";
			this->tbMenueItem_CanvasControl->ToolTipText = S"Canvas Control";
			// 
			// tbSubItem_ClearCanvas
			// 
			this->tbSubItem_ClearCanvas->Text = S"Clear Canvas";
			this->tbSubItem_ClearCanvas->Activate += new System::EventHandler(this, perform_ActionTbMenueButtonItemActivate);
			// 
			// tbSubItem_ResetVD
			// 
			this->tbSubItem_ResetVD->Text = S"Reset View Direction";
			this->tbSubItem_ResetVD->Activate += new System::EventHandler(this, perform_ActionTbMenueButtonItemActivate);
			// 
			// tbSubItem_ResetLight
			// 
			this->tbSubItem_ResetLight->Text = S"Reset Light Direction";
			this->tbSubItem_ResetLight->Activate += new System::EventHandler(this, perform_ActionTbMenueButtonItemActivate);
			// 
			// tbSubItem_ResetLigthAndView
			// 
			this->tbSubItem_ResetLigthAndView->Text = S"Reset Light and View";
			this->tbSubItem_ResetLigthAndView->Activate += new System::EventHandler(this, perform_ActionTbMenueButtonItemActivate);
			// 
			// stb_StatusBar
			// 
			this->stb_StatusBar->Font = new System::Drawing::Font(S"Microsoft Sans Serif", 8.25F, System::Drawing::FontStyle::Regular, System::Drawing::GraphicsUnit::Point, (System::Byte)0);
			this->stb_StatusBar->Location = System::Drawing::Point(0, 880);
			this->stb_StatusBar->Name = S"stb_StatusBar";
			System::Windows::Forms::StatusBarPanel* __mcTemp__77[] = new System::Windows::Forms::StatusBarPanel*[10];
			__mcTemp__77[0] = this->sbp_VolumeDimension;
			__mcTemp__77[1] = this->sbp_ViewDirection;
			__mcTemp__77[2] = this->sbp_LightDirection;
			__mcTemp__77[3] = this->sbp_RayDistance;
			__mcTemp__77[4] = this->sbp_DenityMode;
			__mcTemp__77[5] = this->sbp_GradientMode;
			__mcTemp__77[6] = this->sbp_VoxelIDMode;
			__mcTemp__77[7] = this->sbp_Gradient;
			__mcTemp__77[8] = this->sbp_RenderMode;
			__mcTemp__77[9] = this->sbp_Status;
			this->stb_StatusBar->Panels->AddRange(__mcTemp__77);
			this->stb_StatusBar->ShowPanels = true;
			this->stb_StatusBar->Size = System::Drawing::Size(1016, 22);
			this->stb_StatusBar->TabIndex = 4;
			// 
			// sbp_VolumeDimension
			// 
			this->sbp_VolumeDimension->Text = S"Dataset Dimension:";
			this->sbp_VolumeDimension->ToolTipText = S"Display dimaension of current loaded dataset";
			this->sbp_VolumeDimension->Width = 222;
			// 
			// sbp_ViewDirection
			// 
			this->sbp_ViewDirection->Text = S"View:";
			this->sbp_ViewDirection->ToolTipText = S"Display current view direction";
			this->sbp_ViewDirection->Width = 130;
			// 
			// sbp_LightDirection
			// 
			this->sbp_LightDirection->Text = S"Light:";
			this->sbp_LightDirection->ToolTipText = S"Display current light direction";
			this->sbp_LightDirection->Width = 130;
			// 
			// sbp_RayDistance
			// 
			this->sbp_RayDistance->Text = S"Ray Distance:";
			this->sbp_RayDistance->ToolTipText = S"Display current ray distance";
			this->sbp_RayDistance->Width = 120;
			// 
			// sbp_DenityMode
			// 
			this->sbp_DenityMode->Alignment = System::Windows::Forms::HorizontalAlignment::Center;
			this->sbp_DenityMode->Text = S"Density: TL";
			this->sbp_DenityMode->ToolTipText = S"Current DensityMode is TriLinear";
			this->sbp_DenityMode->Width = 70;
			// 
			// sbp_GradientMode
			// 
			this->sbp_GradientMode->Alignment = System::Windows::Forms::HorizontalAlignment::Center;
			this->sbp_GradientMode->Text = S"Gradient: TL";
			this->sbp_GradientMode->ToolTipText = S"Current GradientMode is TriLienear";
			this->sbp_GradientMode->Width = 70;
			// 
			// sbp_VoxelIDMode
			// 
			this->sbp_VoxelIDMode->Alignment = System::Windows::Forms::HorizontalAlignment::Center;
			this->sbp_VoxelIDMode->Text = S"VoxelID: NN";
			this->sbp_VoxelIDMode->ToolTipText = S"Current VoxelIDMode is Nearest Neighbour";
			this->sbp_VoxelIDMode->Width = 70;
			// 
			// sbp_Gradient
			// 
			this->sbp_Gradient->Alignment = System::Windows::Forms::HorizontalAlignment::Center;
			this->sbp_Gradient->Text = S"CentralD";
			this->sbp_Gradient->ToolTipText = S"Current GradientEstimation by Central Differences";
			this->sbp_Gradient->Width = 70;
			// 
			// sbp_RenderMode
			// 
			this->sbp_RenderMode->Alignment = System::Windows::Forms::HorizontalAlignment::Center;
			this->sbp_RenderMode->Text = S"SW";
			this->sbp_RenderMode->ToolTipText = S"Software renderer is activated";
			this->sbp_RenderMode->Width = 30;
			// 
			// sbp_Status
			// 
			this->sbp_Status->ToolTipText = S"Display currentsystem status";
			this->sbp_Status->Width = 105;
			// 
			// openDia_LoadMaskFile
			// 
			this->openDia_LoadMaskFile->Filter = S"mask files (*.mask)|*.mask|All files (*.*)|*.*";
			this->openDia_LoadMaskFile->RestoreDirectory = true;
			this->openDia_LoadMaskFile->Title = S"Open Mask (.MASK) File";
			this->openDia_LoadMaskFile->FileOk += new System::ComponentModel::CancelEventHandler(this, perform_ActionOpenDialogOK);
			// 
			// openDia_LoadDataFile
			// 
			this->openDia_LoadDataFile->Filter = S"dat files (*.dat)|*.dat|All files (*.*)|*.*";
			this->openDia_LoadDataFile->RestoreDirectory = true;
			this->openDia_LoadDataFile->Title = S"Load Volumedata (.DAT) File";
			this->openDia_LoadDataFile->FileOk += new System::ComponentModel::CancelEventHandler(this, perform_ActionOpenDialogOK);
			// 
			// colDia_LightColor
			// 
			this->colDia_LightColor->Color = System::Drawing::Color::White;
			// 
			// colDia_MipColor
			// 
			this->colDia_MipColor->Color = System::Drawing::Color::White;
			// 
			// colDia_ContourColor
			// 
			this->colDia_ContourColor->Color = System::Drawing::Color::White;
			// 
			// colDia_ToneColorWarm
			// 
			this->colDia_ToneColorWarm->Color = System::Drawing::Color::Yellow;
			// 
			// colDia_ToneColorCold
			// 
			this->colDia_ToneColorCold->Color = System::Drawing::Color::Blue;
			// 
			// openDia_LoadConfigFile
			// 
			this->openDia_LoadConfigFile->Filter = S"xml files (*.xml)|*.xml|All files (*.*)|*.*";
			this->openDia_LoadConfigFile->RestoreDirectory = true;
			this->openDia_LoadConfigFile->Title = S"Load Confiiguration File";
			this->openDia_LoadConfigFile->FileOk += new System::ComponentModel::CancelEventHandler(this, perform_ActionOpenDialogOK);
			// 
			// openDia_LoadTransFunc
			// 
			this->openDia_LoadTransFunc->Filter = S"xml files (*.xml)|*.xml|All files (*.*)|*.*";
			this->openDia_LoadTransFunc->RestoreDirectory = true;
			this->openDia_LoadTransFunc->Title = S"Load Transfer Function";
			this->openDia_LoadTransFunc->FileOk += new System::ComponentModel::CancelEventHandler(this, perform_ActionOpenDialogOK);
			// 
			// saveDia_SaveConfigFile
			// 
			this->saveDia_SaveConfigFile->Filter = S"xml files (*.xml)|*.xml|All files (*.*)|*.*";
			this->saveDia_SaveConfigFile->RestoreDirectory = true;
			this->saveDia_SaveConfigFile->Title = S"Save Configuration File";
			this->saveDia_SaveConfigFile->FileOk += new System::ComponentModel::CancelEventHandler(this, perform_ActionSaveDialogOK);
			// 
			// saveDia_SaveTransFunc
			// 
			this->saveDia_SaveTransFunc->Filter = S"xml files (*.xml)|*.xml|All files (*.*)|*.*";
			this->saveDia_SaveTransFunc->RestoreDirectory = true;
			this->saveDia_SaveTransFunc->Title = S"Save Transfer Function";
			this->saveDia_SaveTransFunc->FileOk += new System::ComponentModel::CancelEventHandler(this, perform_ActionSaveDialogOK);
			// 
			// tot_ToolTips
			// 
			this->tot_ToolTips->AutoPopDelay = 5000;
			this->tot_ToolTips->InitialDelay = 1000;
			this->tot_ToolTips->ReshowDelay = 500;
			// 
			// saveDia_SaveRenderedPicture
			// 
			this->saveDia_SaveRenderedPicture->Filter = S"JPEG Image (*.jpg)|*.jpg|GIF Image (*.gif)|*.gif|PNG Image (*.png)|*.png|TIFF Ima" 
				S"ge (*.tiff)|*.tiff|BMP Image (*.bmp)|*.bmp";
			this->saveDia_SaveRenderedPicture->RestoreDirectory = true;
			this->saveDia_SaveRenderedPicture->Title = S"Save Rendered Picture";
			this->saveDia_SaveRenderedPicture->FileOk += new System::ComponentModel::CancelEventHandler(this, perform_ActionSaveDialogOK);
			// 
			// comboBoxItem1
			// 
			this->comboBoxItem1->BeginGroup = true;
			this->comboBoxItem1->ControlWidth = 30;
			System::Object* __mcTemp__78[] = new System::Object*[3];
			__mcTemp__78[0] = S"X";
			__mcTemp__78[1] = S"Y";
			__mcTemp__78[2] = S"Z";
			this->comboBoxItem1->Items->AddRange(__mcTemp__78);
			this->comboBoxItem1->Padding->Bottom = 0;
			this->comboBoxItem1->Padding->Left = 1;
			this->comboBoxItem1->Padding->Right = 1;
			this->comboBoxItem1->Padding->Top = 0;
			this->comboBoxItem1->Text = S"About axis:";
			// 
			// tbSubItem_AnimationEditor
			// 
			this->tbSubItem_AnimationEditor->Text = S"Animation Editor";
			this->tbSubItem_AnimationEditor->Activate += new System::EventHandler(this, perform_ActionTbMenueButtonItemActivate);
			// 
			// subItem_AnimationEditor
			// 
			this->subItem_AnimationEditor->BeginGroup = true;
			this->subItem_AnimationEditor->Shortcut = System::Windows::Forms::Shortcut::CtrlA;
			this->subItem_AnimationEditor->Text = S"Animation Editor";
			this->subItem_AnimationEditor->Activate += new System::EventHandler(this, perform_ActionSubItemActivate);
			// 
			// Neu
			// 
			this->AutoScaleBaseSize = System::Drawing::Size(5, 13);
			this->ClientSize = System::Drawing::Size(1016, 902);
			this->Controls->Add(this->leftSandBarDock);
			this->Controls->Add(this->stb_StatusBar);
			this->Controls->Add(this->rightSandBarDock);
			this->Controls->Add(this->bottomSandBarDock);
			this->Controls->Add(this->topSandBarDock);
			this->IsMdiContainer = true;
			this->Name = S"Neu";
			this->StartPosition = System::Windows::Forms::FormStartPosition::Manual;
			this->Text = S"IDVR RayCaster";
			this->leftSandBarDock->ResumeLayout(false);
			this->ctb_RenderControl->ResumeLayout(false);
			this->containerBarClientPanel1->ResumeLayout(false);
			this->grb_IDVR->ResumeLayout(false);
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_IDVRGScale))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_IDVRCountersinkSlope))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_IDVRObjectImportance))->EndInit();
			this->grb_RenderControl->ResumeLayout(false);
			this->tab_RenderControl->ResumeLayout(false);
			this->tabPage_DVR->ResumeLayout(false);
			this->pan_Dvr->ResumeLayout(false);
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_SpExponent))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_Specular))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_Diffuse))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_Ambient))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_DepthK2))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_DepthK1))->EndInit();
			this->tabPage_MIP->ResumeLayout(false);
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->trb_MipThreshold))->EndInit();
			this->tabPage_Tone->ResumeLayout(false);
			this->pan_Tone->ResumeLayout(false);
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_ToneKTF))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_ToneKC))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_ToneKW))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_ToneGradient))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->trb_ToneThreshold))->EndInit();
			this->tabPage_Contour->ResumeLayout(false);
			this->pan_Contour->ResumeLayout(false);
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_ContourKTF))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_ContourExponent))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_ContourGradient))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->trb_ContourThreshold))->EndInit();
			this->grb_Rendering->ResumeLayout(false);
			this->tabControl1->ResumeLayout(false);
			this->tabPage_Basic->ResumeLayout(false);
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->trb_ImageRes))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->trb_CanvasSize))->EndInit();
			this->tabPage_Optimation->ResumeLayout(false);
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->trb_SamplingDist))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_OpacityThreshold))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_GradientThreshold))->EndInit();
			this->topSandBarDock->ResumeLayout(false);
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->sbp_VolumeDimension))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->sbp_ViewDirection))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->sbp_LightDirection))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->sbp_RayDistance))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->sbp_DenityMode))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->sbp_GradientMode))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->sbp_VoxelIDMode))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->sbp_Gradient))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->sbp_RenderMode))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->sbp_Status))->EndInit();
			this->ResumeLayout(false);

		}	

		/**
		Starts the actual rendering process for the image.
		The resulting image will be drawn onto the canvas.
		*/
		void startRenderingProcess(void);
		/**
		Handles the loading process of the volume data set.
		This function starts the loading process by creating an instance of the class DataLoader.
		@param n_FileName the user-defined file name and path of the *.dat file.
		*/
		void loadDatasetFile(String* n_FileName);
		/**
		Handles the loading process of the mask data.
		This function starts the loading process by creating an instance of the class DataLoader.
		@param n_MaskFileName the user-defined file name and path of the *.mask file.
		*/
		void loadMaskDataFile(String* n_MaskFileName);
		/**
		Initializes all GUI components with the default values.
		This function will be only called by the constructor Neu:Neu.
		*/
		void initialize_Elements(void);
		/**
		Resets all essential GUI components to the default values.
		*/
		void reinitialize_GUIComponents(void);
		/**
		Sets the GUI components to the READY status if the volume data has been loaded successfully.
		*/
		void setGUIElementsToReadyStatus(void);
		/**
		Sets the GUI components to the READY status if the rendering process has been finished.
		*/
		void setGUIElementsToRenderedStatus(void);
		/**
		Sets the GUI components to the NODATALOADED status if the volume data loading has been interrupted.
		*/
		void setGUIElementsToNoDataLoadedStatus(void);
		/**
		Updates the image plane attributes and the status bar if the user change those attributes over the GUI.
		*/
		void setCanvasImageElements(void);
		/**
		Sets the GUI components to the TWOLEVELRENDERIN status if two-level rendering has been selected.
		*/
		void setGUIElementsToTwoLevelRenderingStatus(void);
		/**
		Sets the GUI components to the NONTWOLEVELRENDERIN status if two-level rendering has not been selected.
		*/
		void setGUIElementsToNonTwoLevelRenderingStatus(void);
		/**
		Updates the status bar with the new choosen attributes of the light direction.
		*/
		void setLightDirectionDisplay(void);
		/**
		Updates the status bar with the new choosen attributes of the view direction.
		*/
		void setViewDirectionDisplay(void);

	// This function controls the FileDialog "Load Dataset" and calls the Loader function of the class DataLoader
	private: System::Void perform_ActionOpenDialogOK(System::Object *  sender, System::ComponentModel::CancelEventArgs *  e)
	{
		if (__try_cast<OpenFileDialog*>(sender)->Equals( this->openDia_LoadDataFile ))
		{
			 FileInfo *fileInfo = new FileInfo(openDia_LoadDataFile->FileName);

			 if(!fileInfo->Exists)
			 {
				 MessageBox::Show(this, S"The choosen file does not exist.", S"Error by reading the .dat-File", MessageBoxButtons::OK, MessageBoxIcon::Error);
				 return;
			 }
			 if(!fileInfo->Extension->ToLower()->EndsWith(S".dat"))
			 {
				 MessageBox::Show(this, S"The choosen file is apparently not a .dat file.", S"Error by reading the .dat-File", MessageBoxButtons::OK, MessageBoxIcon::Error);
				 return;
			 }

			 try
			 {
				 setGUIElementsToNoDataLoadedStatus();
				 loadDatasetFile(openDia_LoadDataFile->FileName);
			 }
			 catch(Exception *e)
			 {
				 MessageBox::Show(this, e->get_Message(), S"Error by reading the .DAT-File", MessageBoxButtons::OK, MessageBoxIcon::Error);
			 }
			 return;
		}

		if (__try_cast<OpenFileDialog*>(sender)->Equals( this->openDia_LoadMaskFile ))
		{
			 if(!openDia_LoadMaskFile->get_CheckFileExists())
			 {
				 MessageBox::Show(this, S"The choosen file does not exist.", S"Error by reading the .MASK-File", MessageBoxButtons::OK, MessageBoxIcon::Error);
				 return;
			 }
			 if(!openDia_LoadMaskFile->FileName->ToLower()->EndsWith(S".mask"))
			 {
				 MessageBox::Show(this, S"The choosen file is apparently not a .mask file.", S"Error by reading the .MASK-File", MessageBoxButtons::OK, MessageBoxIcon::Error);
				 return;
			 }
			 try
			 {
				 loadMaskDataFile(this->openDia_LoadMaskFile->FileName);
			 }
			 catch(Exception *e)
			 {
				MessageBox::Show(this, e->get_Message(), S"Error by reading the Mask-File", MessageBoxButtons::OK, MessageBoxIcon::Error);
			 }
			 return;
		}

		if (__try_cast<OpenFileDialog*>(sender)->Equals( this->openDia_LoadConfigFile ))
		{
			if(!openDia_LoadConfigFile->get_CheckFileExists())
			 {
				 MessageBox::Show(this, S"The choosen file does not exist.", S"Error by reading the .xml-File", MessageBoxButtons::OK, MessageBoxIcon::Error);
				 return;
			 }
			 if(!openDia_LoadConfigFile->FileName->ToLower()->EndsWith(S".xml"))
			 {
				 MessageBox::Show(this, S"The choosen file is apparently not a .xml file.", S"Error by reading the .MSK-File", MessageBoxButtons::OK, MessageBoxIcon::Error);
				 return;
			 }

			 try
			 {
				XMLParser *configFileReader = new XMLParser(this->openDia_LoadConfigFile->get_FileName());
				configFileReader->load_ConfigFile();

				reinitialize_GUIComponents();

				MessageBox::Show(this, S"All configuration values are loaded and activated successfully!", S"Configuration loading process", MessageBoxButtons::OK, MessageBoxIcon::Information);

			 }
			 catch(Exception *e)
			 {
				MessageBox::Show(this, e->get_Message(), S"Error by reading the Config-File", MessageBoxButtons::OK, MessageBoxIcon::Error);
			 }
			 return;	
		}

		if (__try_cast<OpenFileDialog*>(sender)->Equals( this->openDia_LoadTransFunc))
		{
			if(!openDia_LoadTransFunc->get_CheckFileExists())
			 {
				 MessageBox::Show(this, S"The choosen file does not exist.", S"Error by reading the .xml-File", MessageBoxButtons::OK, MessageBoxIcon::Error);
				 return;
			 }
			 if(!openDia_LoadTransFunc->FileName->ToLower()->EndsWith(S".xml"))
			 {
				 MessageBox::Show(this, S"The choosen file is apparently not a .xml file.", S"Error by reading the .MSK-File", MessageBoxButtons::OK, MessageBoxIcon::Error);
				 return;
			 }

			 try
			 {
				
				XMLParser *configFileReader = new XMLParser(this->openDia_LoadTransFunc->get_FileName());
				Transferfunction *newTransFunc = configFileReader->load_Transferfunction();
				
				//reinitialize_GUIComponents();

				this->m_TransFuncInput = newTransFunc;
				this->m_TransFuncInput->calculateOpacityFrequency(DataLoader::DensityScope);
				this->m_TransFuncInput->calculateColorFrequency(DataLoader::DensityScope);

				/*IRenderer *renderer = this->m_VolumeData->m_Dataset->getObjectMember(this->cob_TransObj->get_SelectedIndex())->get_RenderModel();
				if ( __try_cast<RendererSW*>(renderer)->getClassificator()->GetType() == __typeof( TFClassificationModelSW ))
				{
					//Save for actual selected transfer function
					__try_cast<RendererSW*>(renderer)->setClassificator(new TFClassificationModelSW(this->m_TransFuncInput));
				}*/
					
				if (this->tab_RenderControl->get_SelectedTab()->Equals( this->tabPage_DVR ))
					this->pan_TFPreview->Invalidate();
				else if (this->tab_RenderControl->get_SelectedTab()->Equals( this->tabPage_MIP ))
					this->rab_MipTFClassification->set_Checked(true);
				else if (this->tab_RenderControl->get_SelectedTab()->Equals( this->tabPage_Contour ))
					this->rab_ContourDvr->set_Checked(true);
				else if (this->tab_RenderControl->get_SelectedTab()->Equals( this->tabPage_Tone ))
					this->rab_ToneDvr->set_Checked(true);

				MessageBox::Show(this, S"The selected Transfer Function is loaded and activated successfully!", S"Transfer Function loading process", MessageBoxButtons::OK, MessageBoxIcon::Information);
			 }
			 catch(Exception *e)
			 {
				MessageBox::Show(this, e->get_Message(), S"Error by reading the Transfer Function-File", MessageBoxButtons::OK, MessageBoxIcon::Error);
			 }
			 return;
			
		}
	}

	//This function controls all TrackBar elements of the rendering quality group
	private: System::Void perform_ActionTrackBarScroll(System::Object *  sender, System::EventArgs *  e)
	{
			 if (__try_cast<TrackBar*>(sender)->Equals(trb_ImageRes))
			 {
				 int samplingMode = this->cob_SamplingFactor->get_SelectedIndex();
				 int actualValueImage =  this->trb_ImageRes->get_Value() *  this->trb_ImageRes->get_Maximum();
				 int minValue =     this->trb_ImageRes->get_Minimum() *  this->trb_ImageRes->get_Maximum();
				 int maxValue =     this->trb_ImageRes->get_Maximum() *  this->trb_ImageRes->get_Maximum();

				 switch (samplingMode)
				 {
					case SUPERSAMPLING4: 
						
						if ((actualValueImage / 4) < minValue)
						{
							this->trb_ImageRes->set_Value((minValue * 4) / this->trb_ImageRes->get_Maximum());
							break;
						}

						this->trb_CanvasSize->set_Value(this->trb_ImageRes->get_Value() / 4);
						break;

					case SUPERSAMPLING2:

						if ((actualValueImage / 2) < minValue)
						{
							this->trb_ImageRes->set_Value((minValue * 2) / this->trb_ImageRes->get_Maximum());
							break;
						}

						this->trb_CanvasSize->set_Value(this->trb_ImageRes->get_Value() / 2);
						break;

					case NORMALSAMPLING:

						this->trb_CanvasSize->set_Value(this->trb_ImageRes->get_Value());
						break;

					case SUBSAMPLING2:

						if ((actualValueImage * 2) > maxValue)
						{
							this->trb_ImageRes->set_Value((maxValue / 2) / this->trb_ImageRes->get_Maximum());
							break;
						}
		
						this->trb_CanvasSize->set_Value(this->trb_ImageRes->get_Value() * 2);
						break;

					case SUBSAMPLING4:

						if ((actualValueImage * 4) > maxValue)
						{
							this->trb_ImageRes->set_Value((maxValue / 4) / this->trb_ImageRes->get_Maximum());
							break;
						}
						
						this->trb_CanvasSize->set_Value(this->trb_ImageRes->get_Value() * 4);
						break;
				}

				setCanvasImageElements();
				return;
			 }
			 
			 if (__try_cast<TrackBar*>(sender)->Equals(trb_CanvasSize))
			 {
				 int samplingMode = this->cob_SamplingFactor->get_SelectedIndex();
				 int actualValueCanvas  =  this->trb_CanvasSize->get_Value()   *  this->trb_CanvasSize->get_Maximum();
				 int minValue =     this->trb_CanvasSize->get_Minimum() *  this->trb_CanvasSize->get_Maximum();
				 int maxValue =     this->trb_CanvasSize->get_Maximum() *  this->trb_CanvasSize->get_Maximum();

				 switch (samplingMode)
				 {
					case SUPERSAMPLING4:

						if (actualValueCanvas * 4 > maxValue)
						{
							this->trb_CanvasSize->set_Value((maxValue / 4) / this->trb_CanvasSize->get_Maximum());
							break;
						}

						this->trb_ImageRes->set_Value(this->trb_CanvasSize->get_Value() * 4);
						break;

					case SUPERSAMPLING2:

						if (actualValueCanvas * 2 > maxValue)
						{
							this->trb_CanvasSize->set_Value((maxValue / 2) / this->trb_CanvasSize->get_Maximum());
							break;
						}

						this->trb_ImageRes->set_Value(this->trb_CanvasSize->get_Value() * 2);
						break;

					case NORMALSAMPLING:

						this->trb_ImageRes->set_Value(this->trb_CanvasSize->get_Value());
						break;

					case SUBSAMPLING2:

						if (actualValueCanvas / 2 < minValue)
						{
							this->trb_CanvasSize->set_Value((minValue * 2) / this->trb_CanvasSize->get_Maximum());
							break;
						}

						this->trb_ImageRes->set_Value(this->trb_CanvasSize->get_Value() / 2);
						break;

					case SUBSAMPLING4:

						if (actualValueCanvas / 4 < minValue)
						{
							this->trb_CanvasSize->set_Value((minValue * 4) / this->trb_CanvasSize->get_Maximum());
							break;
						}

						this->trb_ImageRes->set_Value(this->trb_CanvasSize->get_Value() / 4);
						break;
				 }
				setCanvasImageElements();
				return;
			 }

			 if (__try_cast<TrackBar*>(sender)->Equals(trb_SamplingDist))
			 {
				//display current sampling distance
				 tex_SamplingDist->set_Text(((float)trb_SamplingDist->Value/10).ToString());
				 return;
			 }

			 if (__try_cast<TrackBar*>(sender)->Equals(trb_MipThreshold))
			 {	
				 //display current Mip Threshold
				 this->tex_MipThreshold->set_Text(this->trb_MipThreshold->get_Value().ToString());
				 return;
			 }

			 if (__try_cast<TrackBar*>(sender)->Equals(trb_ContourThreshold))
			 {	
				 //display current Contour Threshold
				 this->tex_ContourThreshold->set_Text(Convert::ToString((float)this->trb_ContourThreshold->get_Value() / 100.0f));
				 return;
			 }

			 if (__try_cast<TrackBar*>(sender)->Equals(trb_ToneThreshold))
			 {	
				 //display current Contour Threshold
				 this->tex_ToneThreshold->set_Text(Convert::ToString((float)this->trb_ToneThreshold->get_Value() / 100.0f));
				 return;
			 }
	 }

	// This function controls all editable TextBoxes of the rendering quality group
	private: System::Void perform_ActionTextboxKeydown(System::Object *  sender, System::Windows::Forms::KeyEventArgs *  e)
	{
		if (e->get_KeyValue() == 13)
			{
				if (__try_cast<TextBox*>(sender)->Equals(tex_SamplingDist))
					{
					try
					{
						double value = Convert::ToDouble(tex_SamplingDist->get_Text());	
						if (value > 2) value = 2;
						if (value < 0.1 ) value = 0,1;
						tex_SamplingDist->set_Text(value.ToString());
						trb_SamplingDist->set_Value(Convert::ToInt32(value * 10));
					}
					catch(Exception*)
					{
							tex_SamplingDist->set_Text(S"0,7");
							trb_SamplingDist->set_Value(7);
					}
					}
				else if (__try_cast<TextBox*>(sender)->Equals(tex_MipThreshold))
					{
					try
					{
						double value = Convert::ToInt32(tex_MipThreshold->get_Text());	
						if (value > 4095) value = 4095;
						if (value < 0 ) value = 0;
						tex_MipThreshold->set_Text(value.ToString());
						trb_MipThreshold->set_Value(Convert::ToInt32(value));
					}
					catch(Exception*)
					{
							tex_MipThreshold->set_Text(S"4095");
							trb_MipThreshold->set_Value(4095);
					}
					}
				else if (__try_cast<TextBox*>(sender)->Equals(tex_ContourThreshold))
					{
					try
					{
						double value = Convert::ToInt32(tex_ContourThreshold->get_Text());	
						if (value > 4095) value = 4095;
						if (value < 0 ) value = 0;
						tex_ContourThreshold->set_Text(value.ToString());
						trb_ContourThreshold->set_Value(Convert::ToInt32(value));
					}
					catch(Exception*)
					{
							tex_ContourThreshold->set_Text(S"4095");
							trb_ContourThreshold->set_Value(4095);
					}
					}
				else if (__try_cast<TextBox*>(sender)->Equals(tex_ToneThreshold))
					{
					try
					{
						double value = Convert::ToInt32(tex_ToneThreshold->get_Text());	
						if (value > 4095) value = 4095;
						if (value < 0 ) value = 0;
						tex_ToneThreshold->set_Text(value.ToString());
						trb_ToneThreshold->set_Value(Convert::ToInt32(value));
					}
					catch(Exception*)
					{
						tex_ToneThreshold->set_Text(S"4095");
						trb_ToneThreshold->set_Value(4095);
					}
			}		  
		}

	}

	//Load the current rendering settings for the selected volume object
	private: System::Void perform_ActionComboBoxSelectedIndexChanged(System::Object *  sender, System::EventArgs *  e)
	{
		if (__try_cast<ComboBox*>(sender)->Equals( this->cob_TransObj ))
		{
			if (this->cob_TransObj->get_SelectedIndex() < 0 || this->cob_TransObj->get_SelectedIndex() > this->m_VolumeData->m_Dataset->ObjectCount) 
				return;

			IRenderer* renderer = this->m_VolumeData->m_Dataset->getObjectMember(this->cob_TransObj->get_SelectedIndex())->get_RenderModel();

    		//Data object has no valid Rendermodel
			if (renderer == NULL)
			{
				this->tab_RenderControl->set_SelectedTab(this->tabPage_DVR);		
			}		 
			else
			{
				switch (renderer->getRenderMode())
				{
				case RendererSW::TF_PHONG_DVR:
							
					this->tab_RenderControl->set_SelectedTab(this->tabPage_DVR);
					this->perform_ActionTabPanelSelectedIndexChanged(this->tab_RenderControl, new System::EventArgs());
					break;

				case RendererSW::LIN_MIP_LMIP:

					this->tab_RenderControl->set_SelectedTab(this->tabPage_MIP);
					this->perform_ActionTabPanelSelectedIndexChanged(this->tab_RenderControl, new System::EventArgs());
					break;

				case RendererSW::TF_MIP_LMIP:

					this->tab_RenderControl->set_SelectedTab(this->tabPage_MIP);
					this->perform_ActionTabPanelSelectedIndexChanged(this->tab_RenderControl, new System::EventArgs());
					break;

				case RendererSW::LIN_CONTOUR_LMIP:

					this->tab_RenderControl->set_SelectedTab(this->tabPage_Contour);
					this->perform_ActionTabPanelSelectedIndexChanged(this->tab_RenderControl, new System::EventArgs());
					break;

				case RendererSW::TF_CONTOUR_DVR:

					this->tab_RenderControl->set_SelectedTab(this->tabPage_Contour);
					this->perform_ActionTabPanelSelectedIndexChanged(this->tab_RenderControl, new System::EventArgs());
					break;

				case RendererSW::LIN_TONE_LMIP:
	
					this->tab_RenderControl->set_SelectedTab(this->tabPage_Tone);	
					this->perform_ActionTabPanelSelectedIndexChanged(this->tab_RenderControl, new System::EventArgs());
					break;

				case RendererSW::TF_TONE_DVR:
	
					this->tab_RenderControl->set_SelectedTab(this->tabPage_Tone);
					this->perform_ActionTabPanelSelectedIndexChanged(this->tab_RenderControl, new System::EventArgs());
					break;
				}
				return;
			}
		}

		if (__try_cast<ComboBox*>(sender)->Equals( this->cob_IDVRObject ))
		{
			this->nud_IDVRObjectImportance->set_Value(Convert::ToDecimal(this->m_VolumeData->m_Dataset->getObjectMember(this->cob_IDVRObject->get_SelectedIndex())->get_Importance()));
		}

		if (__try_cast<ComboBox*>(sender)->Equals( this->cob_SamplingFactor ))
		{
			int currentValueImage =   this->trb_ImageRes->get_Value()   *  this->trb_ImageRes->get_Maximum();
			int currentValueCanvas =  this->trb_CanvasSize->get_Value()   *  this->trb_CanvasSize->get_Maximum();
			int minValue =     this->trb_ImageRes->get_Minimum() *  this->trb_ImageRes->get_Maximum();
			int maxValue =     this->trb_ImageRes->get_Maximum() *  this->trb_ImageRes->get_Maximum();

			switch(this->cob_SamplingFactor->get_SelectedIndex())
			{
			case SUPERSAMPLING4:

				if (currentValueImage / 4 >= minValue)
					this->trb_CanvasSize->set_Value(this->trb_ImageRes->get_Value() / 4);
				else
					if (currentValueCanvas * 4 <= maxValue)
						this->trb_ImageRes->set_Value(this->trb_CanvasSize->get_Value() * 4);
					else
					{
						this->trb_ImageRes->set_Value(16);
						this->trb_CanvasSize->set_Value(4);
					}
				break;

			case SUPERSAMPLING2:

				if (currentValueImage / 2 >= minValue)
					this->trb_CanvasSize->set_Value(this->trb_ImageRes->get_Value() / 2);
				else
					if (currentValueCanvas * 2 <= maxValue)
						this->trb_ImageRes->set_Value(this->trb_CanvasSize->get_Value() * 2);
					else
					{
						this->trb_ImageRes->set_Value(16);
						this->trb_CanvasSize->set_Value(8);
					}
				break;

			case NORMALSAMPLING:

					this->trb_CanvasSize->set_Value(this->trb_ImageRes->get_Value());
					break;

			case SUBSAMPLING2:

				if (currentValueImage * 2 <= maxValue)
					this->trb_CanvasSize->set_Value(this->trb_ImageRes->get_Value() * 2);
				else
					if (currentValueCanvas / 2 >= minValue)
						this->trb_ImageRes->set_Value(this->trb_CanvasSize->get_Value() / 2);
					else
					{
						this->trb_ImageRes->set_Value(8);
						this->trb_CanvasSize->set_Value(16);
					}
				break;

			case SUBSAMPLING4:

				if (currentValueImage * 4 <= maxValue)
					this->trb_CanvasSize->set_Value(this->trb_ImageRes->get_Value() * 4);
				else
					if (currentValueCanvas / 4 >= minValue)
						this->trb_ImageRes->set_Value(this->trb_CanvasSize->get_Value() / 4);
					else
					{
						this->trb_ImageRes->set_Value(4);
						this->trb_CanvasSize->set_Value(16);
					}
				break;
			}

			setCanvasImageElements();

		}
	}


	private: System::Void perform_ActionTbComboboxSelectedIndexChanged(System::Object *  sender, System::EventArgs *  e)
	{
		if (__try_cast<ComboBox*>(sender)->Equals( this->tbc_FilePath->get_ComboBox() ))
		{
			String* newFileName = __try_cast<String*>(this->tbc_FilePath->get_ComboBox()->get_SelectedItem());
			FileInfo *fileInfo = new FileInfo(newFileName);
			this->openDia_LoadDataFile->set_FileName(newFileName);

			if ( 0 != String::CompareOrdinal( this->m_VolumeData->FileName, newFileName) )
			{
				if(!fileInfo->Exists)
			 {
				 MessageBox::Show(this, S"The choosen file does not exist.", S"Error by reading the .dat-File", MessageBoxButtons::OK, MessageBoxIcon::Error);
				 return;
			 }
			 if(!fileInfo->Extension->ToLower()->EndsWith(S".dat"))
			 {
				 MessageBox::Show(this, S"The choosen file is apparently not a .dat file.", S"Error by reading the .dat-File", MessageBoxButtons::OK, MessageBoxIcon::Error);
				 return;
			 }
				try
			 {
				 setGUIElementsToNoDataLoadedStatus();
				 loadDatasetFile(newFileName);
			 }
			 catch(Exception *e)
			 {
				 MessageBox::Show(this, e->get_Message(), S"Error by reading the .DAT-File", MessageBoxButtons::OK, MessageBoxIcon::Error);
			 }
			}
			return;
		}

		if(__try_cast<ComboBox*>(sender)->Equals( this->tbc_ViewDirection->get_ComboBox()))
		{
			short dimension __gc[] = this->m_VolumeData->m_Dataset->get_VolumeDimension();

			if(this->tbc_ViewDirection->get_ComboBox()->get_SelectedIndex() == 0)
			{
				short width = dimension[Z];
				short heigth = dimension[Y];

				if(width > heigth)
				{
					Vector3 *leftBottom = new Vector3(-Math::Max(Math::Max(dimension[X], dimension[Y]), dimension[Z]), -Math::Round(width - heigth) / 2.0, heigth);

					this->m_ImagePlane = new ImagePlane(new Vector3(leftBottom-> X, 
																    leftBottom-> Y + width,
																    leftBottom-> Z),
														new Vector3(leftBottom-> X, 
																    leftBottom-> Y + width,
																    leftBottom-> Z - width),
														leftBottom,
														new Vector3(leftBottom-> X, 
																    leftBottom-> Y,
																    leftBottom-> Z - width),
														this->trb_ImageRes->get_Value() * this->trb_ImageRes->get_Maximum(),
														dimension);							
					}
				else if(width < heigth)
				{
					
					Vector3 *rightUpper = new Vector3(-Math::Max(Math::Max(dimension[X], dimension[Y]), dimension[Z]), 0.0 , heigth - Math::Round(heigth - width)/ 2.0);

					this->m_ImagePlane = new ImagePlane(new Vector3(rightUpper-> X, 
																    rightUpper-> Y,
																    rightUpper-> Z - heigth),
														rightUpper,
														new Vector3(rightUpper-> X, 
																    rightUpper-> Y + heigth,
																    rightUpper-> Z - heigth),
														new Vector3(rightUpper-> X, 
																    rightUpper-> Y + heigth,
																    rightUpper-> Z),
														this->trb_ImageRes->get_Value() * this->trb_ImageRes->get_Maximum(),
														dimension);							
				}
				else
				{
					Vector3 *leftBottom = new Vector3(-Math::Max(Math::Max(dimension[X], dimension[Y]), dimension[Z]), 0.0, heigth);

					this->m_ImagePlane = new ImagePlane(new Vector3(leftBottom-> X, 
																    leftBottom-> Y + heigth,
																    leftBottom-> Z),
														new Vector3(leftBottom-> X, 
																    leftBottom-> Y + heigth,
																    leftBottom-> Z - heigth),
														leftBottom,
														new Vector3(leftBottom-> X, 
																    leftBottom-> Y,
																    leftBottom-> Z - heigth),
														this->trb_ImageRes->get_Value() * this->trb_ImageRes->get_Maximum(),
														dimension);							
					}
				this->m_Canvas->set_ImagePlaneModel(this->m_ImagePlane);
				return;
			}

			if(this->tbc_ViewDirection->get_ComboBox()->get_SelectedIndex() == 1)
			{
				
				short width = dimension[X];
				short heigth = dimension[Z];

				if(width > heigth)
				{
					Vector3 *leftUpper = new Vector3(0.0, -Math::Max(Math::Max(dimension[X], dimension[Y]), dimension[Z]), width - Math::Round(width - heigth) / 2.0);

					this->m_ImagePlane = new ImagePlane(leftUpper,
														new Vector3(leftUpper->X, 
																    leftUpper->Y,
																    leftUpper->Z - width),
														new Vector3(leftUpper->X + width, 
																    leftUpper->Y,
																    leftUpper->Z),
														new Vector3(leftUpper->X + width, 
																    leftUpper->Y,
																    leftUpper->Z - width),
														this->trb_ImageRes->get_Value() * this->trb_ImageRes->get_Maximum(),
														dimension);							
					}
				else if(width < heigth)
				{
					
					Vector3 *leftUpper = new Vector3(heigth - Math::Round(heigth - width) / 2.0, -Math::Max(Math::Max(dimension[X], dimension[Y]), dimension[Z]), heigth);

					this->m_ImagePlane = new ImagePlane(leftUpper,
														new Vector3(leftUpper->X, 
																   leftUpper->Y,
																   leftUpper->Z - heigth),
														new Vector3(leftUpper->X + heigth, 
																   leftUpper->Y,
																   leftUpper->Z),
														new Vector3(leftUpper->X + heigth, 
																   leftUpper->Y,
																   leftUpper->Z - heigth),
														this->trb_ImageRes->get_Value() * this->trb_ImageRes->get_Maximum(),
														dimension);							
				}
				else
				{
					Vector3 *leftUpper = new Vector3(0.0, -Math::Max(Math::Max(dimension[X], dimension[Y]), dimension[Z]), heigth);

					this->m_ImagePlane = new ImagePlane(leftUpper,
														new Vector3(leftUpper->X, 
																   leftUpper->Y,
																   leftUpper->Z - heigth),
														new Vector3(leftUpper->X + heigth, 
																   leftUpper->Y,
																   leftUpper->Z),
														new Vector3(leftUpper->X + heigth, 
																   leftUpper->Y,
																   leftUpper->Z - heigth),
														this->trb_ImageRes->get_Value() * this->trb_ImageRes->get_Maximum(),
														dimension);							
					}
				this->m_Canvas->set_ImagePlaneModel(this->m_ImagePlane);

				return;
			}

			if(this->tbc_ViewDirection->get_ComboBox()->get_SelectedIndex() == 2)
			{
				short width = dimension[X];
				short heigth = dimension[Y];

				if(width > heigth)
				{
					Vector3 *leftBottom = new Vector3(0.0, -Math::Round((width - heigth) / 2.0), -Math::Max(Math::Max(dimension[X], dimension[Y]), dimension[Z]));

					this->m_ImagePlane = new ImagePlane(new Vector3(leftBottom->X, 
																	leftBottom->Y + heigth,
																	leftBottom->Z),
														new Vector3(leftBottom->X + heigth, 
																	leftBottom->Y + heigth,
																	leftBottom->Z),
														leftBottom,
														new Vector3(leftBottom->X + heigth, 
																	leftBottom->Y,
																	leftBottom->Z),
														this->trb_ImageRes->get_Value() * this->trb_ImageRes->get_Maximum(),
														dimension);	
				}
				else if(width < heigth)
				{
					Vector3 *leftBottom = new Vector3(-Math::Round(heigth - width) / 2.0, 0.0, -Math::Max(Math::Max(dimension[X], dimension[Y]), dimension[Z]));

					this->m_ImagePlane = new ImagePlane(new Vector3(leftBottom->X, 
																	leftBottom->Y + heigth,
																	leftBottom->Z),
														new Vector3(leftBottom->X + heigth, 
																	leftBottom->Y + heigth,
																	leftBottom->Z),
														leftBottom,
														new Vector3(leftBottom->X + heigth, 
																	leftBottom->Y,
																	leftBottom->Z),
														this->trb_ImageRes->get_Value() * this->trb_ImageRes->get_Maximum(),
														dimension);	
				}
				else
				{
					Vector3 *leftBottom = new Vector3(0.0, 0.0, -Math::Max(Math::Max(dimension[X], dimension[Y]), dimension[Z]));

					this->m_ImagePlane = new ImagePlane(new Vector3(leftBottom->X, 
																	leftBottom->Y + heigth,
																	leftBottom->Z),
														new Vector3(leftBottom->X + heigth, 
																	leftBottom->Y + heigth,
																	leftBottom->Z),
														leftBottom,
														new Vector3(leftBottom->X + heigth, 
																	leftBottom->Y,
																	leftBottom->Z),
														this->trb_ImageRes->get_Value() * this->trb_ImageRes->get_Maximum(),
														dimension);		

				}
				this->m_Canvas->set_ImagePlaneModel(this->m_ImagePlane);
				return;
			}
			return;			
		}

		if (__try_cast<ComboBox*>(sender)->Equals( this->tbc_RenderLevel->get_ComboBox()))
		{
			if(this->tbc_RenderLevel->get_ComboBox()->get_SelectedIndex() == 0)
			{
				this->cob_SamplingFactor->set_SelectedIndex(SUPERSAMPLING2);
				this->trb_SamplingDist->set_Value(5);
				this->tex_SamplingDist->set_Text(((float)this->trb_SamplingDist->Value/10).ToString());
				this->nud_OpacityThreshold->set_Value(Convert::ToDecimal(1.00f));
				this->nud_GradientThreshold->set_Value(Convert::ToDecimal(0.01f));

				this->subItem_AllTL->set_Checked(true);
				this->subItem_AllNN->set_Checked(false);
				this->subItem_DensityNN->set_Checked(false);
				this->subItem_DensityTL->set_Checked(true);
				this->subItem_GradientNN->set_Checked(false);
				this->subItem_GradientTL->set_Checked(true);
				this->subItem_VoxelIDNN->set_Checked(false);
				this->subItem_VoxelIDTL->set_Checked(true);

				this->m_DensityInterpolationMode  = InterpolationSW::BY_TRILINIEAR;
				this->m_GradientInterpolationMode = InterpolationSW::BY_TRILINIEAR;
				this->m_VoxelIDInterpolationMode  = InterpolationSW::BY_TRILINIEAR;

				this->sbp_DenityMode->set_Text(S"Density: TL");
				this->sbp_DenityMode->ToolTipText = S"Current DensityMode is TriLinear";
				this->sbp_GradientMode->set_Text(S"Gradient: TL");
				this->sbp_GradientMode->ToolTipText = S"Current GradientMode is TriLinear";
				this->sbp_VoxelIDMode->set_Text(S"VoxelID: TL");
				this->sbp_VoxelIDMode->ToolTipText = S"Current VoxelIDMode is TriLinear";

				if (this->m_SystemStatus == READY)
					perform_ActionSubItemActivate(this->subItem_Regression, new System::EventArgs());
			}

			else if(this->tbc_RenderLevel->get_ComboBox()->get_SelectedIndex() == 1)
			{
				this->cob_SamplingFactor->set_SelectedIndex(NORMALSAMPLING);
				this->trb_SamplingDist->set_Value(7);
				this->tex_SamplingDist->set_Text(((float)this->trb_SamplingDist->Value/10).ToString());
				this->nud_OpacityThreshold->set_Value(Convert::ToDecimal(1.00f));
				this->nud_GradientThreshold->set_Value(Convert::ToDecimal(0.01f));

				this->subItem_AllTL->set_Checked(false);
				this->subItem_AllNN->set_Checked(false);
				this->subItem_DensityNN->set_Checked(false);
				this->subItem_DensityTL->set_Checked(true);
				this->subItem_GradientNN->set_Checked(false);
				this->subItem_GradientTL->set_Checked(true);
				this->subItem_VoxelIDNN->set_Checked(true);
				this->subItem_VoxelIDTL->set_Checked(false);

				this->m_DensityInterpolationMode   = InterpolationSW::BY_TRILINIEAR;
				this->m_GradientEstimationMode     = InterpolationSW::BY_TRILINIEAR;
				this->m_VoxelIDInterpolationMode   = InterpolationSW::BY_NEARESTNEIGHBOUR;

				this->sbp_DenityMode->set_Text(S"Density: TL");
				this->sbp_DenityMode->ToolTipText = S"Current DensityMode is TriLinear";
				this->sbp_GradientMode->set_Text(S"Gradient: TL");
				this->sbp_GradientMode->ToolTipText = S"Current GradientMode is TriLinear";
				this->sbp_VoxelIDMode->set_Text(S"VoxelID: NN");
				this->sbp_VoxelIDMode->ToolTipText = S"Current VoxelIDMode is Nearest Neighbour";
			}

			else if(this->tbc_RenderLevel->get_ComboBox()->get_SelectedIndex() == 2)
			{
				this->cob_SamplingFactor->set_SelectedIndex(NORMALSAMPLING);
				this->trb_SamplingDist->set_Value(15);
				this->tex_SamplingDist->set_Text(((float)this->trb_SamplingDist->Value/10).ToString());
				this->nud_OpacityThreshold->set_Value(Convert::ToDecimal(0.90f));
				this->nud_GradientThreshold->set_Value(Convert::ToDecimal(0.10f));

				this->subItem_AllTL->set_Checked(false);
				this->subItem_AllNN->set_Checked(true);
				this->subItem_DensityNN->set_Checked(true);
				this->subItem_DensityTL->set_Checked(false);
				this->subItem_GradientNN->set_Checked(true);
				this->subItem_GradientTL->set_Checked(false);
				this->subItem_VoxelIDNN->set_Checked(true);
				this->subItem_VoxelIDTL->set_Checked(false);

				this->m_DensityInterpolationMode   = InterpolationSW::BY_NEARESTNEIGHBOUR;
				this->m_GradientEstimationMode     = InterpolationSW::BY_NEARESTNEIGHBOUR;
				this->m_VoxelIDInterpolationMode   = InterpolationSW::BY_NEARESTNEIGHBOUR;

				this->sbp_DenityMode->set_Text(S"Density: NN");
				this->sbp_DenityMode->ToolTipText = S"Current DensityMode is Nearest Neighbour";
				this->sbp_GradientMode->set_Text(S"Gradient: NN");
				this->sbp_GradientMode->ToolTipText = S"Current GradientMode is Nearest Neighbour";
				this->sbp_VoxelIDMode->set_Text(S"VoxelID: NN");
				this->sbp_VoxelIDMode->ToolTipText = S"Current VoxelIDMode is Nearest Neighbour";

				if (this->m_SystemStatus == READY)
					perform_ActionSubItemActivate(this->subItem_Central, new System::EventArgs());
			}
			return;
		}
	}


	// Paint the actual saved transfer function or the default funtion onto the Preview Panel
	private: System::Void perform_ActionPanelPaint(System::Object *  sender, System::Windows::Forms::PaintEventArgs *  e)
	{
		if (__try_cast<Panel*>(sender)->Equals( this->pan_TFPreview ))
		{
			if ((this->tab_RenderControl->get_SelectedIndex() == this->tabPage_DVR->get_TabIndex() - 1) && (this->m_SystemStatus != NODATALOADED))
			{
				Transferfunction *transFunc;
				bool validTF = false;

				//check whether the current transferfunction exsists or is invalid (length == 0)
				if (this->m_TransFuncInput)
				{
					if (this->m_TransFuncInput->get_FunctionLength() != 0)
						validTF = true;
				}

				if(validTF)
				{
					transFunc = this->m_TransFuncInput;
				}
				else
				{
					IRenderer* renderer = this->m_VolumeData->m_Dataset->getObjectMember(this->cob_TransObj->get_SelectedIndex())->get_RenderModel();
					if (renderer == NULL)
							// No valid renderer model is saved for the selected volume object -> default function will be painted
							transFunc = Transferfunction::CreateGeneralTransfer(DataLoader::DensityScope);
					else
						if (renderer->getRenderMode() == RendererSW::TF_PHONG_DVR)
							// Actual loaded function will be painted
							transFunc = __try_cast<TFClassificationModelSW*>(renderer->getClassificator())->get_TransferFunc();
						else
							// Another renderer model is saved for the volume object -> default function will be painted
							transFunc = Transferfunction::CreateGeneralTransfer(DataLoader::DensityScope);
				}

				Hashtable *opacitySamples = Transferfunction::convertOpacitySamplesToDrawMode(transFunc->get_OpacitySamples(),
																								this->pan_TFPreview->get_Size().Width - 5, 
																								this->pan_TFPreview->get_Size().Height - 6);
				Hashtable *colorSamples = Transferfunction::convertColorSamplesToDrawMode(transFunc->get_ColorSamples(),
																							this->pan_TFPreview->get_Size().Width - 5);
				int sortedIndicesOpacity __gc[] = TransferFuncWin::SortOpacityIndices(opacitySamples);;
				int sortedIndicesColor __gc[] =  TransferFuncWin::SortOpacityIndices(colorSamples);

				Graphics *g = this->pan_TFPreview->CreateGraphics();

				for (int i = 0; i < opacitySamples->get_Count() - 1; i++)
				{
					int x1 = sortedIndicesOpacity[i];
					int y1 = Convert::ToInt32(opacitySamples->get_Item(__box(sortedIndicesOpacity[i])));
					int x2 = sortedIndicesOpacity[i + 1];
					int y2 = Convert::ToInt32(opacitySamples->get_Item(__box(sortedIndicesOpacity[i + 1])));

					g->DrawLine(new Pen(Color::Gray, 2), x1, y1, x2, y2);
					float k;

					if ((x2 - x1) != 0)
						k = (float)(y2 - y1) / (float)(x2 - x1);
					else 
						k = 0.0f;

					if (colorSamples->get_Count() != 0)
					{
						for (int i = 0; i < colorSamples->get_Count(); i++)
						{	
							int currentColorX = sortedIndicesColor[i];
							Color currentSampleColor = Color::FromArgb(Convert::ToInt32(colorSamples->get_Item(__box(currentColorX))));

							if (currentColorX > x2) break;
							if (currentColorX >= x1 && currentColorX <= x2)
							{
								SolidBrush* colorBrush = new SolidBrush(currentSampleColor);
								int currentColorY = Convert::ToInt32(Math::Round((float)y1 + ((float)(currentColorX - x1) * k)));
								g->FillEllipse(colorBrush, currentColorX - 3, currentColorY - 3, 6, 6);	
							}
						}
					}
				}

				IDictionaryEnumerator *nextPos = opacitySamples->GetEnumerator();
				while (nextPos->MoveNext())
				{
					int x1 = Convert::ToInt32(nextPos->get_Key());
					int y1 = Convert::ToInt32(nextPos->get_Value());
					g->FillRectangle(new SolidBrush(Color::DarkGray), x1 - 2, y1 - 2, 4, 4);							 
				}

				this->pan_TFPreviewColor->Invalidate();
				return;
			}
		}

		if (__try_cast<Panel*>(sender)->Equals( this->pan_TFPreviewColor ))
		{
			if ((this->tab_RenderControl->get_SelectedIndex() == this->tabPage_DVR->get_TabIndex() - 1) && (this->m_SystemStatus != NODATALOADED))
			{
				Transferfunction *transFunc;
				IRenderer* renderer;
				bool validTF = false;

				//check whether the current transferfunction exsists or is invalid (length == 0)
				if (this->m_TransFuncInput)
				{
					if (this->m_TransFuncInput->get_FunctionLength() != 0)
						validTF = true;
				}

				if(validTF)
				{
					transFunc = this->m_TransFuncInput;
				}
				else
				{
					try
					{
						renderer = this->m_VolumeData->m_Dataset->getObjectMember(this->cob_TransObj->get_SelectedIndex())->get_RenderModel();
					}
					catch(Exception*)
					{
						return;
					}
					if (renderer == NULL)
							// No valid renderer model is saved for the selected volume object -> default function will be painted
							transFunc = Transferfunction::CreateGeneralTransfer(DataLoader::DensityScope);
					else
						if (renderer->getRenderMode() == RendererSW::TF_PHONG_DVR)
							// Actual loaded function will be painted
							transFunc = __try_cast<TFClassificationModelSW*>(renderer->getClassificator())->get_TransferFunc();
						else
							// Another renderer model is saved for the volume object -> default function will be painted
							transFunc = Transferfunction::CreateGeneralTransfer(DataLoader::DensityScope);
				}

				Graphics* g = this->pan_TFPreviewColor->CreateGraphics();
				Hashtable *colorSamples = Transferfunction::convertColorSamplesToDrawMode(transFunc->get_ColorSamples(),
																						this->pan_TFPreviewColor->get_Size().Width);

				if (colorSamples->get_Count() == 0)
					g->Clear(transFunc->get_BackColor());
				else
				{
					int width = this->pan_TFPreviewColor->get_Size().Width;
					int height = this->pan_TFPreviewColor->get_Size().Height;
					Color colorValues __gc[] = TransferFuncWin::DoInterpolation(transFunc, width);
					  
					for (int index  = 0; index < width; index++)
					{ 	
						g->DrawLine(new Pen(colorValues[index]), 
									index, 0, index, height);
					}
				}
				return;
			}
		}
	}

	private: System::Void perform_ActionButtonClick(System::Object *  sender, System::EventArgs *  e)
	{	
		if (__try_cast<Button*>(sender)->Equals(this->but_DVRAssign))
		{
			if (this->m_SystemStatus == RENDERING)
			{
				MessageBox::Show(this, S"Please wait until the rendering process has finished to avoid possible data inconsistency!", S"DVR setting error", MessageBoxButtons::OK, MessageBoxIcon::Warning);
				return;
			}

			unsigned short selectedObjectID = (unsigned short)this->cob_TransObj->get_SelectedIndex();

			if (this->m_TransFuncInput->get_FunctionLength() != 0)
			{
				this->m_VolumeData->m_Dataset->getObjectMember(selectedObjectID)->
					set_RenderModel(new RendererSW(RendererSW::TF_PHONG_DVR, 
									new TFClassificationModelSW(this->m_TransFuncInput),
									new PhongShadingModelSW(Convert::ToDouble(this->nud_Ambient->get_Value()),
															Convert::ToDouble(this->nud_Diffuse->get_Value()),
															Convert::ToDouble(this->nud_Specular->get_Value()),
															Convert::ToDouble(this->nud_SpExponent->get_Value()),
															Convert::ToDouble(this->nud_DepthK1->get_Value()),
															Convert::ToDouble(this->nud_DepthK2->get_Value()),
															this->m_TransFuncInput),
									new DvrCompositingModelSW()));	
			}
			else
			{
				Transferfunction *generalTF = Transferfunction::CreateGeneralTransfer(DataLoader::DensityScope);
				this->m_VolumeData->m_Dataset->getObjectMember(selectedObjectID)->
					set_RenderModel(new RendererSW(RendererSW::TF_PHONG_DVR, 
									new TFClassificationModelSW(generalTF),
									new PhongShadingModelSW(Convert::ToDouble(this->nud_Ambient->get_Value()),
															Convert::ToDouble(this->nud_Diffuse->get_Value()),
															Convert::ToDouble(this->nud_Specular->get_Value()),
															Convert::ToDouble(this->nud_SpExponent->get_Value()),
															Convert::ToDouble(this->nud_DepthK1->get_Value()),
															Convert::ToDouble(this->nud_DepthK2->get_Value()),
															generalTF),
									new DvrCompositingModelSW()));	
			}

			this->pan_TFPreview->Invalidate();
			return;
		}

		if (__try_cast<Button*>(sender)->Equals(this->but_DVRReset))
		{
			if (this->m_SystemStatus == RENDERING)
			{
				MessageBox::Show(this, S"Please wait until the rendering process has finished to avoid possible data inconsistency!", S"DVR setting error", MessageBoxButtons::OK, MessageBoxIcon::Warning);
				return;
			}

			 unsigned short selectedObjectID = (unsigned short)this->cob_TransObj->get_SelectedIndex();
			 IRenderer* renderer = this->m_VolumeData->m_Dataset->getObjectMember(selectedObjectID)->get_RenderModel();

			 if ( renderer == NULL) return;
			 if ( renderer->getRenderMode() == RendererSW::TF_PHONG_DVR)
			 {
				this->nud_Ambient->set_Value(Convert::ToDecimal(__try_cast<PhongShadingModelSW*>(renderer->getShader())->Ambient));
				this->nud_Diffuse->set_Value(Convert::ToDecimal(__try_cast<PhongShadingModelSW*>(renderer->getShader())->Diffuse));
				this->nud_Specular->set_Value(Convert::ToDecimal(__try_cast<PhongShadingModelSW*>(renderer->getShader())->Specular));
				this->nud_SpExponent->set_Value(Convert::ToDecimal(__try_cast<PhongShadingModelSW*>(renderer->getShader())->SpExponent));
				this->nud_DepthK1->set_Value(Convert::ToDecimal(__try_cast<PhongShadingModelSW*>(renderer->getShader())->DepthK1));
				this->nud_DepthK2->set_Value(Convert::ToDecimal(__try_cast<PhongShadingModelSW*>(renderer->getShader())->DepthK2));

				Transferfunction *generalTF = Transferfunction::CreateGeneralTransfer(DataLoader::DensityScope);
				this->m_VolumeData->m_Dataset->getObjectMember(selectedObjectID)->
					set_RenderModel(new RendererSW(RendererSW::TF_PHONG_DVR, 
									new TFClassificationModelSW(generalTF),
									new PhongShadingModelSW(Convert::ToDouble(this->nud_Ambient->get_Value()),
															Convert::ToDouble(this->nud_Diffuse->get_Value()),
															Convert::ToDouble(this->nud_Specular->get_Value()),
															Convert::ToDouble(this->nud_SpExponent->get_Value()),
															Convert::ToDouble(this->nud_DepthK1->get_Value()),
															Convert::ToDouble(this->nud_DepthK2->get_Value()),
															generalTF),
									new DvrCompositingModelSW()));	

				this->m_TransFuncInput = new Transferfunction();
				this->pan_TFPreview->Invalidate();
			 }
			 return;
		}

		if (__try_cast<Button*>(sender)->Equals(this->but_MipAssign))
		{
			if (this->m_SystemStatus == RENDERING)
			{
				MessageBox::Show(this, S"Please wait until the rendering process has finished to avoid possible data inconsistency!", S"MIP setting error", MessageBoxButtons::OK, MessageBoxIcon::Warning);
				return;
			}

			unsigned short selectedObjectID = (unsigned short)this->cob_TransObj->get_SelectedIndex();

			if(this->rab_MipLinClassification->Checked)
			{
				this->m_VolumeData->m_Dataset->getObjectMember(selectedObjectID)->
					set_RenderModel(new RendererSW(RendererSW::LIN_MIP_LMIP, 
									new LinearClassificationModelSW(DataLoader::DensityScope),
									new MipShadingModelSW(DataLoader::DensityScope, this->colDia_MipColor->get_Color()), 
									new LMipCompositingModelSW((float)this->trb_MipThreshold->get_Value(), LMipCompositingModelSW::DENSITY_PROJECTION)));
			}
			else if(this->rab_MipTFClassification->Checked)
			{
				if(this->m_TransFuncInput->get_FunctionLength() != 0)
				{
					this->m_VolumeData->m_Dataset->getObjectMember(selectedObjectID)->
						set_RenderModel(new RendererSW(RendererSW::TF_MIP_LMIP, 
										new TFClassificationModelSW(this->m_TransFuncInput),
										new MipShadingModelSW(DataLoader::DensityScope, this->colDia_MipColor->get_Color()), 
										new LMipCompositingModelSW((float)this->trb_MipThreshold->get_Value(), LMipCompositingModelSW::DENSITY_PROJECTION)));
				}
				else
				{
					this->m_VolumeData->m_Dataset->getObjectMember(selectedObjectID)->
						set_RenderModel(new RendererSW(RendererSW::TF_MIP_LMIP, 
										new TFClassificationModelSW(Transferfunction::CreateGeneralTransfer(DataLoader::DensityScope)),
										new MipShadingModelSW(DataLoader::DensityScope, this->colDia_MipColor->get_Color()), 
										new LMipCompositingModelSW((float)this->trb_MipThreshold->get_Value(), LMipCompositingModelSW::DENSITY_PROJECTION)));
				}
			}
			return;
		}

		if (__try_cast<Button*>(sender)->Equals(this->but_MipReset))
		{
			if (this->m_SystemStatus == RENDERING)
			{
				MessageBox::Show(this, S"Please wait until the rendering process has finished to avoid possible data inconsistency!", S"MIP setting error", MessageBoxButtons::OK, MessageBoxIcon::Warning);
				return;
			}

			 unsigned short selectedObjectID = (unsigned short)this->cob_TransObj->get_SelectedIndex();
			 IRenderer* renderer = this->m_VolumeData->m_Dataset->getObjectMember(selectedObjectID)->get_RenderModel();

			 if ( renderer == NULL) return;
			 if ( renderer->getRenderMode() == RendererSW::LIN_MIP_LMIP || renderer->getRenderMode() == RendererSW::TF_MIP_LMIP)
			 {
				this->rab_MipLinClassification->set_Checked(true);
				this->trb_MipThreshold->set_Value(Convert::ToInt16(Math::Round((double)DataLoader::Default_MipThreshold)));
				this->tex_MipThreshold->set_Text(Convert::ToString(this->trb_MipThreshold->get_Value()));
				this->colDia_MipColor->set_Color(DataLoader::Default_MipColor);
				this->lab_MipColor->set_BackColor(colDia_MipColor->get_Color());

				this->m_VolumeData->m_Dataset->getObjectMember(selectedObjectID)->
					set_RenderModel(new RendererSW(RendererSW::LIN_MIP_LMIP, 
									new LinearClassificationModelSW(DataLoader::DensityScope),
									new MipShadingModelSW(DataLoader::DensityScope, this->colDia_MipColor->get_Color()), 
									new LMipCompositingModelSW((float)this->trb_MipThreshold->get_Value(), LMipCompositingModelSW::DENSITY_PROJECTION)));
			 }
			 return;
		}

		if (__try_cast<Button*>(sender)->Equals(this->but_Render))
		{
			if (this->m_SystemStatus == RENDERING)
			{
				MessageBox::Show(this, S"Please wait until the rendering process has finished to avoid rendering failure!", S"Start rendering", MessageBoxButtons::OK, MessageBoxIcon::Warning);
				return;
			}

			this->sbp_Status->set_Text(S"Rendering data");
			startRenderingProcess();
			return;
		}

		if (__try_cast<Button*>(sender)->Equals(this->but_MipColor))
		{
			this->colDia_MipColor->set_FullOpen(true);
			if (this->colDia_MipColor->ShowDialog(this) == DialogResult::OK)
				this->lab_MipColor->set_BackColor(this->colDia_MipColor->get_Color());
			return;
		}

		if (__try_cast<Button*>(sender)->Equals(this->but_ContourColor))
		{
			this->colDia_ContourColor->set_FullOpen(true);
			if (this->colDia_ContourColor->ShowDialog(this) == DialogResult::OK)
				this->lab_ContourColor->set_BackColor(this->colDia_ContourColor->get_Color());
			return;
		}

		if (__try_cast<Button*>(sender)->Equals(this->but_ToneColWarm))
		{
			this->colDia_ToneColorWarm->set_FullOpen(true);
			if (this->colDia_ToneColorWarm->ShowDialog(this) == DialogResult::OK)
				this->lab_ToneColWarm->set_BackColor(this->colDia_ToneColorWarm->get_Color());
			return;
		}

		if (__try_cast<Button*>(sender)->Equals(this->but_ToneColCold))
		{
			this->colDia_ToneColorCold->set_FullOpen(true);
			if (this->colDia_ToneColorCold->ShowDialog(this) == DialogResult::OK)
				this->lab_ToneColCold->set_BackColor(this->colDia_ToneColorCold->get_Color());
			return;
		}

		if (__try_cast<Button*>(sender)->Equals(this->but_ContourAssign))
		{
			if (this->m_SystemStatus == RENDERING)
			{
				MessageBox::Show(this, S"Please wait until the rendering process has finished to avoid possible data inconsistency!", S"Contour setting error", MessageBoxButtons::OK, MessageBoxIcon::Warning);
				return;
			}

			unsigned short selectedObjectID = (unsigned short)this->cob_TransObj->get_SelectedIndex();

			if(this->rab_ContourDvr->Checked)
			{
				//IRenderer *renderer = this->m_VolumeData->m_Dataset->getObjectMember(Convert::ToInt16(this->cob_TransObj->get_SelectedIndex()))->get_RenderModel();
				
				if (this->m_TransFuncInput->get_FunctionLength() != 0)
				{
					this->m_VolumeData->m_Dataset->getObjectMember(selectedObjectID)->
						set_RenderModel(new RendererSW(RendererSW::TF_CONTOUR_DVR, 
									    new TFClassificationModelSW( this->m_TransFuncInput),
										new ContourShadingModelSW(DataLoader::DensityScope, 
															  Convert::ToDouble(this->nud_ContourGradient->get_Value()), 
															  Convert::ToInt32(this->nud_ContourExponent->get_Value()),
															  this->colDia_ContourColor->get_Color(),
															  (float)Convert::ToDouble(this->nud_ContourKTF->get_Value()),
															  this->m_TransFuncInput),
										new DvrCompositingModelSW()));
				}
				else
				{
					Transferfunction *generalTF = Transferfunction::CreateGeneralTransfer(DataLoader::DensityScope);
					this->m_VolumeData->m_Dataset->getObjectMember(selectedObjectID)->
						set_RenderModel(new RendererSW(RendererSW::TF_CONTOUR_DVR, 
									new TFClassificationModelSW(generalTF),
									new ContourShadingModelSW(DataLoader::DensityScope, 
															  Convert::ToDouble(this->nud_ContourGradient->get_Value()), 
															  Convert::ToInt32(this->nud_ContourExponent->get_Value()),
															  this->colDia_ContourColor->get_Color(),
															  (float)Convert::ToDouble(this->nud_ContourKTF->get_Value()),
															  generalTF),
									new DvrCompositingModelSW()));
				}

			}	
			else if(this->rab_ContourLMip->Checked)
			{
				this->m_VolumeData->m_Dataset->getObjectMember(selectedObjectID)->
					set_RenderModel(new RendererSW(RendererSW::LIN_CONTOUR_LMIP,
					new LinearClassificationModelSW(DataLoader::DensityScope),
					new ContourShadingModelSW(DataLoader::DensityScope, 
											Convert::ToDouble(this->nud_ContourGradient->get_Value()), 
											Convert::ToInt32(this->nud_ContourExponent->get_Value()),
											this->colDia_ContourColor->get_Color()),
											new LMipCompositingModelSW((float)this->trb_ContourThreshold->get_Value() / 100.0f, LMipCompositingModelSW::INTENSITY_PROJECTION)));
			}
			return;
		}


		if (__try_cast<Button*>(sender)->Equals(this->but_ToneAssign))
		{
			if (this->m_SystemStatus == RENDERING)
			{
				MessageBox::Show(this, S"Please wait until the rendering process has finished to avoid possible data inconsistency!", S"Tone setting error", MessageBoxButtons::OK, MessageBoxIcon::Warning);
				return;
			}

			unsigned short selectedObjectID = (unsigned short)this->cob_TransObj->get_SelectedIndex();

			if(this->rab_ToneDvr->Checked)
			{
				if (this->m_TransFuncInput->get_FunctionLength() != 0)
				{
					this->m_VolumeData->m_Dataset->getObjectMember(selectedObjectID)->
						set_RenderModel(new RendererSW(RendererSW::TF_TONE_DVR, 
										new TFClassificationModelSW(this->m_TransFuncInput),
										new ToneShadingModelSW(DataLoader::DensityScope,
															Convert::ToDouble(this->nud_ToneGradient->get_Value()), 
															Convert::ToDouble(this->nud_ToneKW->get_Value()), 
															this->colDia_ToneColorWarm->get_Color(), 
															Convert::ToDouble(this->nud_ToneKC->get_Value()), 
															this->colDia_ToneColorCold->get_Color(), 
															this->m_Light->OffsetLightDirection,
															(float)Convert::ToDouble(this->nud_ToneKTF->get_Value()),
															this->m_TransFuncInput),
										new DvrCompositingModelSW()));
				}
				else
				{
					Transferfunction *generalTF = Transferfunction::CreateGeneralTransfer(DataLoader::DensityScope);
					this->m_VolumeData->m_Dataset->getObjectMember(selectedObjectID)->
						set_RenderModel(new RendererSW(RendererSW::TF_TONE_DVR, 
										new TFClassificationModelSW(generalTF),
										new ToneShadingModelSW(DataLoader::DensityScope,
															Convert::ToDouble(this->nud_ToneGradient->get_Value()), 
															Convert::ToDouble(this->nud_ToneKW->get_Value()), 
															this->colDia_ToneColorWarm->get_Color(), 
															Convert::ToDouble(this->nud_ToneKC->get_Value()), 
															this->colDia_ToneColorCold->get_Color(), 
															this->m_Light->OffsetLightDirection,
															(float)Convert::ToDouble(this->nud_ToneKTF->get_Value()),
															generalTF),
										new DvrCompositingModelSW()));
				}
			}	
			else if(this->rab_ToneLMip->Checked)
			{
				this->m_VolumeData->m_Dataset->getObjectMember(selectedObjectID)->
					set_RenderModel(new RendererSW(RendererSW::LIN_TONE_LMIP,
									new LinearClassificationModelSW(DataLoader::DensityScope), 
									new ToneShadingModelSW(DataLoader::DensityScope,
														Convert::ToDouble(this->nud_ToneGradient->get_Value()), 
														Convert::ToDouble(this->nud_ToneKW->get_Value()), 
														this->colDia_ToneColorWarm->get_Color(), 
														Convert::ToDouble(this->nud_ToneKC->get_Value()), 
														this->colDia_ToneColorCold->get_Color(), 
														this->m_Light->OffsetLightDirection),
														new LMipCompositingModelSW((float)this->trb_ToneThreshold->get_Value() / 100.0f, LMipCompositingModelSW::INTENSITY_PROJECTION)));
			}
			return;
		}

		if(__try_cast<Button*>(sender)->Equals(this->but_MipTransferFunc))
		{
			if (this->get_ActiveMdiChild()->GetType() != __typeof( TransferFuncWin ))
			{
				if (this->cob_TransObj->get_SelectedIndex() == -1)
				{
					MessageBox::Show(this, S"Please choose a specific object first.", S"Loading error", MessageBoxButtons::OK, MessageBoxIcon::Error);
					return;
				}

				if (!this->subItem_EditTransferFunction->Checked)
				{
					TransferFuncWin* transferFunctionWindow = new TransferFuncWin(this->m_VolumeData, this->cob_TransObj->get_SelectedIndex(), TransferFuncWin::TF_PHONG_DVR, this->m_TransFuncInput);
					transferFunctionWindow->setPreViewPanel(this->pan_TFPreview);
					transferFunctionWindow->setTitleAddition(String::Concat(S"of volume object \"", this->m_VolumeData->m_Dataset->getObjectMember(this->cob_TransObj->get_SelectedIndex())->ObjectDescription, S"\""));
					transferFunctionWindow->setControlSubItem(this->subItem_EditTransferFunction);
					transferFunctionWindow->setControlTbSubItem(this->tbSubItem_EditTransferfunction);
					transferFunctionWindow->Show();
					this->subItem_EditTransferFunction->set_Checked(true);
				}
			}
			return;
		}

		if(__try_cast<Button*>(sender)->Equals(this->but_ToneTransferFunc))
		{
			if (this->get_ActiveMdiChild()->GetType() != __typeof( TransferFuncWin ))
			{
				if (this->cob_TransObj->get_SelectedIndex() == -1)
				{
					MessageBox::Show(this, S"Please choose a specific object first.", S"Loading error", MessageBoxButtons::OK, MessageBoxIcon::Error);
					return;
				}

				if (!this->subItem_EditTransferFunction->Checked)
				{	
					TransferFuncWin* transferFunctionWindow = new TransferFuncWin(this->m_VolumeData, this->cob_TransObj->get_SelectedIndex(), TransferFuncWin::TF_PHONG_DVR, this->m_TransFuncInput);
					transferFunctionWindow->setPreViewPanel(this->pan_TFPreview);
					transferFunctionWindow->setTitleAddition(String::Concat(S"of volume object \"", this->m_VolumeData->m_Dataset->getObjectMember(this->cob_TransObj->get_SelectedIndex())->ObjectDescription, S"\""));
					transferFunctionWindow->setControlSubItem(this->subItem_EditTransferFunction);
					transferFunctionWindow->setControlTbSubItem(this->tbSubItem_EditTransferfunction);
					transferFunctionWindow->Show();
					this->subItem_EditTransferFunction->set_Checked(true);
				}
			}
			return;
		}

		if(__try_cast<Button*>(sender)->Equals(this->but_ContourTransferFunc))
		{
			if (this->get_ActiveMdiChild()->GetType() != __typeof( TransferFuncWin ))
			{
				if (this->cob_TransObj->get_SelectedIndex() == -1)
				{
					MessageBox::Show(this, S"Please choose a specific object first.", S"Loading error", MessageBoxButtons::OK, MessageBoxIcon::Error);
					return;
				}

				if (!this->subItem_EditTransferFunction->Checked)
				{
					TransferFuncWin* transferFunctionWindow = new TransferFuncWin(this->m_VolumeData, this->cob_TransObj->get_SelectedIndex(), TransferFuncWin::TF_PHONG_DVR, this->m_TransFuncInput);
					transferFunctionWindow->setPreViewPanel(this->pan_TFPreview);
					transferFunctionWindow->setTitleAddition(String::Concat(S"of volume object \"", this->m_VolumeData->m_Dataset->getObjectMember(this->cob_TransObj->get_SelectedIndex())->ObjectDescription, S"\""));
					transferFunctionWindow->setControlSubItem(this->subItem_EditTransferFunction);
					transferFunctionWindow->setControlTbSubItem(this->tbSubItem_EditTransferfunction);
					transferFunctionWindow->Show();
					this->subItem_EditTransferFunction->set_Checked(true);
				}
			}
			return;
		}

		if (__try_cast<Button*>(sender)->Equals(this->but_ToneReset))
		{
			if (this->m_SystemStatus == RENDERING)
			{
				MessageBox::Show(this, S"Please wait until the rendering process has finished to avoid possible data inconsistency!", S"Tone setting error", MessageBoxButtons::OK, MessageBoxIcon::Warning);
				return;
			}

		    unsigned short selectedObjectID = (unsigned short)this->cob_TransObj->get_SelectedIndex();
			IRenderer* renderer = this->m_VolumeData->m_Dataset->getObjectMember(selectedObjectID)->get_RenderModel();

			if ( renderer == NULL) return;

			if (renderer->getRenderMode() == RendererSW::TF_TONE_DVR || renderer->getRenderMode() == RendererSW::LIN_TONE_LMIP)
			{
				this->rab_ToneLMip->set_Checked(true);
				this->trb_ToneThreshold->set_Value(Convert::ToInt16(Math::Round((double)DataLoader::Default_ToneThreshold)));
				this->tex_ToneThreshold->set_Text(Convert::ToString(this->trb_ToneThreshold->get_Value()));
				this->nud_ToneGradient->set_Value(Convert::ToDecimal(DataLoader::Default_ToneMagnitudeThreshold));
				this->nud_ToneKC->set_Value(Convert::ToDecimal(DataLoader::Default_ToneKCold));
				this->nud_ToneKW->set_Value(Convert::ToDecimal(DataLoader::Default_ToneKWarm));
				this->colDia_ToneColorCold->set_Color(DataLoader::Default_ToneColdColor);
				this->colDia_ToneColorWarm->set_Color(DataLoader::Default_ToneWarmColor);
				this->lab_ToneColCold->set_BackColor(DataLoader::Default_ToneColdColor);
				this->lab_ToneColWarm->set_BackColor(DataLoader::Default_ToneWarmColor);
				this->nud_ToneKTF->set_Value(Convert::ToDecimal(DataLoader::Default_ToneKTF));

				this->m_VolumeData->m_Dataset->getObjectMember(selectedObjectID)->
					set_RenderModel(new RendererSW(RendererSW::LIN_TONE_LMIP,
						            new LinearClassificationModelSW(DataLoader::DensityScope), 
									new ToneShadingModelSW(DataLoader::DensityScope,
								                       Convert::ToDouble(this->nud_ToneGradient->get_Value()), 
													   Convert::ToDouble(this->nud_ToneKW->get_Value()), 
													   this->colDia_ToneColorWarm->get_Color(), 
													   Convert::ToDouble(this->nud_ToneKC->get_Value()), 
													   this->colDia_ToneColorCold->get_Color(), 
													   this->m_Light->OffsetLightDirection),
									new LMipCompositingModelSW((float)this->trb_ToneThreshold->get_Value() / 100.0f, LMipCompositingModelSW::INTENSITY_PROJECTION)));
			}
			return;
		}

		if (__try_cast<Button*>(sender)->Equals(this->but_ContourReset))
		{
			if (this->m_SystemStatus == RENDERING)
			{
				MessageBox::Show(this, S"Please wait until the rendering process has finished to avoid possible data inconsistency!", S"Contour setting error", MessageBoxButtons::OK, MessageBoxIcon::Warning);
				return;
			}

			unsigned short selectedObjectID = (unsigned short)this->cob_TransObj->get_SelectedIndex();
			IRenderer* renderer = this->m_VolumeData->m_Dataset->getObjectMember(selectedObjectID)->get_RenderModel();

			if ( renderer == NULL) return;

			if (renderer->getRenderMode() == RendererSW::TF_CONTOUR_DVR || renderer->getRenderMode() == RendererSW::LIN_CONTOUR_LMIP)
			{
				this->rab_ContourLMip->set_Checked(true);
				this->trb_ContourThreshold->set_Value(Convert::ToInt16(Math::Round((double)DataLoader::Default_ContourThreshold)));
				this->tex_ContourThreshold->set_Text(Convert::ToString(this->trb_ContourThreshold->get_Value()));
				this->colDia_ContourColor->set_Color(DataLoader::Default_ContourColor);
				this->lab_ContourColor->set_BackColor(colDia_ContourColor->get_Color());
				this->nud_ContourGradient->set_Value(Convert::ToDecimal(DataLoader::Default_ContourGradientThreshold));
				this->nud_ContourExponent->set_Value(Convert::ToDecimal(DataLoader::Default_ContourExponent));
				this->nud_ContourKTF->set_Value(Convert::ToDecimal(DataLoader::Default_ContourKTF));

				this->m_VolumeData->m_Dataset->getObjectMember(selectedObjectID)->
						set_RenderModel(new RendererSW(RendererSW::LIN_CONTOUR_LMIP,
										new LinearClassificationModelSW(DataLoader::DensityScope),
										new ContourShadingModelSW(DataLoader::DensityScope, 
																  Convert::ToDouble(this->nud_ContourGradient->get_Value()), 
																  Convert::ToInt32(this->nud_ContourExponent->get_Value()),
																  this->colDia_ContourColor->get_Color()),
										new LMipCompositingModelSW((float)this->trb_ContourThreshold->get_Value() / 100.0f, LMipCompositingModelSW::INTENSITY_PROJECTION)));
			}
			return;
		}

		if (__try_cast<Button*>(sender)->Equals(this->but_IDVRAssign))
		{
			if(this->cob_IDVRCompositing->get_SelectedItem()->Equals(S"Maximum Importance") &&
			   this->cob_IDVRModulation->get_SelectedItem()->Equals(S"Color and Opacity"))
			{
				this->m_VolumeData->m_Dataset->setIIDVRCompositingModel(new IDVRMImPCompositingModelSW(new IDVRColAndOpModulatorSW(), 
																									   Convert::ToDouble(this->nud_IDVRCountersinkSlope->get_Value()),
																									   Convert::ToDouble(this->nud_IDVRGScale->get_Value())));
			}
		}

		if (__try_cast<Button*>(sender)->Equals(this->but_IDVRReset))
		{
			for(int i = 0; i < this->m_VolumeData->m_Dataset->get_ObjectCount(); i++)
			{
				this->m_VolumeData->m_Dataset->getObjectMember(i)->set_Importance(0);
			}	
			
			this->cob_IDVRObject->set_SelectedIndex(0);
			
			this->cob_IDVRCompositing->set_SelectedItem(S"Maximum Importance");
			this->cob_IDVRModulation->set_SelectedItem(S"Color and Opacity");

			this->m_VolumeData->m_Dataset->createGeneralIDVRObjects();

			this->nud_IDVRCountersinkSlope->set_Value(Convert::ToDecimal(1.0));

			return;
		}

		if (__try_cast<Button*>(sender)->Equals(this->but_LoadMaskFile))
		{
			perform_ActionSubItemActivate( this->subItem_LoadMaskFile, new System::EventArgs());
			return;
		}

		if (__try_cast<Button*>(sender)->Equals(this->but_DeleteVolumeObject))
		{
			unsigned short selectedObjectID = (unsigned short)this->cob_TransObj->get_SelectedIndex();
			String *objectName = Convert::ToString(this->cob_TransObj->get_SelectedItem());
			if (selectedObjectID == (unsigned short)0)
			{
				MessageBox::Show(this, S"The overall object must not be deleted.", S"Object could not be deleted", MessageBoxButtons::OK, MessageBoxIcon::Warning);
				return;
			}
			try
			{
				this->m_VolumeData->m_Dataset->removeObject(selectedObjectID);


				if (this->m_VolumeData->m_Dataset->ObjectCount == 1)
				{
					setGUIElementsToNonTwoLevelRenderingStatus();
					this->but_DeleteVolumeObject->Enabled = false;
				}
				else
					setGUIElementsToTwoLevelRenderingStatus();
			}
			catch(Exception *e)
			{
				MessageBox::Show(this, e->Message, S"Object Deletion Error", MessageBoxButtons::OK, MessageBoxIcon::Error);
				return;
			}
			
			MessageBox::Show(this, String::Concat(S"The deletion of the object \"", objectName, S"\" has finished successfully."), S"Object Deletion Success", MessageBoxButtons::OK, MessageBoxIcon::Information);
			return;
		}
	}

	private: System::Void perform_ActionSubItemActivate(System::Object *  sender, System::EventArgs *  e)
	{
		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_Central))
		{
			if (!this->subItem_Central->Checked)
			{
				this->m_GradientEstimationMode = GradientEstimationSW::BY_DIFFERENCES;
				this->subItem_Central->set_Checked(true);
				this->subItem_Regression->set_Checked(false);
				this->sbp_Gradient->set_Text(S"CentralD");
				this->sbp_Gradient->ToolTipText = S"Current GradientEstimation by Central Differences";
				this->m_VolumeData->set_GradientEstimationMode(this->m_GradientEstimationMode);

				if (this->m_SystemStatus == READY && this->m_VolumeData->m_Dataset->GradientsPreCalculated)
				{
					short currentMemorySize = (this->m_VolumeData->m_Dataset->VoxelCountOverall * DataLoader::NumOfBytesPerVoxel) / 1000000;
					if (currentMemorySize <= DataLoader::MaxUsableMemory)
					{
						this->set_Cursor(System::Windows::Forms::Cursors::WaitCursor);
						
						ProgressWin *progressWin  = new ProgressWin();
						System::Drawing::Point newLocation = System::Drawing::Point(
							this->get_Size().Width / 2 - (progressWin->get_Size().Width ),
							this->get_Size().Height / 4 - (progressWin->get_Size().Height / 2));
						progressWin->set_Location(System::Drawing::Point(newLocation));
						progressWin->set_MdiParent(this);
						progressWin->Show();

						this->m_VolumeData->setProgressbarHandler(progressWin);
						this->m_VolumeData->calculateGradients();

						this->set_Cursor(System::Windows::Forms::Cursors::Default);
						this->m_VolumeData->m_Dataset->set_GradientsPreCalculated(true);
					}
				}
			}
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_Regression))
		{
			if (!this->subItem_Regression->Checked)
			{
				this->m_GradientEstimationMode = GradientEstimationSW::BY_4DREGRESSION;
				this->subItem_Regression->set_Checked(true);
				this->subItem_Central->set_Checked(false);
				this->sbp_Gradient->set_Text(S"4DLinReg");
				this->sbp_Gradient->ToolTipText = S"Current GradientEstimation by 4D Reggression";
				this->m_VolumeData->set_GradientEstimationMode(this->m_GradientEstimationMode);

				if (this->m_SystemStatus == READY)
				{
					short currentMemorySize = (this->m_VolumeData->m_Dataset->VoxelCountOverall * DataLoader::NumOfBytesPerVoxel) / 1000000;
					if (currentMemorySize <= DataLoader::MaxUsableMemory)
					{
						this->set_Cursor(System::Windows::Forms::Cursors::WaitCursor);
					
						ProgressWin *progressWin  = new ProgressWin();
						System::Drawing::Point newLocation = System::Drawing::Point(
							this->get_Size().Width / 2 - (progressWin->get_Size().Width ),
							this->get_Size().Height / 4 - (progressWin->get_Size().Height / 2));
						progressWin->set_Location(System::Drawing::Point(newLocation));
						progressWin->set_MdiParent(this);
						progressWin->Show();
						

						this->m_VolumeData->setProgressbarHandler(progressWin);
						this->m_VolumeData->calculateGradients();

						this->set_Cursor(System::Windows::Forms::Cursors::Default);
						this->m_VolumeData->m_Dataset->set_GradientsPreCalculated(true);
					}
				}
			}
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_AllNN))
		{
			if (!this->subItem_AllNN->Checked)
			{
				this->m_DensityInterpolationMode  = InterpolationSW::BY_NEARESTNEIGHBOUR;
				this->m_GradientInterpolationMode = InterpolationSW::BY_NEARESTNEIGHBOUR;
				this->m_VoxelIDInterpolationMode  = InterpolationSW::BY_NEARESTNEIGHBOUR;
				this->subItem_AllNN->set_Checked(true);
				this->subItem_AllTL->set_Checked(false);

				this->subItem_DensityNN->set_Checked(true);
				this->subItem_DensityTL->set_Checked(false);
				this->subItem_GradientNN->set_Checked(true);
				this->subItem_GradientTL->set_Checked(false);
				this->subItem_VoxelIDNN->set_Checked(true);
				this->subItem_VoxelIDTL->set_Checked(false);

				this->sbp_DenityMode->set_Text(S"Density: NN");
				this->sbp_DenityMode->ToolTipText = S"Current DensityMode is Nearest Neighbour";
				this->sbp_GradientMode->set_Text(S"Gradient: NN");
				this->sbp_GradientMode->ToolTipText = S"Current GradientMode is Nearest Neighbour";
				this->sbp_VoxelIDMode->set_Text(S"VoxelID: NN");
				this->sbp_VoxelIDMode->ToolTipText = S"Current VoxelIDMode is Nearest Neighbour";
			}
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_AllTL))
		{
			if (!this->subItem_AllTL->Checked)
			{
				this->m_DensityInterpolationMode  = InterpolationSW::BY_TRILINIEAR;
				this->m_GradientInterpolationMode = InterpolationSW::BY_TRILINIEAR;
				this->m_VoxelIDInterpolationMode  = InterpolationSW::BY_TRILINIEAR;
				this->subItem_AllTL->set_Checked(true);
				this->subItem_AllNN->set_Checked(false);

				this->subItem_DensityNN->set_Checked(false);
				this->subItem_DensityTL->set_Checked(true);
				this->subItem_GradientNN->set_Checked(false);
				this->subItem_GradientTL->set_Checked(true);
				this->subItem_VoxelIDNN->set_Checked(false);
				this->subItem_VoxelIDTL->set_Checked(true);

				this->sbp_DenityMode->set_Text(S"Density: TL");
				this->sbp_DenityMode->ToolTipText = S"Current DensityMode is TriLinear";
				this->sbp_GradientMode->set_Text(S"Gradient: TL");
				this->sbp_GradientMode->ToolTipText = S"Current GradientMode is TriLinear";
				this->sbp_VoxelIDMode->set_Text(S"VoxelID: TL");
				this->sbp_VoxelIDMode->ToolTipText = S"Current VoxelIDMode is TriLinear";
			}
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_DensityNN))
		{
			if (!this->subItem_DensityNN->Checked)
			{
				this->m_DensityInterpolationMode  = InterpolationSW::BY_NEARESTNEIGHBOUR;
				this->subItem_DensityNN->set_Checked(true);
				this->subItem_DensityTL->set_Checked(false);

				this->sbp_DenityMode->set_Text(S"Density: NN");
				this->sbp_DenityMode->ToolTipText = S"Current DensityMode is Nearest Neighbour";

				if (this->subItem_AllTL->Checked)
					this->subItem_AllTL->set_Checked(false);
				if (!this->subItem_AllNN->Checked)
					if (this->subItem_GradientNN->Checked && this->subItem_VoxelIDNN->Checked)
						this->subItem_AllNN->set_Checked(true);
			}
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_DensityTL))
		{
			if (!this->subItem_DensityTL->Checked)
			{
				this->m_DensityInterpolationMode  = InterpolationSW::BY_TRILINIEAR;
				this->subItem_DensityTL->set_Checked(true);
				this->subItem_DensityNN->set_Checked(false);

				this->sbp_DenityMode->set_Text(S"Density: TL");
				this->sbp_DenityMode->ToolTipText = S"Current DensityMode is TriLinear";

				if (this->subItem_AllNN->Checked)
					this->subItem_AllNN->set_Checked(false);
				if (!this->subItem_AllTL->Checked)
					if (this->subItem_GradientTL->Checked && this->subItem_VoxelIDTL->Checked)
						this->subItem_AllTL->set_Checked(true);
			}
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_GradientNN))
		{
			if (!this->subItem_GradientNN->Checked)
			{
				this->m_GradientInterpolationMode  = InterpolationSW::BY_NEARESTNEIGHBOUR;
				this->subItem_GradientNN->set_Checked(true);
				this->subItem_GradientTL->set_Checked(false);

				this->sbp_GradientMode->set_Text(S"Gradient: NN");
				this->sbp_GradientMode->ToolTipText = S"Current GradientMode is Nearest Neighbour";

				if (this->subItem_AllTL->Checked)
					this->subItem_AllTL->set_Checked(false);
				if (!this->subItem_AllNN->Checked)
					if (this->subItem_DensityNN->Checked && this->subItem_VoxelIDNN->Checked)
						this->subItem_AllNN->set_Checked(true);
			}
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_GradientTL))
		{
			if (!this->subItem_GradientTL->Checked)
			{
				this->m_DensityInterpolationMode  = InterpolationSW::BY_TRILINIEAR;
				this->subItem_GradientTL->set_Checked(true);
				this->subItem_GradientNN->set_Checked(false);

				this->sbp_GradientMode->set_Text(S"Gradient: TL");
				this->sbp_GradientMode->ToolTipText = S"Current GradientMode is TriLinear";

				if (this->subItem_AllNN->Checked)
					this->subItem_AllNN->set_Checked(false);
				if (!this->subItem_AllTL->Checked)
					if (this->subItem_DensityTL->Checked && this->subItem_VoxelIDTL->Checked)
						this->subItem_AllTL->set_Checked(true);
			}
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_VoxelIDNN))
		{
			if (!this->subItem_VoxelIDNN->Checked)
			{
				this->m_VoxelIDInterpolationMode  = InterpolationSW::BY_NEARESTNEIGHBOUR;
				this->subItem_VoxelIDNN->set_Checked(true);
				this->subItem_VoxelIDTL->set_Checked(false);

				this->sbp_VoxelIDMode->set_Text(S"VoxelID: NN");
				this->sbp_VoxelIDMode->ToolTipText = S"Current VoxelIDMode is Nearest Neighbour";

				if (this->subItem_AllTL->Checked)
					this->subItem_AllTL->set_Checked(false);
				if (!this->subItem_AllNN->Checked)
					if (this->subItem_DensityNN->Checked && this->subItem_GradientNN->Checked)
						this->subItem_AllNN->set_Checked(true);
			}
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_VoxelIDTL))
		{
			if (!this->subItem_VoxelIDTL->Checked)
			{
				this->m_VoxelIDInterpolationMode  = InterpolationSW::BY_TRILINIEAR;
				this->subItem_VoxelIDTL->set_Checked(true);
				this->subItem_VoxelIDNN->set_Checked(false);

				this->sbp_VoxelIDMode->set_Text(S"VoxelID: TL");
				this->sbp_VoxelIDMode->ToolTipText = S"Current VoxelIDMode is TriLinear";

				if (this->subItem_AllNN->Checked)
					this->subItem_AllNN->set_Checked(false);
				if (!this->subItem_AllTL->Checked)
					if (this->subItem_DensityTL->Checked && this->subItem_GradientTL->Checked)
						this->subItem_AllTL->set_Checked(true);
			}
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_Software))
		{
			if (!this->subItem_Software->Checked)
			{
				this->m_RenderType = SOFTWARE;
				this->subItem_Software->set_Checked(true);
				this->subItem_Hardware->set_Checked(false);
				this->sbp_RenderMode->set_Text(S"SW");
				this->sbp_RenderMode->ToolTipText = S"Software renderer is activated";
			}
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_Hardware))
		{
			if (!this->subItem_Hardware->Checked)
			{
				this->m_RenderType = HARDWARE;
				this->subItem_Hardware->set_Checked(true);
				this->subItem_Software->set_Checked(false);
				this->sbp_RenderMode->set_Text(S"HW");
				this->sbp_RenderMode->ToolTipText = S"Hardware renderer is activated";
			}
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_LoadDataFile))
		{
			if (this->m_SystemStatus == RENDERING)
			{
				MessageBox::Show(this, S"Please wait until the rendering process has finished to avoid possible data inconsistency!", S"Load Datafile", MessageBoxButtons::OK, MessageBoxIcon::Warning);
				return;
			}

			openDia_LoadDataFile->ShowDialog(this);
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_LoadMaskFile))
		{
			if (this->m_SystemStatus == RENDERING)
			{
				MessageBox::Show(this, S"Please wait until the rendering process has finished to avoid possible data inconsistency!", S"Load Maskfile", MessageBoxButtons::OK, MessageBoxIcon::Warning);
				return;
			}

			openDia_LoadMaskFile->ShowDialog(this);
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_LoadTransFunc))
		{
			openDia_LoadTransFunc->ShowDialog(this);
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_LoadConfigFile))
		{
			openDia_LoadConfigFile->ShowDialog(this);
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_SaveConfigFile))
		{
			saveDia_SaveConfigFile->ShowDialog(this);
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_SaveTransFunc))
		{
			saveDia_SaveTransFunc->ShowDialog(this);
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_SaveRenderedPicture))
		{
			saveDia_SaveRenderedPicture->ShowDialog(this);
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_BackColor))
		{
			this->colDia_BackColor->FullOpen = true;
			if (this->colDia_BackColor->ShowDialog(this) == DialogResult::OK)
			{
				this->m_Canvas->set_CanvasBackColor(this->colDia_BackColor->get_Color());
				this->m_Canvas->clearCanvas();
				this->m_PreviewCanvas->set_CanvasBackColor(this->colDia_BackColor->get_Color());
				this->m_PreviewCanvas->clearCanvas();
				this->tbl_BackColor->set_ForeColor(this->colDia_BackColor->get_Color());
			}
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_LightColor))
		{
			this->colDia_LightColor->FullOpen = true;
			if (this->colDia_LightColor->ShowDialog(this) == DialogResult::OK)
			{
				this->tbl_LightColor->set_ForeColor(this->colDia_LightColor->get_Color());
				this->m_Light->set_LightColor(this->colDia_LightColor->get_Color().ToArgb());
				this->m_Canvas->set_LightModel(this->m_Light);
				this->m_PreviewCanvas->set_LightModel(this->m_Light);
			}
			return;
		}	

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_PreviewCanvasDefaultPosition))
		{
			if (this->m_PreviewCanvas)
			{	
				this->m_PreviewCanvas->set_Location(Point(0,0));
				this->m_PreviewCanvas->Invalidate();
			}
			
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_PreviewWindow))
		{
			if (this->subItem_PreviewWindow->Checked)
			{
				this->m_PreviewCanvas->Hide();
				this->subItem_PreviewWindow->set_Checked(false);
				this->tbSubItem_PreviewWindow->set_Checked(false);
			}
			else
			{
				try
				{
					this->m_PreviewCanvas->Show();
				}
				catch(Exception*)
				{
					//Initialize new OpenGLPreviewCanvas object
					this->m_PreviewCanvas = new OpenGLPreviewCanvas(PREVIEWCANVAS_DEFAULTPOSX,
																	PREVIEWCANVAS_DEFAULTPOSY, 
																	this->colDia_BackColor->get_Color(), 
																	Convert::ToSingle(this->nud_OpacityThreshold->Value),
																	Convert::ToSingle(this->nud_GradientThreshold->Value),
																    this);
					this->m_PreviewCanvas->setControlSubItem(this->subItem_PreviewWindow);
					this->m_PreviewCanvas->setControlTbSubItem(this->tbSubItem_PreviewWindow);
					this->m_PreviewCanvas->set_ShowInTaskbar(true);
					this->m_PreviewCanvas->Show();

					if (this->m_SystemStatus != NODATALOADED)
					{
						this->m_PreviewCanvas->set_Dataset(this->m_VolumeData->getDataset());
						this->m_PreviewCanvas->set_SystemStatus(this->m_SystemStatus);
						this->m_PreviewCanvas->set_OriginalImagePlane(this->m_ImagePlane);
						this->m_PreviewCanvas->set_CanvasBackColor(this->colDia_BackColor->get_Color());
						this->m_PreviewCanvas->set_LightModel(this->m_Light);
						this->m_PreviewCanvas->set_TwoLevelRendering(this->chb_TwoLevelVR->Checked);
						this->m_PreviewCanvas->set_IDVR(this->chb_IDVR->Checked);
					}
				
				}
				this->subItem_PreviewWindow->set_Checked(true);
				this->tbSubItem_PreviewWindow->set_Checked(true);
			}
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_EditTransferFunction))
		{

			if (!this->subItem_EditTransferFunction->Checked)
			{
				if (this->cob_TransObj->get_SelectedIndex() == -1)
					{
						MessageBox::Show(this, S"Please choose a specific object first.", S"Loading error", MessageBoxButtons::OK, MessageBoxIcon::Error);
						return;
					}
		
					TransferFuncWin* transferFunctionWindow = new TransferFuncWin(this->m_VolumeData, this->cob_TransObj->get_SelectedIndex(), TransferFuncWin::TF_PHONG_DVR, this->m_TransFuncInput);
					transferFunctionWindow->setPreViewPanel(this->pan_TFPreview);
					transferFunctionWindow->setTitleAddition(String::Concat(S"of volume object \"", this->m_VolumeData->m_Dataset->getObjectMember(this->cob_TransObj->get_SelectedIndex())->ObjectDescription, S"\""));
					transferFunctionWindow->setControlSubItem(this->subItem_EditTransferFunction);
					transferFunctionWindow->setControlTbSubItem(this->tbSubItem_EditTransferfunction);
					transferFunctionWindow->Show();
					this->subItem_EditTransferFunction->set_Checked(true);
					this->tbSubItem_EditTransferfunction->set_Checked(true);
			}

			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_SliceViewer))
		{
			if (!this->subItem_SliceViewer->Checked)
			{
				//Set title addition with the name of current loaded file
				FileInfo *info = new FileInfo(openDia_LoadDataFile->FileName);
				SliceViewer *sliceViewer = new SliceViewer(this->m_VolumeData->m_Dataset, info->Name);

				sliceViewer->setControlSubItem(this->subItem_SliceViewer);
				sliceViewer->setControlTbSubItem(this->tbSubItem_SliceViewer);
				sliceViewer->Show();
				this->subItem_SliceViewer->Checked = true;
				this->tbSubItem_SliceViewer->Checked = true;
			}
			
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_AnimationEditor))
		{
			if (!this->subItem_AnimationEditor->Checked)
			{
				if (this->m_RenderType == SOFTWARE)
				{
					this->m_ImagePlane->set_Resolution(this->trb_ImageRes->get_Value()*this->trb_ImageRes->get_Maximum());

					ImagePlane *animationImagePlane = new ImagePlane(this->m_ImagePlane);
					//Calculate the new imageplane position
					animationImagePlane->transformImagePlane();

					RenderMachineSW* rayCaster = new RenderMachineSW(this->m_VolumeData->m_Dataset, animationImagePlane, this->colDia_BackColor->get_Color(), NULL);

					rayCaster->set_DensityInterpolationMode(this->m_DensityInterpolationMode);
					rayCaster->set_GradientEstimationMode(this->m_GradientEstimationMode);
					rayCaster->set_VoxelIDInterpolationMode(this->m_VoxelIDInterpolationMode);
					rayCaster->set_GradientEstimationMode(this->m_GradientEstimationMode);
					rayCaster->set_SampingDistance(double(this->trb_SamplingDist->get_Value() / ((double)this->trb_SamplingDist->get_Maximum() / 2.0)));
					rayCaster->set_TwoLevelRendering(this->chb_TwoLevelVR->Checked);
					rayCaster->set_ImportanceDrivenRendering(this->chb_IDVR->Checked);
					rayCaster->set_OpacityThreshold(Convert::ToDouble(this->nud_OpacityThreshold->get_Value()));
					rayCaster->set_GradientThreshold(Convert::ToDouble(this->nud_GradientThreshold->get_Value()));

					//Set title addition with the name of current loaded file
					FileInfo *info = new FileInfo(openDia_LoadDataFile->FileName);
					AnimationWin *animationEditor = new AnimationWin(rayCaster, info->Name, this->m_Canvas->getCanvas()->Width , this->m_Canvas->getCanvas()->Height, new Light(this->m_Light));

					animationEditor->setControlSubItem(this->subItem_AnimationEditor);
					animationEditor->setControlTbSubItem(this->tbSubItem_AnimationEditor);
					animationEditor->Show();
					this->subItem_AnimationEditor->Checked = true;
					this->tbSubItem_AnimationEditor->Checked = true;
				}
			}
			
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_LightViewModeABS))
		{
			if (!this->subItem_LightViewModeABS->Checked)
			{
				this->subItem_LightViewModeABS->Checked = true;
				this->subItem_LightViewModeREL->Checked = false;
				this->m_LightViewMode = LightObjectGL::ABSOLUTELIGHTVIEW;

				if (this->m_Canvas)
					this->m_Canvas->setLightViewMode(this->m_LightViewMode);
			}
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_LightViewModeREL))
		{
			if (!this->subItem_LightViewModeREL->Checked)
			{
				this->subItem_LightViewModeREL->Checked = true;
				this->subItem_LightViewModeABS->Checked = false;
				this->m_LightViewMode = LightObjectGL::RELATIVLIGHTVIEW;

				if (this->m_Canvas)
					this->m_Canvas->setLightViewMode(this->m_LightViewMode);
			}
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals(this->subItem_About))
		{
			AboutWin *about = new AboutWin();
			about->Show();
			return;
		}

	}

	private: System::Void perform_ActionPanelClick(System::Object *  sender, System::EventArgs *  e)
	{
		if (this->cob_TransObj->get_SelectedIndex() == -1)
		{
			MessageBox::Show(this, S"Please choose a specific object first.", S"Loading error", MessageBoxButtons::OK, MessageBoxIcon::Error);
			return;
		}
		if (!this->subItem_EditTransferFunction->Checked)
		{
			TransferFuncWin* transferFunctionWindow = new TransferFuncWin(this->m_VolumeData, this->cob_TransObj->get_SelectedIndex(), TransferFuncWin::TF_PHONG_DVR, this->m_TransFuncInput);
			transferFunctionWindow->setPreViewPanel(this->pan_TFPreview);
			transferFunctionWindow->setTitleAddition(String::Concat(S"of volume object \"", this->m_VolumeData->m_Dataset->getObjectMember(this->cob_TransObj->get_SelectedIndex())->ObjectDescription, S"\""));
			transferFunctionWindow->setControlSubItem(this->subItem_EditTransferFunction);
			transferFunctionWindow->setControlTbSubItem(this->tbSubItem_EditTransferfunction);
			transferFunctionWindow->Show();
			this->subItem_EditTransferFunction->set_Checked(true);
		}
		
		return;
	}

	private: System::Void perform_ActionTbMenueButtonItemActivate(System::Object *  sender, System::EventArgs *  e)
	{
		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals( this->tbSubItem_ClearCanvas ))
		{
			this->m_Canvas->clearCanvas();
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals( this->tbSubItem_ResetVD ))
		{
			this->m_Canvas->resetViewPointPosition();
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals( this->tbSubItem_ResetLight ))
		{
			this->m_Canvas->resetLightDirection();
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals( this->tbSubItem_ResetLigthAndView ))
		{
			this->m_Canvas->resetLightDirection();
			this->m_Canvas->resetViewPointPosition();
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals( this->tbSubItem_StartRendering ))
		{
			perform_ActionButtonClick( this->but_Render, new System::EventArgs());
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals( this->tbSubItem_StopRendering ))
		{
			if (this->get_ActiveMdiChild()->GetType() == __typeof(ProgressWin))
			{
				this->get_ActiveMdiChild()->Close();
			}

			Thread *currentThread = Thread::CurrentThread;
			if (0 == String::CompareOrdinal(currentThread->Name, RENDERINGTHREAD))
				currentThread->Abort();

			setGUIElementsToRenderedStatus();
			
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals( this->tbSubItem_PreviewWindow ))
		{
			perform_ActionSubItemActivate(this->subItem_PreviewWindow, new System::EventArgs());
		
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals( this->tbSubItem_EditTransferfunction ))
		{
			perform_ActionSubItemActivate(this->subItem_EditTransferFunction, new System::EventArgs());
		
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals( this->tbSubItem_SliceViewer ))
		{
			perform_ActionSubItemActivate(this->subItem_SliceViewer, new System::EventArgs());
		
			return;
		}

		if (__try_cast<TD::SandBar::MenuButtonItem*>(sender)->Equals( this->tbSubItem_AnimationEditor ))
		{
			perform_ActionSubItemActivate(this->subItem_AnimationEditor, new System::EventArgs());
		
			return;
		}
	}

	private: System::Void perform_ActionTbButtonActivate(System::Object *  sender, System::EventArgs *  e)
	{
		if (__try_cast<TD::SandBar::ButtonItem*>(sender)->Equals( this->tbb_TranslateLeft ))
		{
			if (this->m_SystemStatus != READY) return;
			//ImagePlane
			Vector3 *transDirectionIP = this->m_ImagePlane->getHorizontalViewVector()->multSkalar(this->stepLength);
			Matrix4 *translationMatrixIP = Matrix4::CreateTranslationMatrix(transDirectionIP);
			this->m_ImagePlane->modifyTranslateMatrix(translationMatrixIP);
			this->m_ImagePlane->transformImagePlane();

			//ProxyObject
			Vector3 *transDirectionPO = new Vector3(-this->stepLength, 0.0, 0.0);
			Matrix4 *translationMatrixPO = Matrix4::CreateTranslationMatrix(transDirectionPO);
			this->m_Canvas->modifyProxyObjectTransMatrix(translationMatrixPO);
			this->m_Canvas->drawProxy();
			
			return;
		}
		if (__try_cast<TD::SandBar::ButtonItem*>(sender)->Equals( this->tbb_TranslateRight ))
		{
			if (this->m_SystemStatus != READY) return;
			//ImagePlane
			Vector3 *transDirectionIP = this->m_ImagePlane->getHorizontalViewVector()->multSkalar(this->stepLength)->getNegativeVector();
			Matrix4 *translationMatrixIP = Matrix4::CreateTranslationMatrix(transDirectionIP);
			this->m_ImagePlane->modifyTranslateMatrix(translationMatrixIP);
			this->m_ImagePlane->transformImagePlane();

			//ProxyObject
			Vector3 *transDirectionPO = new Vector3(this->stepLength, 0.0, 0.0);
			Matrix4 *translationMatrixPO = Matrix4::CreateTranslationMatrix(transDirectionPO);
			this->m_Canvas->modifyProxyObjectTransMatrix(translationMatrixPO);
			this->m_Canvas->drawProxy();

			return;
		}
		if (__try_cast<TD::SandBar::ButtonItem*>(sender)->Equals( this->tbb_TranslateUp ))
		{
			if (this->m_SystemStatus != READY) return;
			//ImagePlane
			Vector3 *transDirectionIP = this->m_ImagePlane->getVerticalViewVector()->multSkalar(this->stepLength);
			Matrix4 *translationMatrixIP = Matrix4::CreateTranslationMatrix(transDirectionIP);
			this->m_ImagePlane->modifyTranslateMatrix(translationMatrixIP);
			this->m_ImagePlane->transformImagePlane();

			//ProxyObject
			Vector3 *transDirectionPO = new Vector3(0.0, this->stepLength, 0.0);
			Matrix4 *translationMatrixPO = Matrix4::CreateTranslationMatrix(transDirectionPO);
			this->m_Canvas->modifyProxyObjectTransMatrix(translationMatrixPO);
			this->m_Canvas->drawProxy();

			return;
		}
		if (__try_cast<TD::SandBar::ButtonItem*>(sender)->Equals( this->tbb_TranslateDown ))
		{
			if (this->m_SystemStatus != READY) return;
			//ImagePlane
			Vector3 *transDirectionIP = this->m_ImagePlane->getVerticalViewVector()->multSkalar(this->stepLength)->getNegativeVector();
			Matrix4 *translationMatrixIP = Matrix4::CreateTranslationMatrix(transDirectionIP);
			this->m_ImagePlane->modifyTranslateMatrix(translationMatrixIP);
			this->m_ImagePlane->transformImagePlane();

			//ProxyObject
			Vector3 *transDirectionPO = new Vector3(0.0, -this->stepLength, 0.0);
			Matrix4 *translationMatrixPO = Matrix4::CreateTranslationMatrix(transDirectionPO);
			this->m_Canvas->modifyProxyObjectTransMatrix(translationMatrixPO);
			this->m_Canvas->drawProxy();
			
			return;
		}

		if (__try_cast<TD::SandBar::ButtonItem*>(sender)->Equals( this->tbb_ZoomIn ))
		{
			if (this->m_SystemStatus != READY) return;

			Vector3 *middlePoint = this->m_ImagePlane->get_NullPoint();
			
			//ImagePlane
			Matrix4 *scaleMatrixIP = Matrix4::CreateScaleUniformMatrix(0.9, middlePoint);
			this->m_ImagePlane->modifyScaleMatrix(scaleMatrixIP);
			this->m_ImagePlane->transformImagePlane();

			//ProxyObject
			Matrix4 *scaleMatrixPO = Matrix4::CreateScaleUniformMatrix(1/0.9);
			this->m_Canvas->modifyProxyObjectTransMatrix(scaleMatrixPO);
			this->m_Canvas->drawProxy();

			return;
		}

		if (__try_cast<TD::SandBar::ButtonItem*>(sender)->Equals( this->tbb_ZoomOut ))
		{
			if (this->m_SystemStatus != READY) return;

			Vector3 *middlePoint = this->m_ImagePlane->get_NullPoint();

			//ImagePlane
			Matrix4 *scaleMatrixIP = Matrix4::CreateScaleUniformMatrix(1/0.9, middlePoint);
			this->m_ImagePlane->modifyScaleMatrix(scaleMatrixIP);
			this->m_ImagePlane->transformImagePlane();

			//ProxyObject
			Matrix4 *scaleMatrixPO = Matrix4::CreateScaleUniformMatrix(0.9);
			this->m_Canvas->modifyProxyObjectTransMatrix(scaleMatrixPO);
			this->m_Canvas->drawProxy();
			
			return;
		}

		if (__try_cast<TD::SandBar::ButtonItem*>(sender)->Equals( this->tbb_VPCCWRotation ))
		{	
			if (this->m_SystemStatus != READY) return;

			if(__try_cast<String*>(this->tbc_VPRotationAxis->get_ComboBox()->get_SelectedItem())->Equals(S"X"))
			{
				//Rotationmatrix about x-axis for the transformation in world coord system (Imageplane, Light)
				Matrix4 *rotXWorld   = Matrix4::CreateRotXMatrix(ROTATIONSTEP);
				//Rotationmatrix about x-axis for the transformation in the display coord system (ProxyObject, LightArrowObject)
				Matrix4 *rotXDisplay = Matrix4::CreateRotXMatrix(-ROTATIONSTEP);

				//ImagePlane
				this->m_ImagePlane->modifyRotMatrix(rotXWorld);
				this->m_ImagePlane->transformImagePlane();
				setViewDirectionDisplay();

				//Lightsource
				this->m_Light->modifyBaseRotation(rotXWorld);
				this->m_Light->calculateLightDirection();
				setLightDirectionDisplay();

				//ProxyObject
				this->m_Canvas->modifyProxyObjectTransMatrix(rotXDisplay);
				this->m_Canvas->drawProxy();

				//LightObject
				this->m_Canvas->modifyLightObjectGLBaseRotation(rotXDisplay);
				this->m_Canvas->drawLightArrow();
			}

			if(__try_cast<String*>(this->tbc_VPRotationAxis->get_ComboBox()->get_SelectedItem())->Equals(S"Y"))
			{
				//Rotationmatrix about y-axis for the transformation in world coord system (Imageplane, Light)
				Matrix4 *rotYWorld   = Matrix4::CreateRotYMatrix(ROTATIONSTEP);
				//Rotationmatrix about y-axis for the transformation in the display coord system (ProxyObject, LightArrowObject)
				Matrix4 *rotYDisplay = Matrix4::CreateRotYMatrix(ROTATIONSTEP);

				//ImagePlane
				this->m_ImagePlane->modifyRotMatrix(rotYWorld);
				this->m_ImagePlane->transformImagePlane();
				setViewDirectionDisplay();
			
				//Lightsource
				this->m_Light->modifyBaseRotation(rotYWorld);
				this->m_Light->calculateLightDirection();
				setLightDirectionDisplay();

				//ProxyObject
				this->m_Canvas->modifyProxyObjectTransMatrix(rotYDisplay);
				this->m_Canvas->drawProxy();

				//LightObject
				this->m_Canvas->modifyLightObjectGLBaseRotation(rotYDisplay);
				this->m_Canvas->drawLightArrow();
			}

			if(__try_cast<String*>(this->tbc_VPRotationAxis->get_ComboBox()->get_SelectedItem())->Equals(S"Z"))
			{
				//Rotationmatrix about z-axis for the transformation in world coord system (Imageplane, Light)
				Matrix4 *rotZWorld   = Matrix4::CreateRotZMatrix(ROTATIONSTEP);
				//Rotationmatrix about z-axis for the transformation in the display coord system (ProxyObject, LightArrowObject)
				Matrix4 *rotZDisplay = Matrix4::CreateRotZMatrix(ROTATIONSTEP);

				//ImagePlane
				this->m_ImagePlane->modifyRotMatrix(rotZWorld);
				this->m_ImagePlane->transformImagePlane();
				setViewDirectionDisplay();
			
				//Lightsource
				this->m_Light->modifyBaseRotation(rotZWorld);
				this->m_Light->calculateLightDirection();
				setLightDirectionDisplay();
				
				//ProxyObject
				this->m_Canvas->modifyProxyObjectTransMatrix(rotZDisplay);
				this->m_Canvas->drawProxy();

				//LightObject
				this->m_Canvas->modifyLightObjectGLBaseRotation(rotZDisplay);
				this->m_Canvas->drawLightArrow();
			}
			return;
		}

		if (__try_cast<TD::SandBar::ButtonItem*>(sender)->Equals( this->tbb_VPCWRotation ))
		{	
			if (this->m_SystemStatus != READY) return;

			if(__try_cast<String*>(this->tbc_VPRotationAxis->get_ComboBox()->get_SelectedItem())->Equals(S"X"))
			{
				//Rotationmatrix about x-axis for the transformation in world coord system (Imageplane, Light)
				Matrix4 *rotXWorld   = Matrix4::CreateRotXMatrix(-ROTATIONSTEP);
				//Rotationmatrix about x-axis for the transformation in the display coord system (ProxyObject, LightArrowObject)
				Matrix4 *rotXDisplay = Matrix4::CreateRotXMatrix(ROTATIONSTEP);

				//ImagePlane
				this->m_ImagePlane->modifyRotMatrix(rotXWorld);
				this->m_ImagePlane->transformImagePlane();
				setViewDirectionDisplay();

				//Lightsource
				this->m_Light->modifyBaseRotation(rotXWorld);
				this->m_Light->calculateLightDirection();
				setLightDirectionDisplay();

				//ProxyObject
				this->m_Canvas->modifyProxyObjectTransMatrix(rotXDisplay);
				this->m_Canvas->drawProxy();
				
				//LightObject
				this->m_Canvas->modifyLightObjectGLBaseRotation(rotXDisplay);
				this->m_Canvas->drawLightArrow();
			}

			if(__try_cast<String*>(this->tbc_VPRotationAxis->get_ComboBox()->get_SelectedItem())->Equals(S"Y"))
			{
				//Rotationmatrix about y-axis for the transformation in world coord system (Imageplane, Light)
				Matrix4 *rotYWorld   = Matrix4::CreateRotYMatrix(-ROTATIONSTEP);
				//Rotationmatrix about y-axis for the transformation in the display coord system (ProxyObject, LightArrowObject)
				Matrix4 *rotYDisplay = Matrix4::CreateRotYMatrix(-ROTATIONSTEP);

				//ImagePlane
				this->m_ImagePlane->modifyRotMatrix(rotYWorld);
				this->m_ImagePlane->transformImagePlane();
				setViewDirectionDisplay();

				//Lightsource
				this->m_Light->modifyBaseRotation(rotYWorld);
				this->m_Light->calculateLightDirection();
				setLightDirectionDisplay();
			
				//ProxyObject
				this->m_Canvas->modifyProxyObjectTransMatrix(rotYDisplay);
				this->m_Canvas->drawProxy();

				//LightObject
				this->m_Canvas->modifyLightObjectGLBaseRotation(rotYDisplay);
				this->m_Canvas->drawLightArrow();
			}

			if(__try_cast<String*>(this->tbc_VPRotationAxis->get_ComboBox()->get_SelectedItem())->Equals(S"Z"))
			{
				//Rotationmatrix about z-axis for the transformation in world coord system (Imageplane, Light)
				Matrix4 *rotZWorld   = Matrix4::CreateRotZMatrix(-ROTATIONSTEP);
				//Rotationmatrix about z-axis for the transformation in the display coord system (ProxyObject, LightArrowObject)
				Matrix4 *rotZDisplay = Matrix4::CreateRotZMatrix(-ROTATIONSTEP);

				//ImagePlane
				this->m_ImagePlane->modifyRotMatrix(rotZWorld);
				this->m_ImagePlane->transformImagePlane();
				setViewDirectionDisplay();

				//Lightsource
				this->m_Light->modifyBaseRotation(rotZWorld);
				this->m_Light->calculateLightDirection();
				setLightDirectionDisplay();
			
				//ProxyObject
				this->m_Canvas->modifyProxyObjectTransMatrix(rotZDisplay);
				this->m_Canvas->drawProxy();
				
				//LightObject
				this->m_Canvas->modifyLightObjectGLBaseRotation(rotZDisplay);
				this->m_Canvas->drawLightArrow();
			}
			return;
		}

		if (__try_cast<TD::SandBar::ButtonItem*>(sender)->Equals( this->tbb_LightCWRotation ))
		{	
			if (this->m_SystemStatus != READY) return;

			if(__try_cast<String*>(this->tbc_LightRotationAxis->get_ComboBox()->get_SelectedItem())->Equals(S"X"))
			{
				//Rotationmatrix about x-axis for the transformation in world coord system (Light)
				Matrix4 *rotXWorld   = Matrix4::CreateRotXMatrix(-ROTATIONSTEP);
				//Rotationmatrix about x-axis for the transformation in the display coord system (LightArrowObject)
				Matrix4 *rotXDisplay = Matrix4::CreateRotXMatrix(ROTATIONSTEP);

				//Set new Light Direction
				this->m_Light->modifyOffsetRotation(rotXWorld);
				this->m_Light->calculateLightDirection();
				setLightDirectionDisplay();

				//Draw new Light Object
				this->m_Canvas->modifyLightObjectGLOffsetRotation(rotXDisplay);
				this->m_Canvas->drawLightArrow();
			}

			if(__try_cast<String*>(this->tbc_LightRotationAxis->get_ComboBox()->get_SelectedItem())->Equals(S"Y"))
			{
				//Rotationmatrix about y-axis for the transformation in world coord system (Light)
				Matrix4 *rotYWorld   = Matrix4::CreateRotYMatrix(-ROTATIONSTEP);
				//Rotationmatrix about y-axis for the transformation in the display coord system (LightArrowObject)
				Matrix4 *rotYDisplay = Matrix4::CreateRotYMatrix(-ROTATIONSTEP);

				//Set new Light Direction
				this->m_Light->modifyOffsetRotation(rotYWorld);
				this->m_Light->calculateLightDirection();
				setLightDirectionDisplay();

				//Draw new Light Object
				this->m_Canvas->modifyLightObjectGLOffsetRotation(rotYDisplay);
				this->m_Canvas->drawLightArrow();
			}

			if(__try_cast<String*>(this->tbc_LightRotationAxis->get_ComboBox()->get_SelectedItem())->Equals(S"Z"))
			{
				//Rotationmatrix about z-axis for the transformation in world coord system (Light)
				Matrix4 *rotZWorld   = Matrix4::CreateRotZMatrix(-ROTATIONSTEP);
				//Rotationmatrix about z-axis for the transformation in the display coord system (LightArrowObject)
				Matrix4 *rotZDisplay = Matrix4::CreateRotZMatrix(-ROTATIONSTEP);

				//Set new Light Direction
				this->m_Light->modifyOffsetRotation(rotZWorld);
				this->m_Light->calculateLightDirection();
				setLightDirectionDisplay();

				//Draw new Light Object
				this->m_Canvas->modifyLightObjectGLOffsetRotation(rotZDisplay);
				this->m_Canvas->drawLightArrow();
			}
			return;
		}

		if (__try_cast<TD::SandBar::ButtonItem*>(sender)->Equals( this->tbb_LightCCWRotation ))
		{	
			if (this->m_SystemStatus != READY) return;

			if(__try_cast<String*>(this->tbc_LightRotationAxis->get_ComboBox()->get_SelectedItem())->Equals(S"X"))
			{
				//Rotationmatrix about x-axis for the transformation in world coord system (Light)
				Matrix4 *rotXWorld   = Matrix4::CreateRotXMatrix(ROTATIONSTEP);
				//Rotationmatrix about x-axis for the transformation in the display coord system (LightArrowObject)
				Matrix4 *rotXDisplay = Matrix4::CreateRotXMatrix(-ROTATIONSTEP);

				//Set new Light Direction
				this->m_Light->modifyOffsetRotation(rotXWorld);
				this->m_Light->calculateLightDirection();
				setLightDirectionDisplay();

				//Draw new Light Object
				this->m_Canvas->modifyLightObjectGLOffsetRotation(rotXDisplay);
				this->m_Canvas->drawLightArrow();
			}

			if(__try_cast<String*>(this->tbc_LightRotationAxis->get_ComboBox()->get_SelectedItem())->Equals(S"Y"))
			{
				//Rotationmatrix about y-axis for the transformation in world coord system (Light)
				Matrix4 *rotYWorld   = Matrix4::CreateRotYMatrix(ROTATIONSTEP);
				//Rotationmatrix about y-axis for the transformation in the display coord system (LightArrowObject)
				Matrix4 *rotYDisplay = Matrix4::CreateRotYMatrix(ROTATIONSTEP);

				//Set new Light Direction
				this->m_Light->modifyOffsetRotation(rotYWorld);
				this->m_Light->calculateLightDirection();
				setLightDirectionDisplay();

				//Draw new Light Object
				this->m_Canvas->modifyLightObjectGLOffsetRotation(rotYDisplay);
				this->m_Canvas->drawLightArrow();
			}

			if(__try_cast<String*>(this->tbc_LightRotationAxis->get_ComboBox()->get_SelectedItem())->Equals(S"Z"))
			{
				//Rotationmatrix about z-axis for the transformation in world coord system (Light)
				Matrix4 *rotZWorld   = Matrix4::CreateRotZMatrix(ROTATIONSTEP);
				//Rotationmatrix about z-axis for the transformation in the display coord system (LightArrowObject)
				Matrix4 *rotZDisplay = Matrix4::CreateRotZMatrix(ROTATIONSTEP);

				//Set new Light Direction
				this->m_Light->modifyOffsetRotation(rotZWorld);
				this->m_Light->calculateLightDirection();
				setLightDirectionDisplay();

				//Draw new Light Object
				this->m_Canvas->modifyLightObjectGLOffsetRotation(rotZDisplay);
				this->m_Canvas->drawLightArrow();
			}
			return;
		}
	}

	private: System::Void perform_ActionTabPanelSelectedIndexChanged(System::Object *  sender, System::EventArgs *  e)
	{
		if (this->cob_TransObj->get_SelectedIndex() < 0) return;

		IRenderer* renderer = this->m_VolumeData->m_Dataset->getObjectMember(this->cob_TransObj->get_SelectedIndex())->get_RenderModel();

		if (this->tab_RenderControl->get_SelectedTab()->Equals( this->tabPage_DVR ))
		{
			this->m_TransFuncInput = NULL;
			this->pan_TFPreview->Invalidate();
			this->pan_TFPreviewColor->Invalidate();

			if (renderer->getRenderMode() == RendererSW::TF_PHONG_DVR)
			{
				this->m_TransFuncInput = __try_cast<PhongShadingModelSW*>(renderer->getShader())->TransferFunc;

				this->nud_Ambient->set_Value(Convert::ToDecimal(__try_cast<PhongShadingModelSW*>(renderer->getShader())->Ambient));
				this->nud_Diffuse->set_Value(Convert::ToDecimal(__try_cast<PhongShadingModelSW*>(renderer->getShader())->Diffuse));
				this->nud_Specular->set_Value(Convert::ToDecimal(__try_cast<PhongShadingModelSW*>(renderer->getShader())->Specular));
				this->nud_SpExponent->set_Value(Convert::ToDecimal(__try_cast<PhongShadingModelSW*>(renderer->getShader())->SpExponent));
				this->nud_DepthK1->set_Value(Convert::ToDecimal(__try_cast<PhongShadingModelSW*>(renderer->getShader())->DepthK1));
				this->nud_DepthK2->set_Value(Convert::ToDecimal(__try_cast<PhongShadingModelSW*>(renderer->getShader())->DepthK2));
			}
			else
			{
				this->m_TransFuncInput = Transferfunction::CreateGeneralTransfer(DataLoader::DensityScope);

				this->nud_Ambient->set_Value(Convert::ToDecimal(DataLoader::Default_Ambient));
				this->nud_Diffuse->set_Value(Convert::ToDecimal(DataLoader::Default_Diffuse));
				this->nud_Specular->set_Value(Convert::ToDecimal(DataLoader::Default_Specular));
				this->nud_SpExponent->set_Value(Convert::ToDecimal(DataLoader::Default_SpExponent));
				this->nud_DepthK1->set_Value(Convert::ToDecimal(DataLoader::Default_K1DepthCueing));
				this->nud_DepthK2->set_Value(Convert::ToDecimal(DataLoader::Default_K2DepthCueing));
			}

			return;
		}

		if (this->tab_RenderControl->get_SelectedTab()->Equals( this->tabPage_MIP ))
		{
			this->m_TransFuncInput = NULL;

			if (renderer->getRenderMode() == RendererSW::LIN_MIP_LMIP)
			{
				this->m_TransFuncInput = new Transferfunction();

				this->rab_MipLinClassification->set_Checked(true);
				this->trb_MipThreshold->set_Value(Convert::ToInt16(Math::Round((double)__try_cast<LMipCompositingModelSW*>(renderer->getCompositor())->get_Threshold())));
				this->tex_MipThreshold->set_Text(Convert::ToString(this->trb_MipThreshold->get_Value()));

				MipShadingModelSW *shader = __try_cast<MipShadingModelSW*>(renderer->getShader());

				this->colDia_MipColor->set_Color(shader->get_Col());
				this->lab_MipColor->set_BackColor(this->colDia_MipColor->get_Color());
			}

			else if(renderer->getRenderMode() == RendererSW::TF_MIP_LMIP)
			{
				this->m_TransFuncInput = __try_cast<TFClassificationModelSW*>(renderer->getClassificator())->TransferFunc;
				
				this->rab_MipTFClassification->set_Checked(true);
				this->trb_MipThreshold->set_Value(Convert::ToInt16(Math::Round((double)__try_cast<LMipCompositingModelSW*>(renderer->getCompositor())->get_Threshold())));
				this->tex_MipThreshold->set_Text(Convert::ToString(this->trb_MipThreshold->get_Value()));

				MipShadingModelSW *shader = __try_cast<MipShadingModelSW*>(renderer->getShader());

				this->colDia_MipColor->set_Color(shader->get_Col());
				this->lab_MipColor->set_BackColor(this->colDia_MipColor->get_Color());
			}

			else
			{
				this->m_TransFuncInput = new Transferfunction();

				this->rab_MipLinClassification->set_Checked(true);
				this->trb_MipThreshold->set_Value(Convert::ToInt16(Math::Round((double)DataLoader::Default_MipThreshold)));
				this->tex_MipThreshold->set_Text(Convert::ToString(this->trb_MipThreshold->get_Value()));
				this->colDia_MipColor->set_Color(DataLoader::Default_MipColor);
				this->lab_MipColor->set_BackColor(this->colDia_MipColor->get_Color());
			}
			return;
		}

		if (this->tab_RenderControl->get_SelectedTab()->Equals( this->tabPage_Contour ))
		{
			this->m_TransFuncInput = NULL;

			if (renderer->getRenderMode() == RendererSW::LIN_CONTOUR_LMIP)
			{
				this->m_TransFuncInput = new Transferfunction();

				this->rab_ContourLMip->set_Checked(true);
				this->trb_ContourThreshold->set_Value(Convert::ToInt16(Math::Round((double)__try_cast<LMipCompositingModelSW*>(renderer->getCompositor())->get_Threshold() * 100.0f)));
				this->tex_ContourThreshold->set_Text(Convert::ToString(this->trb_ContourThreshold->get_Value() / 100.0f));

				ContourShadingModelSW* shader = __try_cast<ContourShadingModelSW*>(renderer->getShader());

				this->colDia_ContourColor->set_Color(shader->get_Col());
				this->lab_ContourColor->set_BackColor(this->colDia_ContourColor->get_Color());
				this->nud_ContourGradient->set_Value(Convert::ToDecimal(shader->get_ContourGradientMagnitudeThreshold()));
				this->nud_ContourExponent->set_Value(Convert::ToDecimal(shader->get_Exponent()));
				this->nud_ContourKTF->set_Value(Convert::ToDecimal(DataLoader::Default_ContourKTF));
	
			}
			else if (renderer->getRenderMode() == RendererSW::TF_CONTOUR_DVR)
			{
				this->m_TransFuncInput = __try_cast<TFClassificationModelSW*>(renderer->getClassificator())->TransferFunc;
				
				this->rab_ContourDvr->set_Checked(true);
				this->trb_ContourThreshold->set_Value(Convert::ToInt16(Math::Round((double)DataLoader::Default_ContourThreshold * 100.0f)));
				this->tex_ContourThreshold->set_Text(Convert::ToString((float)this->trb_ContourThreshold->get_Value() / 100.0f));

				ContourShadingModelSW* shader = __try_cast<ContourShadingModelSW*>(renderer->getShader());

				this->colDia_ContourColor->set_Color(shader->get_Col());
				this->lab_ContourColor->set_BackColor(this->colDia_ContourColor->get_Color());
				this->nud_ContourGradient->set_Value(Convert::ToDecimal(shader->get_ContourGradientMagnitudeThreshold()));
				this->nud_ContourExponent->set_Value(Convert::ToDecimal(shader->get_Exponent()));
				this->nud_ContourKTF->set_Value(Convert::ToDecimal(shader->get_ContourKTF()));
			}	 
			else
			{
				this->m_TransFuncInput = new Transferfunction();

				this->rab_ContourLMip->set_Checked(true);
				this->trb_ContourThreshold->set_Value(Convert::ToInt16(Math::Round((double)DataLoader::Default_ContourThreshold * 100.0f)));
				this->tex_ContourThreshold->set_Text(Convert::ToString((float)this->trb_ContourThreshold->get_Value() / 100.0f));
				this->colDia_ContourColor->set_Color(DataLoader::Default_ContourColor);
				this->lab_ContourColor->set_BackColor(this->colDia_ContourColor->get_Color());
				this->nud_ContourGradient->set_Value(Convert::ToDecimal(DataLoader::Default_ContourGradientThreshold));
				this->nud_ContourExponent->set_Value(Convert::ToDecimal(DataLoader::Default_ContourExponent));
				this->nud_ContourKTF->set_Value(Convert::ToDecimal(DataLoader::Default_ContourKTF));
			}
			return;
		}

		if (this->tab_RenderControl->get_SelectedTab()->Equals( this->tabPage_Tone ))
		{
			this->m_TransFuncInput = NULL;

			if (renderer->getRenderMode() == RendererSW::LIN_TONE_LMIP)
			{
				this->m_TransFuncInput = new Transferfunction();

				this->rab_ToneLMip->set_Checked(true);
				this->trb_ToneThreshold->set_Value(Convert::ToInt16(Math::Round((double)__try_cast<LMipCompositingModelSW*>(renderer->getCompositor())->get_Threshold() * 100.0f)));
				this->tex_ToneThreshold->set_Text(Convert::ToString(this->trb_ToneThreshold->get_Value() / 100.0f));
				ToneShadingModelSW* shader = __try_cast<ToneShadingModelSW*>(renderer->getShader());
				this->nud_ToneGradient->set_Value(Convert::ToDecimal(shader->get_ToneGradientMagnitudeThreshold()));
				this->nud_ToneKW->set_Value(Convert::ToDecimal(shader->get_ToneKWarm()));
				this->nud_ToneKC->set_Value(Convert::ToDecimal(shader->get_ToneKCold()));
				this->colDia_ToneColorCold->set_Color(shader->get_ToneColdColor());
				this->colDia_ToneColorWarm->set_Color(shader->get_ToneWarmColor());
				this->nud_ToneKTF->set_Value(Convert::ToDecimal(DataLoader::Default_ToneKTF));
			}
			else if (renderer->getRenderMode() == RendererSW::TF_TONE_DVR)
			{
				this->m_TransFuncInput = __try_cast<TFClassificationModelSW*>(renderer->getClassificator())->TransferFunc;
				
				this->rab_ToneDvr->set_Checked(true);
				this->trb_ToneThreshold->set_Value(Convert::ToInt16(Math::Round((double)DataLoader::Default_ToneThreshold * 100.0f)));
				this->tex_ToneThreshold->set_Text(Convert::ToString((float)this->trb_ToneThreshold->get_Value() / 100.0f));
				ToneShadingModelSW* shader = __try_cast<ToneShadingModelSW*>(renderer->getShader());
				this->nud_ToneGradient->set_Value(Convert::ToDecimal(shader->get_ToneGradientMagnitudeThreshold()));
				this->nud_ToneKW->set_Value(Convert::ToDecimal(shader->get_ToneKWarm()));
				this->nud_ToneKC->set_Value(Convert::ToDecimal(shader->get_ToneKCold()));
				this->colDia_ToneColorCold->set_Color(shader->get_ToneColdColor());
				this->colDia_ToneColorWarm->set_Color(shader->get_ToneWarmColor());
				this->nud_ToneKTF->set_Value(Convert::ToDecimal(shader->get_ToneKTF()));
				
			}
			else
			{
				this->m_TransFuncInput = new Transferfunction();

				this->rab_ToneLMip->set_Checked(true);
				this->trb_ToneThreshold->set_Value(Convert::ToInt16(Math::Round((double)DataLoader::Default_ToneThreshold * 100.0f)));
				this->tex_ToneThreshold->set_Text(Convert::ToString((float)this->trb_ToneThreshold->get_Value() / 100.0f));
				this->nud_ToneGradient->set_Value(Convert::ToDecimal(DataLoader::Default_ToneMagnitudeThreshold));
				this->nud_ToneKW->set_Value(Convert::ToDecimal(DataLoader::Default_ToneKWarm));
				this->nud_ToneKC->set_Value(Convert::ToDecimal(DataLoader::Default_ToneKCold));
				this->colDia_ToneColorCold->set_Color(DataLoader::Default_ToneColdColor);
				this->colDia_ToneColorWarm->set_Color(DataLoader::Default_ToneWarmColor);
				this->nud_ToneKTF->set_Value(Convert::ToDecimal(DataLoader::Default_ToneKTF));
			}

		}
	}

	private: System::Void perform_ActionRadioButtonCheckedChanged(System::Object *  sender, System::EventArgs *  e)
		{
			if (__try_cast<RadioButton*>(sender)->Equals(this->rab_ToneLMip))
			{
				this->but_ToneTransferFunc->Enabled = false;

				this->lab_ToneThreshold->Enabled = true;
				this->tex_ToneThreshold->Enabled = true;
				this->trb_ToneThreshold->Enabled = true;
			}

			else if (__try_cast<RadioButton*>(sender)->Equals(this->rab_ToneDvr))
			{
				this->but_ToneTransferFunc->Enabled = true;

				this->lab_ToneThreshold->Enabled = false;
				this->tex_ToneThreshold->Enabled = false;
				this->trb_ToneThreshold->Enabled = false;
			}

			else if (__try_cast<RadioButton*>(sender)->Equals(this->rab_ContourLMip))
			{
				this->but_ContourTransferFunc->Enabled = false;

				this->lab_ContourThreshold->Enabled = true;
				this->tex_ContourThreshold->Enabled = true;
				this->trb_ContourThreshold->Enabled = true;
			}

			else if (__try_cast<RadioButton*>(sender)->Equals(this->rab_ContourDvr))
			{
				this->but_ContourTransferFunc->Enabled = true;

				this->lab_ContourThreshold->Enabled = false;
				this->tex_ContourThreshold->Enabled = false;
				this->trb_ContourThreshold->Enabled = false;
			}

			else if (__try_cast<RadioButton*>(sender)->Equals(this->rab_MipLinClassification))
			{
				this->lab_MipNote->Enabled = false;
				this->but_MipTransferFunc->Enabled = false;
			}

			else if (__try_cast<RadioButton*>(sender)->Equals(this->rab_MipTFClassification))
			{
				this->lab_MipNote->Enabled = true;
				this->but_MipTransferFunc->Enabled = true;
			}
		}

	private: System::Void perform_ActionSaveDialogOK(System::Object *  sender, System::ComponentModel::CancelEventArgs *  e)
	{
		if (__try_cast<SaveFileDialog*>(sender)->Equals( this->saveDia_SaveConfigFile ))
		{
			 if(!saveDia_SaveConfigFile->FileName->ToLower()->EndsWith(S".xml"))
			 {
				 MessageBox::Show(this, S"The choosen file is apparently not a .xml file.", S"Error by writing the .xml-File", MessageBoxButtons::OK, MessageBoxIcon::Error);
				 return;
			 }
			 try
			 {
				XMLParser *configFileWriter = new XMLParser(this->saveDia_SaveConfigFile->get_FileName());

				Array *newDefaultValues = Array::CreateInstance(__typeof(System::Object), 36);
				unsigned short index = 0;

				//DataFile Settings
				newDefaultValues->SetValue(__box(DataLoader::DensityScope), index++);
				newDefaultValues->SetValue(DataLoader::Default_DataFileDirectory, index++);
				newDefaultValues->SetValue(DataLoader::Default_MaskFileDirectory, index++);
				newDefaultValues->SetValue(DataLoader::Default_TransFuncFileDirectory, index++);
				newDefaultValues->SetValue(DataLoader::Default_ConfigFileDirectory, index++);
				newDefaultValues->SetValue(__box(DataLoader::MaxUsableMemory), index++);

				//Phongshading Settings
				newDefaultValues->SetValue(__box(this->nud_Ambient->get_Value()), index++);
				newDefaultValues->SetValue(__box(this->nud_Diffuse->get_Value()), index++);
				newDefaultValues->SetValue(__box(this->nud_Specular->get_Value()), index++);
				newDefaultValues->SetValue(__box(this->nud_SpExponent->get_Value()), index++);
				
				//Optimation Settings
				newDefaultValues->SetValue(__box((float)this->trb_SamplingDist->get_Value() / (float)(this->trb_SamplingDist->get_Maximum() * 0.5)), index++);
				newDefaultValues->SetValue(__box(this->nud_GradientThreshold->get_Value()), index++);
				newDefaultValues->SetValue(__box(this->nud_OpacityThreshold->get_Value()), index++);

				//Depth Cueing Settings
				newDefaultValues->SetValue(__box(this->nud_DepthK1->get_Value()), index++);
				newDefaultValues->SetValue(__box(this->nud_DepthK2->get_Value()), index++);
				
				//MIP Shading Settings
				newDefaultValues->SetValue(__box(this->trb_MipThreshold->get_Value()), index++);
				newDefaultValues->SetValue(__box(this->colDia_MipColor->get_Color().get_R()), index++);
				newDefaultValues->SetValue(__box(this->colDia_MipColor->get_Color().get_G()), index++);
				newDefaultValues->SetValue(__box(this->colDia_MipColor->get_Color().get_B()), index++);

				//Contour Shading Settings
				newDefaultValues->SetValue(__box(this->trb_ContourThreshold->get_Value()), index++);
				newDefaultValues->SetValue(__box(this->nud_ContourGradient->get_Value()), index++);
				newDefaultValues->SetValue(__box(this->nud_ContourExponent->get_Value()), index++);
				newDefaultValues->SetValue(__box(this->nud_ContourKTF->get_Value()), index++);
				newDefaultValues->SetValue(__box(this->colDia_ContourColor->get_Color().get_R()), index++);
				newDefaultValues->SetValue(__box(this->colDia_ContourColor->get_Color().get_G()), index++);
				newDefaultValues->SetValue(__box(this->colDia_ContourColor->get_Color().get_B()), index++);

				//Contour Shading Settings
				newDefaultValues->SetValue(__box(this->trb_ToneThreshold->get_Value()), index++);
				newDefaultValues->SetValue(__box(this->nud_ToneGradient->get_Value()), index++);
				newDefaultValues->SetValue(__box(this->nud_ToneKW->get_Value()), index++);
				newDefaultValues->SetValue(__box(this->nud_ToneKC->get_Value()), index++);
				newDefaultValues->SetValue(__box(this->nud_ToneKTF->get_Value()), index++);
				newDefaultValues->SetValue(__box(this->colDia_ToneColorWarm->get_Color().get_R()), index++);
				newDefaultValues->SetValue(__box(this->colDia_ToneColorWarm->get_Color().get_G()), index++);
				newDefaultValues->SetValue(__box(this->colDia_ToneColorWarm->get_Color().get_B()), index++);
				newDefaultValues->SetValue(__box(this->colDia_ToneColorCold->get_Color().get_R()), index++);
				newDefaultValues->SetValue(__box(this->colDia_ToneColorCold->get_Color().get_G()), index++);
				newDefaultValues->SetValue(__box(this->colDia_ToneColorCold->get_Color().get_B()), index++);

				//Start writing all the configuration data to file
				configFileWriter->save_ConfigFile(newDefaultValues);

				MessageBox::Show(this, 
								 S"The configuration data is saved successfully!",
								 S"Configuration saving process", MessageBoxButtons::OK, MessageBoxIcon::Information);
			 }
			 catch(Exception *e)
			 {
				MessageBox::Show(this, e->get_Message(), S"Configuration saving process", MessageBoxButtons::OK, MessageBoxIcon::Error);
			 }
			 return;
		}

		
		if (__try_cast<SaveFileDialog*>(sender)->Equals( this->saveDia_SaveTransFunc ))
		{
			if(!saveDia_SaveTransFunc->FileName->ToLower()->EndsWith(S".xml"))
			{
				 MessageBox::Show(this, S"The given file name has apparently not a .xml extension.", S"Error by writing the .xml-File", MessageBoxButtons::OK, MessageBoxIcon::Error);
				 return;
			}

			IRenderer *renderer = this->m_VolumeData->m_Dataset->getObjectMember(this->cob_TransObj->get_SelectedIndex())->get_RenderModel();

			if (renderer->getClassificator()->GetType() == __typeof( TFClassificationModelSW ))
			{
				Transferfunction *transFunc = __try_cast<TFClassificationModelSW*>(renderer->getClassificator())->get_TransferFunc();
				try
				{
					XMLParser *transFuncWriter = new XMLParser(saveDia_SaveTransFunc->get_FileName());

					//start writung the transfer function to file
					transFuncWriter->save_Transferfunction(transFunc);

					MessageBox::Show(this, 
								 String::Concat(S"The selected transfer function of the object ",
			    								 this->cob_TransObj->get_SelectedItem(),
												 S" is saved successfully!"),
								 S"Transfer Function saving process", MessageBoxButtons::OK, MessageBoxIcon::Information);
				}
				catch(Exception *e)
				{
					MessageBox::Show(this, e->get_Message(), S"Error by writing the .xml-File", MessageBoxButtons::OK, MessageBoxIcon::Error);
				}
			}
			else
			{
				MessageBox::Show(this, 
								 S"Please choose an object with a valid included transfer function!",
								 S"Transfer Function saving process", MessageBoxButtons::OK, MessageBoxIcon::Error);
			}
			return;
		}

		if (__try_cast<SaveFileDialog*>(sender)->Equals( this->saveDia_SaveRenderedPicture ))
		{	
			Bitmap *tempBitmap = DrawModelGLSW::GetBitmap(this->m_Canvas->get_DrawModel()->getFrameBuffer(),
															this->m_Canvas->get_DrawModel()->CanvasWidth,
															this->m_Canvas->get_DrawModel()->CanvasHeight,
															this->m_Canvas->get_DrawModel()->TextureWidth);

			if(saveDia_SaveRenderedPicture->FileName->ToLower()->EndsWith(S".jpg"))
			{
				tempBitmap->Save(this->saveDia_SaveRenderedPicture->get_FileName(), ImageFormat::Jpeg);
				//this->m_Canvas->get_DrawModel()->getBitmap()->Save(this->saveDia_SaveRenderedPicture->get_FileName(), ImageFormat::Jpeg);
			}
			else
			if(saveDia_SaveRenderedPicture->FileName->ToLower()->EndsWith(S".gif"))
			{
				tempBitmap->Save(this->saveDia_SaveRenderedPicture->get_FileName(), ImageFormat::Gif);
				/*this->m_Canvas->get_DrawModel()->getBitmap()->Save(this->saveDia_SaveRenderedPicture->get_FileName(), ImageFormat::Gif);*/
			}
			else
			if(saveDia_SaveRenderedPicture->FileName->ToLower()->EndsWith(S".tiff"))
			{
				tempBitmap->Save(this->saveDia_SaveRenderedPicture->get_FileName(), ImageFormat::Tiff);
				/*this->m_Canvas->get_DrawModel()->getBitmap()->Save(this->saveDia_SaveRenderedPicture->get_FileName(), ImageFormat::Tiff);*/
			}
			else
			if(saveDia_SaveRenderedPicture->FileName->ToLower()->EndsWith(S".png"))
			{
				tempBitmap->Save(this->saveDia_SaveRenderedPicture->get_FileName(), ImageFormat::Png);
				/*this->m_Canvas->get_DrawModel()->getBitmap()->Save(this->saveDia_SaveRenderedPicture->get_FileName(), ImageFormat::Png);*/
			}
			else
			if(saveDia_SaveRenderedPicture->FileName->ToLower()->EndsWith(S".bmp"))
			{
				tempBitmap->Save(this->saveDia_SaveRenderedPicture->get_FileName(), ImageFormat::Bmp);
				/*this->m_Canvas->get_DrawModel()->getBitmap()->Save(this->saveDia_SaveRenderedPicture->get_FileName(), ImageFormat::Bmp);*/
			}
			else
			{
				MessageBox::Show(this, 
								 String::Concat(S"The choosen file format \"", 
											    saveDia_SaveRenderedPicture->FileName->Substring(saveDia_SaveRenderedPicture->FileName->LastIndexOf('.'), 
																								 saveDia_SaveRenderedPicture->FileName->Length-1), 
									            "\" is an unknown image file format."),
								 S"Image Saving Error", MessageBoxButtons::OK, MessageBoxIcon::Error);
			}
			return;
		}
	}

	private: System::Void perform_ActionNudValueChanged(System::Object *  sender, System::EventArgs *  e) 
		 {
			 if (__try_cast<NumericUpDown*>(sender)->Equals( this->nud_OpacityThreshold ))
			 {
				 if (this->m_PreviewCanvas)
					 this->m_PreviewCanvas->set_OpacityThreshold(Convert::ToSingle(this->nud_OpacityThreshold->Value));
				 return;
			 }

			 if (__try_cast<NumericUpDown*>(sender)->Equals( this->nud_GradientThreshold ))
			 {
				 if (this->m_PreviewCanvas)
					 this->m_PreviewCanvas->set_GradientThreshold(Convert::ToSingle(this->nud_GradientThreshold->Value));
				 return;
			 }

			 if (__try_cast<NumericUpDown*>(sender)->Equals( this->nud_IDVRObjectImportance ))
			 {
				 this->m_VolumeData->m_Dataset->getObjectMember((unsigned short)this->cob_IDVRObject->get_SelectedIndex())->set_Importance(Convert::ToUInt16(this->nud_IDVRObjectImportance->get_Value()));
			 }
		 }

	private: System::Void perform_ActionCheckBoxCheckedChanged(System::Object *  sender, System::EventArgs *  e)
	{

		if (__try_cast<CheckBox*>(sender)->Equals( this->chb_TwoLevelVR ))
		{
			if (this->m_PreviewCanvas)
				this->m_PreviewCanvas->set_TwoLevelRendering(this->chb_TwoLevelVR->Checked);
			
			if(!this->chb_TwoLevelVR->Checked)
			{
				this->chb_IDVR->set_Checked(false);
			}
			return;
		}

		if (__try_cast<CheckBox*>(sender)->Equals( this->chb_IDVR ))
		{
			if(this->chb_IDVR->Checked)
			{
				this->chb_TwoLevelVR->set_Checked(true);
				if (this->m_PreviewCanvas)
				{
					this->m_PreviewCanvas->set_TwoLevelRendering(this->chb_TwoLevelVR->Checked);
					this->m_PreviewCanvas->set_IDVR(this->chb_TwoLevelVR->Checked);

				}

				//Activate Importance controls
				this->lab_IDVRObject->Enabled = true;
				this->cob_IDVRObject->Enabled = true;
				this->nud_IDVRObjectImportance->Enabled = true;
				this->lab_IDVRCompositing->Enabled = true;
				this->cob_IDVRCompositing->Enabled = true;
				this->lab_IDVRModulation->Enabled = true;
				this->cob_IDVRModulation->Enabled = true;
				this->but_IDVRAssign->Enabled = true;
				this->but_IDVRReset->Enabled = true;
				this->lab_IDVRCountersinkSlope->Enabled = true;
				this->nud_IDVRCountersinkSlope->Enabled = true;
				this->lab_IDVRGScale->Enabled = true;
				this->nud_IDVRGScale->Enabled = true;
	    	}

			else
	    	{
				//Deactivate Importance controls
				this->lab_IDVRObject->Enabled = false;
				this->cob_IDVRObject->Enabled = false;
				this->nud_IDVRObjectImportance->Enabled = false;
				this->lab_IDVRCompositing->Enabled = false;
				this->cob_IDVRCompositing->Enabled = false;
				this->lab_IDVRModulation->Enabled = false;
				this->cob_IDVRModulation->Enabled = false;
				this->but_IDVRAssign->Enabled = false;
				this->but_IDVRReset->Enabled = false;
				this->lab_IDVRCountersinkSlope->Enabled = false;
				this->nud_IDVRCountersinkSlope->Enabled = false;
				this->lab_IDVRGScale->Enabled = false;
				this->nud_IDVRGScale->Enabled = false;
			 }
			 return;
		}
	}
};
}